@@ -1,4 +1,4 @@
-/* Copyright 2021 Stanford University, NVIDIA Corporation
+/* Copyright 2023 Stanford University, NVIDIA Corporation
  *
  * Licensed under the Apache License, Version 2.0 (the "License");
  * you may not use this file except in compliance with the License.
@@ -234,7 +234,7 @@ legion_terra_index_cross_product_create_multi(
                              partitions[0],
                              partitions.begin() + 1, partitions.end(), colors, 0);
 
-  colors_[0] = runtime->get_index_partition_color(ctx, partitions[0]);
+  colors_[0] = runtime->get_index_partition_color(partitions[0]);
   std::copy(colors.begin(), colors.end(), colors_+1);
 
   legion_terra_index_cross_product_t result;
@@ -243,6 +243,19 @@ legion_terra_index_cross_product_create_multi(
   return result;
 }
 
+legion_terra_index_cross_product_t
+legion_terra_index_cross_product_import(
+  legion_index_partition_t first_partition_,
+  legion_color_t second_color)
+{
+  IndexPartition first_partition = CObjectWrapper::unwrap(first_partition_);
+
+  legion_terra_index_cross_product_t result;
+  result.partition = CObjectWrapper::wrap(first_partition);
+  result.other_color = second_color;
+  return result;
+}
+
 static IndexSpace
 unwrap_list(legion_terra_index_space_list_t list,
             std::vector<IndexSpace> &subspaces)
@@ -621,17 +634,15 @@ legion_terra_index_cross_product_get_partition(
 legion_index_partition_t
 legion_terra_index_cross_product_get_subpartition_by_color_domain_point(
   legion_runtime_t runtime_,
-  legion_context_t ctx_,
   legion_terra_index_cross_product_t prod,
   legion_domain_point_t color_)
 {
   Runtime *runtime = CObjectWrapper::unwrap(runtime_);
-  Context ctx = CObjectWrapper::unwrap(ctx_)->context();
   IndexPartition partition = CObjectWrapper::unwrap(prod.partition);
   DomainPoint color = CObjectWrapper::unwrap(color_);
 
-  IndexSpace is = runtime->get_index_subspace(ctx, partition, color);
-  IndexPartition ip = runtime->get_index_partition(ctx, is, prod.other_color);
+  IndexSpace is = runtime->get_index_subspace(partition, color);
+  IndexPartition ip = runtime->get_index_partition(is, prod.other_color);
   return CObjectWrapper::wrap(ip);
 }
 
