@@ -1,4 +1,4 @@
-/* Copyright 2021 Stanford University, Los Alamos National Laboratory
+/* Copyright 2023 Stanford University, Los Alamos National Laboratory
  *
  * Licensed under the Apache License, Version 2.0 (the "License");
  * you may not use this file except in compliance with the License.
@@ -18,6 +18,8 @@
 #include <cassert>
 #include <cstdlib>
 #include <sys/time.h>
+#include <math.h>
+#include <limits>
 #include "legion.h"
 using namespace Legion;
 
@@ -56,6 +58,10 @@ double get_cur_time() {
   return cur_time;
 }
 
+bool compare_double(double a, double b)
+{
+  return fabs(a - b) < std::numeric_limits<double>::epsilon();
+}
 
 void top_level_task(const Task *task,
                     const std::vector<PhysicalRegion> &regions,
@@ -107,10 +113,6 @@ void top_level_task(const Task *task,
   double *z_ptr = NULL;
   double *xy_ptr = NULL;
   double *xyz_ptr = NULL;
-  const Memory local_sysmem = Machine::MemoryQuery(Machine::get_machine())
-      .has_affinity_to(runtime->get_executing_processor(ctx))
-      .only_kind(Memory::SYSTEM_MEM)
-      .first();
   if (soa_flag == 0) 
   { // SOA
     xy_ptr = (double*)malloc(2*sizeof(double)*(num_elements));
@@ -127,8 +129,7 @@ void top_level_task(const Task *task,
       std::vector<FieldID> attach_fields(2);
       attach_fields[0] = FID_X;
       attach_fields[1] = FID_Y;
-      launcher.attach_array_soa(xy_ptr, false/*column major*/,
-                                attach_fields, local_sysmem);
+      launcher.attach_array_soa(xy_ptr, false/*column major*/, attach_fields);
       xy_pr = runtime->attach_external_resource(ctx, launcher);
     }
     { 
@@ -136,8 +137,7 @@ void top_level_task(const Task *task,
       AttachLauncher launcher(EXTERNAL_INSTANCE, output_lr, output_lr);
       std::vector<FieldID> attach_fields(1);
       attach_fields[0] = FID_Z;
-      launcher.attach_array_soa(z_ptr, false/*column major*/,
-                                attach_fields, local_sysmem);
+      launcher.attach_array_soa(z_ptr, false/*column major*/, attach_fields);
       z_pr = runtime->attach_external_resource(ctx, launcher);
     }
   } 
@@ -155,14 +155,14 @@ void top_level_task(const Task *task,
     {
       AttachLauncher launcher(EXTERNAL_INSTANCE, input_lr, input_lr);
       launcher.attach_array_aos(xyz_ptr, false/*column major*/,
-                                layout_constraint_fields, local_sysmem);
+                                layout_constraint_fields);
       launcher.privilege_fields.erase(FID_Z);
       xy_pr = runtime->attach_external_resource(ctx, launcher);
     }
     {
       AttachLauncher launcher(EXTERNAL_INSTANCE, output_lr, output_lr);
       launcher.attach_array_aos(xyz_ptr, false/*column major*/,
-                                layout_constraint_fields, local_sysmem);
+                                layout_constraint_fields);
       launcher.privilege_fields.erase(FID_X);
       launcher.privilege_fields.erase(FID_Y);
       z_pr = runtime->attach_external_resource(ctx, launcher);
@@ -399,13 +399,17 @@ void check_task(const Task *task,
     // Probably shouldn't check for floating point equivalence but
     // the order of operations are the same should they should
     // be bitwise equal.
-    if (expected != received)
+    if (!compare_double(expected, received)) {
       all_passed = false;
+      printf("expected %f, received %f\n", expected, received);
+    }
   }
   if (all_passed)
     printf("SUCCESS!\n");
-  else
+  else {
     printf("FAILURE!\n");
+    abort();
+  }
 }
 
 int main(int argc, char **argv)
