@@ -1,4 +1,4 @@
-/* Copyright 2021 Stanford University
+/* Copyright 2023 Stanford University
  *
  * Licensed under the Apache License, Version 2.0 (the "License");
  * you may not use this file except in compliance with the License.
@@ -141,7 +141,7 @@ void InitWiresTask::cpu_base_impl(const Task *task,
     // Pick a random node within our piece
     int in_index = int(erand48(xsubi) * nodes_per_piece);
     fa_wire_in_ptr[*itr] = Point<1>(task->index_point[0] * nodes_per_piece + in_index);
-    if ((100 * erand48(xsubi)) < pct_wire_in_piece)
+    if ((num_pieces == 1) || ((100 * erand48(xsubi)) < pct_wire_in_piece))
     {
       // Stay within our piece
       int out_index = int(erand48(xsubi) * nodes_per_piece);
@@ -588,16 +588,16 @@ Partitions load_circuit(Circuit &ckt, std::vector<CircuitPiece> &pieces, Context
   for (int n = 0; n < num_pieces; n++)
   {
     pieces[n].pvt_nodes = runtime->get_logical_subregion_by_color(ctx, result.pvt_nodes, n);
-    sprintf(buf, "private_nodes_of_piece_%d", n);
+    snprintf(buf, sizeof(buf), "private_nodes_of_piece_%d", n);
     runtime->attach_name(pieces[n].pvt_nodes, buf);
     pieces[n].shr_nodes = runtime->get_logical_subregion_by_color(ctx, result.shr_nodes, n);
-    sprintf(buf, "shared_nodes_of_piece_%d", n);
+    snprintf(buf, sizeof(buf), "shared_nodes_of_piece_%d", n);
     runtime->attach_name(pieces[n].shr_nodes, buf);
     pieces[n].ghost_nodes = runtime->get_logical_subregion_by_color(ctx, result.ghost_nodes, n);
-    sprintf(buf, "ghost_nodes_of_piece_%d", n);
+    snprintf(buf, sizeof(buf), "ghost_nodes_of_piece_%d", n);
     runtime->attach_name(pieces[n].ghost_nodes, buf);
     pieces[n].pvt_wires = runtime->get_logical_subregion_by_color(ctx, result.pvt_wires, n);
-    sprintf(buf, "private_wires_of_piece_%d", n);
+    snprintf(buf, sizeof(buf), "private_wires_of_piece_%d", n);
     runtime->attach_name(pieces[n].pvt_wires, buf);
     pieces[n].num_wires = wires_per_piece;
     pieces[n].first_wire = Point<1>(n * wires_per_piece);
