@@ -1,6 +1,6 @@
 #!/usr/bin/env python3
 
-# Copyright 2021 Stanford University
+# Copyright 2023 Stanford University
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -19,18 +19,12 @@ from __future__ import print_function
 import argparse, hashlib, multiprocessing, os, platform, re, subprocess, sys, tempfile, traceback
 
 def discover_llvm_version():
-    if os.environ.get('LMOD_SYSTEM_NAME') == 'summit': # Summit doesn't set hostname
-        return '60'
-    elif os.environ.get('NERSC_HOST') == 'perlmutter':
-        return '110'
-    else:
-        return '60'
+    # standardize on LLVM 13.0 everywhere
+    return '130'
 
 def discover_skip_certificate_check():
-    if platform.node().startswith('titan'):
-        return True
-    else:
-        return False
+    # Elliott: I don't think any systems are sensitive to this anymore
+    return False
 
 def discover_conduit():
     if 'CONDUIT' in os.environ:
@@ -45,11 +39,18 @@ def discover_conduit():
         return 'psm'
     elif os.environ.get('LMOD_SYSTEM_NAME') == 'summit': # Summit doesn't set hostname
         return 'ibv'
+    elif os.environ.get('LMOD_SYSTEM_NAME') == 'crusher': # Crusher doesn't set hostname
+        return 'ofi-slingshot11'
     elif os.environ.get('NERSC_HOST') == 'perlmutter':
-        return 'ucx'
+        return 'ofi-slingshot11'
     else:
         raise Exception('Please set CONDUIT in your environment')
 
+def short_conduit(conduit):
+    if conduit is not None and conduit.startswith('ofi-'):
+        return 'ofi'
+    return conduit
+
 def gasnet_enabled():
     if 'USE_GASNET' in os.environ:
         return os.environ['USE_GASNET'] == '1'
@@ -87,9 +88,13 @@ def extract(dest_dir, archive_path, format):
 def apply_patch(dest_dir, diff_path, strip_levels=1):
     subprocess.check_call(['patch', '-p%d' % strip_levels, '-i', diff_path], cwd=dest_dir)
 
-def git_clone(repo_dir, url, branch=None):
+def git_clone(repo_dir, url, branch=None, commit_id=None):
     if branch is not None:
+        assert commit_id is None
         subprocess.check_call(['git', 'clone', '-b', branch, url, repo_dir])
+    elif commit_id is not None:
+        subprocess.check_call(['git', 'clone', url, repo_dir])
+        subprocess.check_call(['git', '-C', repo_dir, 'checkout', commit_id])
     else:
         subprocess.check_call(['git', 'clone', url, repo_dir])
 
@@ -98,38 +103,39 @@ def git_update(repo_dir):
         ['git', 'pull', '--ff-only'],
         cwd=repo_dir)
 
-def build_gasnet(gasnet_dir, conduit):
-    subprocess.check_call(['make', 'CONDUIT=%s' % conduit], cwd=gasnet_dir)
+def build_gasnet(gasnet_dir, conduit, gasnet_version):
+    subprocess.check_call(
+        ['make',
+         'CONDUIT=%s' % conduit,
+         'GASNET_VERSION=%s' % gasnet_version],
+        cwd=gasnet_dir)
 
-def build_llvm(source_dir, build_dir, install_dir, use_cmake, cmake_exe, thread_count, is_cray):
+def build_llvm(source_dir, build_dir, install_dir, is_project_build, cmake_exe, thread_count, is_cray):
     env = None
     if is_cray:
         env = dict(list(os.environ.items()) + [
             ('CC', os.environ['HOST_CC']),
             ('CXX', os.environ['HOST_CXX']),
         ])
-    if use_cmake:
-        subprocess.check_call(
-            [cmake_exe,
-             '-DCMAKE_INSTALL_PREFIX=%s' % install_dir,
-             '-DCMAKE_BUILD_TYPE=Release',
-             '-DLLVM_ENABLE_ASSERTIONS=OFF',
-             '-DLLVM_ENABLE_ZLIB=OFF',
-             '-DLLVM_ENABLE_TERMINFO=OFF',
-             '-DLLVM_ENABLE_LIBEDIT=OFF',
-             source_dir],
-            cwd=build_dir,
-            env=env)
-    else:
-        subprocess.check_call(
-            [os.path.join(source_dir, 'configure'),
-             '--prefix=%s' % install_dir,
-             '--enable-optimized',
-             '--disable-assertions',
-             '--disable-zlib',
-             '--disable-terminfo'],
-            cwd=build_dir,
-            env=env)
+    extra_flags = []
+    if is_project_build:
+        extra_flags = [
+            '-DLLVM_ENABLE_PROJECTS=clang;lld',
+            '-DLLVM_ENABLE_RUNTIMES=libunwind',
+        ]
+    subprocess.check_call(
+        [cmake_exe,
+         '-DCMAKE_INSTALL_PREFIX=%s' % install_dir,
+         '-DCMAKE_BUILD_TYPE=Release',
+         '-DLLVM_ENABLE_ASSERTIONS=OFF',
+         '-DLLVM_ENABLE_ZLIB=OFF',
+         '-DLLVM_ENABLE_LIBXML2=OFF',
+         '-DLLVM_ENABLE_TERMINFO=OFF',
+         '-DLLVM_ENABLE_LIBEDIT=OFF'] +
+        extra_flags +
+        [source_dir],
+        cwd=build_dir,
+        env=env)
     subprocess.check_call(['make', '-j', str(thread_count)], cwd=build_dir)
     subprocess.check_call(['make', 'install'], cwd=build_dir)
 
@@ -196,6 +202,8 @@ def build_hdf(source_dir, install_dir, thread_count, is_cray):
 
 def build_regent(root_dir, use_cmake, cmake_exe, extra_flags,
                  gasnet_dir, llvm_dir, terra_dir, hdf_dir, conduit, thread_count):
+    conduit = short_conduit(conduit)
+
     env = dict(list(os.environ.items()) +
         ([('CONDUIT', conduit),
           ('GASNET', gasnet_dir),
@@ -208,6 +216,9 @@ def build_regent(root_dir, use_cmake, cmake_exe, extra_flags,
          ('CMAKE_PREFIX_PATH', llvm_dir)]
     )
 
+    if use_cmake and hdf_enabled():
+        extra_flags = extra_flags + ['-DHDF5_ROOT=%s' % hdf_dir]
+
     subprocess.check_call(
         [os.path.join(root_dir, 'install.py'),
          '--with-terra', terra_dir,
@@ -218,73 +229,49 @@ def build_regent(root_dir, use_cmake, cmake_exe, extra_flags,
         ['--extra=%s' % flag for flag in extra_flags],
         env=env)
 
-def install_llvm(llvm_dir, llvm_install_dir, scratch_dir, llvm_version, llvm_use_cmake, cmake_exe, thread_count, cache, is_cray, insecure):
+def install_llvm(llvm_dir, llvm_install_dir, scratch_dir, llvm_version, cmake_exe, thread_count, cache, is_cray, insecure):
     try:
         os.mkdir(llvm_dir)
     except OSError:
         pass # Hope this means it already exists
     assert(os.path.isdir(llvm_dir))
 
-    # mirror = 'http://sapling.stanford.edu/~eslaught/llvm'
-    mirror = 'https://releases.llvm.org'
-    if llvm_version == '35':
-        llvm_tarball = os.path.join(llvm_dir, 'llvm-3.5.2.src.tar.xz')
-        llvm_source_dir = os.path.join(llvm_dir, 'llvm-3.5.2.src')
-        clang_tarball = os.path.join(llvm_dir, 'cfe-3.5.2.src.tar.xz')
-        clang_source_dir = os.path.join(llvm_dir, 'cfe-3.5.2.src')
-        download(llvm_tarball, '%s/3.5.2/llvm-3.5.2.src.tar.xz' % mirror, '44196156d5749eb4b4224fe471a29cc3984df92570a4a89fa859f7394fc0c575', insecure=insecure)
-        download(clang_tarball, '%s/3.5.2/cfe-3.5.2.src.tar.xz' % mirror, '4feb575f74fb3a74b6245400460230141bf610f235ef3a25008cfe6137828620', insecure=insecure)
-    elif llvm_version == '38':
-        llvm_tarball = os.path.join(llvm_dir, 'llvm-3.8.1.src.tar.xz')
-        llvm_source_dir = os.path.join(llvm_dir, 'llvm-3.8.1.src')
-        clang_tarball = os.path.join(llvm_dir, 'cfe-3.8.1.src.tar.xz')
-        clang_source_dir = os.path.join(llvm_dir, 'cfe-3.8.1.src')
-        download(llvm_tarball, '%s/3.8.1/llvm-3.8.1.src.tar.xz' % mirror, '6e82ce4adb54ff3afc18053d6981b6aed1406751b8742582ed50f04b5ab475f9', insecure=insecure)
-        download(clang_tarball, '%s/3.8.1/cfe-3.8.1.src.tar.xz' % mirror, '4cd3836dfb4b88b597e075341cae86d61c63ce3963e45c7fe6a8bf59bb382cdf', insecure=insecure)
-    elif llvm_version == '39':
-        llvm_tarball = os.path.join(llvm_dir, 'llvm-3.9.1.src.tar.xz')
-        llvm_source_dir = os.path.join(llvm_dir, 'llvm-3.9.1.src')
-        clang_tarball = os.path.join(llvm_dir, 'cfe-3.9.1.src.tar.xz')
-        clang_source_dir = os.path.join(llvm_dir, 'cfe-3.9.1.src')
-        download(llvm_tarball, '%s/3.9.1/llvm-3.9.1.src.tar.xz' % mirror, '1fd90354b9cf19232e8f168faf2220e79be555df3aa743242700879e8fd329ee', insecure=insecure)
-        download(clang_tarball, '%s/3.9.1/cfe-3.9.1.src.tar.xz' % mirror, 'e6c4cebb96dee827fa0470af313dff265af391cb6da8d429842ef208c8f25e63', insecure=insecure)
-    elif llvm_version == '60':
+    mirror = 'https://github.com/llvm/llvm-project/releases/download'
+    if llvm_version == '60':
+        mirror = 'https://releases.llvm.org'
+        if not os.environ.get('SETUP_ENV_ENABLE_LLVM_60') == '1':
+            raise Exception('LLVM 6.0 is deprecated in Terra. If you still rely on this version is it VERY IMPORANT that you contact the Legion team IMMEDIATELY so that your use case can be addressed. If you want to TEMPORARILY work around this warning, you can set the environment variable SETUP_ENV_ENABLE_LLVM_60=1')
         llvm_tarball = os.path.join(llvm_dir, 'llvm-6.0.1.src.tar.xz')
         llvm_source_dir = os.path.join(llvm_dir, 'llvm-6.0.1.src')
         clang_tarball = os.path.join(llvm_dir, 'cfe-6.0.1.src.tar.xz')
         clang_source_dir = os.path.join(llvm_dir, 'cfe-6.0.1.src')
         download(llvm_tarball, '%s/6.0.1/llvm-6.0.1.src.tar.xz' % mirror, 'b6d6c324f9c71494c0ccaf3dac1f16236d970002b42bb24a6c9e1634f7d0f4e2', insecure=insecure)
         download(clang_tarball, '%s/6.0.1/cfe-6.0.1.src.tar.xz' % mirror, '7c243f1485bddfdfedada3cd402ff4792ea82362ff91fbdac2dae67c6026b667', insecure=insecure)
-    elif llvm_version == '90':
-        llvm_tarball = os.path.join(llvm_dir, 'llvm-9.0.0.src.tar.xz')
-        llvm_source_dir = os.path.join(llvm_dir, 'llvm-9.0.0.src')
-        clang_tarball = os.path.join(llvm_dir, 'cfe-9.0.0.src.tar.xz')
-        clang_source_dir = os.path.join(llvm_dir, 'cfe-9.0.0.src')
-        download(llvm_tarball, '%s/9.0.0/llvm-9.0.0.src.tar.xz' % mirror, 'd6a0565cf21f22e9b4353b2eb92622e8365000a9e90a16b09b56f8157eabfe84', insecure=insecure)
-        download(clang_tarball, '%s/9.0.0/cfe-9.0.0.src.tar.xz' % mirror, '7ba81eef7c22ca5da688fdf9d88c20934d2d6b40bfe150ffd338900890aa4610', insecure=insecure)
     elif llvm_version == '110':
-        mirror = 'https://github.com/llvm/llvm-project/releases/download'
         llvm_tarball = os.path.join(llvm_dir, 'llvm-11.1.0.src.tar.xz')
         llvm_source_dir = os.path.join(llvm_dir, 'llvm-11.1.0.src')
         clang_tarball = os.path.join(llvm_dir, 'clang-11.1.0.src.tar.xz')
         clang_source_dir = os.path.join(llvm_dir, 'clang-11.1.0.src')
         download(llvm_tarball, '%s/llvmorg-11.1.0/llvm-11.1.0.src.tar.xz' % mirror, 'ce8508e318a01a63d4e8b3090ab2ded3c598a50258cc49e2625b9120d4c03ea5', insecure=insecure)
         download(clang_tarball, '%s/llvmorg-11.1.0/clang-11.1.0.src.tar.xz' % mirror, '0a8288f065d1f57cb6d96da4d2965cbea32edc572aa972e466e954d17148558b', insecure=insecure)
+    elif llvm_version == '130':
+        llvm_tarball = os.path.join(llvm_dir, 'llvm-project-13.0.0.src.tar.xz')
+        llvm_source_dir = os.path.join(llvm_dir, 'llvm-project-13.0.0.src', 'llvm')
+        clang_tarball = None
+        download(llvm_tarball, '%s/llvmorg-13.0.0/llvm-project-13.0.0.src.tar.xz' % mirror, '6075ad30f1ac0e15f07c1bf062c1e1268c241d674f11bd32cdf0e040c71f2bf3', insecure=insecure)
     else:
         assert False
 
     if not cache:
         extract(llvm_dir, llvm_tarball, 'xz')
-        extract(llvm_dir, clang_tarball, 'xz')
-        if llvm_version == '35':
-            apply_patch(llvm_source_dir, os.path.join(os.path.dirname(os.path.realpath(__file__)), 'llvm-3.5-gcc.patch'))
-        elif llvm_version == '38':
-            apply_patch(llvm_source_dir, os.path.join(os.path.dirname(os.path.realpath(__file__)), 'llvm-3.8-gcc.patch'))
-        os.rename(clang_source_dir, os.path.join(llvm_source_dir, 'tools', 'clang'))
+        if clang_tarball:
+            extract(llvm_dir, clang_tarball, 'xz')
+        if clang_tarball:
+            os.rename(clang_source_dir, os.path.join(llvm_source_dir, 'tools', 'clang'))
 
         llvm_build_dir = tempfile.mkdtemp(prefix='setup_env_llvm_build', dir=scratch_dir or llvm_dir)
         os.mkdir(llvm_install_dir)
-        build_llvm(llvm_source_dir, llvm_build_dir, llvm_install_dir, llvm_use_cmake, cmake_exe, thread_count, is_cray)
+        build_llvm(llvm_source_dir, llvm_build_dir, llvm_install_dir, clang_tarball is None, cmake_exe, thread_count, is_cray)
 
 def install_hdf(hdf_dir, hdf_install_dir, thread_count, cache, is_cray, insecure):
     try:
@@ -292,9 +279,9 @@ def install_hdf(hdf_dir, hdf_install_dir, thread_count, cache, is_cray, insecure
     except OSError:
         pass # Hope this means it already exists
     assert(os.path.isdir(hdf_dir))
-    hdf_tarball = os.path.join(hdf_dir, 'hdf5-1.10.1.tar.gz')
-    hdf_source_dir = os.path.join(hdf_dir, 'hdf5-1.10.1')
-    download(hdf_tarball, 'http://sapling.stanford.edu/~manolis/hdf/hdf5-1.10.1.tar.gz', '048a9d149fb99aaa1680a712963f5a78e9c43b588d0e79d55e06760ec377c172', insecure=insecure)
+    hdf_tarball = os.path.join(hdf_dir, 'hdf5-1_10_1.tar.gz')
+    hdf_source_dir = os.path.join(hdf_dir, 'hdf5-hdf5-1_10_1')
+    download(hdf_tarball, 'https://github.com/HDFGroup/hdf5/archive/refs/tags/hdf5-1_10_1.tar.gz', '1658e734d209b653c84461cd198366fdb741384c31e49490612f26028c3d3258', insecure=insecure)
     if not cache:
         extract(hdf_dir, hdf_tarball, 'gz')
         build_hdf(hdf_source_dir, hdf_install_dir, thread_count, is_cray)
@@ -347,6 +334,7 @@ def check_dirty_build(name, build_result, component_dir):
 def driver(prefix_dir=None, scratch_dir=None, cache=False,
            legion_use_cmake=False, extra_flags=[], llvm_version=None,
            terra_url=None, terra_branch=None, terra_lua=None, terra_use_cmake=None,
+           gasnet_version=None, gasnet_config_version=None,
            thread_count=None, insecure=False):
     if not cache:
         if 'CC' not in os.environ:
@@ -371,15 +359,8 @@ def driver(prefix_dir=None, scratch_dir=None, cache=False,
         if 'HOST_CXX' not in os.environ:
             raise Exception('Please set HOST_CXX in your environment')
 
-    if llvm_version in ('35', '38'):
-        llvm_use_cmake = False
-    elif llvm_version in ('39', '60', '90', '110'):
-        llvm_use_cmake = True
-    else:
-        raise Exception('Unrecognized LLVM version %s' % llvm_version)
-
     if terra_use_cmake is None:
-        terra_use_cmake = llvm_use_cmake
+        terra_use_cmake = True
 
     root_dir = os.path.dirname(os.path.dirname(os.path.realpath(__file__)))
     legion_dir = os.path.dirname(root_dir)
@@ -397,21 +378,26 @@ def driver(prefix_dir=None, scratch_dir=None, cache=False,
             # this works on macos
             thread_count = multiprocessing.cpu_count()
 
-    gasnet_release_dir = None
+    gasnet_build_dir = None
     conduit = None
     if gasnet_enabled():
         gasnet_dir = os.path.realpath(os.path.join(prefix_dir, 'gasnet'))
         if not os.path.exists(gasnet_dir):
-            git_clone(gasnet_dir, 'https://github.com/StanfordLegion/gasnet.git')
+            git_clone(
+                gasnet_dir,
+                'https://github.com/StanfordLegion/gasnet.git',
+                commit_id=gasnet_config_version)
         if not cache:
             conduit = discover_conduit()
-            gasnet_release_dir = os.path.join(gasnet_dir, 'release')
+            conduit_short = short_conduit(conduit)
+            gasnet_build_type = 'debug' if os.environ.get('GASNET_DEBUG') == '1' else 'release'
+            gasnet_build_dir = os.path.join(gasnet_dir, gasnet_build_type)
             gasnet_build_result = os.path.join(
-                gasnet_release_dir, '%s-conduit' % conduit,
-                'libgasnet-%s-par.a' % conduit)
-            if not os.path.exists(gasnet_release_dir):
+                gasnet_build_dir, '%s-conduit' % conduit_short,
+                'libgasnet-%s-par.a' % conduit_short)
+            if not os.path.exists(gasnet_build_dir):
                 try:
-                    build_gasnet(gasnet_dir, conduit)
+                    build_gasnet(gasnet_dir, conduit, gasnet_version)
                 except Exception as e:
                     report_build_failure('gasnet', gasnet_dir, e)
             else:
@@ -425,42 +411,52 @@ def driver(prefix_dir=None, scratch_dir=None, cache=False,
         pass # Can't find CMake, continue to download
     else:
         m = re.match(r'cmake version (\d+)[.](\d+)', cmake_version)
-        if m is not None and (int(m.group(1)) < 3 or int(m.group(2)) < 1):
+        # LLVM requires at least CMake 13: https://github.com/llvm/llvm-project/blob/main/llvm/CMakeLists.txt
+        if m is not None and (int(m.group(1)) < 3 or int(m.group(2)) < 13):
             pass # CMake is too old, continue to download
         elif m is None:
             raise Exception('Cannot parse CMake version:\n\n%s' % cmake_version)
         else:
             cmake_exe = 'cmake' # CMake is ok, use it
-    if cache or ((legion_use_cmake or llvm_use_cmake or terra_use_cmake) and cmake_exe is None):
-        cmake_stem = 'cmake-3.7.2-%s-x86_64' % platform.system()
+    if cache or (cmake_exe is None):
+        cmake_system = platform.system()
+        if cmake_system == 'Darwin':
+            cmake_system = 'macos'
+        else:
+            cmake_system = cmake_system.lower()
+        cmake_processor = platform.machine()
+
+        if cmake_system == 'macos':
+            cmake_processor = 'universal'
+
+        cmake_stem = 'cmake-3.23.4-%s-%s' % (cmake_system, cmake_processor)
         cmake_basename = '%s.tar.gz' % cmake_stem
-        cmake_url = 'https://cmake.org/files/v3.7/%s' % cmake_basename
-        if cmake_stem == 'cmake-3.7.2-Linux-x86_64':
-            cmake_shasum = '0e6ec35d4fa9bf79800118916b51928b6471d5725ff36f1d0de5ebb34dcd5406'
-        elif cmake_stem == 'cmake-3.7.2-Darwin-x86_64':
-            cmake_shasum = '0175e97748052dfc15ebd3c0aa65286e5ec20ca22ed606ce88940e699496b03c'
+        cmake_url = 'https://github.com/Kitware/CMake/releases/download/v3.23.4/%s' % cmake_basename
+        if cmake_stem == 'cmake-3.23.4-linux-x86_64':
+            cmake_shasum = '3fbcbff85043d63a8a83c8bdf8bd5b1b2fd5768f922de7dc4443de7805a2670d'
+        elif cmake_stem == 'cmake-3.23.4-macos-universal':
+            cmake_shasum = '98cac043cdf321caa4fd07f27da3316db6c8bc48c39997bf78e27e5c46c4eb68'
+        else:
+            raise Exception("Don't know how to download CMake binary for %s %s" % (cmake_system, cmake_processor))
 
         cmake_dir = os.path.realpath(os.path.join(prefix_dir, 'cmake'))
         cmake_install_dir = os.path.join(cmake_dir, cmake_stem)
         if not os.path.exists(cmake_dir):
             os.mkdir(cmake_dir)
-
-            proc_type = subprocess.check_output(['uname', '-p']).decode('utf-8').strip()
-            if proc_type != 'x86_64' and proc_type != 'i386':
-                raise Exception("Don't know how to download CMake binary for %s" % proc_type)
-
             cmake_tarball = os.path.join(cmake_dir, cmake_basename)
             download(cmake_tarball, cmake_url, cmake_shasum, insecure=insecure)
             extract(cmake_dir, cmake_tarball, 'gz')
         assert os.path.exists(cmake_install_dir)
         cmake_exe = os.path.join(cmake_install_dir, 'bin', 'cmake')
+        if cmake_system == 'macos':
+            cmake_exe = os.path.join(cmake_install_dir, 'CMake.app', 'Contents', 'bin', 'cmake')
 
     llvm_dir = os.path.realpath(os.path.join(prefix_dir, 'llvm'))
     llvm_install_dir = os.path.join(llvm_dir, 'install')
     llvm_build_result = os.path.join(llvm_install_dir, 'bin', 'llvm-config')
     if not os.path.exists(llvm_install_dir):
         try:
-            install_llvm(llvm_dir, llvm_install_dir, scratch_dir, llvm_version, llvm_use_cmake, cmake_exe, thread_count, cache, is_cray, insecure)
+            install_llvm(llvm_dir, llvm_install_dir, scratch_dir, llvm_version, cmake_exe, thread_count, cache, is_cray, insecure)
         except Exception as e:
             report_build_failure('llvm', llvm_dir, e)
     else:
@@ -469,11 +465,10 @@ def driver(prefix_dir=None, scratch_dir=None, cache=False,
         assert os.path.exists(llvm_build_result)
 
     terra_dir = os.path.join(prefix_dir, 'terra.build')
-    terra_build_dir = os.path.join(terra_dir, 'build', 'bin')
     terra_build_result = os.path.join(terra_dir, 'release', 'bin', 'terra')
     if not os.path.exists(terra_dir):
         git_clone(terra_dir, terra_url, terra_branch)
-    if not os.path.exists(terra_build_dir):
+    if not os.path.exists(terra_build_result):
         try:
             build_terra(terra_dir, terra_branch, terra_lua, terra_use_cmake, cmake_exe, llvm_install_dir, cache, is_cray, thread_count)
         except Exception as e:
@@ -487,7 +482,7 @@ def driver(prefix_dir=None, scratch_dir=None, cache=False,
     if hdf_enabled():
         hdf_dir = os.path.join(prefix_dir, 'hdf')
         hdf_install_dir = os.path.join(hdf_dir, 'install')
-        hdf_build_result = os.path.join(hdf_install_dir, 'lib', 'libhdf5.so')
+        hdf_build_result = os.path.join(hdf_install_dir, 'lib', 'libhdf5.a')
         if not os.path.exists(hdf_install_dir):
             try:
                 install_hdf(hdf_dir, hdf_install_dir, thread_count, cache, is_cray, insecure)
@@ -500,7 +495,7 @@ def driver(prefix_dir=None, scratch_dir=None, cache=False,
 
     if not cache:
         build_regent(root_dir, legion_use_cmake, cmake_exe, extra_flags,
-                     gasnet_release_dir, llvm_install_dir, terra_dir, hdf_install_dir,
+                     gasnet_build_dir, llvm_install_dir, terra_dir, hdf_install_dir,
                      conduit, thread_count)
 
 if __name__ == '__main__':
@@ -528,16 +523,16 @@ if __name__ == '__main__':
         default=[],
         help='Extra flags for Make/CMake command.')
     parser.add_argument(
-        '--llvm-version', dest='llvm_version', required=False, choices=('35', '38', '39', '60', '90', '110'),
+        '--llvm-version', dest='llvm_version', required=False, choices=('60', '110', '130'),
         default=discover_llvm_version(),
         help='Select LLVM version.')
     parser.add_argument(
         '--terra-url', dest='terra_url', required=False,
-        default='https://github.com/StanfordLegion/terra.git',
+        default='https://github.com/terralang/terra.git',
         help='URL of Terra repository to clone from.')
     parser.add_argument(
         '--terra-branch', dest='terra_branch', required=False,
-        default='luajit2.1',
+        default='release-1.0.6',
         help='Branch of Terra repository to checkout.')
     parser.add_argument(
         '--terra-lua', dest='terra_lua', required=False,
@@ -549,6 +544,14 @@ if __name__ == '__main__':
     parser.add_argument(
         '--no-terra-cmake', dest='terra_use_cmake', action='store_false', default=None,
         help='Use CMake to build Terra.')
+    parser.add_argument(
+        '--gasnet-version', dest='gasnet_version', required=False,
+        default=os.environ.get('GASNET_VERSION', 'GASNet-2022.9.2'),
+        help='Select GASNet version.')
+    parser.add_argument(
+        '--gasnet-config-version', dest='gasnet_config_version', required=False,
+        default='10c55f5d53d803f5305e6983a0cf581b8e467aed',
+        help='Select version of the GASNet configuration/build tool.')
     parser.add_argument(
         '-j', dest='thread_count', nargs='?', type=int,
         help='Number threads used to compile.')
