@@ -1,4 +1,4 @@
-/* Copyright 2021 Stanford University, NVIDIA Corporation
+/* Copyright 2023 Stanford University, NVIDIA Corporation
  *
  * Licensed under the Apache License, Version 2.0 (the "License");
  * you may not use this file except in compliance with the License.
@@ -38,8 +38,8 @@ namespace Legion {
 
     enum DistCollectableType {
       PHYSICAL_MANAGER_DC = 0x1,
-      //= 0x2,
-      //= 0x3,
+      INDEX_SPACE_NODE_DC = 0x2,
+      INDEX_PART_NODE_DC = 0x3,
       MATERIALIZED_VIEW_DC = 0x4,
       REDUCTION_VIEW_DC = 0x5,
       FILL_VIEW_DC = 0x6,
@@ -47,7 +47,7 @@ namespace Legion {
       VERSION_STATE_DC = 0x8,
       FUTURE_DC = 0x9,
       FUTURE_MAP_DC = 0xA,
-      INDEX_TREE_NODE_DC = 0xB,
+      INDEX_EXPR_NODE_DC = 0xB,
       FIELD_SPACE_DC = 0xC,
       REGION_TREE_NODE_DC = 0xD,
       EQUIVALENCE_SET_DC = 0xE,
@@ -59,14 +59,14 @@ namespace Legion {
       FUTURE_HANDLE_REF = 0,
       DEFERRED_TASK_REF = 1,
       VERSION_MANAGER_REF = 2,
-      VERSION_INFO_REF = 3,
+      PHYSICAL_ANALYSIS_REF = 3,
       PHYSICAL_STATE_REF = 4,
       PHYSICAL_REGION_REF = 5,
       PENDING_GC_REF = 6,
       REMOTE_DID_REF = 7,
       PENDING_COLLECTIVE_REF = 8,
       MEMORY_MANAGER_REF = 9,
-      COMPOSITE_NODE_REF = 10,
+      INSTANCE_BUILDER_REF = 10,
       FIELD_ALLOCATOR_REF = 11,
       REMOTE_CREATE_REF = 12,
       INSTANCE_MAPPER_REF = 13,
@@ -75,17 +75,17 @@ namespace Legion {
       NEVER_GC_REF = 16,
       CONTEXT_REF = 17,
       RESTRICTED_REF = 18,
-      VERSION_STATE_TREE_REF = 19,
-      PHYSICAL_MANAGER_REF = 20,
+      META_TASK_REF = 19,
+      PHYSICAL_USER_REF = 20,
       LOGICAL_VIEW_REF = 21,
       REGION_TREE_REF = 22,
       LAYOUT_DESC_REF = 23,
       RUNTIME_REF = 24,
-      IS_EXPR_REF = 25,
+      LIVE_EXPR_REF = 25,
       TRACE_REF = 26,
-      AGGREGATORE_REF = 27,
+      AGGREGATOR_REF = 27,
       FIELD_STATE_REF = 28,
-      CANONICAL_REF = 29,
+      COPY_ACROSS_REF = 29,
       LAST_SOURCE_REF = 30,
     };
 
@@ -107,7 +107,7 @@ namespace Legion {
       "Remote Distributed ID Reference",            \
       "Pending Collective Reference",               \
       "Memory Manager Reference",                   \
-      "Composite Node Reference",                   \
+      "Instance Builder Reference",                 \
       "Field Allocator Reference",                  \
       "Remote Creation Reference",                  \
       "Instance Mapper Reference",                  \
@@ -116,17 +116,17 @@ namespace Legion {
       "Never GC Reference",                         \
       "Context Reference",                          \
       "Restricted Reference",                       \
-      "Version State Tree Reference",               \
-      "Physical Manager Reference",                 \
+      "Meta-Task Reference",                        \
+      "Physical User Reference",                    \
       "Logical View Reference",                     \
       "Region Tree Reference",                      \
       "Layout Description Reference",               \
       "Runtime Reference",                          \
-      "Index Space Expression Reference",           \
+      "Live Index Space Expression Reference",      \
       "Physical Trace Reference",                   \
       "Aggregator Reference",                       \
       "Field State Reference",                      \
-      "Canonical Index Space Expression Reference", \
+      "Copy Across Executor Reference",             \
     }
 
     extern Realm::Logger log_garbage;
@@ -154,7 +154,7 @@ namespace Legion {
       inline void add_reference(unsigned cnt = 1);
       inline bool remove_reference(unsigned cnt = 1);
     protected:
-      unsigned int references;
+      std::atomic<unsigned int> references;
     };
 
     /**
@@ -205,7 +205,7 @@ namespace Legion {
     public:
       RtEvent get_done_event(void);
     private:
-      std::set<RtEvent> mutation_effects;
+      std::vector<RtEvent> mutation_effects;
     };
 
     /**
@@ -227,12 +227,24 @@ namespace Legion {
     };
 
     /**
-     * \class IgnoreReferenceMutator
-     * This will ignore any reference effects
+     * \class ImplicitReferenceTracker
+     * This class tracks implicit references that are held either by
+     * an application runtime API call or a meta-task. At the end of the
+     * runtime API call or meta-task the references are updated.
      */
-    class IgnoreReferenceMutator : public ReferenceMutator {
+    class ImplicitReferenceTracker {
     public:
-      virtual void record_reference_mutation_effect(RtEvent event) { }
+      ImplicitReferenceTracker(void) { }
+      ImplicitReferenceTracker(const ImplicitReferenceTracker&) = delete;
+      ~ImplicitReferenceTracker(void);
+    public:
+      ImplicitReferenceTracker& operator=(
+                               const ImplicitReferenceTracker&) = delete;
+    public:
+      inline void record_live_expression(IndexSpaceExpression *expr) 
+        { live_expressions.emplace_back(expr); }
+    private:
+      std::vector<IndexSpaceExpression*> live_expressions;
     };
 
     /**
@@ -251,6 +263,7 @@ namespace Legion {
         ACTIVE_INVALID_STATE,
         VALID_STATE,
         DELETED_STATE,
+        // Make sure all these come after deleted state
         PENDING_ACTIVE_STATE,
         PENDING_INACTIVE_STATE,
         PENDING_VALID_STATE,
@@ -261,16 +274,13 @@ namespace Legion {
     public:
       class UnregisterFunctor {
       public:
-        UnregisterFunctor(Runtime *rt, const DistributedID d,
-                          VirtualChannelKind v, std::set<RtEvent> &done)
-          : runtime(rt), did(d), vc(v), done_events(done) { }
+        UnregisterFunctor(Runtime *rt, Serializer &r)
+          : runtime(rt), rez(r) { }
       public:
         void apply(AddressSpaceID target);
       protected:
         Runtime *const runtime;
-        const DistributedID did;
-        const VirtualChannelKind vc;
-        std::set<RtEvent> &done_events;
+        Serializer &rez;
       };
       struct DeferRemoteReferenceUpdateArgs : 
         public LgTaskArgs<DeferRemoteReferenceUpdateArgs> {
@@ -278,29 +288,26 @@ namespace Legion {
         static const LgTaskID TASK_ID = LG_DEFER_REMOTE_REF_UPDATE_TASK_ID;
       public:
         DeferRemoteReferenceUpdateArgs(DistributedCollectable *d, 
-            AddressSpaceID t, RtUserEvent e, unsigned c, bool v)
+            AddressSpaceID t, RtUserEvent e, unsigned c, ReferenceKind k)
           : LgTaskArgs<DeferRemoteReferenceUpdateArgs>(implicit_provenance),
             did(d->did), target(t), done_event(e), count(c),
-            owner(d->owner_space == t), valid(v) { } 
+            kind(k), owner(d->owner_space == t) { } 
       public:
         const DistributedID did;
         const AddressSpaceID target;
         const RtUserEvent done_event;
         const int count;
+        const ReferenceKind kind;
         const bool owner;
-        const bool valid;
       };
       struct DeferRemoteUnregisterArgs :
         public LgTaskArgs<DeferRemoteUnregisterArgs> {
       public:
         static const LgTaskID TASK_ID = LG_DEFER_REMOTE_UNREGISTER_TASK_ID;
       public:
-        DeferRemoteUnregisterArgs(DistributedID id, VirtualChannelKind c,
-                                  const NodeSet &nodes);
+        DeferRemoteUnregisterArgs(DistributedID id, const NodeSet &nodes);
       public:
-        const RtUserEvent done;
         const DistributedID did;
-        const VirtualChannelKind vc;
         NodeSet *const nodes;
       };
     public:
@@ -333,18 +340,28 @@ namespace Legion {
       inline bool remove_base_resource_ref(ReferenceSource source, int cnt = 1);
       inline bool remove_nested_resource_ref(DistributedID source, int cnt = 1);
     public:
+#ifdef DEBUG_LEGION
+      bool check_valid(void);
+      // Better be called while holding the lock
+      inline bool in_stable_state(void) const 
+        { return (current_state <= DELETED_STATE); }
+#endif
       // Atomic check and increment operations 
-      inline bool check_valid_and_increment(ReferenceSource source,int cnt = 1);
-      bool check_resource_and_increment(ReferenceSource source ,int cnt = 1);
+      bool check_valid_and_increment(ReferenceSource source,int cnt = 1);
+      bool check_valid_and_increment(DistributedID source, int cnt = 1);
+      bool check_active_and_increment(ReferenceSource source, int cnt = 1);
+      bool check_active_and_increment(DistributedID source, int cnt = 1);
+#ifndef DEBUG_LEGION_GC
     private:
-      void add_gc_reference(ReferenceMutator *mutator);
-      bool remove_gc_reference(ReferenceMutator *mutator);
+      void add_gc_reference(ReferenceMutator *mutator, int cnt);
+      bool remove_gc_reference(ReferenceMutator *mutator, int cnt);
     private:
-      void add_valid_reference(ReferenceMutator *mutator);
-      bool remove_valid_reference(ReferenceMutator *mutator);
+      void add_valid_reference(ReferenceMutator *mutator, int cnt);
+      bool remove_valid_reference(ReferenceMutator *mutator, int cnt);
     private:
-      void add_resource_reference(void);
-      bool remove_resource_reference(void);
+      void add_resource_reference(int cnt);
+      bool remove_resource_reference(int cnt);
+#endif
 #ifdef USE_REMOTE_REFERENCES
     private:
       bool add_create_reference(AddressSpaceID source, 
@@ -391,17 +408,18 @@ namespace Legion {
       void filter_remote_instances(AddressSpaceID remote_space);
     public:
       inline bool has_remote_instances(void) const;
+      inline size_t count_remote_instances(void) const;
       template<typename FUNCTOR>
       inline void map_over_remote_instances(FUNCTOR &functor);
     public:
-      // This is for the owner node only
       void register_with_runtime(ReferenceMutator *mutator);
+      bool confirm_deletion(void);
     protected:
-      void unregister_with_runtime(void) const;
-      RtEvent send_unregister_messages(VirtualChannelKind vc) const;
+      bool try_unregister(void);
+      bool unregister_with_runtime(void) const;
+      void send_unregister_messages(void) const;
     public:
       // This for remote nodes only
-      void unregister_collectable(void);
       static void handle_unregister_collectable(Runtime *runtime,
                                                 Deserializer &derez);
     public:
@@ -423,6 +441,9 @@ namespace Legion {
                                     ReferenceMutator *mutator = NULL,
                                     RtEvent precondition = RtEvent::NO_RT_EVENT,
                                     unsigned count = 1);
+      void send_remote_resource_decrement(AddressSpaceID target,
+                                    RtEvent precondition = RtEvent::NO_RT_EVENT,
+                                    unsigned count = 1);
 #ifdef USE_REMOTE_REFERENCES
     public:
       ReferenceKind send_create_reference(AddressSpaceID target);
@@ -437,6 +458,8 @@ namespace Legion {
                                                  Deserializer &derez);
       static void handle_did_remote_gc_update(Runtime *runtime,
                                               Deserializer &derez);
+      static void handle_did_remote_resource_update(Runtime *runtime,
+                                                    Deserializer &derez);
       static void handle_defer_remote_reference_update(Runtime *runtime,
                                                       const void *args);
       static void handle_defer_remote_unregister(Runtime *runtime,
@@ -477,9 +500,15 @@ namespace Legion {
       bool has_resource_references;
       bool reentrant_update;
     private: // derived users can't see the references
+#ifdef DEBUG_LEGION_GC
       int gc_references;
       int valid_references;
       int resource_references;
+#else
+      std::atomic<int> gc_references;
+      std::atomic<int> valid_references;
+      std::atomic<int> resource_references;
+#endif
 #ifdef USE_REMOTE_REFERENCES
     protected:
       // These are only valid on the owner node
@@ -543,14 +572,14 @@ namespace Legion {
     inline void Collectable::add_reference(unsigned cnt /*= 1*/)
     //--------------------------------------------------------------------------
     {
-      __sync_add_and_fetch(&references,cnt);
+      references.fetch_add(cnt);
     }
 
     //--------------------------------------------------------------------------
     inline bool Collectable::remove_reference(unsigned cnt /*= 1*/)
     //--------------------------------------------------------------------------
     {
-      unsigned prev = __sync_fetch_and_sub(&references,cnt);
+      unsigned prev = references.fetch_sub(cnt);
 #ifdef DEBUG_LEGION
       assert(prev >= cnt); // check for underflow
 #endif
@@ -567,6 +596,14 @@ namespace Legion {
       return !remote_instances.empty();
     }
 
+    //--------------------------------------------------------------------------
+    inline size_t DistributedCollectable::count_remote_instances(void) const
+    //--------------------------------------------------------------------------
+    {
+      AutoLock gc(gc_lock,1,false/*exclusive*/);
+      return remote_instances.size();
+    }
+
     //--------------------------------------------------------------------------
     template<typename FUNCTOR>
     void DistributedCollectable::map_over_remote_instances(FUNCTOR &functor)
@@ -587,15 +624,17 @@ namespace Legion {
 #ifdef LEGION_GC
       log_base_ref<true>(GC_REF_KIND, did, local_space, source, cnt);
 #endif
-#ifndef DEBUG_LEGION_GC
-      int previous = __sync_fetch_and_add(&gc_references, cnt);
-#ifdef DEBUG_LEGION
-      assert(previous >= 0);
-#endif
-      if (previous == 0)
-        add_gc_reference(mutator);
-#else
+#ifdef DEBUG_LEGION_GC
       add_base_gc_ref_internal(source, mutator, cnt); 
+#else
+      int current = gc_references.load();
+      while (current > 0)
+      {
+        int next = current + cnt;
+        if (gc_references.compare_exchange_weak(current, next))
+          return;
+      }
+      add_gc_reference(mutator, cnt);
 #endif
     }
 
@@ -610,16 +649,18 @@ namespace Legion {
 #ifdef LEGION_GC
       log_nested_ref<true>(GC_REF_KIND, did, local_space, source, cnt);
 #endif
-#ifndef DEBUG_LEGION_GC
-      int previous = __sync_fetch_and_add(&gc_references, cnt);
-#ifdef DEBUG_LEGION
-      assert(previous >= 0);
-#endif
-      if (previous == 0)
-        add_gc_reference(mutator);
-#else
+#ifdef DEBUG_LEGION_GC
       add_nested_gc_ref_internal(LEGION_DISTRIBUTED_ID_FILTER(source), 
                                  mutator, cnt);
+#else
+      int current = gc_references.load();
+      while (current > 0)
+      {
+        int next = current + cnt;
+        if (gc_references.compare_exchange_weak(current, next))
+          return;
+      }
+      add_gc_reference(mutator, cnt);
 #endif
     }
 
@@ -634,16 +675,20 @@ namespace Legion {
 #ifdef LEGION_GC
       log_base_ref<false>(GC_REF_KIND, did, local_space, source, cnt);
 #endif
-#ifndef DEBUG_LEGION_GC
-      int previous = __sync_fetch_and_add(&gc_references, -cnt);
-#ifdef DEBUG_LEGION
-      assert(previous >= cnt);
-#endif
-      if (previous == cnt)
-        return remove_gc_reference(mutator);
-      return false;
-#else
+#ifdef DEBUG_LEGION_GC
       return remove_base_gc_ref_internal(source, mutator, cnt);
+#else
+      int current = gc_references.load();
+#ifdef DEBUG_LEGION
+      assert(current >= cnt);
+#endif
+      while (current > cnt)
+      {
+        int next = current - cnt;
+        if (gc_references.compare_exchange_weak(current, next))
+          return false;
+      }
+      return remove_gc_reference(mutator, cnt);
 #endif
     }
 
@@ -658,17 +703,21 @@ namespace Legion {
 #ifdef LEGION_GC
       log_nested_ref<false>(GC_REF_KIND, did, local_space, source, cnt);
 #endif
-#ifndef DEBUG_LEGION_GC
-      int previous = __sync_fetch_and_add(&gc_references, -cnt);
-#ifdef DEBUG_LEGION
-      assert(previous >= cnt);
-#endif
-      if (previous == cnt)
-        return remove_gc_reference(mutator);
-      return false;
-#else
+#ifdef DEBUG_LEGION_GC
       return remove_nested_gc_ref_internal(
           LEGION_DISTRIBUTED_ID_FILTER(source), mutator, cnt);
+#else
+      int current = gc_references.load();
+#ifdef DEBUG_LEGION
+      assert(current >= cnt);
+#endif
+      while (current > cnt)
+      {
+        int next = current - cnt;
+        if (gc_references.compare_exchange_weak(current, next))
+          return false;
+      }
+      return remove_gc_reference(mutator, cnt);
 #endif
     }
 
@@ -683,15 +732,17 @@ namespace Legion {
 #ifdef LEGION_GC
       log_base_ref<true>(VALID_REF_KIND, did, local_space, source, cnt);
 #endif
-#ifndef DEBUG_LEGION_GC
-      int previous = __sync_fetch_and_add(&valid_references, cnt);
-#ifdef DEBUG_LEGION
-      assert(previous >= 0);
-#endif
-      if (previous == 0)
-        add_valid_reference(mutator);
-#else
+#ifdef DEBUG_LEGION_GC
       add_base_valid_ref_internal(source, mutator, cnt);
+#else
+      int current = valid_references.load();
+      while (current > 0)
+      {
+        int next = current + cnt;
+        if (valid_references.compare_exchange_weak(current, next))
+          return;
+      }
+      add_valid_reference(mutator, cnt);
 #endif
     }
 
@@ -706,16 +757,18 @@ namespace Legion {
 #ifdef LEGION_GC
       log_nested_ref<true>(VALID_REF_KIND, did, local_space, source, cnt);
 #endif
-#ifndef DEBUG_LEGION_GC
-      int previous = __sync_fetch_and_add(&valid_references, cnt);
-#ifdef DEBUG_LEGION
-      assert(previous >= 0);
-#endif
-      if (previous == 0)
-        add_valid_reference(mutator);
-#else
+#ifdef DEBUG_LEGION_GC
       add_nested_valid_ref_internal(LEGION_DISTRIBUTED_ID_FILTER(source), 
                                     mutator, cnt);
+#else
+      int current = valid_references.load();
+      while (current > 0)
+      {
+        int next = current + cnt;
+        if (valid_references.compare_exchange_weak(current, next))
+          return;
+      }
+      add_valid_reference(mutator, cnt);
 #endif
     }
 
@@ -730,16 +783,20 @@ namespace Legion {
 #ifdef LEGION_GC
       log_base_ref<false>(VALID_REF_KIND, did, local_space, source, cnt);
 #endif
-#ifndef DEBUG_LEGION_GC
-      int previous = __sync_fetch_and_add(&valid_references, -cnt);
-#ifdef DEBUG_LEGION
-      assert(previous >= cnt);
-#endif
-      if (previous == cnt)
-        return remove_valid_reference(mutator);
-      return false;
-#else
+#ifdef DEBUG_LEGION_GC
       return remove_base_valid_ref_internal(source, mutator, cnt);
+#else
+      int current = valid_references.load();
+#ifdef DEBUG_LEGION
+      assert(current >= cnt);
+#endif
+      while (current > cnt)
+      {
+        int next = current - cnt;
+        if (valid_references.compare_exchange_weak(current, next))
+          return false;
+      }
+      return remove_valid_reference(mutator, cnt);
 #endif
     }
 
@@ -754,17 +811,21 @@ namespace Legion {
 #ifdef LEGION_GC
       log_nested_ref<false>(VALID_REF_KIND, did, local_space, source, cnt);
 #endif
-#ifndef DEBUG_LEGION_GC
-      int previous = __sync_fetch_and_add(&valid_references, -cnt);
-#ifdef DEBUG_LEGION
-      assert(previous >= cnt);
-#endif
-      if (previous == cnt)
-        return remove_valid_reference(mutator);
-      return false;
-#else
+#ifdef DEBUG_LEGION_GC
       return remove_nested_valid_ref_internal(
           LEGION_DISTRIBUTED_ID_FILTER(source), mutator, cnt);
+#else
+      int current = valid_references.load();
+#ifdef DEBUG_LEGION
+      assert(current >= cnt);
+#endif
+      while (current > cnt)
+      {
+        int next = current - cnt;
+        if (valid_references.compare_exchange_weak(current, next))
+          return false;
+      }
+      return remove_valid_reference(mutator, cnt);
 #endif
     }
 
@@ -779,15 +840,17 @@ namespace Legion {
 #ifdef LEGION_GC
       log_base_ref<true>(RESOURCE_REF_KIND, did, local_space, source, cnt);
 #endif
-#ifndef DEBUG_LEGION_GC
-      int previous = __sync_fetch_and_add(&resource_references, cnt);
-#ifdef DEBUG_LEGION
-      assert(previous >= 0);
-#endif
-      if (previous == 0)
-        add_resource_reference();
-#else
+#ifdef DEBUG_LEGION_GC
       add_base_resource_ref_internal(source, cnt);
+#else
+      int current = resource_references.load();
+      while (current > 0)
+      {
+        int next = current + cnt;
+        if (resource_references.compare_exchange_weak(current, next))
+          return;
+      }
+      add_resource_reference(cnt);
 #endif
     }
 
@@ -802,16 +865,18 @@ namespace Legion {
 #ifdef LEGION_GC
       log_nested_ref<true>(RESOURCE_REF_KIND, did, local_space, source, cnt);
 #endif
-#ifndef DEBUG_LEGION_GC
-      int previous = __sync_fetch_and_add(&resource_references, cnt);
-#ifdef DEBUG_LEGION
-      assert(previous >= 0);
-#endif
-      if (previous == 0)
-        add_resource_reference();
-#else
+#ifdef DEBUG_LEGION_GC
       add_nested_resource_ref_internal(
           LEGION_DISTRIBUTED_ID_FILTER(source), cnt);
+#else
+      int current = resource_references.load();
+      while (current > 0)
+      {
+        int next = current + cnt;
+        if (resource_references.compare_exchange_weak(current, next))
+          return;
+      }
+      add_resource_reference(cnt);
 #endif
     }
 
@@ -826,16 +891,20 @@ namespace Legion {
 #ifdef LEGION_GC
       log_base_ref<false>(RESOURCE_REF_KIND, did, local_space, source, cnt);
 #endif
-#ifndef DEBUG_LEGION_GC
-      int previous = __sync_fetch_and_add(&resource_references, -cnt);
-#ifdef DEBUG_LEGION
-      assert(previous >= cnt);
-#endif
-      if (previous == cnt)
-        return remove_resource_reference();
-      return false;
-#else
+#ifdef DEBUG_LEGION_GC
       return remove_base_resource_ref_internal(source, cnt);
+#else
+      int current = resource_references.load();
+#ifdef DEBUG_LEGION
+      assert(current >= cnt);
+#endif
+      while (current > cnt)
+      {
+        int next = current - cnt;
+        if (resource_references.compare_exchange_weak(current, next))
+          return false;
+      }
+      return remove_resource_reference(cnt);
 #endif
     }
 
@@ -850,50 +919,22 @@ namespace Legion {
 #ifdef LEGION_GC
       log_nested_ref<false>(RESOURCE_REF_KIND, did, local_space, source, cnt);
 #endif
-#ifndef DEBUG_LEGION_GC
-      int previous = __sync_fetch_and_add(&resource_references, -cnt);
-#ifdef DEBUG_LEGION
-      assert(previous >= cnt);
-#endif
-      if (previous == cnt)
-        return remove_resource_reference();
-      return false;
-#else
+#ifdef DEBUG_LEGION_GC
       return remove_nested_resource_ref_internal(
           LEGION_DISTRIBUTED_ID_FILTER(source), cnt);
-#endif
-    }
-
-    //--------------------------------------------------------------------------
-    inline bool DistributedCollectable::check_valid_and_increment(
-                                         ReferenceSource source, int cnt /*=1*/)
-    //--------------------------------------------------------------------------
-    {
+#else
+      int current = resource_references.load();
 #ifdef DEBUG_LEGION
-      assert(cnt >= 0);
+      assert(current >= cnt);
 #endif
-      // Don't support this if we are debugging GC
-#ifndef DEBUG_LEGION_GC
-      // Read the value in an unsafe way at first 
-      int current_cnt = valid_references;
-      // Don't even both trying if the count is zero
-      while (current_cnt > 0)
+      while (current > cnt)
       {
-        const int next_cnt = current_cnt + cnt;
-        const int prev_cnt = 
-          __sync_val_compare_and_swap(&valid_references, current_cnt, next_cnt);
-        if (prev_cnt == current_cnt)
-        {
-#ifdef LEGION_GC
-          log_base_ref<true>(VALID_REF_KIND, did, local_space, source, cnt);
-#endif
-          return true;
-        }
-        // Update the current count
-        current_cnt = prev_cnt;
+        int next = current - cnt;
+        if (resource_references.compare_exchange_weak(current, next))
+          return false;
       }
+      return remove_resource_reference(cnt);
 #endif
-      return false;
     }
 
   }; // namespace Internal 
