@@ -1,4 +1,4 @@
-/* Copyright 2021 Stanford University
+/* Copyright 2023 Stanford University
  *
  * Licensed under the Apache License, Version 2.0 (the "License");
  * you may not use this file except in compliance with the License.
@@ -263,6 +263,7 @@ namespace Legion {
       wrap(Domain domain) {
         legion_domain_t domain_;
         domain_.is_id = domain.is_id;
+        domain_.is_type = domain.is_type;
         domain_.dim = domain.dim;
         std::copy(domain.rect_data, domain.rect_data + 2 * LEGION_MAX_DIM, domain_.rect_data);
         return domain_;
@@ -272,6 +273,7 @@ namespace Legion {
       unwrap(legion_domain_t domain_) {
         Domain domain;
         domain.is_id = domain_.is_id;
+        domain.is_type = domain_.is_type;
         domain.dim = domain_.dim;
         std::copy(domain_.rect_data, domain_.rect_data + 2 * LEGION_MAX_DIM, domain.rect_data);
         return domain;
@@ -711,6 +713,10 @@ namespace Legion {
         assert(false);
         return NULL;
       }
+      virtual const std::string& get_provenance_string(bool human=true) const {
+        assert(false);
+        return (*new std::string());
+      }
       virtual bool has_parent_task(void) const {
         assert(false);
         return false;
@@ -719,6 +725,10 @@ namespace Legion {
         assert(false);
         return NULL;
       }
+      virtual Domain get_slice_domain(void) const {
+        assert(false);
+        return Domain::NO_DOMAIN;
+      }
     };
 };
 
