@@ -1,4 +1,4 @@
-/* Copyright 2021 Stanford University, NVIDIA Corporation
+/* Copyright 2023 Stanford University, NVIDIA Corporation
  *
  * Licensed under the Apache License, Version 2.0 (the "License");
  * you may not use this file except in compliance with the License.
@@ -36,11 +36,12 @@ namespace Legion {
   __op__(MEMORY CONSTRAINT, "Memory") \
   __op__(FIELD_CONSTRAINT, "Field") \
   __op__(ORDERING_CONSTRAINT, "Ordering") \
-  __op__(SPLITTING_CONSTRAINT, "Splitting") \
+  __op__(TILING_CONSTRAINT, "Tiling") \
   __op__(DIMENSION_CONSTRAINT, "Dimension") \
   __op__(ALIGNMENT_CONSTRAINT, "Alignment") \
   __op__(OFFSET_CONSTRAINT, "Offset") \
-  __op__(POINTER_CONSTRAINT, "Pointer")
+  __op__(POINTER_CONSTRAINT, "Pointer") \
+  __op__(PADDING_CONSTRAINT, "Padding")
 
     /**
      * \class ISAConstraint
@@ -254,7 +255,7 @@ namespace Legion {
         { return NULL; }
       virtual OrderingConstraint* as_ordering_constraint(void)
         { return NULL; }
-      virtual SplittingConstraint* as_splitting_constraint(void)
+      virtual TilingConstraint* as_tiling_constraint(void)
         { return NULL; }
       virtual DimensionConstraint* as_dimension_constraint(void)
         { return NULL; } 
@@ -264,6 +265,8 @@ namespace Legion {
         { return NULL; }
       virtual PointerConstraint* as_pointer_constraint(void)
         { return NULL; }
+      virtual PaddingConstraint* as_padding_constraint(void)
+        { return NULL; }
     public:
       virtual const SpecializedConstraint* 
         as_specialized_constraint(void) const { return NULL; }
@@ -273,8 +276,8 @@ namespace Legion {
         as_field_constraint(void) const { return NULL; }
       virtual const OrderingConstraint* 
         as_ordering_constraint(void) const { return NULL; }
-      virtual const SplittingConstraint* 
-        as_splitting_constraint(void) const { return NULL; }
+      virtual const TilingConstraint* 
+        as_tiling_constraint(void) const { return NULL; }
       virtual const DimensionConstraint* 
         as_dimension_constraint(void) const { return NULL; }
       virtual const AlignmentConstraint* 
@@ -283,6 +286,8 @@ namespace Legion {
         as_offset_constraint(void) const { return NULL; }
       virtual const PointerConstraint* 
         as_pointer_constraint(void) const { return NULL; }
+      virtual const PaddingConstraint*
+        as_padding_constraint(void) const { return NULL; }
     };
 
     /**
@@ -451,14 +456,6 @@ namespace Legion {
      * almost certainly be to specify a total order. Users
      * can also specify whether or not the related constraints
      * are contiguous with each other.
-     *
-     * It is important to note that users cannot mix full
-     * dimensions with split dimensions. For example, either
-     * 'DIM_X' can be used or 'INNER_DIM_X' and 'OUTER_DIM_X'
-     * but never both at the same time. The 'INNER' and 
-     * 'OUTER' dims may only be specified in a dimension
-     * constraint if there is an associated split constraint
-     * saying how to split the logical dimension.
      */
     class OrderingConstraint : public LayoutConstraint {
     public:
@@ -493,44 +490,43 @@ namespace Legion {
     };
 
     /**
-     * \class SplittingConstraint
-     * Specify how to split a normal index space dimension into
-     * an inner and outer dimension. The split can be specified
-     * in one of two ways: either by saying how many chunks to
-     * break the dimension into, or by specifying a splitting
-     * factor that will create as many chunks as necessary.
+     * \class TilingConstraint 
+     * Specify how to tile a dimension of an index space, The tile can 
+     * be specified in one of two ways: either by saying how many tiles
+     * to break the dimension into, or by specifying a tiling
+     * factor that will create as many tiles as necessary.
      * These two constructors provide both top-down and bottom-up
      * ways of saying how to break a dimension apart.
      */
-    class SplittingConstraint : public LayoutConstraint { 
+    class TilingConstraint : public LayoutConstraint { 
     public:
       static const LayoutConstraintKind constraint_kind = 
-                                            LEGION_SPLITTING_CONSTRAINT;
+                                            LEGION_TILING_CONSTRAINT;
     public:
-      SplittingConstraint(void);
-      SplittingConstraint(DimensionKind dim); // chunks
-      SplittingConstraint(DimensionKind dim, size_t value);
+      TilingConstraint(void);
+      TilingConstraint(DimensionKind dim); // tiles
+      TilingConstraint(DimensionKind dim, size_t value, bool tiles = false);
     public:
-      inline bool operator==(const SplittingConstraint &o) const
-      { return kind == o.kind && value == o.value && chunks == o.chunks; }
+      inline bool operator==(const TilingConstraint &o) const
+      { return dim == o.dim && value == o.value && tiles == o.tiles; }
     public:
       virtual LayoutConstraintKind get_constraint_kind(void) const
         { return constraint_kind; }
-      virtual SplittingConstraint* as_splitting_constraint(void) 
+      virtual TilingConstraint* as_tiling_constraint(void) 
         { return this; }
-      virtual const SplittingConstraint* as_splitting_constraint(void) const
+      virtual const TilingConstraint* as_tiling_constraint(void) const
         { return this; }
     public:
-      bool entails(const SplittingConstraint &other) const;
-      bool conflicts(const SplittingConstraint &other) const;
+      bool entails(const TilingConstraint &other) const;
+      bool conflicts(const TilingConstraint &other) const;
     public:
-      void swap(SplittingConstraint &rhs);
+      void swap(TilingConstraint &rhs);
       void serialize(Serializer &rez) const;
       void deserialize(Deserializer &derez);
     public:
-      DimensionKind kind;
+      DimensionKind dim;
       size_t value;
-      bool chunks;
+      bool tiles;
     };
 
     /**
@@ -674,6 +670,41 @@ namespace Legion {
       uintptr_t ptr;
     };
 
+    /**
+     * \class PaddingConstraint
+     * Specify additional scratch space padding around the instance.
+     * This can only be specified for non-compact instances
+     */
+    class PaddingConstraint : public LayoutConstraint {
+    public:
+      static const LayoutConstraintKind constraint_kind =
+                                          LEGION_PADDING_CONSTRAINT;
+    public:
+      PaddingConstraint(void) = default;
+      PaddingConstraint(const Domain &delta);
+      PaddingConstraint(const DomainPoint &lower, const DomainPoint &upper);
+    public:
+      inline bool operator==(const PaddingConstraint &rhs) const 
+      { return (delta == rhs.delta); }
+    public:
+      virtual LayoutConstraintKind get_constraint_kind(void) const
+        { return constraint_kind; }
+      virtual PaddingConstraint* as_padding_constraint(void) 
+        { return this; }
+      virtual const PaddingConstraint* as_padding_constraint(void) const
+        { return this; }
+    public:
+      bool entails(const PaddingConstraint &other) const;
+      bool conflicts(const PaddingConstraint &other) const;
+    public:
+      void swap(PaddingConstraint &rhs);
+      void serialize(Serializer &rez) const;
+      void deserialize(Deserializer &derez);
+    public:
+      // Should be positive integers for offsets
+      Domain delta;
+    };
+
     /**
      * \class LayoutConstraintSet
      * Provide a class for tracking all the associated 
@@ -693,7 +724,7 @@ namespace Legion {
       LayoutConstraintSet&
         add_constraint(const OrderingConstraint &constraint);
       LayoutConstraintSet&
-        add_constraint(const SplittingConstraint &constraint);
+        add_constraint(const TilingConstraint &constraint);
       LayoutConstraintSet&
         add_constraint(const DimensionConstraint &constraint);
       LayoutConstraintSet&
@@ -702,13 +733,16 @@ namespace Legion {
         add_constraint(const OffsetConstraint &constraint);
       LayoutConstraintSet&
         add_constraint(const PointerConstraint &constraint);
+      LayoutConstraintSet&
+        add_constraint(const PaddingConstraint &constraint);
     public:
       bool operator==(const LayoutConstraintSet &other) const;
     public:
       // failed_constraint will be the one from 'other' that wasn't entailed
       bool entails(const LayoutConstraintSet &other, 
                    unsigned total_dims = 0,
-                   const LayoutConstraint **failed_constraint = NULL) const;
+                   const LayoutConstraint **failed_constraint = NULL,
+                   bool test_pointer = true) const;
       // conflict_constraint will be the one from 'this' that conficted
       // with a constraint from 'other'
       bool conflicts(const LayoutConstraintSet &other,
@@ -725,8 +759,9 @@ namespace Legion {
       FieldConstraint                  field_constraint;
       MemoryConstraint                 memory_constraint;
       PointerConstraint                pointer_constraint;
+      PaddingConstraint                padding_constraint;
       OrderingConstraint               ordering_constraint;
-      std::vector<SplittingConstraint> splitting_constraints;
+      std::vector<TilingConstraint>    tiling_constraints;
       std::vector<DimensionConstraint> dimension_constraints;
       std::vector<AlignmentConstraint> alignment_constraints; 
       std::vector<OffsetConstraint>    offset_constraints;
