@@ -1,4 +1,4 @@
-/* Copyright 2021 Stanford University, NVIDIA Corporation
+/* Copyright 2023 Stanford University, NVIDIA Corporation
  *
  * Licensed under the Apache License, Version 2.0 (the "License");
  * you may not use this file except in compliance with the License.
@@ -44,6 +44,10 @@ namespace Legion {
    */
   template<int M, int N, typename T = coord_t>
   struct AffineTransform {
+  private:
+    static_assert(M > 0, "M must be positive");
+    static_assert(N > 0, "N must be positive");
+    static_assert(std::is_integral<T>::value, "must be integral type");
   public:
     __CUDA_HD__
     AffineTransform(void); // default to identity transform
@@ -86,6 +90,10 @@ namespace Legion {
    */
   template<int M, int N, typename T = coord_t>
   struct ScaleTransform {
+  private:
+    static_assert(M > 0, "M must be positive");
+    static_assert(M > 0, "N must be positive");
+    static_assert(std::is_integral<T>::value, "must be integral type");
   public:
     __CUDA_HD__
     ScaleTransform(void); // default to identity transform
@@ -123,7 +131,7 @@ namespace Legion {
    */
   class DomainPoint {
   public:
-    enum { MAX_POINT_DIM = LEGION_MAX_DIM };
+    static constexpr int MAX_POINT_DIM = LEGION_MAX_DIM;
 
     __CUDA_HD__
     DomainPoint(void);
@@ -141,6 +149,8 @@ namespace Legion {
 
     __CUDA_HD__
     DomainPoint& operator=(const DomainPoint &rhs);
+    template<int DIM, typename T> __CUDA_HD__
+    DomainPoint& operator=(const Point<DIM,T> &rhs);
     __CUDA_HD__
     bool operator==(const DomainPoint &rhs) const;
     __CUDA_HD__
@@ -233,6 +243,8 @@ namespace Legion {
     static DomainPoint nil(void);
 
   protected:
+    template<typename T> __CUDA_HD__
+    static inline coord_t check_for_overflow(const T &value);
   public:
     int dim;
     coord_t point_data[MAX_POINT_DIM];
@@ -250,7 +262,7 @@ namespace Legion {
     typedef ::realm_id_t IDType;
     // Keep this in sync with legion_domain_max_rect_dim_t
     // in legion_config.h
-    enum { MAX_RECT_DIM = LEGION_MAX_DIM };
+    static constexpr int MAX_RECT_DIM = LEGION_MAX_DIM;
     __CUDA_HD__
     Domain(void);
     __CUDA_HD__
@@ -266,6 +278,10 @@ namespace Legion {
 
     __CUDA_HD__
     Domain& operator=(const Domain& other);
+    template<int DIM, typename T> __CUDA_HD__
+    Domain& operator=(const Rect<DIM,T> &other);
+    template<int DIM, typename T> __CUDA_HD__
+    Domain& operator=(const DomainT<DIM,T> &other);
 
     __CUDA_HD__
     bool operator==(const Domain &rhs) const;
@@ -318,11 +334,11 @@ namespace Legion {
     __CUDA_HD__
     bool is_valid(void) const;
 
-    bool contains(DomainPoint point) const;
+    bool contains(const DomainPoint &point) const;
 
     // This will only check the bounds and not the sparsity map
     __CUDA_HD__
-    bool contains_bounds_only(DomainPoint point) const;
+    bool contains_bounds_only(const DomainPoint &point) const;
 
     __CUDA_HD__
     int get_dim(void) const;
@@ -361,27 +377,176 @@ namespace Legion {
       DomainPointIterator operator++(int /*i am postfix*/);
     public:
       DomainPoint p;
-      // Some buffers that we will do in-place new statements to in
-      // order to not have to call new/delete in our implementation
-      char is_iterator[
+      // Note: GCC 4.9 breaks even with C++11, so for now peg this on
+      // C++14 until we deprecate GCC 4.9 support.
+#if __cplusplus >= 201402L
+      // Realm's iterators are copyable by value so we can just always
+      // copy them in and out of some buffers
+      static_assert(std::is_trivially_copyable<
+          Realm::IndexSpaceIterator<MAX_RECT_DIM,coord_t> >::value, "very bad");
+      static_assert(std::is_trivially_copyable<
+          Realm::PointInRectIterator<MAX_RECT_DIM,coord_t> >::value,"very bad");
+#endif
+      uint8_t is_iterator[
               sizeof(Realm::IndexSpaceIterator<MAX_RECT_DIM,coord_t>)];
-      char rect_iterator[
+      uint8_t rect_iterator[
               sizeof(Realm::PointInRectIterator<MAX_RECT_DIM,coord_t>)];
+      TypeTag is_type;
       bool is_valid, rect_valid;
     };
   protected:
   public:
     IDType is_id;
+    // For Realm index spaces we need to have a type tag to know
+    // what the type of the original sparsity map was
+    // Without it you can get undefined behavior trying to interpret
+    // the sparsity map incorrectly. This doesn't matter for the bounds
+    // data because we've done the conversion for ourselves.
+    // Technically this is redundant with the dimension since it also
+    // encodes the dimension, but we'll keep them separate for now for
+    // backwards compatibility
+    TypeTag is_type;
 #if defined(__CUDA_ARCH__) || defined(__HIP_DEVICE_COMPILE__)
     // Work around an internal nvcc bug by marking this volatile 
     volatile
 #endif
     int dim;
     coord_t rect_data[2 * MAX_RECT_DIM];
+  private:
+    // Helper functor classes for demux-ing templates when we have
+    // non-trivial sparsity maps with unusual types
+    // User's should never need to look at these hence they are private
+    template<typename T> __CUDA_HD__
+    static inline coord_t check_for_overflow(const T &value);
+    struct ContainsFunctor {
+    public: 
+      ContainsFunctor(const Domain &d, const DomainPoint &p, bool &res)
+        : domain(d), point(p), result(res) { }
+      template<typename N, typename T>
+      static inline void demux(ContainsFunctor *functor)
+      {
+        DomainT<N::N,T> is = functor->domain;
+        Point<N::N,T> p = functor->point;
+        functor->result = is.contains(p);
+      }
+    public:
+      const Domain &domain;
+      const DomainPoint &point;
+      bool &result;
+    };
+    struct VolumeFunctor {
+    public:
+      VolumeFunctor(const Domain &d, size_t &r)
+        : domain(d), result(r) { }
+      template<typename N, typename T>
+      static inline void demux(VolumeFunctor *functor)
+      {
+        DomainT<N::N,T> is = functor->domain;
+        functor->result = is.volume();
+      }
+    public:
+      const Domain &domain;
+      size_t &result;
+    };
+    struct IntersectionFunctor {
+    public:
+      IntersectionFunctor(const Domain &l, const Domain &r, Domain &res)
+        : lhs(l), rhs(r), result(res) { }
+    public:
+      template<typename N, typename T>
+      static inline void demux(IntersectionFunctor *functor)
+      {
+        DomainT<N::N,T> is1 = functor->lhs;
+        DomainT<N::N,T> is2 = functor->rhs;
+        Realm::ProfilingRequestSet dummy_requests;
+        DomainT<N::N,T> temp;
+        Internal::LgEvent wait_on(
+            DomainT<N::N,T>::compute_intersection(is1, is2,
+              temp, dummy_requests));
+        if (wait_on.exists())
+          wait_on.wait();
+        functor->result = Domain(temp.tighten());
+        temp.destroy();
+      }
+    public:
+      const Domain &lhs;
+      const Domain &rhs;
+      Domain &result;
+    };
+    struct IteratorInitFunctor {
+    public:
+      IteratorInitFunctor(const Domain &d, DomainPointIterator &i)
+        : domain(d), iterator(i) { }
+    public:
+      template<typename N, typename T>
+      static inline void demux(IteratorInitFunctor *functor)
+      {
+        DomainT<N::N,T> is = functor->domain;
+        Realm::IndexSpaceIterator<N::N,T> is_itr(is);
+        static_assert(sizeof(is_itr) <=
+            sizeof(functor->iterator.is_iterator), "very bad");
+        functor->iterator.is_valid = is_itr.valid;
+        if (is_itr.valid)
+        {
+          // Always use coord_t for the rect so we don't demux unnecessarily
+          Realm::Rect<N::N,coord_t> rect = is_itr.rect;
+          Realm::PointInRectIterator<N::N,coord_t> rect_itr(rect);
+          static_assert(sizeof(rect_itr) <=
+              sizeof(functor->iterator.rect_iterator), "very bad");
+          assert(rect_itr.valid);
+          functor->iterator.rect_valid = true;
+          functor->iterator.p = rect_itr.p;
+          memcpy(functor->iterator.rect_iterator, &rect_itr, sizeof(rect_itr));
+          memcpy(functor->iterator.is_iterator, &is_itr, sizeof(is_itr));
+        }
+        else
+          functor->iterator.rect_valid = false; \
+      }
+    public:
+      const Domain &domain;
+      DomainPointIterator &iterator;
+    };
+    struct IteratorStepFunctor {
+    public:
+      IteratorStepFunctor(DomainPointIterator &i)
+        : iterator(i) { }
+    public:
+      template<typename N, typename T>
+      static inline void demux(IteratorStepFunctor *functor)
+      {
+        // We already know the rect iterator is not valid here
+#ifdef DEBUG_LEGION
+        assert(!functor->iterator.rect_valid);
+#endif
+        Realm::IndexSpaceIterator<N::N,T> is_itr;
+        memcpy(&is_itr, functor->iterator.is_iterator, sizeof(is_itr));
+        is_itr.step(); \
+        functor->iterator.is_valid = is_itr.valid;
+        if (is_itr.valid)
+        {
+          // Convert to rect with coord_t
+          Realm::Rect<N::N,coord_t> rect = is_itr.rect;
+          Realm::PointInRectIterator<N::N,coord_t> new_rectitr(rect);
+#ifdef DEBUG_LEGION
+          assert(new_rectitr.valid);
+#endif
+          functor->iterator.rect_valid = true;
+          functor->iterator.p = new_rectitr.p;
+          memcpy(functor->iterator.rect_iterator, &new_rectitr,
+                  sizeof(new_rectitr));
+          memcpy(functor->iterator.is_iterator, &is_itr, sizeof(is_itr));
+        }
+      }
+    public:
+      DomainPointIterator &iterator;
+    };
   };
 
   template<int DIM, typename COORD_T = coord_t>
   class PointInRectIterator {
+  private:
+    static_assert(DIM > 0, "DIM must be positive");
+    static_assert(std::is_integral<COORD_T>::value, "must be integral type");
   public:
     __CUDA_HD__
     PointInRectIterator(void);
@@ -412,6 +577,9 @@ namespace Legion {
 
   template<int DIM, typename COORD_T = coord_t>
   class RectInDomainIterator {
+  private:
+    static_assert(DIM > 0, "DIM must be positive");
+    static_assert(std::is_integral<COORD_T>::value, "must be integral type");
   public:
     RectInDomainIterator(void);
     RectInDomainIterator(const DomainT<DIM,COORD_T> &d);
@@ -430,6 +598,9 @@ namespace Legion {
 
   template<int DIM, typename COORD_T = coord_t>
   class PointInDomainIterator {
+  private:
+    static_assert(DIM > 0, "DIM must be positive");
+    static_assert(std::is_integral<COORD_T>::value, "must be integral type");
   public:
     PointInDomainIterator(void);
     PointInDomainIterator(const DomainT<DIM,COORD_T> &d,
@@ -581,9 +752,15 @@ namespace Legion {
   template<typename FT, PrivilegeMode PM = LEGION_READ_WRITE>
   class Span {
   public:
-    class iterator : 
-      public std::iterator<std::random_access_iterator_tag,FT> {
+    class iterator {
     public:
+      // explicitly set iterator traits
+      typedef std::random_access_iterator_tag iterator_category;
+      typedef FT value_type;
+      typedef std::ptrdiff_t difference_type;
+      typedef FT *pointer;
+      typedef FT& reference;
+
       iterator(void) : ptr(NULL), stride(0) { } 
     private:
       iterator(uint8_t *p, size_t s) : ptr(p), stride(s) { }
@@ -642,9 +819,15 @@ namespace Legion {
       uint8_t *ptr;
       size_t stride;
     };
-    class reverse_iterator : 
-      public std::iterator<std::random_access_iterator_tag,FT> {
+    class reverse_iterator {
     public:
+      // explicitly set iterator traits
+      typedef std::random_access_iterator_tag iterator_category;
+      typedef FT value_type;
+      typedef std::ptrdiff_t difference_type;
+      typedef FT *pointer;
+      typedef FT& reference;
+
       reverse_iterator(void) : ptr(NULL), stride(0) { } 
     private:
       reverse_iterator(uint8_t *p, size_t s) : ptr(p), stride(s) { }
