@@ -1,4 +1,4 @@
-/* Copyright 2021 Stanford University, NVIDIA Corporation
+/* Copyright 2023 Stanford University, NVIDIA Corporation
  *
  * Licensed under the Apache License, Version 2.0 (the "License");
  * you may not use this file except in compliance with the License.
@@ -54,19 +54,13 @@ namespace Legion {
     public:
       LayoutDescription& operator=(const LayoutDescription &rhs);
     public:
-      void log_instance_layout(ApEvent inst_event) const;
+      void log_instance_layout(LgEvent inst_event) const;
     public:
       void compute_copy_offsets(const FieldMask &copy_mask, 
                                 const PhysicalInstance instance,  
-#ifdef LEGION_SPY
-                                const ApEvent inst_event, 
-#endif
                                 std::vector<CopySrcDstField> &fields);
       void compute_copy_offsets(const std::vector<FieldID> &copy_fields,
                                 const PhysicalInstance instance,
-#ifdef LEGION_SPY
-                                const ApEvent inst_event,
-#endif
                                 std::vector<CopySrcDstField> &fields);
     public:
       void get_fields(std::set<FieldID> &fields) const;
@@ -102,7 +96,7 @@ namespace Legion {
     protected:
       mutable LocalLock layout_lock; 
       std::map<LEGION_FIELD_MASK_FIELD_TYPE,
-               LegionList<std::pair<FieldMask,FieldMask> >::aligned> comp_cache;
+               LegionList<std::pair<FieldMask,FieldMask> > > comp_cache;
     }; 
 
     /**
@@ -132,7 +126,7 @@ namespace Legion {
           get_field_accessor(FieldID fid) const = 0;
     public: 
       virtual ApEvent get_use_event(void) const = 0;
-      virtual ApEvent get_unique_event(void) const = 0;
+      virtual LgEvent get_unique_event(void) const = 0;
       virtual PhysicalInstance get_instance(const DomainPoint &key) const = 0;
       virtual PointerConstraint 
                      get_pointer_constraint(const DomainPoint &key) const = 0;
@@ -218,6 +212,13 @@ namespace Legion {
         // Events added since the last collection of view events
         unsigned events_added;
       };
+      enum GarbageCollectionState {
+        VALID_GC_STATE,
+        ACQUIRED_GC_STATE,
+        COLLECTABLE_GC_STATE,
+        PENDING_COLLECTED_GC_STATE,
+        COLLECTED_GC_STATE,
+      };
     public:
       PhysicalManager(RegionTreeForest *ctx, LayoutDescription *layout, 
                       DistributedID did, AddressSpaceID owner_space, 
@@ -225,11 +226,11 @@ namespace Legion {
                       const ReductionOp *rop, FieldSpaceNode *node,
                       IndexSpaceExpression *index_domain, 
                       const void *piece_list, size_t piece_list_size,
-                      RegionTreeID tree_id, ApEvent unique, 
-                      bool register_now, bool shadow_instance = false);
+                      RegionTreeID tree_id, LgEvent unique, 
+                      bool register_now);
       virtual ~PhysicalManager(void);
     public:
-      virtual ApEvent get_unique_event(void) const { return unique_event; }
+      virtual LgEvent get_unique_event(void) const { return unique_event; }
     public:
       void log_instance_creation(UniqueID creator_id, Processor proc,
                      const std::vector<LogicalRegion> &regions) const; 
@@ -259,29 +260,28 @@ namespace Legion {
       static void handle_manager_request(Deserializer &derez, 
                           Runtime *runtime, AddressSpaceID source);
     public:
-      virtual bool acquire_instance(ReferenceSource source, 
-                                    ReferenceMutator *mutator) = 0;
-      virtual void perform_deletion(RtEvent deferred_event) = 0;
+      virtual void notify_active(ReferenceMutator *mutator);
+      virtual void notify_inactive(ReferenceMutator *mutator);
+      virtual void notify_valid(ReferenceMutator *mutator);
+      virtual void notify_invalid(ReferenceMutator *mutator);
+    public:
+      bool acquire_instance(ReferenceSource source, ReferenceMutator *mutator);
+      bool can_collect(AddressSpaceID source, bool &already_collected);
+      bool collect(RtEvent &collected);
+      RtEvent set_garbage_collection_priority(MapperID mapper_id,
+                                              Processor p, GCPriority priority);
+      virtual void get_instance_pointers(Memory memory, 
+                                    std::vector<uintptr_t> &pointers) const = 0;
+      virtual RtEvent perform_deletion(AddressSpaceID source, 
+                                       AutoLock *i_lock = NULL) = 0;
       virtual void force_deletion(void) = 0;
-      virtual void set_garbage_collection_priority(MapperID mapper_id, 
-                                Processor p, GCPriority priority) = 0; 
+      virtual RtEvent update_garbage_collection_priority(
+                                                       GCPriority priority) = 0;
       virtual RtEvent attach_external_instance(void) = 0;
       virtual RtEvent detach_external_instance(void) = 0;
       virtual bool has_visible_from(const std::set<Memory> &memories) const = 0;
       virtual Memory get_memory(void) const = 0; 
       inline size_t get_instance_size(void) const { return instance_footprint; }
-#ifdef LEGION_GPU_REDUCTIONS
-    public:
-      virtual bool is_gpu_visible(PhysicalManager *other) const = 0;
-      virtual ReductionView* find_or_create_shadow_reduction(unsigned fidx,
-          ReductionOpID redop, AddressSpaceID request_space, UniqueID opid) = 0;
-      virtual void record_remote_shadow_reduction(unsigned fidx,
-          ReductionOpID redop, ReductionView *view) = 0;
-      static void handle_create_shadow_request(Runtime *runtime,
-                                AddressSpaceID source, Deserializer &derez);
-      static void handle_create_shadow_response(Runtime *runtime,
-                                                Deserializer &derez);
-#endif
     public:
       // Methods for creating/finding/destroying logical top views
       virtual InstanceView* create_instance_top_view(InnerContext *context,
@@ -299,32 +299,67 @@ namespace Legion {
                          bool tight_region_bounds = false) const;
       bool meets_expression(IndexSpaceExpression *expr, 
                             bool tight_bounds = false) const;
+    public:
+      void find_padded_reservations(const FieldMask &mask,
+                                    Operation *op, unsigned index);
+      void find_field_reservations(const FieldMask &mask,
+                                   std::vector<Reservation> &results);
+      static void handle_padded_reservation_request(Runtime *runtime,
+                                  Deserializer &derez, AddressSpaceID source);
+      void update_field_reservations(const FieldMask &mask,
+                                  const std::vector<Reservation> &reservations);
+      static void handle_padded_reservation_response(Runtime *runtime,
+                                                     Deserializer &derez);
     protected:
       void prune_gc_events(void);
+      void pack_garbage_collection_state(Serializer &rez,AddressSpaceID target);
+      void initialize_remote_gc_state(GarbageCollectionState state);
     public: 
       static ApEvent fetch_metadata(PhysicalInstance inst, ApEvent use_event);
+      static void handle_acquire_request(Runtime *runtime,
+          Deserializer &derez, AddressSpaceID source);
+      static void handle_acquire_response(Deserializer &derez, 
+          AddressSpaceID source);
+      static void handle_garbage_collection_request(Runtime *runtime,
+          Deserializer &derez, AddressSpaceID source);
+      static void handle_garbage_collection_response(Deserializer &derez);
+      static void handle_garbage_collection_acquire(Runtime *runtime,
+          Deserializer &derez, AddressSpaceID source);
+      static void handle_garbage_collection_acquired(Deserializer &derez);
+      static void handle_garbage_collection_priority_update(Runtime *runtime,
+          Deserializer &derez, AddressSpaceID source);
+      static void handle_garbage_collection_debug_request(Runtime *runtime,
+          Deserializer &derez, AddressSpaceID source);
+      static void handle_garbage_collection_debug_response(Deserializer &derez);
     public:
       const size_t instance_footprint;
       const ReductionOp *reduction_op;
       const ReductionOpID redop;
       // Unique identifier event that is common across nodes
-      const ApEvent unique_event;
+      // Note this is just an LgEvent which suggests you shouldn't be using
+      // it for anything other than logging
+      const LgEvent unique_event;
       const void *const piece_list;
       const size_t piece_list_size;
-      const bool shadow_instance;
     protected:
       mutable LocalLock inst_lock;
       std::set<InnerContext*> active_contexts;
-#ifdef LEGION_GPU_REDUCTIONS
     protected:
-      std::map<std::pair<unsigned/*fidx*/,ReductionOpID>,ReductionView*>
-                                              shadow_reduction_instances;
-      std::map<std::pair<unsigned/*fidx*/,ReductionOpID>,RtEvent>
-                                              pending_reduction_shadows;
-#endif
+      // Stuff for garbage collection
+      GarbageCollectionState gc_state; 
+      unsigned pending_changes;
+      std::atomic<unsigned> remaining_collection_guards;
+      RtEvent collection_ready;
+      RtUserEvent deferred_deletion;
+      bool currently_active;
+      // Garbage collection priorities
+      GCPriority min_gc_priority;
+      RtEvent priority_update_done;
+      std::map<std::pair<MapperID,Processor>,GCPriority> mapper_gc_priorities;
     private:
       // Events that have to trigger before we can remove our GC reference
       std::map<CollectableView*,CollectableInfo> gc_events;
+      std::map<unsigned,Reservation> *padded_reservations;
     };
 
     /**
@@ -351,7 +386,7 @@ namespace Legion {
       FieldMask convert_dst_to_src(const FieldMask &dst_mask);
     public:
       std::vector<CopySrcDstField> offsets; 
-      LegionDeque<std::pair<FieldMask,FieldMask> >::aligned compressed_cache;
+      LegionDeque<std::pair<FieldMask,FieldMask> > compressed_cache;
     };
 
     /**
@@ -371,31 +406,28 @@ namespace Legion {
         static const LgTaskID TASK_ID = LG_DEFER_INDIVIDUAL_MANAGER_TASK_ID;
       public:
         DeferIndividualManagerArgs(DistributedID d, AddressSpaceID own, 
-            Memory m, PhysicalInstance i, size_t f, bool local, 
-            IndexSpaceExpression *lx, bool is, IndexSpace dh, 
-            IndexSpaceExprID dx, FieldSpace h, RegionTreeID tid, 
-            LayoutConstraintID l, ApEvent use, ReductionOpID redop, 
-            const void *piece_list, size_t piece_list_size, 
-            bool shadow_instance);
+            Memory m, PhysicalInstance i, size_t f, IndexSpaceExpression *lx,
+            const PendingRemoteExpression &pending, FieldSpace h, 
+            RegionTreeID tid, LayoutConstraintID l, LgEvent use,
+            ReductionOpID redop, const void *piece_list, size_t piece_list_size,
+            AddressSpaceID src, GarbageCollectionState state);
       public:
         const DistributedID did;
         const AddressSpaceID owner;
         const Memory mem;
         const PhysicalInstance inst;
         const size_t footprint;
-        const bool local_is;
-        const bool domain_is;
+        const PendingRemoteExpression pending;
         IndexSpaceExpression *local_expr;
-        const IndexSpace domain_handle;
-        const IndexSpaceExprID domain_expr;
         const FieldSpace handle;
         const RegionTreeID tree_id;
         const LayoutConstraintID layout_id;
-        const ApEvent use_event;
+        const LgEvent unique_event;
         const ReductionOpID redop;
         const void *const piece_list;
         const size_t piece_list_size;
-        const bool shadow_instance;
+        const AddressSpaceID source;
+        const GarbageCollectionState state;
       };
     public:
       IndividualManager(RegionTreeForest *ctx, DistributedID did,
@@ -406,18 +438,12 @@ namespace Legion {
                         FieldSpaceNode *node, RegionTreeID tree_id,
                         LayoutDescription *desc, ReductionOpID redop, 
                         bool register_now, size_t footprint,
-                        ApEvent use_event, bool external_instance,
-                        const ReductionOp *op = NULL,
-                        bool shadow_instance = false);
-      IndividualManager(const IndividualManager &rhs);
+                        ApEvent use_event, LgEvent unique_event,
+                        bool external_instance, const ReductionOp *op = NULL);
+      IndividualManager(const IndividualManager &rhs) = delete;
       virtual ~IndividualManager(void);
     public:
-      IndividualManager& operator=(const IndividualManager &rhs);
-    public:
-      virtual void notify_active(ReferenceMutator *mutator);
-      virtual void notify_inactive(ReferenceMutator *mutator);
-      virtual void notify_valid(ReferenceMutator *mutator);
-      virtual void notify_invalid(ReferenceMutator *mutator);
+      IndividualManager& operator=(const IndividualManager &rhs) = delete;
     public:
       virtual LegionRuntime::Accessor::RegionAccessor<
         LegionRuntime::Accessor::AccessorType::Generic>
@@ -468,27 +494,19 @@ namespace Legion {
           size_t inst_footprint, IndexSpaceExpression *inst_domain,
           const void *piece_list, size_t piece_list_size,
           FieldSpaceNode *space_node, RegionTreeID tree_id,
-          LayoutConstraints *constraints, ApEvent use_event,
-          ReductionOpID redop, bool shadow_instance);
+          LayoutConstraints *constraints, LgEvent unique_event,
+          ReductionOpID redop, GarbageCollectionState state);
     public:
-      virtual bool acquire_instance(ReferenceSource source, 
-                                    ReferenceMutator *mutator);
-      virtual void perform_deletion(RtEvent deferred_event);
+      virtual void get_instance_pointers(Memory memory, 
+                                    std::vector<uintptr_t> &pointers) const;
+      virtual RtEvent perform_deletion(AddressSpaceID source, 
+                                       AutoLock *i_lock = NULL);
       virtual void force_deletion(void);
-      virtual void set_garbage_collection_priority(MapperID mapper_id, 
-                                Processor p, GCPriority priority); 
+      virtual RtEvent update_garbage_collection_priority(GCPriority);
       virtual RtEvent attach_external_instance(void);
       virtual RtEvent detach_external_instance(void);
       virtual bool has_visible_from(const std::set<Memory> &memories) const;
       virtual Memory get_memory(void) const;
-#ifdef LEGION_GPU_REDUCTIONS
-    public:
-      virtual bool is_gpu_visible(PhysicalManager *other) const;
-      virtual ReductionView* find_or_create_shadow_reduction(unsigned fidx,
-          ReductionOpID redop, AddressSpaceID request_space, UniqueID opid); 
-      virtual void record_remote_shadow_reduction(unsigned fidx,
-          ReductionOpID redop, ReductionView *view);
-#endif
     public:
       MemoryManager *const memory_manager;
       const PhysicalInstance instance;
@@ -510,13 +528,8 @@ namespace Legion {
       static const AllocationType alloc_type = COLLECTIVE_INST_MANAGER_ALLOC;
     public:
       enum MessageKind {
-        ACTIVATE_MESSAGE,
-        DEACTIVATE_MESSAGE,
-        VALIDATE_MESSAGE,
-        INVALIDATE_MESSAGE,
         PERFORM_DELETE_MESSAGE,
         FORCE_DELETE_MESSAGE,
-        SET_GC_PRIORITY_MESSAGE,
         DETACH_EXTERNAL_MESSAGE,
         FINALIZE_MESSAGE,
       };
@@ -527,20 +540,19 @@ namespace Legion {
         static const LgTaskID TASK_ID = LG_DEFER_COLLECTIVE_MANAGER_TASK_ID;
       public:
         DeferCollectiveManagerArgs(DistributedID d, AddressSpaceID own, 
-            IndexSpace p, size_t f, bool local, IndexSpaceExpression *lx, 
-            bool is, IndexSpace dh, IndexSpaceExprID dx, FieldSpace h, 
+            IndexSpace p, size_t f, IndexSpaceExpression *lx, 
+            const PendingRemoteExpression &pending, FieldSpace h, 
             RegionTreeID tid, LayoutConstraintID l, ApEvent use, 
-            ReductionOpID redop, const void *piece_list,size_t piece_list_size);
+            ReductionOpID redop, const void *piece_list,
+            size_t piece_list_size, AddressSpaceID source,
+            GarbageCollectionState state);
       public:
         const DistributedID did;
         const AddressSpaceID owner;
         IndexSpace point_space;
         const size_t footprint;
-        const bool local_is;
-        const bool domain_is;
         IndexSpaceExpression *const local_expr;
-        const IndexSpace domain_handle;
-        const IndexSpaceExprID domain_expr;
+        const PendingRemoteExpression pending;
         const FieldSpace handle;
         const RegionTreeID tree_id;
         const LayoutConstraintID layout_id;
@@ -548,6 +560,8 @@ namespace Legion {
         const ReductionOpID redop;
         const void *const piece_list;
         const size_t piece_list_size;
+        const AddressSpaceID source;
+        const GarbageCollectionState state;
       };
     public:
       CollectiveManager(RegionTreeForest *ctx, DistributedID did,
@@ -557,23 +571,18 @@ namespace Legion {
                         FieldSpaceNode *node, RegionTreeID tree_id,
                         LayoutDescription *desc, ReductionOpID redop, 
                         bool register_now, size_t footprint,
-                        ApEvent unique_event, bool external_instance);
-      CollectiveManager(const CollectiveManager &rhs);
+                        LgEvent unique_event, bool external_instance);
+      CollectiveManager(const CollectiveManager &rhs) = delete;
       virtual ~CollectiveManager(void);
     public:
-      CollectiveManager& operator=(const CollectiveManager &rh);
+      CollectiveManager& operator=(const CollectiveManager &rh) = delete;
     public:
       void finalize_collective_instance(ApUserEvent instance_event);
     public:
       virtual ApEvent get_use_event(void) const;
       virtual PhysicalInstance get_instance(const DomainPoint &key) const;
       virtual PointerConstraint
-                     get_pointer_constraint(const DomainPoint &key) const;
-    public:
-      virtual void notify_active(ReferenceMutator *mutator);
-      virtual void notify_inactive(ReferenceMutator *mutator);
-      virtual void notify_valid(ReferenceMutator *mutator);
-      virtual void notify_invalid(ReferenceMutator *mutator);
+                     get_pointer_constraint(const DomainPoint &key) const; 
     public:
       virtual LegionRuntime::Accessor::RegionAccessor<
         LegionRuntime::Accessor::AccessorType::Generic>
@@ -581,18 +590,13 @@ namespace Legion {
       virtual LegionRuntime::Accessor::RegionAccessor<
         LegionRuntime::Accessor::AccessorType::Generic>
           get_field_accessor(FieldID fid) const;
-    protected:
-      void activate_collective(ReferenceMutator *mutator);
-      void deactivate_collective(ReferenceMutator *mutator);
-      void validate_collective(ReferenceMutator *mutator);
-      void invalidate_collective(ReferenceMutator *mutator);
     public:
-      virtual bool acquire_instance(ReferenceSource source, 
-                                    ReferenceMutator *mutator);
-      virtual void perform_deletion(RtEvent deferred_event);
+      virtual void get_instance_pointers(Memory memory, 
+                                    std::vector<uintptr_t> &pointers) const;
+      virtual RtEvent perform_deletion(AddressSpaceID source,
+                                       AutoLock *i_lock = NULL);
       virtual void force_deletion(void);
-      virtual void set_garbage_collection_priority(MapperID mapper_id, 
-                                    Processor p, GCPriority priority); 
+      virtual RtEvent update_garbage_collection_priority(GCPriority);
       virtual RtEvent attach_external_instance(void);
       virtual RtEvent detach_external_instance(void);
       virtual bool has_visible_from(const std::set<Memory> &memories) const;
@@ -600,16 +604,12 @@ namespace Legion {
     protected:
       void perform_delete(RtEvent deferred_event, bool left); 
       void force_delete(bool left);
-      void set_gc_priority(MapperID mapper_id, Processor p, 
-                           GCPriority priority, bool left);
       void detach_external(RtUserEvent to_trigger, bool left, 
                   RtEvent full_detach = RtEvent::NO_RT_EVENT);
       bool finalize_message(void);
     protected:
       void collective_deletion(RtEvent deferred_event);
       void collective_force(void);
-      void collective_set_gc_priority(MapperID mapper_id, Processor proc,
-                                      GCPriority priority);
       void collective_detach(std::set<RtEvent> &detach_events);
     public:
       virtual ApEvent fill_from(FillView *fill_view,
@@ -632,14 +632,6 @@ namespace Legion {
                                 CopyAcrossHelper *across_helper = NULL);
       virtual void compute_copy_offsets(const FieldMask &copy_mask,
                                 std::vector<CopySrcDstField> &fields);
-#ifdef LEGION_GPU_REDUCTIONS
-    public:
-      virtual bool is_gpu_visible(PhysicalManager *other) const;
-      virtual ReductionView* find_or_create_shadow_reduction(unsigned fidx,
-          ReductionOpID redop, AddressSpaceID request_space, UniqueID opid);
-      virtual void record_remote_shadow_reduction(unsigned fidx,
-          ReductionOpID redop, ReductionView *view);
-#endif
     public:
       virtual void send_manager(AddressSpaceID target);
     public:
@@ -654,7 +646,8 @@ namespace Legion {
           size_t inst_footprint, IndexSpaceExpression *inst_domain, 
           const void *piece_list, size_t piece_list_size, 
           FieldSpaceNode *space_node, RegionTreeID tree_id, 
-          LayoutConstraints *constraints,ApEvent use_event,ReductionOpID redop);
+          LayoutConstraints *constraints, ApEvent use_event,
+          ReductionOpID redop, GarbageCollectionState state);
     public:
       IndexSpaceNode *const point_space;
     protected:
@@ -697,7 +690,7 @@ namespace Legion {
       virtual void notify_invalid(ReferenceMutator *mutator);
     public: 
       virtual ApEvent get_use_event(void) const;
-      virtual ApEvent get_unique_event(void) const;
+      virtual LgEvent get_unique_event(void) const;
       virtual PhysicalInstance get_instance(const DomainPoint &key) const;
       virtual PointerConstraint
                      get_pointer_constraint(const DomainPoint &key) const;
@@ -719,20 +712,20 @@ namespace Legion {
           creator_id(cid), instance(PhysicalInstance::NO_INST), 
           field_space_node(NULL), instance_domain(NULL), tree_id(0),
           redop_id(0), reduction_op(NULL), realm_layout(NULL), piece_list(NULL),
-          piece_list_size(0), shadow_instance(false), valid(false) { }
+          piece_list_size(0), valid(false) { }
       InstanceBuilder(const std::vector<LogicalRegion> &regs,
                       IndexSpaceExpression *expr, FieldSpaceNode *node,
                       RegionTreeID tree_id, const LayoutConstraintSet &cons, 
                       Runtime *rt, MemoryManager *memory, UniqueID cid,
-                      const void *piece_list, size_t piece_list_size, 
-                      bool shadow_instance);
+                      const void *piece_list, size_t piece_list_size); 
       virtual ~InstanceBuilder(void);
     public:
       void initialize(RegionTreeForest *forest);
       PhysicalManager* create_physical_instance(RegionTreeForest *forest,
                         CollectiveManager *collective, DomainPoint *point,
                         LayoutConstraintKind *unsat_kind,
-                        unsigned *unsat_index, size_t *footprint = NULL);
+                        unsigned *unsat_index, size_t *footprint = NULL,
+                        RtEvent precondition = RtEvent::NO_RT_EVENT);
       CollectiveManager* create_collective_instance(RegionTreeForest *forest,
                         Memory::Kind mem_kind, IndexSpaceNode *point_space,
                         LayoutConstraintKind *unsat_kind, unsigned *unsat_index,
@@ -768,7 +761,6 @@ namespace Legion {
       Realm::InstanceLayoutGeneric *realm_layout;
       void *piece_list;
       size_t piece_list_size;
-      bool shadow_instance;
     public:
       bool valid;
     };
