@@ -1,4 +1,4 @@
-/* Copyright 2021 Stanford University, NVIDIA Corporation
+/* Copyright 2023 Stanford University, NVIDIA Corporation
  *
  * Licensed under the Apache License, Version 2.0 (the "License");
  * you may not use this file except in compliance with the License.
@@ -836,7 +836,7 @@ namespace Legion {
 
     //--------------------------------------------------------------------------
     IndexSpace MapperRuntime::create_index_space(MapperContext ctx, 
-                                   const Domain &bounds, TypeTag type_tag) const
+           const Domain &bounds, TypeTag type_tag, const char *provenance) const
     //--------------------------------------------------------------------------
     {
       if (type_tag == 0)
@@ -856,12 +856,12 @@ namespace Legion {
             assert(false);
         }
       }
-      return ctx->manager->create_index_space(ctx, bounds, type_tag);
+      return ctx->manager->create_index_space(ctx, bounds, type_tag,provenance);
     }
 
     //--------------------------------------------------------------------------
     IndexSpace MapperRuntime::create_index_space(MapperContext ctx,
-                                   const std::vector<DomainPoint> &points) const
+           const std::vector<DomainPoint> &points, const char *provenance) const
     //--------------------------------------------------------------------------
     {
       switch (points[0].get_dim())
@@ -876,7 +876,7 @@ namespace Legion {
               (Realm::IndexSpace<DIM,coord_t>(realm_points))); \
           const Domain domain(realm_is); \
           return ctx->manager->create_index_space(ctx, domain, \
-                    Internal::NT_TemplateHelper::encode_tag<DIM,coord_t>()); \
+           Internal::NT_TemplateHelper::encode_tag<DIM,coord_t>(),provenance); \
         }
         LEGION_FOREACH_N(DIMFUNC)
 #undef DIMFUNC
@@ -888,7 +888,7 @@ namespace Legion {
 
     //--------------------------------------------------------------------------
     IndexSpace MapperRuntime::create_index_space(MapperContext ctx,
-                                         const std::vector<Domain> &rects) const
+                 const std::vector<Domain> &rects, const char *provenance) const
     //--------------------------------------------------------------------------
     {
       switch (rects[0].get_dim())
@@ -903,7 +903,8 @@ namespace Legion {
                 (Realm::IndexSpace<DIM,coord_t>(realm_rects))); \
             const Domain domain(realm_is); \
             return ctx->manager->create_index_space(ctx, domain, \
-                      Internal::NT_TemplateHelper::encode_tag<DIM,coord_t>()); \
+                      Internal::NT_TemplateHelper::encode_tag<DIM,coord_t>(), \
+                      provenance); \
           }
         LEGION_FOREACH_N(DIMFUNC)
 #undef DIMFUNC
@@ -915,26 +916,26 @@ namespace Legion {
 
     //--------------------------------------------------------------------------
     IndexSpace MapperRuntime::union_index_spaces(MapperContext ctx,
-                                   const std::vector<IndexSpace> &sources) const
+           const std::vector<IndexSpace> &sources, const char *provenance) const
     //--------------------------------------------------------------------------
     {
-      return ctx->manager->union_index_spaces(ctx, sources);
+      return ctx->manager->union_index_spaces(ctx, sources, provenance);
     }
 
     //--------------------------------------------------------------------------
     IndexSpace MapperRuntime::intersect_index_spaces(MapperContext ctx,
-                                   const std::vector<IndexSpace> &sources) const
+           const std::vector<IndexSpace> &sources, const char *provenance) const
     //--------------------------------------------------------------------------
     {
-      return ctx->manager->intersect_index_spaces(ctx, sources);
+      return ctx->manager->intersect_index_spaces(ctx, sources, provenance);
     }
 
     //--------------------------------------------------------------------------
     IndexSpace MapperRuntime::subtract_index_spaces(MapperContext ctx,
-                                        IndexSpace left, IndexSpace right) const
+                IndexSpace left, IndexSpace right, const char *provenance) const
     //--------------------------------------------------------------------------
     {
-      return ctx->manager->subtract_index_spaces(ctx, left, right);
+      return ctx->manager->subtract_index_spaces(ctx, left, right, provenance);
     }
 
     //--------------------------------------------------------------------------
