@@ -63,6 +63,8 @@ namespace Legion {
       ss << "MetaDesc {" 
          << "id:" << META_DESC_ID                   << delim
          << "kind:unsigned:"     << sizeof(unsigned) << delim
+         << "message:bool:"      << sizeof(bool) << delim
+         << "ordered_vc:bool:"   << sizeof(bool) << delim
          << "name:string:" << "-1"
          << "}" << std::endl;
 
@@ -83,6 +85,12 @@ namespace Legion {
          << "max_dim:maxdim:" << sizeof(unsigned)
          << "}" << std::endl;
 
+      ss << "MachineDesc {"
+         << "id:" << MACHINE_DESC_ID                 << delim
+         << "node_id:unsigned:" << sizeof(unsigned)  << delim
+         << "num_nodes:unsigned:" << sizeof(unsigned)
+         << "}" << std::endl;
+
       ss << "MemDesc {" 
          << "id:" << MEM_DESC_ID                               << delim
          << "mem_id:MemID:"                << sizeof(MemID)    << delim
@@ -93,7 +101,9 @@ namespace Legion {
       ss << "ProcMDesc {"
          << "id:" << PROC_MEM_DESC_ID                          << delim
          << "proc_id:ProcID:"              << sizeof(ProcID)   << delim
-         << "mem_id:MemID:"                << sizeof(MemID)
+         << "mem_id:MemID:"                << sizeof(MemID)    << delim
+         << "bandwidth:unsigned:"          << sizeof(unsigned) << delim
+         << "latency:unsigned:"            << sizeof(unsigned)
          << "}" << std::endl;
 
       ss << "IndexSpacePointDesc {"
@@ -175,8 +185,7 @@ namespace Legion {
 
       ss << "PhysicalInstRegionDesc {"
          << "id:" << PHYSICAL_INST_REGION_ID                      << delim
-	 << "op_id:UniqueID:"              << sizeof(UniqueID)    << delim
-	 << "inst_id:InstID:"              << sizeof(InstID)      << delim
+         << "inst_uid:unsigned long long:" << sizeof(LgEvent)     << delim
 	 << "ispace_id:IDType:"            << sizeof(IDType)      << delim
 	 << "fspace_id:unsigned:"          << sizeof(unsigned)    << delim
 	 << "tree_id:unsigned:"            << sizeof(unsigned)
@@ -184,8 +193,7 @@ namespace Legion {
 
       ss << "PhysicalInstLayoutDesc {"
          << "id:" << PHYSICAL_INST_LAYOUT_ID                  << delim
-         << "op_id:UniqueID:"           << sizeof(UniqueID)   << delim
-         << "inst_id:InstID:"           << sizeof(InstID)     << delim
+         << "inst_uid:unsigned long long:" << sizeof(LgEvent) << delim
          << "field_id:unsigned:"        << sizeof(unsigned)   << delim
          << "fspace_id:unsigned:"       << sizeof(unsigned)   << delim
          << "has_align:bool:"           << sizeof(bool)       << delim
@@ -195,12 +203,19 @@ namespace Legion {
 
       ss << "PhysicalInstDimOrderDesc {"
          << "id:" << PHYSICAL_INST_LAYOUT_DIM_ID              << delim
-         << "op_id:UniqueID:"           << sizeof(UniqueID)   << delim
-         << "inst_id:InstID:"           << sizeof(InstID)     << delim
+         << "inst_uid:unsigned long long:" << sizeof(LgEvent) << delim
          << "dim:unsigned:"             << sizeof(unsigned)   << delim
          << "dim_kind:unsigned:"        << sizeof(unsigned)
          << "}" << std::endl;
 
+      ss << "PhysicalInstanceUsage {"
+         << "id:" << PHYSICAL_INST_USAGE_ID                   << delim
+         << "inst_uid:unsigned long long:" << sizeof(LgEvent) << delim
+         << "op_id:UniqueID:"           << sizeof(UniqueID)   << delim
+         << "index_id:unsigned:"        << sizeof(unsigned)   << delim
+         << "field_id:unsigned:"        << sizeof(unsigned)
+         << "}" << std::endl;
+
       ss << "TaskKind {" 
          << "id:" << TASK_KIND_ID                 << delim
          << "task_id:TaskID:"   << sizeof(TaskID) << delim
@@ -218,7 +233,9 @@ namespace Legion {
       ss << "OperationInstance {"
          << "id:" << OPERATION_INSTANCE_ID        << delim
          << "op_id:UniqueID:" << sizeof(UniqueID) << delim
-         << "kind:unsigned:"  << sizeof(unsigned)
+         << "parent_id:UniqueID:" << sizeof(UniqueID) << delim
+         << "kind:unsigned:"  << sizeof(unsigned) << delim
+         << "provenance:string:" << "-1"
          << "}" << std::endl;
 
       ss << "MultiTask {"
@@ -307,15 +324,14 @@ namespace Legion {
       ss << "CopyInfo {"
          << "id:" << COPY_INFO_ID                                    << delim
          << "op_id:UniqueID:"          << sizeof(UniqueID)           << delim
-         << "src:MemID:"               << sizeof(MemID)              << delim
-         << "dst:MemID:"               << sizeof(MemID)              << delim
          << "size:unsigned long long:" << sizeof(unsigned long long) << delim
          << "create:timestamp_t:"      << sizeof(timestamp_t)        << delim
          << "ready:timestamp_t:"       << sizeof(timestamp_t)        << delim
          << "start:timestamp_t:"       << sizeof(timestamp_t)        << delim
          << "stop:timestamp_t:"        << sizeof(timestamp_t)        << delim
-         << "fevent:unsigned long long:" << sizeof(LgEvent) << delim
-         << "num_requests:unsigned:" << sizeof(unsigned)
+         << "num_hops:unsigned:"       << sizeof(unsigned)           << delim
+         << "request_type:unsigned:"   << sizeof(unsigned)           << delim
+         << "fevent:unsigned long long:" << sizeof(LgEvent)
 #ifdef LEGION_PROF_PROVENANCE
          << delim
          << "provenance:"              << sizeof(LgEvent)
@@ -324,53 +340,48 @@ namespace Legion {
 
       ss << "CopyInstInfo {"
          << "id:" << COPY_INST_INFO_ID                           << delim
-         << "op_id:UniqueID:"          << sizeof(UniqueID)       << delim
-         << "src_inst:InstID:"         << sizeof(InstID)         << delim
-         << "dst_inst:InstID:"         << sizeof(InstID)         << delim
+         << "src:MemID:"                 << sizeof(MemID)        << delim
+         << "dst:MemID:"                 << sizeof(MemID)        << delim
+         << "src_fid:unsigned:"          << sizeof(FieldID)      << delim
+         << "dst_fid:unsigned:"          << sizeof(FieldID)      << delim
+         << "src_inst:unsigned long long:"  << sizeof(LgEvent)   << delim
+         << "dst_inst:unsigned long long:"  << sizeof(LgEvent)   << delim
          << "fevent:unsigned long long:" << sizeof(LgEvent)      << delim
-         << "num_fields:unsigned:"       << sizeof(unsigned)     << delim
-         << "request_type:unsigned:"     << sizeof(unsigned)     << delim
-         << "num_hops:unsigned:"         << sizeof(unsigned)
+         << "indirect:bool:"             << sizeof(bool)
          << "}" << std::endl;
 
       ss << "FillInfo {"
          << "id:" << FILL_INFO_ID                        << delim
          << "op_id:UniqueID:"     << sizeof(UniqueID)    << delim
-         << "dst:MemID:"          << sizeof(MemID)       << delim
+         << "size:unsigned long long:" << sizeof(unsigned long long) << delim
          << "create:timestamp_t:" << sizeof(timestamp_t) << delim
          << "ready:timestamp_t:"  << sizeof(timestamp_t) << delim
          << "start:timestamp_t:"  << sizeof(timestamp_t) << delim
-         << "stop:timestamp_t:"   << sizeof(timestamp_t)
+         << "stop:timestamp_t:"   << sizeof(timestamp_t) << delim
+         << "fevent:unsigned long long:" << sizeof(LgEvent)
 #ifdef LEGION_PROF_PROVENANCE
          << delim
          << "provenance:"         << sizeof(LgEvent)
 #endif
          << "}" << std::endl;
 
-      ss << "InstCreateInfo {"
-         << "id:" << INST_CREATE_INFO_ID                 << delim
-         << "op_id:UniqueID:"     << sizeof(UniqueID)    << delim
-         << "inst_id:InstID:"     << sizeof(InstID)      << delim
-         << "create:timestamp_t:" << sizeof(timestamp_t)
-#ifdef LEGION_PROF_PROVENANCE
-         << delim
-         << "provenance:"         << sizeof(LgEvent)
-#endif
-         << "}" << std::endl;
-
-      ss << "InstUsageInfo {"
-         << "id:" << INST_USAGE_INFO_ID                    << delim
-         << "op_id:UniqueID:"          << sizeof(UniqueID) << delim
-         << "inst_id:InstID:"          << sizeof(InstID)   << delim
-         << "mem_id:MemID:"            << sizeof(MemID)    << delim
-         << "size:unsigned long long:" << sizeof(unsigned long long)
+      ss << "FillInstInfo {"
+         << "id:" << FILL_INST_INFO_ID                           << delim
+         << "dst:MemID:"                    << sizeof(MemID)     << delim
+         << "fid:unsigned:"                 << sizeof(FieldID)   << delim
+         << "dst_inst:unsigned long long:"  << sizeof(LgEvent)   << delim
+         << "fevent:unsigned long long:" << sizeof(LgEvent)
          << "}" << std::endl;
 
       ss << "InstTimelineInfo {"
          << "id:" << INST_TIMELINE_INFO_ID                << delim
-         << "op_id:UniqueID:"      << sizeof(UniqueID)    << delim
-         << "inst_id:InstID:"      << sizeof(InstID)      << delim
+         << "inst_uid:unsigned long long:" << sizeof(LgEvent) << delim
+         << "inst_id:InstID:"          << sizeof(InstID)   << delim
+         << "mem_id:MemID:"            << sizeof(MemID)    << delim
+         << "size:unsigned long long:" << sizeof(unsigned long long) << delim
+         << "op_id:UniqueID:"       << sizeof(UniqueID) << delim
          << "create:timestamp_t:"  << sizeof(timestamp_t) << delim
+         << "ready:timestamp_t:"  << sizeof(timestamp_t) << delim
          << "destroy:timestamp_t:" << sizeof(timestamp_t)
          << "}" << std::endl;
 
@@ -455,6 +466,8 @@ namespace Legion {
       int ID = META_DESC_ID;
       lp_fwrite(f, (char*)&ID, sizeof(ID));
       lp_fwrite(f, (char*)&(meta_desc.kind), sizeof(meta_desc.kind));
+      lp_fwrite(f, (char*)&(meta_desc.message), sizeof(meta_desc.message));
+      lp_fwrite(f, (char*)&(meta_desc.ordered_vc),sizeof(meta_desc.ordered_vc));
       lp_fwrite(f, meta_desc.name, strlen(meta_desc.name) + 1);
     }
 
@@ -469,16 +482,6 @@ namespace Legion {
       lp_fwrite(f, op_desc.name, strlen(op_desc.name) + 1);
     }
 
-    //--------------------------------------------------------------------------
-    void LegionProfBinarySerializer::serialize(
-                                      const LegionProfDesc::ProcDesc& proc_desc)
-    //--------------------------------------------------------------------------
-    {
-      int ID = PROC_DESC_ID;
-      lp_fwrite(f, (char*)&ID, sizeof(ID));
-      lp_fwrite(f, (char*)&(proc_desc.proc_id), sizeof(proc_desc.proc_id));
-      lp_fwrite(f, (char*)&(proc_desc.kind),    sizeof(proc_desc.kind));
-    }
 
     //--------------------------------------------------------------------------
     void LegionProfBinarySerializer::serialize(
@@ -492,28 +495,19 @@ namespace Legion {
 		sizeof(max_dim_desc.max_dim));
 
     }
-    //--------------------------------------------------------------------------
-    void LegionProfBinarySerializer::serialize(
-                                        const LegionProfDesc::MemDesc& mem_desc)
-    //--------------------------------------------------------------------------
-    {
-      int ID = MEM_DESC_ID;
-      lp_fwrite(f, (char*)&ID, sizeof(ID));
-      lp_fwrite(f, (char*)&(mem_desc.mem_id),   sizeof(mem_desc.mem_id));
-      lp_fwrite(f, (char*)&(mem_desc.kind),     sizeof(mem_desc.kind));
-      lp_fwrite(f, (char*)&(mem_desc.capacity), sizeof(mem_desc.capacity));
-    }
 
-    // Serialize Methods
     //--------------------------------------------------------------------------
     void LegionProfBinarySerializer::serialize(
-                                          const LegionProfDesc::ProcMemDesc &pm)
+                                      const LegionProfDesc::MachineDesc
+				      &machine_desc)
     //--------------------------------------------------------------------------
     {
-      int ID = PROC_MEM_DESC_ID;
+      int ID = MACHINE_DESC_ID;
       lp_fwrite(f, (char*)&ID, sizeof(ID));
-      lp_fwrite(f, (char*) &(pm.proc_id), sizeof(pm.proc_id));
-      lp_fwrite(f, (char*) &(pm.mem_id), sizeof(pm.mem_id));
+      lp_fwrite(f, (char*)&(machine_desc.node_id),
+		sizeof(machine_desc.node_id));
+      lp_fwrite(f, (char*)&(machine_desc.num_nodes),
+		sizeof(machine_desc.num_nodes));
     }
 
     //--------------------------------------------------------------------------
@@ -660,8 +654,8 @@ namespace Legion {
     {
       int ID = PHYSICAL_INST_REGION_ID;
       lp_fwrite(f, (char*)&ID, sizeof(ID));
-      lp_fwrite(f, (char*)&(phy_instance_rdesc.op_id), sizeof(UniqueID));
-      lp_fwrite(f, (char*)&(phy_instance_rdesc.inst_id), sizeof(IDType));
+      lp_fwrite(f, (char*)&(phy_instance_rdesc.inst_uid.id), 
+                sizeof(phy_instance_rdesc.inst_uid.id));
       lp_fwrite(f, (char*)&(phy_instance_rdesc.ispace_id), sizeof(IDType));
       lp_fwrite(f, (char*)&(phy_instance_rdesc.fspace_id), sizeof(unsigned));
       lp_fwrite(f, (char*)&(phy_instance_rdesc.tree_id), sizeof(unsigned));
@@ -675,15 +669,14 @@ namespace Legion {
     {
       int ID = PHYSICAL_INST_LAYOUT_DIM_ID;
       lp_fwrite(f, (char*)&ID, sizeof(ID));
-      lp_fwrite(f, (char*)&(phy_instance_dim_order_rdesc.op_id),
-                sizeof(UniqueID));
-      lp_fwrite(f, (char*)&(phy_instance_dim_order_rdesc.inst_id),
-                sizeof(IDType));
+      lp_fwrite(f, (char*)&(phy_instance_dim_order_rdesc.inst_uid.id),
+                sizeof(phy_instance_dim_order_rdesc.inst_uid.id));
       lp_fwrite(f, (char*)&(phy_instance_dim_order_rdesc.dim),
                 sizeof(unsigned));
       lp_fwrite(f, (char*)&(phy_instance_dim_order_rdesc.k),
                 sizeof(unsigned));
     }
+
     //--------------------------------------------------------------------------
     void LegionProfBinarySerializer::serialize(
                 const LegionProfInstance::PhysicalInstLayoutDesc 
@@ -692,8 +685,8 @@ namespace Legion {
     {
       int ID = PHYSICAL_INST_LAYOUT_ID;
       lp_fwrite(f, (char*)&ID, sizeof(ID));
-      lp_fwrite(f, (char*)&(phy_instance_layout_rdesc.op_id),sizeof(UniqueID));
-      lp_fwrite(f, (char*)&(phy_instance_layout_rdesc.inst_id),sizeof(InstID));
+      lp_fwrite(f, (char*)&(phy_instance_layout_rdesc.inst_uid.id),
+                sizeof(phy_instance_layout_rdesc.inst_uid.id));
       lp_fwrite(f, (char*)&(phy_instance_layout_rdesc.field_id),
                 sizeof(unsigned));
       lp_fwrite(f, (char*)&(phy_instance_layout_rdesc.fspace_id),
@@ -706,6 +699,19 @@ namespace Legion {
                 sizeof(unsigned));
     }
 
+    //--------------------------------------------------------------------------
+    void LegionProfBinarySerializer::serialize(
+                         const LegionProfInstance::PhysicalInstanceUsage &usage)
+    //--------------------------------------------------------------------------
+    {
+      int ID = PHYSICAL_INST_USAGE_ID;
+      lp_fwrite(f, (char*)&ID, sizeof(ID));
+      lp_fwrite(f, (char*)&(usage.inst_uid.id), sizeof(usage.inst_uid.id));
+      lp_fwrite(f, (char*)&(usage.op_id), sizeof(UniqueID));
+      lp_fwrite(f, (char*)&(usage.index), sizeof(unsigned));
+      lp_fwrite(f, (char*)&(usage.field), sizeof(unsigned));
+    }
+
     //--------------------------------------------------------------------------
     void LegionProfBinarySerializer::serialize(
                               const LegionProfInstance::IndexSpaceSizeDesc
@@ -754,8 +760,15 @@ namespace Legion {
       lp_fwrite(f, (char*)&ID, sizeof(ID));
       lp_fwrite(f, (char*)&(operation_instance.op_id), 
                 sizeof(operation_instance.op_id));
+      lp_fwrite(f, (char*)&(operation_instance.parent_id),
+                sizeof(operation_instance.parent_id));
       lp_fwrite(f, (char*)&(operation_instance.kind),
                 sizeof(operation_instance.kind));
+      if (operation_instance.provenance != NULL)
+        lp_fwrite(f, operation_instance.provenance,
+            strlen(operation_instance.provenance) + 1);
+      else
+        lp_fwrite(f, "", 1);
     }
 
     //--------------------------------------------------------------------------
@@ -904,35 +917,36 @@ namespace Legion {
       lp_fwrite(f, (char*)&ID, sizeof(ID));
 
       lp_fwrite(f, (char*)&(copy_info.op_id),  sizeof(copy_info.op_id));
-      lp_fwrite(f, (char*)&(copy_info.src),    sizeof(copy_info.src));
-      lp_fwrite(f, (char*)&(copy_info.dst),    sizeof(copy_info.dst));
       lp_fwrite(f, (char*)&(copy_info.size),   sizeof(copy_info.size));
       lp_fwrite(f, (char*)&(copy_info.create), sizeof(copy_info.create));
       lp_fwrite(f, (char*)&(copy_info.ready),  sizeof(copy_info.ready));
       lp_fwrite(f, (char*)&(copy_info.start),  sizeof(copy_info.start));
       lp_fwrite(f, (char*)&(copy_info.stop),   sizeof(copy_info.stop));
+      lp_fwrite(f, (char*)&(copy_info.num_hops), sizeof(copy_info.num_hops));
+      lp_fwrite(f, (char*)&(copy_info.request_type), sizeof(copy_info.request_type));
       lp_fwrite(f, (char*)&(copy_info.fevent),sizeof(copy_info.fevent.id));
-      lp_fwrite(f, (char*)&(copy_info.num_requests),   sizeof(copy_info.num_requests));
 #ifdef LEGION_PROF_PROVENANCE
       lp_fwrite(f, (char*)&(copy_info.provenance),sizeof(copy_info.provenance));
 #endif
     }
 
-        //--------------------------------------------------------------------------
+    //--------------------------------------------------------------------------
     void LegionProfBinarySerializer::serialize(
-                                  const LegionProfInstance::CopyInstInfo &copy_inst,
-                                  const LegionProfInstance::CopyInfo& copy_info)
+                              const LegionProfInstance::CopyInstInfo &copy_inst)
     //--------------------------------------------------------------------------
     {
       int ID = COPY_INST_INFO_ID;
       lp_fwrite(f, (char*)&ID, sizeof(ID));
-      lp_fwrite(f, (char*)&(copy_info.op_id),     sizeof(copy_info.op_id));
-      lp_fwrite(f, (char*)&(copy_inst.src_inst_id),sizeof(copy_inst.src_inst_id));
-      lp_fwrite(f, (char*)&(copy_inst.dst_inst_id),sizeof(copy_inst.dst_inst_id));
-      lp_fwrite(f, (char*)&(copy_info.fevent),sizeof(copy_info.fevent.id));
-      lp_fwrite(f, (char*)&(copy_inst.num_fields),sizeof(copy_inst.num_fields));
-      lp_fwrite(f, (char*)&(copy_inst.request_type),sizeof(copy_inst.request_type));
-      lp_fwrite(f, (char*)&(copy_inst.num_hops),sizeof(copy_inst.num_hops));
+      lp_fwrite(f, (char*)&(copy_inst.src), sizeof(copy_inst.src));
+      lp_fwrite(f, (char*)&(copy_inst.dst), sizeof(copy_inst.dst));
+      lp_fwrite(f, (char*)&(copy_inst.src_fid), sizeof(copy_inst.src_fid));
+      lp_fwrite(f, (char*)&(copy_inst.dst_fid), sizeof(copy_inst.dst_fid));
+      lp_fwrite(f, (char*)&(copy_inst.src_inst_uid.id),
+          sizeof(copy_inst.src_inst_uid.id));
+      lp_fwrite(f, (char*)&(copy_inst.dst_inst_uid.id),
+          sizeof(copy_inst.dst_inst_uid.id));
+      lp_fwrite(f, (char*)&(copy_inst.fevent),sizeof(copy_inst.fevent.id));
+      lp_fwrite(f, (char*)&(copy_inst.indirect), sizeof(copy_inst.indirect));
     }
 
     //--------------------------------------------------------------------------
@@ -942,13 +956,13 @@ namespace Legion {
     {
       int ID = FILL_INFO_ID;
       lp_fwrite(f, (char*)&ID, sizeof(ID));
-
       lp_fwrite(f, (char*)&(fill_info.op_id),  sizeof(fill_info.op_id));
-      lp_fwrite(f, (char*)&(fill_info.dst),    sizeof(fill_info.dst));
+      lp_fwrite(f, (char*)&(fill_info.size),   sizeof(fill_info.size));
       lp_fwrite(f, (char*)&(fill_info.create), sizeof(fill_info.create));
       lp_fwrite(f, (char*)&(fill_info.ready),  sizeof(fill_info.ready));
       lp_fwrite(f, (char*)&(fill_info.start),  sizeof(fill_info.start));
       lp_fwrite(f, (char*)&(fill_info.stop),   sizeof(fill_info.stop));
+      lp_fwrite(f, (char*)&(fill_info.fevent), sizeof(fill_info.fevent.id));
 #ifdef LEGION_PROF_PROVENANCE
       lp_fwrite(f, (char*)&(fill_info.provenance),sizeof(fill_info.provenance));
 #endif
@@ -956,38 +970,16 @@ namespace Legion {
 
     //--------------------------------------------------------------------------
     void LegionProfBinarySerializer::serialize(
-                     const LegionProfInstance::InstCreateInfo& inst_create_info)
+                              const LegionProfInstance::FillInstInfo &fill_inst)
     //--------------------------------------------------------------------------
     {
-      int ID = INST_CREATE_INFO_ID;
+      int ID = FILL_INST_INFO_ID;
       lp_fwrite(f, (char*)&ID, sizeof(ID));
-      lp_fwrite(f, (char*)&(inst_create_info.op_id),   
-                sizeof(inst_create_info.op_id));
-      lp_fwrite(f, (char*)&(inst_create_info.inst_id), 
-                sizeof(inst_create_info.inst_id));
-      lp_fwrite(f, (char*)&(inst_create_info.create),  
-                sizeof(inst_create_info.create));
-#ifdef LEGION_PROF_PROVENANCE
-      lp_fwrite(f, (char*)&(inst_create_info.provenance),
-                sizeof(inst_create_info.provenance));
-#endif
-    }
-
-    //--------------------------------------------------------------------------
-    void LegionProfBinarySerializer::serialize(
-                       const LegionProfInstance::InstUsageInfo& inst_usage_info)
-    //--------------------------------------------------------------------------
-    {
-      int ID = INST_USAGE_INFO_ID;
-      lp_fwrite(f, (char*)&ID, sizeof(ID));
-      lp_fwrite(f, (char*)&(inst_usage_info.op_id),   
-                sizeof(inst_usage_info.op_id));
-      lp_fwrite(f, (char*)&(inst_usage_info.inst_id), 
-                sizeof(inst_usage_info.inst_id));
-      lp_fwrite(f, (char*)&(inst_usage_info.mem_id),  
-                sizeof(inst_usage_info.mem_id));
-      lp_fwrite(f, (char*)&(inst_usage_info.size),    
-                sizeof(inst_usage_info.size));
+      lp_fwrite(f, (char*)&(fill_inst.dst), sizeof(fill_inst.dst));
+      lp_fwrite(f, (char*)&(fill_inst.fid),    sizeof(fill_inst.fid));
+      lp_fwrite(f, (char*)&(fill_inst.dst_inst_uid.id),
+          sizeof(fill_inst.dst_inst_uid.id));
+      lp_fwrite(f, (char*)&(fill_inst.fevent),sizeof(fill_inst.fevent.id));
     }
 
     //--------------------------------------------------------------------------
@@ -997,12 +989,20 @@ namespace Legion {
     {
       int ID = INST_TIMELINE_INFO_ID;
       lp_fwrite(f, (char*)&ID, sizeof(ID));
-      lp_fwrite(f, (char*)&(inst_timeline_info.op_id),   
-                sizeof(inst_timeline_info.op_id));
-      lp_fwrite(f, (char*)&(inst_timeline_info.inst_id), 
+      lp_fwrite(f, (char*)&(inst_timeline_info.inst_uid.id),
+                sizeof(inst_timeline_info.inst_uid.id));
+      lp_fwrite(f, (char*)&(inst_timeline_info.inst_id),
                 sizeof(inst_timeline_info.inst_id));
+      lp_fwrite(f, (char*)&(inst_timeline_info.mem_id),  
+                sizeof(inst_timeline_info.mem_id));
+      lp_fwrite(f, (char*)&(inst_timeline_info.size),    
+                sizeof(inst_timeline_info.size));
+      lp_fwrite(f, (char*)&(inst_timeline_info.op_id),
+                sizeof(inst_timeline_info.op_id));
       lp_fwrite(f, (char*)&(inst_timeline_info.create),  
                 sizeof(inst_timeline_info.create));
+      lp_fwrite(f, (char*)&(inst_timeline_info.ready),  
+                sizeof(inst_timeline_info.ready));
       lp_fwrite(f, (char*)&(inst_timeline_info.destroy), 
                 sizeof(inst_timeline_info.destroy));
     }
@@ -1068,6 +1068,42 @@ namespace Legion {
                 sizeof(runtime_call_info.proc_id));
     }
 
+    //--------------------------------------------------------------------------
+    void LegionProfBinarySerializer::serialize(
+                                      const LegionProfInstance::ProcDesc& proc_desc)
+    //--------------------------------------------------------------------------
+    {
+      int ID = PROC_DESC_ID;
+      lp_fwrite(f, (char*)&ID, sizeof(ID));
+      lp_fwrite(f, (char*)&(proc_desc.proc_id), sizeof(proc_desc.proc_id));
+      lp_fwrite(f, (char*)&(proc_desc.kind),    sizeof(proc_desc.kind));
+    }
+    //--------------------------------------------------------------------------
+    void LegionProfBinarySerializer::serialize(
+                                        const LegionProfInstance::MemDesc& mem_desc)
+    //--------------------------------------------------------------------------
+    {
+      int ID = MEM_DESC_ID;
+      lp_fwrite(f, (char*)&ID, sizeof(ID));
+      lp_fwrite(f, (char*)&(mem_desc.mem_id),   sizeof(mem_desc.mem_id));
+      lp_fwrite(f, (char*)&(mem_desc.kind),     sizeof(mem_desc.kind));
+      lp_fwrite(f, (char*)&(mem_desc.capacity), sizeof(mem_desc.capacity));
+    }
+
+    //--------------------------------------------------------------------------
+    void LegionProfBinarySerializer::serialize(
+                                          const LegionProfInstance::ProcMemDesc &pm)
+    //--------------------------------------------------------------------------
+    {
+      int ID = PROC_MEM_DESC_ID;
+      lp_fwrite(f, (char*)&ID, sizeof(ID));
+      lp_fwrite(f, (char*) &(pm.proc_id), sizeof(pm.proc_id));
+      lp_fwrite(f, (char*) &(pm.mem_id), sizeof(pm.mem_id));
+      lp_fwrite(f, (char*) &(pm.bandwidth), sizeof(pm.bandwidth));
+      lp_fwrite(f, (char*) &(pm.latency), sizeof(pm.latency));
+    }
+
+
 #ifdef LEGION_PROF_SELF_PROFILE
     //--------------------------------------------------------------------------
     void LegionProfBinarySerializer::serialize(
@@ -1433,9 +1469,8 @@ namespace Legion {
            const LegionProfInstance::PhysicalInstRegionDesc &phy_instance_rdesc)
     //--------------------------------------------------------------------------
     {
-      log_prof.print("Physical Inst Region Desc " "%llu "  IDFMT " %llu %u %u",
-		     phy_instance_rdesc.op_id,
-		     phy_instance_rdesc.inst_id,
+      log_prof.print("Physical Inst Region Desc "  IDFMT " %llu %u %u",
+                     phy_instance_rdesc.inst_uid.id,
 		     phy_instance_rdesc.ispace_id,
 		     phy_instance_rdesc.fspace_id,
 		     phy_instance_rdesc.tree_id);
@@ -1447,9 +1482,8 @@ namespace Legion {
            &phy_instance_dim_order_rdesc)
     //--------------------------------------------------------------------------
     {
-      log_prof.print("Physical Inst Dim Order Desc " "%llu " IDFMT " %u %u",
-                     phy_instance_dim_order_rdesc.op_id,
-                     phy_instance_dim_order_rdesc.inst_id,
+      log_prof.print("Physical Inst Dim Order Desc " IDFMT " %u %u",
+                     phy_instance_dim_order_rdesc.inst_uid.id,
                      phy_instance_dim_order_rdesc.dim,
                      phy_instance_dim_order_rdesc.k
                      );
@@ -1476,10 +1510,9 @@ namespace Legion {
               &phy_instance_layout_rdesc)
     //--------------------------------------------------------------------------
     {
-      log_prof.print("Physical Inst Layout Desc " "%llu " IDFMT " %u %u %u %u "
+      log_prof.print("Physical Inst Layout Desc " IDFMT " %u %u %u %u "
                      "%u",
-                     phy_instance_layout_rdesc.op_id,
-                     phy_instance_layout_rdesc.inst_id,
+                     phy_instance_layout_rdesc.inst_uid.id,
                      phy_instance_layout_rdesc.field_id,
                      phy_instance_layout_rdesc.fspace_id,
                      phy_instance_layout_rdesc.has_align,
@@ -1488,6 +1521,15 @@ namespace Legion {
                      );
     }
 
+    //--------------------------------------------------------------------------
+    void LegionProfASCIISerializer::serialize(
+                         const LegionProfInstance::PhysicalInstanceUsage &usage)
+    //--------------------------------------------------------------------------
+    {
+      log_prof.print("Physical Inst Usage " IDFMT " %llu %u %u",
+                      usage.inst_uid.id, usage.op_id, usage.index, usage.field);
+    }
+
     //--------------------------------------------------------------------------
     void LegionProfASCIISerializer::serialize(
                          const LegionProfDesc::MapperCallDesc &mapper_call_desc)
@@ -1511,7 +1553,8 @@ namespace Legion {
                                       const LegionProfDesc::MetaDesc &meta_desc)
     //--------------------------------------------------------------------------
     {
-      log_prof.print("Prof Meta Desc %u %s", meta_desc.kind, meta_desc.name);
+      log_prof.print("Prof Meta Desc %u %d %d %s", meta_desc.kind,
+       meta_desc.message ? 1 : 0, meta_desc.ordered_vc ? 1 : 0, meta_desc.name);
     }
 
     //--------------------------------------------------------------------------
@@ -1522,15 +1565,6 @@ namespace Legion {
       log_prof.print("Prof Op Desc %u %s", op_desc.kind, op_desc.name);
     }
 
-    //--------------------------------------------------------------------------
-    void LegionProfASCIISerializer::serialize(
-                                      const LegionProfDesc::ProcDesc &proc_desc)
-    //--------------------------------------------------------------------------
-    {
-      log_prof.print("Prof Proc Desc " IDFMT " %d", 
-                     proc_desc.proc_id, proc_desc.kind);
-    }
-
     //--------------------------------------------------------------------------
     void LegionProfASCIISerializer::serialize(
                                       const LegionProfDesc::MaxDimDesc
@@ -1543,20 +1577,12 @@ namespace Legion {
 
     //--------------------------------------------------------------------------
     void LegionProfASCIISerializer::serialize(
-                                        const LegionProfDesc::MemDesc &mem_desc)
+                                      const LegionProfDesc::MachineDesc
+				      &machine_desc)
     //--------------------------------------------------------------------------
     {
-      log_prof.print("Prof Mem Desc " IDFMT " %d %llu", 
-                      mem_desc.mem_id, mem_desc.kind, mem_desc.capacity);
-    }
-
-    //--------------------------------------------------------------------------
-    void LegionProfASCIISerializer::serialize(
-                                          const LegionProfDesc::ProcMemDesc &pm)
-    //--------------------------------------------------------------------------
-    {
-      log_prof.print("Prof Mem Proc Affinity Desc " IDFMT " " IDFMT,
-		     pm.proc_id, pm.mem_id);
+      log_prof.print("Machine Desc %d %d",
+                     machine_desc.node_id, machine_desc.num_nodes);
     }
 
     //--------------------------------------------------------------------------
@@ -1582,7 +1608,9 @@ namespace Legion {
                            const LegionProfInstance::OperationInstance& op_inst)
     //--------------------------------------------------------------------------
     {
-      log_prof.print("Prof Operation %llu %u", op_inst.op_id, op_inst.kind);
+      log_prof.print("Prof Operation %llu %llu %u %s", 
+          op_inst.op_id, op_inst.parent_id, op_inst.kind,
+          op_inst.provenance == NULL ? "" : op_inst.provenance);
     }
 
     //--------------------------------------------------------------------------
@@ -1703,34 +1731,30 @@ namespace Legion {
     //--------------------------------------------------------------------------
     {
 #ifdef LEGION_PROF_PROVENANCE
-      log_prof.print("Prof Copy Info %llu " IDFMT " " IDFMT " %llu"
-                     " %llu %llu %llu %llu " IDFMT " " IDFMT " %u",
-                     copy_info.op_id, copy_info.src,
-                     copy_info.dst, copy_info.size, copy_info.create,
+      log_prof.print("Prof Copy Info %llu %luu %llu %llu %llu %llu %u %u " IDFMT " " 
+                     IDFMT, copy_info.op_id, copy_info.size, copy_info.create,
                      copy_info.ready, copy_info.start, copy_info.stop,
-                     copy_info.fevent.id,
-                     copy_info.provenance.id,
-                     copy_info.num_requests);
+                     copy_info.num_hops, copy_info.request_type,
+                     copy_info.fevent.id, copy_info.provenance.id);
 #else
-      log_prof.print("Prof Copy Info %llu " IDFMT " " IDFMT " %llu"
-                     " %llu %llu %llu %llu " IDFMT " %u", copy_info.op_id,
-                     copy_info.src,
-                     copy_info.dst, copy_info.size, copy_info.create,
+      log_prof.print("Prof Copy Info %llu %llu %llu %llu %llu %llu %u %u " IDFMT,
+                     copy_info.op_id, copy_info.size, copy_info.create,
                      copy_info.ready, copy_info.start, copy_info.stop,
-                     copy_info.fevent.id,
-                     copy_info.num_requests);
+                     copy_info.num_hops, copy_info.request_type,
+                     copy_info.fevent.id);
 #endif
     }
 
     //--------------------------------------------------------------------------
     void LegionProfASCIISerializer::serialize(
-                          const LegionProfInstance::CopyInstInfo& copy_inst,
-                          const LegionProfInstance::CopyInfo& copy_info)
+                              const LegionProfInstance::CopyInstInfo& copy_inst)
     //--------------------------------------------------------------------------
     {
-      log_prof.print("Prof Copy Inst Info %llu " IDFMT " " IDFMT " " IDFMT " %u %u %u",
-                     copy_info.op_id, copy_inst.src_inst_id, copy_inst.dst_inst_id,
-                     copy_info.fevent.id, copy_inst.num_fields, copy_inst.request_type, copy_inst.num_hops);
+      log_prof.print("Prof Copy Inst Info " IDFMT " " IDFMT " %d %d " IDFMT " " 
+                     IDFMT " " IDFMT " %u", copy_inst.src, copy_inst.dst,
+                     copy_inst.src_fid, copy_inst.dst_fid,
+                     copy_inst.src_inst_uid.id, copy_inst.dst_inst_uid.id,
+                     copy_inst.fevent.id, copy_inst.indirect ? 1 : 0); 
     }
 
     //--------------------------------------------------------------------------
@@ -1739,40 +1763,25 @@ namespace Legion {
     //--------------------------------------------------------------------------
     {
 #ifdef LEGION_PROF_PROVENANCE
-      log_prof.print("Prof Fill Info %llu " IDFMT " %llu %llu %llu %llu " IDFMT,
-        fill_info.op_id, fill_info.dst, fill_info.create, fill_info.ready, 
-        fill_info.start, fill_info.stop, fill_info.provenance.id);
+      log_prof.print("Prof Fill Info %llu %llu %llu %llu %llu %llu " IDFMT " "
+          IDFMT, fill_info.op_id, fill_info.size, fill_info.create, 
+          fill_info.ready, fill_info.start, fill_info.stop,
+          fill_info.fevent.id, fill_info.provenance.id);
 #else
-      log_prof.print("Prof Fill Info %llu " IDFMT 
-         " %llu %llu %llu %llu", fill_info.op_id, fill_info.dst, 
-         fill_info.create, fill_info.ready, fill_info.start, fill_info.stop);
+      log_prof.print("Prof Fill Info %llu %llu %llu %llu %llu %llu " IDFMT, 
+          fill_info.op_id, fill_info.size, fill_info.create, fill_info.ready,
+          fill_info.start, fill_info.stop, fill_info.fevent.id);
 #endif
     }
 
     //--------------------------------------------------------------------------
     void LegionProfASCIISerializer::serialize(
-                     const LegionProfInstance::InstCreateInfo& inst_create_info)
+                              const LegionProfInstance::FillInstInfo& fill_inst)
     //--------------------------------------------------------------------------
     {
-#ifdef LEGION_PROF_PROVENANCE
-      log_prof.print("Prof Inst Create %llu " IDFMT " %llu " IDFMT "", 
-                     inst_create_info.op_id, inst_create_info.inst_id, 
-                     inst_create_info.create, inst_create_info.provenance.id);
-#else
-      log_prof.print("Prof Inst Create %llu " IDFMT " %llu", 
-                     inst_create_info.op_id, inst_create_info.inst_id, 
-                     inst_create_info.create);
-#endif
-    }
-
-    //--------------------------------------------------------------------------
-    void LegionProfASCIISerializer::serialize(
-                       const LegionProfInstance::InstUsageInfo& inst_usage_info)
-    //--------------------------------------------------------------------------
-    {
-      log_prof.print("Prof Inst Usage %llu " IDFMT " " IDFMT " %llu",
-                     inst_usage_info.op_id, inst_usage_info.inst_id, 
-                     inst_usage_info.mem_id, inst_usage_info.size);
+      log_prof.print("Prof Fill Inst Info " IDFMT " %d " IDFMT " " IDFMT,
+                      fill_inst.dst, fill_inst.fid, fill_inst.dst_inst_uid.id,
+                      fill_inst.fevent.id);
     }
 
     //--------------------------------------------------------------------------
@@ -1780,9 +1789,12 @@ namespace Legion {
                  const LegionProfInstance::InstTimelineInfo& inst_timeline_info)
     //--------------------------------------------------------------------------
     {
-      log_prof.print("Prof Inst Timeline %llu " IDFMT " %llu %llu",
-         inst_timeline_info.op_id, inst_timeline_info.inst_id,
-         inst_timeline_info.create, inst_timeline_info.destroy);
+      log_prof.print("Prof Inst Timeline " IDFMT " " IDFMT " " IDFMT 
+                     " %llu %llu %llu %llu %llu",
+         inst_timeline_info.inst_uid.id, inst_timeline_info.inst_id,
+         inst_timeline_info.mem_id, inst_timeline_info.size,
+         inst_timeline_info.op_id, inst_timeline_info.create,
+         inst_timeline_info.ready, inst_timeline_info.destroy);
     }
 
     //--------------------------------------------------------------------------
@@ -1824,6 +1836,33 @@ namespace Legion {
                      runtime_call_info.start, runtime_call_info.stop);
     }
 
+    //--------------------------------------------------------------------------
+    void LegionProfASCIISerializer::serialize(
+                                 const LegionProfInstance::ProcDesc &proc_desc)
+    //--------------------------------------------------------------------------
+    {
+      log_prof.print("Prof Proc Desc " IDFMT " %d",
+                     proc_desc.proc_id, proc_desc.kind);
+    }
+
+    //--------------------------------------------------------------------------
+    void LegionProfASCIISerializer::serialize(
+                                   const LegionProfInstance::MemDesc &mem_desc)
+    //--------------------------------------------------------------------------
+    {
+      log_prof.print("Prof Mem Desc " IDFMT " %d %llu",
+                      mem_desc.mem_id, mem_desc.kind, mem_desc.capacity);
+    }
+
+    //--------------------------------------------------------------------------
+    void LegionProfASCIISerializer::serialize(
+                                    const LegionProfInstance::ProcMemDesc &pm)
+    //--------------------------------------------------------------------------
+    {
+      log_prof.print("Prof Mem Proc Affinity Desc " IDFMT " " IDFMT " %u %u",
+		     pm.proc_id, pm.mem_id, pm.bandwidth, pm.latency);
+    }
+
 #ifdef LEGION_PROF_SELF_PROFILE
     //--------------------------------------------------------------------------
     void LegionProfASCIISerializer::serialize(
