@@ -1,4 +1,4 @@
-/* Copyright 2021 Stanford University, NVIDIA Corporation
+/* Copyright 2023 Stanford University, NVIDIA Corporation
  *
  * Licensed under the Apache License, Version 2.0 (the "License");
  * you may not use this file except in compliance with the License.
@@ -38,6 +38,97 @@ namespace Legion {
      * as part of the execution of the task.
      */
     class ResourceTracker {
+    public:
+      struct DeletedRegion {
+      public:
+        DeletedRegion(void);
+        DeletedRegion(LogicalRegion r, Provenance *provenance = NULL);
+        DeletedRegion(const DeletedRegion &rhs);
+        DeletedRegion(DeletedRegion &&rhs);
+        ~DeletedRegion(void);
+      public:
+        DeletedRegion& operator=(const DeletedRegion &rhs);
+        DeletedRegion& operator=(DeletedRegion &&rhs);
+      public:
+        void serialize(Serializer &rez) const;
+        void deserialize(Deserializer &derez);
+      public:
+        LogicalRegion region;
+        Provenance *provenance;
+      };
+      struct DeletedField {
+      public:
+        DeletedField(void);
+        DeletedField(FieldSpace sp, FieldID f, Provenance *provenance = NULL);
+        DeletedField(const DeletedField &rhs);
+        DeletedField(DeletedField &&rhs);
+        ~DeletedField(void);
+      public:
+        DeletedField& operator=(const DeletedField &rhs);
+        DeletedField& operator=(DeletedField &&rhs);
+      public:
+        void serialize(Serializer &rez) const;
+        void deserialize(Deserializer &derez);
+      public:
+        FieldSpace space;
+        FieldID fid;
+        Provenance *provenance;
+      };
+      struct DeletedFieldSpace {
+      public:
+        DeletedFieldSpace(void);
+        DeletedFieldSpace(FieldSpace sp, Provenance *provenance = NULL);
+        DeletedFieldSpace(const DeletedFieldSpace &rhs);
+        DeletedFieldSpace(DeletedFieldSpace &&rhs);
+        ~DeletedFieldSpace(void);
+      public:
+        DeletedFieldSpace& operator=(const DeletedFieldSpace &rhs);
+        DeletedFieldSpace& operator=(DeletedFieldSpace &&rhs);
+      public:
+        void serialize(Serializer &rez) const;
+        void deserialize(Deserializer &derez);
+      public:
+        FieldSpace space;
+        Provenance *provenance;
+      };
+      struct DeletedIndexSpace {
+      public:
+        DeletedIndexSpace(void);
+        DeletedIndexSpace(IndexSpace sp, bool recurse, 
+                          Provenance *provenance = NULL);
+        DeletedIndexSpace(const DeletedIndexSpace &rhs);
+        DeletedIndexSpace(DeletedIndexSpace &&rhs);
+        ~DeletedIndexSpace(void);
+      public:
+        DeletedIndexSpace& operator=(const DeletedIndexSpace &rhs);
+        DeletedIndexSpace& operator=(DeletedIndexSpace &&rhs);
+      public:
+        void serialize(Serializer &rez) const;
+        void deserialize(Deserializer &derez);
+      public:
+        IndexSpace space;
+        Provenance *provenance;
+        bool recurse;
+      };
+      struct DeletedPartition {
+      public:
+        DeletedPartition(void);
+        DeletedPartition(IndexPartition p, bool recurse,
+                         Provenance *provenance = NULL);
+        DeletedPartition(const DeletedPartition &rhs);
+        DeletedPartition(DeletedPartition &&rhs);
+        ~DeletedPartition(void);
+      public:
+        DeletedPartition& operator=(const DeletedPartition &rhs);
+        DeletedPartition& operator=(DeletedPartition &&rhs);
+      public:
+        void serialize(Serializer &rez) const;
+        void deserialize(Deserializer &derez);
+      public:
+        IndexPartition partition;
+        Provenance *provenance;
+        bool recurse;
+      };
     public:
       ResourceTracker(void);
       ResourceTracker(const ResourceTracker &rhs);
@@ -49,16 +140,16 @@ namespace Legion {
                             std::set<RtEvent> &preconditions);
       virtual void receive_resources(size_t return_index,
               std::map<LogicalRegion,unsigned> &created_regions,
-              std::vector<LogicalRegion> &deleted_regions,
+              std::vector<DeletedRegion> &deleted_regions,
               std::set<std::pair<FieldSpace,FieldID> > &created_fields,
-              std::vector<std::pair<FieldSpace,FieldID> > &deleted_fields,
+              std::vector<DeletedField> &deleted_fields,
               std::map<FieldSpace,unsigned> &created_field_spaces,
               std::map<FieldSpace,std::set<LogicalRegion> > &latent_spaces,
-              std::vector<FieldSpace> &deleted_field_spaces,
+              std::vector<DeletedFieldSpace> &deleted_field_spaces,
               std::map<IndexSpace,unsigned> &created_index_spaces,
-              std::vector<std::pair<IndexSpace,bool> > &deleted_index_spaces,
+              std::vector<DeletedIndexSpace> &deleted_index_spaces,
               std::map<IndexPartition,unsigned> &created_partitions,
-              std::vector<std::pair<IndexPartition,bool> > &deleted_partitions,
+              std::vector<DeletedPartition> &deleted_partitions,
               std::set<RtEvent> &preconditions) = 0;
       void pack_resources_return(Serializer &rez, size_t return_index);
       static RtEvent unpack_resources_return(Deserializer &derez,
@@ -72,12 +163,12 @@ namespace Legion {
       std::map<IndexSpace,unsigned>                    created_index_spaces;
       std::map<IndexPartition,unsigned>                created_index_partitions;
     protected:
-      std::vector<LogicalRegion>                       deleted_regions;
-      std::vector<std::pair<FieldSpace,FieldID> >      deleted_fields;
-      std::vector<FieldSpace>                          deleted_field_spaces;
+      std::vector<DeletedRegion>                       deleted_regions;
+      std::vector<DeletedField>                        deleted_fields;
+      std::vector<DeletedFieldSpace>                   deleted_field_spaces;
       std::map<FieldSpace,std::set<LogicalRegion> >    latent_field_spaces;
-      std::vector<std::pair<IndexSpace,bool> >         deleted_index_spaces;
-      std::vector<std::pair<IndexPartition,bool> >     deleted_index_partitions;
+      std::vector<DeletedIndexSpace>                   deleted_index_spaces;
+      std::vector<DeletedPartition>                    deleted_index_partitions;
     };
 
     /**
@@ -121,16 +212,6 @@ namespace Legion {
       public:
         TaskOp *const op;
       };
-      struct DeferDistributeArgs : public LgTaskArgs<DeferDistributeArgs> {
-      public:
-        static const LgTaskID TASK_ID = LG_DEFER_DISTRIBUTE_TASK_ID;
-      public:
-        DeferDistributeArgs(TaskOp *op)
-          : LgTaskArgs<DeferDistributeArgs>(op->get_unique_op_id()),
-            proxy_this(op) { }
-      public:
-        TaskOp *const proxy_this;
-      };
       struct DeferMappingArgs : public LgTaskArgs<DeferMappingArgs> {
       public:
         static const LgTaskID TASK_ID = LG_DEFER_PERFORM_MAPPING_TASK_ID;
@@ -151,16 +232,6 @@ namespace Legion {
         std::vector<unsigned> *const performed_regions;
         std::vector<ApEvent> *const effects;
       };
-      struct DeferLaunchArgs : public LgTaskArgs<DeferLaunchArgs> {
-      public:
-        static const LgTaskID TASK_ID = LG_DEFER_LAUNCH_TASK_ID;
-      public:
-        DeferLaunchArgs(TaskOp *op)
-          : LgTaskArgs<DeferLaunchArgs>(op->get_unique_op_id()),
-            proxy_this(op) { }
-      public:
-        TaskOp *const proxy_this;
-      };
       struct DeferredFutureSetArgs : public LgTaskArgs<DeferredFutureSetArgs> {
       public:
         static const LgTaskID TASK_ID = LG_DEFERRED_FUTURE_SET_ID;
@@ -188,18 +259,6 @@ namespace Legion {
         const Domain domain;
         TaskOp *const task_op;
       };
-      struct DeferredEnqueueArgs : public LgTaskArgs<DeferredEnqueueArgs> {
-      public:
-        static const LgTaskID TASK_ID = LG_DEFERRED_ENQUEUE_TASK_ID;
-      public:
-        DeferredEnqueueArgs(ProcessorManager *man, TaskOp *t, bool select)
-          : LgTaskArgs<DeferredEnqueueArgs>(t->get_unique_op_id()),
-            manager(man), task(t), select_options(select) { }
-      public:
-        ProcessorManager *const manager;
-        TaskOp *const task;
-        const bool select_options;
-      };
     public:
       TaskOp(Runtime *rt);
       virtual ~TaskOp(void);
@@ -209,6 +268,7 @@ namespace Legion {
       virtual void set_context_index(size_t index);
       virtual bool has_parent_task(void) const;
       virtual const Task* get_parent_task(void) const;
+      virtual const std::string& get_provenance_string(bool human = true) const;
       virtual const char* get_task_name(void) const;
       virtual bool is_reducing_future(void) const;
       virtual void pack_remote_operation(Serializer &rez, AddressSpaceID target,
@@ -241,7 +301,12 @@ namespace Legion {
       void mark_stolen(void);
       void initialize_base_task(InnerContext *ctx, bool track,
             const std::vector<StaticDependence> *dependences,
-            const Predicate &p, Processor::TaskFuncID tid);
+            const Predicate &p, Processor::TaskFuncID tid,
+            const char *provenance);
+      void initialize_base_task(InnerContext *ctx, bool track,
+            const std::vector<StaticDependence> *dependences,
+            const Predicate &p, Processor::TaskFuncID tid,
+            Provenance *provenance);
       void check_empty_field_requirements(void);
       size_t check_future_size(FutureImpl *impl);
     public:
@@ -294,14 +359,13 @@ namespace Legion {
       virtual void perform_inlining(VariantImpl *variant,
                     const std::deque<InstanceSet> &parent_regions) = 0;
     public:
-      RtEvent defer_distribute_task(RtEvent precondition);
+      void defer_distribute_task(RtEvent precondition);
       RtEvent defer_perform_mapping(RtEvent precondition, MustEpochOp *op,
                                     const DeferMappingArgs *args,
                                     unsigned invocation_count,
                                     std::vector<unsigned> *performed = NULL,
                                     std::vector<ApEvent> *effects = NULL);
-      RtEvent defer_launch_task(RtEvent precondition);
-    protected:
+      void defer_launch_task(RtEvent precondition);
       void enqueue_ready_task(bool use_target_processor,
                               RtEvent wait_on = RtEvent::NO_RT_EVENT);
     public:
@@ -406,7 +470,9 @@ namespace Legion {
       virtual int get_depth(void) const;
       virtual bool has_parent_task(void) const;
       virtual const Task* get_parent_task(void) const;
+      virtual const std::string& get_provenance_string(bool human = true) const;
       virtual const char* get_task_name(void) const;
+      virtual Domain get_slice_domain(void) const;
       virtual void set_context_index(size_t index);
     public:
       virtual const char* get_logging_name(void) const;
@@ -493,8 +559,9 @@ namespace Legion {
     public:
       virtual void pack_profiling_requests(Serializer &rez,
                                            std::set<RtEvent> &applied) const;
-      virtual void add_copy_profiling_request(const PhysicalTraceInfo &info,
-                               Realm::ProfilingRequestSet &requests, bool fill);
+      virtual int add_copy_profiling_request(const PhysicalTraceInfo &info,
+                               Realm::ProfilingRequestSet &requests,
+                               bool fill, unsigned count = 1);
       virtual void handle_profiling_response(const ProfilingResponseBase *base,
                                       const Realm::ProfilingResponse &respone,
                                       const void *orig, size_t orig_length);
@@ -556,7 +623,7 @@ namespace Legion {
       // Regions which are NO_ACCESS or have no privilege fields
       std::vector<bool>                     no_access_regions;
       // The version infos for this operation
-      LegionVector<VersionInfo>::aligned    version_infos;
+      LegionVector<VersionInfo>             version_infos;
     protected:
       std::vector<Processor>                target_processors;
       // Hold the result of the mapping
@@ -605,8 +672,9 @@ namespace Legion {
       std::vector<SingleProfilingInfo>                  profiling_info;
       RtUserEvent                                   profiling_reported;
       int                                           profiling_priority;
-      int                               outstanding_profiling_requests;
-      int                               outstanding_profiling_reported;
+      int                                           copy_fill_priority;
+      std::atomic<int>                  outstanding_profiling_requests;
+      std::atomic<int>                  outstanding_profiling_reported;
 #ifdef DEBUG_LEGION
     protected:
       // For checking that premapped instances didn't change during mapping
@@ -642,6 +710,7 @@ namespace Legion {
       virtual void activate(void) = 0;
       virtual void deactivate(void) = 0;
       virtual bool is_reducing_future(void) const { return (redop > 0); }
+      virtual Domain get_slice_domain(void) const;
     public:
       virtual void trigger_dependence_analysis(void) = 0;
     public:
@@ -738,6 +807,7 @@ namespace Legion {
       virtual void activate(void);
       virtual void deactivate(void);
       virtual SingleTask* get_origin_task(void) const { return orig_task; }
+      virtual Domain get_slice_domain(void) const { return Domain::NO_DOMAIN; }
     public:
       Future initialize_task(InnerContext *ctx,
                              const TaskLauncher &launcher,
@@ -847,6 +917,8 @@ namespace Legion {
       virtual void activate(void);
       virtual void deactivate(void);
       virtual SingleTask* get_origin_task(void) const { return orig_task; }
+      virtual Domain get_slice_domain(void) const 
+        { return Domain(index_point,index_point); }
       virtual bool is_reducing_future(void) const;
     public:
       virtual void trigger_dependence_analysis(void);
@@ -882,6 +954,9 @@ namespace Legion {
       // ProjectionPoint methods
       virtual const DomainPoint& get_domain_point(void) const;
       virtual void set_projection_result(unsigned idx, LogicalRegion result);
+      virtual void record_intra_space_dependences(unsigned index,
+             const std::vector<DomainPoint> &dependences);
+      virtual const Mappable* as_mappable(void) const { return this; }
     public:
       void initialize_point(SliceTask *owner, const DomainPoint &point,
                             const FutureMap &point_arguments,
@@ -910,9 +985,7 @@ namespace Legion {
                                                 unsigned index, bool success);
       virtual void report_total_collective_instance_calls(MappingCallKind call,
                                                           unsigned total_calls);
-    public:
-      void record_intra_space_dependences(unsigned index,
-             const std::vector<DomainPoint> &dependences);
+    public: 
       bool has_remaining_inlining_dependences(
             std::map<PointTask*,unsigned> &remaining,
             std::map<RtEvent,std::vector<PointTask*> > &event_deps) const;
@@ -1001,8 +1074,9 @@ namespace Legion {
     public:
       virtual void pack_profiling_requests(Serializer &rez,
                                            std::set<RtEvent> &applied) const;
-      virtual void add_copy_profiling_request(const PhysicalTraceInfo &info,
-                               Realm::ProfilingRequestSet &requests, bool fill);
+      virtual int add_copy_profiling_request(const PhysicalTraceInfo &info,
+                               Realm::ProfilingRequestSet &requests,
+                               bool fill, unsigned count = 1);
       virtual void handle_profiling_response(const ProfilingResponseBase *base,
                                       const Realm::ProfilingResponse &respone,
                                       const void *orig, size_t orig_length);
@@ -1054,7 +1128,7 @@ namespace Legion {
       unsigned committed_points;
       RtUserEvent future_map_ready;
     protected:
-      LegionMap<unsigned/*idx*/,VersionInfo>::aligned version_infos;
+      LegionMap<unsigned/*idx*/,VersionInfo> version_infos;
       std::vector<RegionTreePath> privilege_paths;
       std::set<SliceTask*> origin_mapped_slices;
     protected:
@@ -1076,8 +1150,9 @@ namespace Legion {
       std::vector<IndexProfilingInfo>                   profiling_info;
       RtUserEvent                                   profiling_reported;
       int                                           profiling_priority;
-      int                               outstanding_profiling_requests;
-      int                               outstanding_profiling_reported;
+      int                                           copy_fill_priority;
+      std::atomic<int>                  outstanding_profiling_requests;
+      std::atomic<int>                  outstanding_profiling_reported;
     protected:
       // Whether we have to do intra-task alias analysis
       bool need_intra_task_alias_analysis;
@@ -1181,16 +1256,16 @@ namespace Legion {
     public: // Privilege tracker methods
       virtual void receive_resources(size_t return_index,
               std::map<LogicalRegion,unsigned> &created_regions,
-              std::vector<LogicalRegion> &deleted_regions,
+              std::vector<DeletedRegion> &deleted_regions,
               std::set<std::pair<FieldSpace,FieldID> > &created_fields,
-              std::vector<std::pair<FieldSpace,FieldID> > &deleted_fields,
+              std::vector<DeletedField> &deleted_fields,
               std::map<FieldSpace,unsigned> &created_field_spaces,
               std::map<FieldSpace,std::set<LogicalRegion> > &latent_spaces,
-              std::vector<FieldSpace> &deleted_field_spaces,
+              std::vector<DeletedFieldSpace> &deleted_field_spaces,
               std::map<IndexSpace,unsigned> &created_index_spaces,
-              std::vector<std::pair<IndexSpace,bool> > &deleted_index_spaces,
+              std::vector<DeletedIndexSpace> &deleted_index_spaces,
               std::map<IndexPartition,unsigned> &created_partitions,
-              std::vector<std::pair<IndexPartition,bool> > &deleted_partitions,
+              std::vector<DeletedPartition> &deleted_partitions,
               std::set<RtEvent> &preconditions);
     public:
       // From MemoizableOp
