@@ -1,4 +1,4 @@
-/* Copyright 2021 Stanford University, NVIDIA Corporation
+/* Copyright 2023 Stanford University, NVIDIA Corporation
  *
  * Licensed under the Apache License, Version 2.0 (the "License");
  * you may not use this file except in compliance with the License.
@@ -78,7 +78,8 @@ namespace Legion {
         PHYSICAL_REPLAY,
       };
     public:
-      LegionTrace(InnerContext *ctx, TraceID tid, bool logical_only);
+      LegionTrace(InnerContext *ctx, TraceID tid, bool logical_only, 
+                  Provenance *provenance);
       virtual ~LegionTrace(void);
     public:
       virtual bool is_static_trace(void) const = 0;
@@ -107,7 +108,7 @@ namespace Legion {
     public:
       // Called by task execution thread
       inline bool is_fixed(void) const { return fixed; }
-      void fix_trace(void);
+      void fix_trace(Provenance *provenance);
     public:
       bool has_physical_trace(void) { return physical_trace != NULL; }
       PhysicalTrace* get_physical_trace(void) { return physical_trace; }
@@ -117,10 +118,10 @@ namespace Legion {
       void end_trace_execution(FenceOp *fence_op);
     public:
       void initialize_tracing_state(void) { state = LOGICAL_ONLY; }
-      void set_state_record(void) { state = PHYSICAL_RECORD; }
-      void set_state_replay(void) { state = PHYSICAL_REPLAY; }
-      bool is_recording(void) const { return state == PHYSICAL_RECORD; }
-      bool is_replaying(void) const { return state == PHYSICAL_REPLAY; }
+      void set_state_record(void) { state.store(PHYSICAL_RECORD); }
+      void set_state_replay(void) { state.store(PHYSICAL_REPLAY); }
+      bool is_recording(void) const { return state.load() == PHYSICAL_RECORD; }
+      bool is_replaying(void) const { return state.load() == PHYSICAL_REPLAY; }
     public:
       inline void clear_blocking_call(void) { blocking_call_observed = false; }
       inline void record_blocking_call(void) { blocking_call_observed = true; }
@@ -140,13 +141,16 @@ namespace Legion {
     public:
       InnerContext *const ctx;
       const TraceID tid;
+      Provenance *const begin_provenance;
+      // Set after end_trace is called
+      Provenance *end_provenance;
     protected:
       std::vector<std::pair<Operation*,GenerationID> > operations; 
       // We also need a data structure to record when there are
       // aliased but non-interfering region requirements. This should
       // be pretty sparse so we'll make it a map
-      std::map<unsigned,LegionVector<AliasChildren>::aligned> aliased_children;
-      volatile TracingState state;
+      std::map<unsigned,LegionVector<AliasChildren> > aliased_children;
+      std::atomic<TracingState> state;
       // Pointer to a physical trace
       PhysicalTrace *physical_trace;
       unsigned last_memoized;
@@ -174,7 +178,7 @@ namespace Legion {
       static const AllocationType alloc_type = STATIC_TRACE_ALLOC;
     public:
       StaticTrace(TraceID tid, InnerContext *ctx, bool logical_only,
-                  const std::set<RegionTreeID> *trees);
+                  Provenance *p, const std::set<RegionTreeID> *trees);
       StaticTrace(const StaticTrace &rhs);
       virtual ~StaticTrace(void);
     public:
@@ -208,11 +212,11 @@ namespace Legion {
                           UniqueID prev_fence_uid, UniqueID curr_fence_uid);
 #endif
     protected:
-      const LegionVector<DependenceRecord>::aligned&
+      const LegionVector<DependenceRecord>&
                   translate_dependence_records(Operation *op, unsigned index);
     protected:
       std::deque<std::vector<StaticDependence> > static_dependences;
-      std::deque<LegionVector<DependenceRecord>::aligned> translated_deps;
+      std::deque<LegionVector<DependenceRecord> > translated_deps;
       std::set<RegionTreeID> application_trees;
     };
 
@@ -236,7 +240,8 @@ namespace Legion {
         unsigned count;
       }; 
     public:
-      DynamicTrace(TraceID tid, InnerContext *ctx, bool logical_only);
+      DynamicTrace(TraceID tid, InnerContext *ctx, 
+                   bool logical_only, Provenance *p);
       DynamicTrace(const DynamicTrace &rhs);
       virtual ~DynamicTrace(void);
     public:
@@ -288,12 +293,12 @@ namespace Legion {
       // trace that we would have interfered with had the internal operations
       // not been necessary.
       std::map<std::pair<InternalOp*,GenerationID>,
-               LegionVector<DependenceRecord>::aligned> internal_dependences;
+               LegionVector<DependenceRecord> > internal_dependences;
     protected: 
       // This is the generalized form of the dependences
       // For each operation, we remember a list of operations that
       // it dependens on and whether it is a validates the region
-      std::deque<LegionVector<DependenceRecord>::aligned> dependences;
+      std::deque<LegionVector<DependenceRecord> > dependences;
       // Metadata for checking the validity of a trace when it is replayed
       std::vector<OperationInfo> op_info;
     protected:
@@ -331,7 +336,7 @@ namespace Legion {
       TraceCaptureOp& operator=(const TraceCaptureOp &rhs);
     public:
       void initialize_capture(InnerContext *ctx, bool has_blocking_call,
-                              bool remove_trace_reference);
+                    bool remove_trace_reference, const char *provenance);
     public:
       virtual void activate(void);
       virtual void deactivate(void);
@@ -364,7 +369,8 @@ namespace Legion {
     public:
       TraceCompleteOp& operator=(const TraceCompleteOp &rhs);
     public:
-      void initialize_complete(InnerContext *ctx, bool has_blocking_call);
+      void initialize_complete(InnerContext *ctx, bool has_blocking_call,
+                               const char *provenance);
     public:
       virtual void activate(void);
       virtual void deactivate(void);
@@ -396,7 +402,8 @@ namespace Legion {
     public:
       TraceReplayOp& operator=(const TraceReplayOp &rhs);
     public:
-      void initialize_replay(InnerContext *ctx, LegionTrace *trace);
+      void initialize_replay(InnerContext *ctx, LegionTrace *trace,
+                             Provenance *provenance);
     public:
       virtual void activate(void);
       virtual void deactivate(void);
@@ -423,7 +430,8 @@ namespace Legion {
     public:
       TraceBeginOp& operator=(const TraceBeginOp &rhs);
     public:
-      void initialize_begin(InnerContext *ctx, LegionTrace *trace);
+      void initialize_begin(InnerContext *ctx, LegionTrace *trace,
+                            Provenance *provenance);
     public:
       virtual void activate(void);
       virtual void deactivate(void);
@@ -443,7 +451,8 @@ namespace Legion {
     public:
       void initialize_summary(InnerContext *ctx,
                               PhysicalTemplate *tpl,
-                              Operation *invalidator);
+                              Operation *invalidator,
+                              Provenance *provenance);
       void perform_logging(void);
     public:
       virtual void activate(void);
@@ -468,10 +477,10 @@ namespace Legion {
     class PhysicalTrace {
     public:
       PhysicalTrace(Runtime *runtime, LegionTrace *logical_trace);
-      PhysicalTrace(const PhysicalTrace &rhs);
+      PhysicalTrace(const PhysicalTrace &rhs) = delete;
       ~PhysicalTrace(void);
     public:
-      PhysicalTrace& operator=(const PhysicalTrace &rhs);
+      PhysicalTrace& operator=(const PhysicalTrace &rhs) = delete;
     public:
       void clear_cached_template(void) { current_template = NULL; }
       void check_template_preconditions(TraceReplayOp *op,
@@ -503,6 +512,7 @@ namespace Legion {
     public:
       Runtime * const runtime;
       const LegionTrace *logical_trace;
+      const bool perform_fence_elision;
     private:
       mutable LocalLock trace_lock;
       FenceOp *previous_replay;
@@ -552,7 +562,7 @@ namespace Legion {
       void dump(void) const;
     protected:
       typedef LegionMap<InstanceView*,
-                        FieldMaskSet<EquivalenceSet>  >::aligned ViewSet;
+                        FieldMaskSet<EquivalenceSet>  > ViewSet;
     protected:
       RegionTreeForest * const forest;
     protected:
@@ -574,11 +584,15 @@ namespace Legion {
     public:
       bool require(Operation *op, std::set<RtEvent> &applied_events);
       void ensure(Operation *op, std::set<RtEvent> &applied_events);
+      bool modifies_fields(const LegionMap<InstanceView*,
+              FieldMaskSet<IndexSpaceExpression> > &instances) const;
     private:
       bool cached;
       // The following containers are populated only when the 'cached' is true.
-      LegionVector<FieldMaskSet<InstanceView> >::aligned views;
-      LegionVector<VersionInfo>::aligned                 version_infos;
+      LegionVector<FieldMaskSet<InstanceView> >          views;
+      LegionVector<VersionInfo>                          version_infos;
+      // Managers that we keep valid
+      std::vector<PhysicalManager*>                      valid_instances; 
     };
 
     /**
@@ -596,12 +610,13 @@ namespace Legion {
       public:
         static const LgTaskID TASK_ID = LG_REPLAY_SLICE_TASK_ID;
       public:
-        ReplaySliceArgs(PhysicalTemplate *t, unsigned si)
+        ReplaySliceArgs(PhysicalTemplate *t, unsigned si, bool recurrent)
           : LgTaskArgs<ReplaySliceArgs>(implicit_provenance),
-            tpl(t), slice_index(si) { }
+            tpl(t), slice_index(si), recurrent_replay(recurrent) { }
       public:
         PhysicalTemplate *const tpl;
-        unsigned slice_index;
+        const unsigned slice_index;
+        const bool recurrent_replay;
       };
       struct TransitiveReductionArgs :
         public LgTaskArgs<TransitiveReductionArgs> {
@@ -640,12 +655,12 @@ namespace Legion {
         std::vector<Processor>  target_procs;
         std::deque<InstanceSet> physical_instances;
       };
-      typedef LegionMap<TraceLocalID,CachedMapping>::aligned CachedMappings;
+      typedef LegionMap<TraceLocalID,CachedMapping> CachedMappings;
     private:
       typedef LegionMap<InstanceView*,
-                        FieldMaskSet<IndexSpaceExpression> >::aligned ViewExprs;
+                        FieldMaskSet<IndexSpaceExpression> >         ViewExprs;
       typedef LegionMap<InstanceView*,
-                        FieldMaskSet<ViewUser> >::aligned             ViewUsers;
+                        FieldMaskSet<ViewUser> >                     ViewUsers;
       typedef std::map<RegionTreeID,std::set<InstanceView*> >        ViewGroups;
     public:
       PhysicalTemplate(PhysicalTrace *trace, ApEvent fence_event);
@@ -699,10 +714,11 @@ namespace Legion {
                                std::set<RtEvent> &applied_events);
     public:
       void register_operation(Operation *op);
-      void execute_slice(unsigned slice_idx);
+      void execute_slice(unsigned slice_idx, bool recurrent_replay);
     public:
       void issue_summary_operations(InnerContext* context,
-                                    Operation *invalidator);
+                                    Operation *invalidator,
+                                    Provenance *provenance);
     public:
       void dump_template(void);
     private:
@@ -713,12 +729,12 @@ namespace Legion {
       UniqueID get_fence_uid(void) const { return prev_fence_uid; }
 #endif
     public:
-      inline bool is_replaying(void) const { return !recording; }
+      inline bool is_replaying(void) const { return !recording.load(); }
       inline bool is_replayable(void) const { return replayable.replayable; }
       inline const std::string& get_replayable_message(void) const
         { return replayable.message; }
     public:
-      virtual bool is_recording(void) const { return recording; }
+      virtual bool is_recording(void) const { return recording.load(); }
       virtual void add_recorder_reference(void) { /*do nothing*/ }
       virtual bool remove_recorder_reference(void) 
         { /*do nothing, never delete*/ return false; }
@@ -735,6 +751,8 @@ namespace Legion {
                              bool &postmap_task,
                              std::vector<Processor> &target_proc,
                              std::deque<InstanceSet> &physical_instances) const;
+      void get_task_reservations(SingleTask *task,
+                             std::map<Reservation,bool> &reservations) const;
     public:
       virtual void record_get_term_event(Memoizable *memo);
       virtual void record_create_ap_user_event(ApUserEvent lhs, 
@@ -757,27 +775,26 @@ namespace Legion {
                              IndexSpaceExpression *expr,
                              const std::vector<CopySrcDstField>& src_fields,
                              const std::vector<CopySrcDstField>& dst_fields,
+                             const std::vector<Reservation> &reservations,
 #ifdef LEGION_SPY
                              RegionTreeID src_tree_id, RegionTreeID dst_tree_id,
 #endif
                              ApEvent precondition, PredEvent pred_guard,
-                             ReductionOpID redop, bool reduction_fold);
-      virtual void record_issue_indirect(Memoizable *memo, ApEvent &lhs,
-                             IndexSpaceExpression *expr,
-                             const std::vector<CopySrcDstField>& src_fields,
-                             const std::vector<CopySrcDstField>& dst_fields,
-                             const std::vector<CopyIndirection*> &indirections,
-#ifdef LEGION_SPY
-                             unsigned unique_indirections_identifier,
-#endif
-                             ApEvent precondition, PredEvent pred_guard,
-                             ApEvent tracing_precondition);
+                             LgEvent src_unique, LgEvent dst_unique, 
+                             int priority);
+      virtual void record_issue_across(Memoizable *memo, ApEvent &lhs, 
+                             ApEvent collective_precondition,
+                             ApEvent copy_precondition,
+                             ApEvent src_indirect_precondition,
+                             ApEvent dst_indirect_precondition,
+                             CopyAcrossExecutor *executor);
       virtual void record_copy_views(ApEvent lhs, Memoizable *memo,
                            unsigned src_idx, unsigned dst_idx,
                            IndexSpaceExpression *expr,
                            const FieldMaskSet<InstanceView> &tracing_srcs,
                            const FieldMaskSet<InstanceView> &tracing_dsts,
                            PrivilegeMode src_mode, PrivilegeMode dst_mode,
+                           bool src_indirect, bool dst_indirect,
                            std::set<RtEvent> &applied);
       virtual void record_indirect_views(ApEvent indirect_done,ApEvent all_done,
                            Memoizable *memo, unsigned index, 
@@ -792,17 +809,8 @@ namespace Legion {
                              FieldSpace handle, 
                              RegionTreeID tree_id,
 #endif
-                             ApEvent precondition, PredEvent pred_guard);
-#ifdef LEGION_GPU_REDUCTIONS
-      virtual void record_gpu_reduction(Memoizable *memo, ApEvent &lhs,
-                           IndexSpaceExpression *expr,
-                           const std::vector<CopySrcDstField> &src_fields,
-                           const std::vector<CopySrcDstField> &dst_fields,
-                           Processor gpu, TaskID gpu_task_id,
-                           PhysicalManager *src, PhysicalManager *dst,
-                           ApEvent precondition, PredEvent pred_guard,
-                           ReductionOpID redop, bool reduction_fold);
-#endif
+                             ApEvent precondition, PredEvent pred_guard,
+                             LgEvent unique_event, int priority);
     public:
       virtual void record_op_view(Memoizable *memo,
                                   unsigned idx,
@@ -822,7 +830,7 @@ namespace Legion {
                         IndexSpaceExpression *expr,
                         const RegionUsage &usage,
                         const FieldMaskSet<InstanceView> &views,
-                    const LegionList<FieldSet<EquivalenceSet*> >::aligned &eqs);
+                        const LegionList<FieldSet<EquivalenceSet*> > &eqs);
       void update_valid_views(InstanceView *view,
                               EquivalenceSet *eq,
                               const RegionUsage &usage,
@@ -832,7 +840,7 @@ namespace Legion {
                          const RegionUsage &usage,
                          unsigned user, IndexSpaceExpression *user_expr,
                          const FieldMask &user_mask);
-      void record_copy_views(unsigned copy_id,
+      static void record_expression_views(ViewExprs &cviews,
                              IndexSpaceExpression *expr,
                              const FieldMaskSet<InstanceView> &views);
       void record_fill_views(const FieldMaskSet<FillView> &views);
@@ -840,9 +848,9 @@ namespace Legion {
       virtual void record_set_op_sync_event(ApEvent &lhs, Memoizable *memo);
       virtual void record_set_effects(Memoizable *memo, ApEvent &rhs);
       virtual void record_complete_replay(Memoizable *memo, ApEvent rhs);
-      virtual void record_reservations(Memoizable *memo, ApEvent &lhs,
-                              const std::map<Reservation,bool> &locks, 
-                              ApEvent precondition, ApEvent postcondition);
+      virtual void record_reservations(const TraceLocalID &tlid,
+                                const std::map<Reservation,bool> &locks,
+                                std::set<RtEvent> &applied_events); 
     public:
       void perform_replay(Runtime *rt, std::set<RtEvent> &replayed_events);
       void finish_replay(std::set<ApEvent> &postconditions);
@@ -864,7 +872,6 @@ namespace Legion {
     private:
       unsigned convert_event(const ApEvent &event);
       unsigned find_event(const ApEvent &event) const;
-      unsigned find_or_convert_event(const ApEvent &event);
       void insert_instruction(Instruction *inst);
     private:
       // Returns the set of last users for all <view,field mask,index expr>
@@ -877,6 +884,19 @@ namespace Legion {
                            IndexSpaceExpression *expr,
                            const FieldMask &mask,
                            std::set<unsigned> &users);
+      void rewrite_preconditions(unsigned &precondition,
+                           std::set<unsigned> &users,
+                           const std::vector<Instruction*> &instructions,
+                           std::vector<Instruction*> &new_instructions,
+                           std::vector<unsigned> &gen,
+                           unsigned &merge_starts);
+      void parallelize_replay_event(unsigned &event_to_check,
+                           unsigned slice_index,
+                           const std::vector<unsigned> &gen,
+                           const std::vector<unsigned> &slice_indices_by_inst,
+                           std::map<unsigned,
+                              std::pair<unsigned,unsigned> > &crossing_counts,
+                           std::vector<Instruction*> &crossing_instructions);
     public:
       inline void update_last_fence(GetTermEvent *fence)
         { last_fence = fence; }
@@ -885,7 +905,7 @@ namespace Legion {
       void release_remote_memos(void);
     private:
       PhysicalTrace * const trace;
-      volatile bool recording;
+      std::atomic<bool> recording;
       Replayable replayable;
       mutable LocalLock template_lock;
       const unsigned fence_completion_id;
@@ -898,6 +918,7 @@ namespace Legion {
       std::vector<Memoizable*> remote_memos;
     private:
       CachedMappings cached_mappings;
+      std::map<TraceLocalID,std::map<Reservation,bool> > cached_reservations;
       bool has_virtual_mapping;
     private:
       GetTermEvent                    *last_fence;
@@ -909,7 +930,6 @@ namespace Legion {
     private:
       std::vector<Instruction*>               instructions;
       std::vector<std::vector<Instruction*> > slices;
-      std::vector<std::vector<TraceLocalID> > slice_tasks;
     private:
       std::map<unsigned/*event*/,unsigned/*consumers*/> crossing_events;
       // Frontiers of a template are a set of users whose events must
@@ -925,11 +945,15 @@ namespace Legion {
     private:
       RtUserEvent recording_done;
       RtEvent transitive_reduction_done;
-      std::vector<unsigned> *volatile pending_inv_topo_order;
-      std::vector<std::vector<unsigned> >*volatile pending_transitive_reduction;
+      std::atomic<std::vector<unsigned>*> pending_inv_topo_order;
+      std::atomic<
+        std::vector<std::vector<unsigned> >*> pending_transitive_reduction;
     private:
       std::map<TraceLocalID,ViewExprs> op_views;
       std::map<unsigned,ViewExprs>     copy_views;
+      std::map<unsigned,ViewExprs>     src_indirect_views;
+      std::map<unsigned,ViewExprs>     dst_indirect_views;
+      std::vector<IssueAcross*>        across_copies;
     private:
       TraceConditionSet   pre, post;
       ViewGroups          view_groups;
@@ -963,15 +987,10 @@ namespace Legion {
       friend class AssignFenceCompletion;
       friend class IssueCopy;
       friend class IssueFill;
-      friend class IssueIndirect;
+      friend class IssueAcross;
       friend class SetOpSyncEvent;
       friend class SetEffects;
       friend class CompleteReplay;
-      friend class AcquireReplay;
-      friend class ReleaseReplay;
-#ifdef LEGION_GPU_REDUCTIONS
-      friend class GPUReduction;
-#endif
 #endif
     };
 
@@ -983,16 +1002,11 @@ namespace Legion {
       MERGE_EVENT,
       ISSUE_COPY,
       ISSUE_FILL,
-      ISSUE_INDIRECT,
+      ISSUE_ACROSS,
       SET_OP_SYNC_EVENT,
       SET_EFFECTS,
       ASSIGN_FENCE_COMPLETION,
       COMPLETE_REPLAY,
-      ACQUIRE_REPLAY,
-      RELEASE_REPLAY,
-#ifdef LEGION_GPU_REDUCTIONS
-      GPU_REDUCTION,
-#endif
     };
 
     /**
@@ -1005,7 +1019,8 @@ namespace Legion {
       virtual ~Instruction(void) {};
       virtual void execute(std::vector<ApEvent> &events,
                            std::map<unsigned,ApUserEvent> &user_events,
-                           std::map<TraceLocalID,Memoizable*> &operations) = 0;
+                           std::map<TraceLocalID,Memoizable*> &operations,
+                           const bool recurrent_replay) = 0;
       virtual std::string to_string(
                            std::map<TraceLocalID,Memoizable*> &operations) = 0;
 
@@ -1018,15 +1033,10 @@ namespace Legion {
         { return NULL; }
       virtual IssueCopy* as_issue_copy(void) { return NULL; }
       virtual IssueFill* as_issue_fill(void) { return NULL; }
-      virtual IssueIndirect* as_issue_indirect(void) { return NULL; }
+      virtual IssueAcross* as_issue_across(void) { return NULL; }
       virtual SetOpSyncEvent* as_set_op_sync_event(void) { return NULL; }
       virtual SetEffects* as_set_effects(void) { return NULL; }
       virtual CompleteReplay* as_complete_replay(void) { return NULL; }
-      virtual AcquireReplay* as_acquire_replay(void) { return NULL; }
-      virtual ReleaseReplay* as_release_replay(void) { return NULL; }
-#ifdef LEGION_GPU_REDUCTIONS
-      virtual GPUReduction* as_gpu_reduction(void) { return NULL; }
-#endif
     public:
       const TraceLocalID owner;
     };
@@ -1042,7 +1052,8 @@ namespace Legion {
                    const TraceLocalID& rhs, bool fence);
       virtual void execute(std::vector<ApEvent> &events,
                            std::map<unsigned,ApUserEvent> &user_events,
-                           std::map<TraceLocalID,Memoizable*> &operations);
+                           std::map<TraceLocalID,Memoizable*> &operations,
+                           const bool recurrent_replay);
       virtual std::string to_string(
                            std::map<TraceLocalID,Memoizable*> &operations);
 
@@ -1066,7 +1077,8 @@ namespace Legion {
                         const TraceLocalID &owner);
       virtual void execute(std::vector<ApEvent> &events,
                            std::map<unsigned,ApUserEvent> &user_events,
-                           std::map<TraceLocalID,Memoizable*> &operations);
+                           std::map<TraceLocalID,Memoizable*> &operations,
+                           const bool recurrent_replay);
       virtual std::string to_string(
                            std::map<TraceLocalID,Memoizable*> &operations);
 
@@ -1090,7 +1102,8 @@ namespace Legion {
                    const TraceLocalID &owner);
       virtual void execute(std::vector<ApEvent> &events,
                            std::map<unsigned,ApUserEvent> &user_events,
-                           std::map<TraceLocalID,Memoizable*> &operations);
+                           std::map<TraceLocalID,Memoizable*> &operations,
+                           const bool recurrent_replay);
       virtual std::string to_string(
                            std::map<TraceLocalID,Memoizable*> &operations);
 
@@ -1116,7 +1129,8 @@ namespace Legion {
                  const TraceLocalID &owner);
       virtual void execute(std::vector<ApEvent> &events,
                            std::map<unsigned,ApUserEvent> &user_events,
-                           std::map<TraceLocalID,Memoizable*> &operations);
+                           std::map<TraceLocalID,Memoizable*> &operations,
+                           const bool recurrent_replay);
       virtual std::string to_string(
                            std::map<TraceLocalID,Memoizable*> &operations);
 
@@ -1140,7 +1154,8 @@ namespace Legion {
                             const TraceLocalID &owner);
       virtual void execute(std::vector<ApEvent> &events,
                            std::map<unsigned,ApUserEvent> &user_events,
-                           std::map<TraceLocalID,Memoizable*> &operations);
+                           std::map<TraceLocalID,Memoizable*> &operations,
+                           const bool recurrent_replay);
       virtual std::string to_string(
                            std::map<TraceLocalID,Memoizable*> &operations);
 
@@ -1171,11 +1186,13 @@ namespace Legion {
 #ifdef LEGION_SPY
                 FieldSpace handle, RegionTreeID tree_id,
 #endif
-                unsigned precondition_idx);
+                unsigned precondition_idx, LgEvent unique_event,
+                int priority);
       virtual ~IssueFill(void);
       virtual void execute(std::vector<ApEvent> &events,
                            std::map<unsigned,ApUserEvent> &user_events,
-                           std::map<TraceLocalID,Memoizable*> &operations);
+                           std::map<TraceLocalID,Memoizable*> &operations,
+                           const bool recurrent_replay);
       virtual std::string to_string(
                            std::map<TraceLocalID,Memoizable*> &operations);
 
@@ -1195,6 +1212,8 @@ namespace Legion {
       RegionTreeID tree_id;
 #endif
       unsigned precondition_idx;
+      LgEvent unique_event;
+      int priority;
     };
 
     /**
@@ -1212,15 +1231,17 @@ namespace Legion {
                 const TraceLocalID &op_key,
                 const std::vector<CopySrcDstField>& src_fields,
                 const std::vector<CopySrcDstField>& dst_fields,
+                const std::vector<Reservation>& reservations,
 #ifdef LEGION_SPY
                 RegionTreeID src_tree_id, RegionTreeID dst_tree_id,
 #endif
                 unsigned precondition_idx,
-                ReductionOpID redop, bool reduction_fold);
+                LgEvent src_unique, LgEvent dst_unique, int priority);
       virtual ~IssueCopy(void);
       virtual void execute(std::vector<ApEvent> &events,
                            std::map<unsigned,ApUserEvent> &user_events,
-                           std::map<TraceLocalID,Memoizable*> &operations);
+                           std::map<TraceLocalID,Memoizable*> &operations,
+                           const bool recurrent_replay);
       virtual std::string to_string(
                            std::map<TraceLocalID,Memoizable*> &operations);
 
@@ -1234,107 +1255,53 @@ namespace Legion {
       IndexSpaceExpression *expr;
       std::vector<CopySrcDstField> src_fields;
       std::vector<CopySrcDstField> dst_fields;
+      std::vector<Reservation> reservations;
 #ifdef LEGION_SPY
       RegionTreeID src_tree_id;
       RegionTreeID dst_tree_id;
 #endif
       unsigned precondition_idx;
-      ReductionOpID redop;
-      bool reduction_fold;
+      LgEvent src_unique, dst_unique;
+      int priority;
     };
 
     /**
-     * \class IssueIndirect
+     * \class IssueAcross
      * This instruction has the following semantics:
-     *  events[lhs] = expr->issue_indirect(dst_fields, src_fields,
-     *                                     indirections,
-     *                                     events[precondition_idx],
-     *                                     predicate_guard)
+     *  events[lhs] = executor->execute(ops[key], predicate_guard,
+     *                                  events[copy_precondition],
+     *                                  events[src_indirect_precondition],
+     *                                  events[dst_indirect_precondition])
      */
-    class IssueIndirect : public Instruction {
+    class IssueAcross : public Instruction {
     public:
-      IssueIndirect(PhysicalTemplate &tpl,
-                    unsigned lhs, IndexSpaceExpression *expr,
-                    const TraceLocalID &op_key,
-                    const std::vector<CopySrcDstField>& src_fields,
-                    const std::vector<CopySrcDstField>& dst_fields,
-                    const std::vector<CopyIndirection*>& indirects,
-#ifdef LEGION_SPY
-                    unsigned unique_indirections_identifier,
-#endif
-                    unsigned precondition_idx,
-                    unsigned tracing_pre_idx);
-      virtual ~IssueIndirect(void);
+      IssueAcross(PhysicalTemplate &tpl, unsigned lhs,
+                  unsigned copy_pre, unsigned collective_pre,
+                  unsigned src_indirect_pre, unsigned dst_indirect_pre,
+                  const TraceLocalID &op_key,
+                  CopyAcrossExecutor *executor);
+      virtual ~IssueAcross(void);
       virtual void execute(std::vector<ApEvent> &events,
                            std::map<unsigned,ApUserEvent> &user_events,
-                           std::map<TraceLocalID,Memoizable*> &operations);
+                           std::map<TraceLocalID,Memoizable*> &operations,
+                           const bool recurrent_replay);
       virtual std::string to_string(
                            std::map<TraceLocalID,Memoizable*> &operations);
 
       virtual InstructionKind get_kind(void)
-        { return ISSUE_INDIRECT; }
-      virtual IssueIndirect* as_issue_indirect(void)
+        { return ISSUE_ACROSS; }
+      virtual IssueAcross* as_issue_across(void)
         { return this; }
     private:
       friend class PhysicalTemplate;
       unsigned lhs;
-      IndexSpaceExpression *expr;
-      std::vector<CopySrcDstField> src_fields;
-      std::vector<CopySrcDstField> dst_fields;
-      std::vector<CopyIndirection*> indirections;
-#ifdef LEGION_SPY
-      unsigned unique_indirections_identifier;
-#endif
-      unsigned precondition_idx;
-      unsigned tracing_pre_idx;
+      unsigned copy_precondition;
+      unsigned collective_precondition;
+      unsigned src_indirect_precondition;
+      unsigned dst_indirect_precondition;
+      CopyAcrossExecutor *const executor;
     };
 
-#ifdef LEGION_GPU_REDUCTIONS
-    /**
-     * \class GPUReduction
-     * This instruction has the following semantics:
-     * events[lhs] = expr->gpu_reduction(dst_fields, src_fields,
-     *                                   gpu, dst, src,
-     *                                   events[precondition_idx],
-     *                                   predicate_guard,
-     *                                   redop, reduction_fold)
-     */
-    class GPUReduction : public Instruction {
-    public:
-      GPUReduction(PhysicalTemplate &tpl,
-                   unsigned lhs, IndexSpaceExpression *expr,
-                   const TraceLocalID &op_key,
-                   const std::vector<CopySrcDstField>& src_fields,
-                   const std::vector<CopySrcDstField>& dst_fields,
-                   Processor gpu, TaskID gpu_task_id,
-                   PhysicalManager *src, PhysicalManager *dst,
-                   unsigned precondition_idx,
-                   ReductionOpID redop, bool reduction_fold);
-      virtual ~GPUReduction(void);
-      virtual void execute(std::vector<ApEvent> &events,
-                           std::map<unsigned,ApUserEvent> &user_events,
-                           std::map<TraceLocalID,Memoizable*> &operations);
-      virtual std::string to_string(
-                           std::map<TraceLocalID,Memoizable*> &operations);
-
-      virtual InstructionKind get_kind(void)
-        { return GPU_REDUCTION; }
-      virtual GPUReduction* as_gpu_reduction(void)
-        { return this; }
-    private:
-      friend class PhysicalTemplate;
-      unsigned lhs;
-      IndexSpaceExpression *expr;
-      std::vector<CopySrcDstField> src_fields, dst_fields;
-      Processor gpu;
-      TaskID gpu_task_id;
-      PhysicalManager *src, *dst;
-      unsigned precondition_idx;
-      ReductionOpID redop;
-      bool reduction_fold;
-    };
-#endif
-
     /**
      * \class SetOpSyncEvent
      * This instruction has the following semantics:
@@ -1346,7 +1313,8 @@ namespace Legion {
                      const TraceLocalID& rhs);
       virtual void execute(std::vector<ApEvent> &events,
                            std::map<unsigned,ApUserEvent> &user_events,
-                           std::map<TraceLocalID,Memoizable*> &operations);
+                           std::map<TraceLocalID,Memoizable*> &operations,
+                           const bool recurrent_replay);
       virtual std::string to_string(
                            std::map<TraceLocalID,Memoizable*> &operations);
 
@@ -1369,7 +1337,8 @@ namespace Legion {
       SetEffects(PhysicalTemplate& tpl, const TraceLocalID& lhs, unsigned rhs);
       virtual void execute(std::vector<ApEvent> &events,
                            std::map<unsigned,ApUserEvent> &user_events,
-                           std::map<TraceLocalID,Memoizable*> &operations);
+                           std::map<TraceLocalID,Memoizable*> &operations,
+                           const bool recurrent_replay);
       virtual std::string to_string(
                            std::map<TraceLocalID,Memoizable*> &operations);
 
@@ -1393,7 +1362,8 @@ namespace Legion {
                      unsigned rhs);
       virtual void execute(std::vector<ApEvent> &events,
                            std::map<unsigned,ApUserEvent> &user_events,
-                           std::map<TraceLocalID,Memoizable*> &operations);
+                           std::map<TraceLocalID,Memoizable*> &operations,
+                           const bool recurrent_replay);
       virtual std::string to_string(
                            std::map<TraceLocalID,Memoizable*> &operations);
 
@@ -1406,59 +1376,6 @@ namespace Legion {
       unsigned rhs;
     };
 
-    /**
-     * \class AcquireReplay
-     * This instruction has the following semantics:
-     *   events[lhs] = acquire_reservations(events[pre])
-     */
-    class AcquireReplay : public Instruction {
-    public:
-      AcquireReplay(PhysicalTemplate &tpl, unsigned lhs,
-          unsigned rhs, const TraceLocalID &tld,
-          const std::map<Reservation,bool> &reservations);
-      virtual void execute(std::vector<ApEvent> &events,
-                           std::map<unsigned,ApUserEvent> &user_events,
-                           std::map<TraceLocalID,Memoizable*> &operations);
-      virtual std::string to_string(
-                           std::map<TraceLocalID,Memoizable*> &operations);
-
-      virtual InstructionKind get_kind(void)
-        { return ACQUIRE_REPLAY; }
-      virtual AcquireReplay * as_acquire_replay(void)
-        { return this; }
-    private:
-      friend class PhysicalTemplate;
-      const std::map<Reservation,bool> reservations;
-      unsigned lhs;
-      unsigned rhs;
-    };
-
-    /**
-     * \class ReleaseReplay
-     * This instruction has the following semantics:
-     *   release_reservations(events[pre])
-     */
-    class ReleaseReplay : public Instruction {
-    public:
-      ReleaseReplay(PhysicalTemplate &tpl, 
-          unsigned rhs, const TraceLocalID &tld,
-          const std::map<Reservation,bool> &reservations);
-      virtual void execute(std::vector<ApEvent> &events,
-                           std::map<unsigned,ApUserEvent> &user_events,
-                           std::map<TraceLocalID,Memoizable*> &operations);
-      virtual std::string to_string(
-                           std::map<TraceLocalID,Memoizable*> &operations);
-
-      virtual InstructionKind get_kind(void)
-        { return RELEASE_REPLAY; }
-      virtual ReleaseReplay * as_release_replay(void)
-        { return this; }
-    private:
-      friend class PhysicalTemplate;
-      const std::map<Reservation,bool> reservations;
-      unsigned rhs;
-    };
-
   }; // namespace Internal
 }; // namespace Legion
 
