@@ -1,4 +1,4 @@
-/* Copyright 2021 Stanford University, NVIDIA Corporation
+/* Copyright 2023 Stanford University, NVIDIA Corporation
  *
  * Licensed under the Apache License, Version 2.0 (the "License");
  * you may not use this file except in compliance with the License.
@@ -105,8 +105,14 @@ namespace Legion {
       template<unsigned int MAX>
       inline void serialize(const PPCTLBitMask<MAX> &mask);
 #endif
-      template<typename IT, typename DT, bool BIDIR>
-      inline void serialize(const IntegerSet<IT,DT,BIDIR> &integer_set);
+#ifdef __ARM_NEON
+      template<unsigned int MAX>
+      inline void serialize(const NeonBitMask<MAX> &mask);
+      template<unsigned int MAX>
+      inline void serialize(const NeonTLBitMask<MAX> &mask);
+#endif
+      template<typename DT, unsigned BLOAT, bool BIDIR>
+      inline void serialize(const CompoundBitMask<DT,BLOAT,BIDIR> &mask);
       inline void serialize(const Domain &domain);
       inline void serialize(const DomainPoint &dp);
       inline void serialize(const void *src, size_t bytes);
@@ -184,8 +190,14 @@ namespace Legion {
       template<unsigned int MAX>
       inline void deserialize(PPCTLBitMask<MAX> &mask);
 #endif
-      template<typename IT, typename DT, bool BIDIR>
-      inline void deserialize(IntegerSet<IT,DT,BIDIR> &integer_set);
+#ifdef __ARM_NEON
+      template<unsigned int MAX>
+      inline void deserialize(NeonBitMask<MAX> &mask);
+      template<unsigned int MAX>
+      inline void deserialize(NeonTLBitMask<MAX> &mask);
+#endif
+      template<typename DT, unsigned BLOAT, bool BIDIR>
+      inline void deserialize(CompoundBitMask<DT,BLOAT,BIDIR> &mask);
       inline void deserialize(Domain &domain);
       inline void deserialize(DomainPoint &dp);
       inline void deserialize(void *dst, size_t bytes);
@@ -498,22 +510,23 @@ namespace Legion {
         : DynamicTableNodeBase<IT>(_level, _first_index, _last_index) 
       { 
         for (size_t i = 0; i < SIZE; i++)
-          elems[i] = 0;
+          elems[i].store(NULL);
       }
       DynamicTableNode(const DynamicTableNode &rhs) { assert(false); }
       virtual ~DynamicTableNode(void)
       {
         for (size_t i = 0; i < SIZE; i++)
         {
-          if (elems[i] != 0)
-            delete elems[i];
+          ET *elem = elems[i].load();
+          if (elem != NULL)
+            delete elem;
         }
       }
     public:
       DynamicTableNode& operator=(const DynamicTableNode &rhs)
         { assert(false); return *this; }
     public:
-      ET*volatile elems[SIZE];
+      std::atomic<ET*> elems[SIZE];
     };
 
     template<typename ET, size_t _SIZE, typename IT>
@@ -525,24 +538,23 @@ namespace Legion {
         : DynamicTableNodeBase<IT>(_level, _first_index, _last_index) 
       { 
         for (size_t i = 0; i < SIZE; i++)
-          elems[i] = 0;
+          elems[i].store(NULL);
       }
       LeafTableNode(const LeafTableNode &rhs) { assert(false); }
       virtual ~LeafTableNode(void)
       {
         for (size_t i = 0; i < SIZE; i++)
         {
-          if (elems[i] != 0)
-          {
-            delete elems[i];
-          }
+          ET *elem = elems[i].load();
+          if (elem != NULL)
+            delete elem;
         }
       }
     public:
       LeafTableNode& operator=(const LeafTableNode &rhs)
         { assert(false); return *this; }
     public:
-      ET*volatile elems[SIZE];
+      std::atomic<ET*> elems[SIZE];
     };
 
     template<typename ALLOCATOR>
@@ -569,7 +581,7 @@ namespace Legion {
       NodeBase* new_tree_node(int level, IT first_index, IT last_index);
       NodeBase* lookup_leaf(IT index);
     protected:
-      NodeBase *volatile root;
+      std::atomic<NodeBase*> root;
       mutable LocalLock lock; 
     };
 
@@ -629,14 +641,8 @@ namespace Legion {
     inline void Serializer::serialize<bool>(const bool &element)
     //--------------------------------------------------------------------------
     {
-      static_assert(sizeof(bool) <= 4, "huge bool");
-      while ((index + 4) > total_bytes)
-        resize();
-      memcpy(buffer+index, (const void*)&element, sizeof(bool));
-      index += 4;
-#ifdef DEBUG_LEGION
-      context_bytes += 4;
-#endif
+      const uint32_t flag = element ? 1 : 0;
+      serialize<uint32_t>(flag);
     }
 
     //--------------------------------------------------------------------------
@@ -709,12 +715,30 @@ namespace Legion {
     }
 #endif
 
+#ifdef __ARM_NEON
     //--------------------------------------------------------------------------
-    template<typename IT, typename DT, bool BIDIR>
-    inline void Serializer::serialize(const IntegerSet<IT,DT,BIDIR> &int_set)
+    template<unsigned int MAX>
+    inline void Serializer::serialize(const NeonBitMask<MAX> &mask)
     //--------------------------------------------------------------------------
     {
-      int_set.serialize(*this);
+      mask.serialize(*this);
+    }
+
+    //--------------------------------------------------------------------------
+    template<unsigned int MAX>
+    inline void Serializer::serialize(const NeonTLBitMask<MAX> &mask)
+    //--------------------------------------------------------------------------
+    {
+      mask.serialize(*this);
+    }
+#endif
+
+    //--------------------------------------------------------------------------
+    template<typename DT, unsigned BLOAT, bool BIDIR>
+    inline void Serializer::serialize(const CompoundBitMask<DT,BLOAT,BIDIR> &m)
+    //--------------------------------------------------------------------------
+    {
+      m.serialize(*this);
     }
 
     //--------------------------------------------------------------------------
@@ -722,6 +746,8 @@ namespace Legion {
     //--------------------------------------------------------------------------
     {
       serialize(dom.is_id);
+      if (dom.is_id > 0)
+        serialize(dom.is_type);
       serialize(dom.dim);
       for (int i = 0; i < 2*dom.dim; i++)
         serialize(dom.rect_data[i]);
@@ -855,16 +881,9 @@ namespace Legion {
     inline void Deserializer::deserialize<bool>(bool &element)
     //--------------------------------------------------------------------------
     {
-      static_assert(sizeof(bool) <= 4, "huge bool");
-#ifdef DEBUG_LEGION
-      // Check to make sure we don't read past the end
-      assert((index+4) <= total_bytes);
-#endif
-      memcpy(&element, buffer+index, sizeof(bool));
-      index += 4;
-#ifdef DEBUG_LEGION
-      context_bytes += 4;
-#endif
+      uint32_t flag;
+      deserialize<uint32_t>(flag);
+      element = (flag != 0);
     }
 
     //--------------------------------------------------------------------------
@@ -937,12 +956,30 @@ namespace Legion {
     }
 #endif
 
+#ifdef __ARM_NEON
     //--------------------------------------------------------------------------
-    template<typename IT, typename DT, bool BIDIR>
-    inline void Deserializer::deserialize(IntegerSet<IT,DT,BIDIR> &int_set)
+    template<unsigned int MAX>
+    inline void Deserializer::deserialize(NeonBitMask<MAX> &mask)
     //--------------------------------------------------------------------------
     {
-      int_set.deserialize(*this);
+      mask.deserialize(*this);
+    }
+
+    //--------------------------------------------------------------------------
+    template<unsigned int MAX>
+    inline void Deserializer::deserialize(NeonTLBitMask<MAX> &mask)
+    //--------------------------------------------------------------------------
+    {
+      mask.deserialize(*this);
+    }
+#endif
+
+    //--------------------------------------------------------------------------
+    template<typename DT, unsigned BLOAT, bool BIDIR>
+    inline void Deserializer::deserialize(CompoundBitMask<DT,BLOAT,BIDIR> &mask)
+    //--------------------------------------------------------------------------
+    {
+      mask.deserialize(*this);
     }
 
     //--------------------------------------------------------------------------
@@ -950,6 +987,8 @@ namespace Legion {
     //--------------------------------------------------------------------------
     {
       deserialize(dom.is_id);
+      if (dom.is_id > 0)
+        deserialize(dom.is_type);
       deserialize(dom.dim);
       for (int i = 0; i < 2*dom.dim; i++)
         deserialize(dom.rect_data[i]);
@@ -1529,9 +1568,9 @@ namespace Legion {
     //-------------------------------------------------------------------------
     template<typename ALLOCATOR>
     DynamicTable<ALLOCATOR>::DynamicTable(void)
-      : root(0)
     //-------------------------------------------------------------------------
     {
+      root.store(NULL);
     }
 
     //-------------------------------------------------------------------------
@@ -1548,10 +1587,11 @@ namespace Legion {
     DynamicTable<ALLOCATOR>::~DynamicTable(void)
     //-------------------------------------------------------------------------
     {
-      if (root != 0)
+      NodeBase *r = root.load();
+      if (r != NULL)
       {
-        delete root;
-        root = NULL;
+        delete r;
+        root.store(NULL);
       }
     }
 
@@ -1588,10 +1628,10 @@ namespace Legion {
     size_t DynamicTable<ALLOCATOR>::max_entries(void) const
     //-------------------------------------------------------------------------
     {
-      if (!root)
+      if (root.load() == NULL)
         return 0;
       size_t elems_addressable = 1 << ALLOCATOR::LEAF_BITS;
-      for (int i = 0; i < root->level; i++)
+      for (int i = 0; i < root.load()->level; i++)
         elems_addressable <<= ALLOCATOR::INNER_BITS;
       return elems_addressable;
     }
@@ -1609,7 +1649,7 @@ namespace Legion {
 	elems_addressable <<= ALLOCATOR::INNER_BITS;
       }
 
-      NodeBase *n = root;
+      NodeBase *n = root.load();
       if (!n || (n->level < level_needed))
         return false;
 
@@ -1630,7 +1670,7 @@ namespace Legion {
 #ifdef DEBUG_LEGION
         assert((i >= 0) && (((size_t)i) < ALLOCATOR::INNER_TYPE::SIZE));
 #endif
-        NodeBase *child = inner->elems[i];
+        NodeBase *child = inner->elems[i].load();
         if (child == 0)
           return false;
 #ifdef DEBUG_LEGION
@@ -1655,22 +1695,20 @@ namespace Legion {
       typename ALLOCATOR::LEAF_TYPE *leaf = 
         static_cast<typename ALLOCATOR::LEAF_TYPE*>(n);
       int offset = (index & ((((IT)1) << ALLOCATOR::LEAF_BITS) - 1));
-      ET *result = leaf->elems[offset];
-      if (result == 0)
+      ET *result = leaf->elems[offset].load();
+      if (result == NULL)
       {
         AutoLock l(leaf->lock);
         // Now that we have the lock, check to see if we lost the race
-        if (leaf->elems[offset] == 0)
+        if (leaf->elems[offset].load() == NULL)
         {
           ET *elem = new ET();
-          // Enforce total store ordering
-          __sync_synchronize();
-          leaf->elems[offset] = elem;
+          leaf->elems[offset].store(elem);
         }
-        result = leaf->elems[offset];
+        result = leaf->elems[offset].load();
       }
 #ifdef DEBUG_LEGION
-      assert(result != 0);
+      assert(result != NULL);
 #endif
       return result;
     }
@@ -1686,22 +1724,20 @@ namespace Legion {
       typename ALLOCATOR::LEAF_TYPE *leaf = 
         static_cast<typename ALLOCATOR::LEAF_TYPE*>(n);
       int offset = (index & ((((IT)1) << ALLOCATOR::LEAF_BITS) - 1));
-      ET *result = leaf->elems[offset];
-      if (result == 0)
+      ET *result = leaf->elems[offset].load();
+      if (result == NULL)
       {
         AutoLock l(leaf->lock);
         // Now that we have the lock, check to see if we lost the race
-        if (leaf->elems[offset] == 0)
+        if (leaf->elems[offset].load() == NULL)
         {
           ET *elem = new ET(arg);
-          // Enforce total store ordering
-          __sync_synchronize();
-          leaf->elems[offset] = elem;
+          leaf->elems[offset].store(elem);
         }
-        result = leaf->elems[offset];
+        result = leaf->elems[offset].load();
       }
 #ifdef DEBUG_LEGION
-      assert(result != 0);
+      assert(result != NULL);
 #endif
       return result;
     }
@@ -1718,22 +1754,20 @@ namespace Legion {
       typename ALLOCATOR::LEAF_TYPE *leaf = 
         static_cast<typename ALLOCATOR::LEAF_TYPE*>(n);
       int offset = (index & ((((IT)1) << ALLOCATOR::LEAF_BITS) - 1));
-      ET *result = leaf->elems[offset];
-      if (result == 0)
+      ET *result = leaf->elems[offset].load();
+      if (result == NULL)
       {
         AutoLock l(leaf->lock);
         // Now that we have the lock, check to see if we lost the race
-        if (leaf->elems[offset] == 0)
+        if (leaf->elems[offset].load() == NULL)
         {
           ET *elem = new ET(arg1, arg2);
-          // Enforce total store ordering
-          __sync_synchronize();
-          leaf->elems[offset] = elem;
+          leaf->elems[offset].store(elem);
         }
-        result = leaf->elems[offset];
+        result = leaf->elems[offset].load();
       }
 #ifdef DEBUG_LEGION
-      assert(result != 0);
+      assert(result != NULL);
 #endif
       return result;
     }
@@ -1755,31 +1789,31 @@ namespace Legion {
 
       // In most cases we won't need to add levels to the tree, but
       // if we do, then do it now
-      NodeBase *n = root;
+      NodeBase *n = root.load();
       if (!n || (n->level < level_needed)) 
       {
         AutoLock l(lock); 
-        if (root)
+        n = root.load();
+        if (n != NULL)
         {
           // some of the tree exists - add new layers on top
-          while (root->level < level_needed)
+          while (n->level < level_needed)
           {
-            int parent_level = root->level + 1;
+            int parent_level = n->level + 1;
             IT parent_first = 0;
             IT parent_last = 
-              (((root->last_index + 1) << ALLOCATOR::INNER_BITS) - 1);
+              (((n->last_index + 1) << ALLOCATOR::INNER_BITS) - 1);
             NodeBase *parent = new_tree_node(parent_level, 
                                              parent_first, parent_last);
             typename ALLOCATOR::INNER_TYPE *inner = 
               static_cast<typename ALLOCATOR::INNER_TYPE*>(parent);
-            __sync_synchronize();
-            inner->elems[0] = root;
-            root = parent;
+            inner->elems[0].store(n);
+            n = parent;
           }
         }
         else
-          root = new_tree_node(level_needed, 0, elems_addressable - 1);
-        n = root;
+          n = new_tree_node(level_needed, 0, elems_addressable - 1);
+        root.store(n);
       }
       // root should be high-enough now
 #ifdef DEBUG_LEGION
@@ -1799,12 +1833,12 @@ namespace Legion {
 #ifdef DEBUG_LEGION
         assert((i >= 0) && (((size_t)i) < ALLOCATOR::INNER_TYPE::SIZE));
 #endif
-        NodeBase *child = inner->elems[i];
-        if (child == 0)
+        NodeBase *child = inner->elems[i].load();
+        if (child == NULL)
         {
           AutoLock l(inner->lock);
           // Now that the lock is held, check to see if we lost the race
-          if (inner->elems[i] == 0)
+          if (inner->elems[i].load() == NULL)
           {
             int child_level = inner->level - 1;
             int child_shift = 
@@ -1814,13 +1848,12 @@ namespace Legion {
 
             NodeBase *next = new_tree_node(child_level, 
                                            child_first, child_last);
-            __sync_synchronize();
-            inner->elems[i] = next;
+            inner->elems[i].store(next);
           }
-          child = inner->elems[i];
+          child = inner->elems[i].load();
         }
 #ifdef DEBUG_LEGION
-        assert((child != 0) &&
+        assert((child != NULL) &&
                (child->level == (n->level - 1)) &&
                (index >= child->first_index) &&
                (index <= child->last_index));
@@ -1852,8 +1885,8 @@ namespace Legion {
     //--------------------------------------------------------------------------
     template<typename T>
     inline void compute_field_sets(FieldMask universe_mask,
-                        const typename LegionMap<T,FieldMask>::aligned &inputs,
-                        typename LegionList<FieldSet<T> >::aligned &output_sets)
+                                   const LegionMap<T,FieldMask> &inputs,
+                                   LegionList<FieldSet<T> > &output_sets)
     //--------------------------------------------------------------------------
     {
       // Special cases for empty and size 1 sets
@@ -1865,7 +1898,7 @@ namespace Legion {
       }
       else if (inputs.size() == 1)
       {
-        typename LegionMap<T,FieldMask>::aligned::const_iterator first = 
+        typename LegionMap<T,FieldMask>::const_iterator first = 
           inputs.begin();
         output_sets.push_back(FieldSet<T>(first->second));
         FieldSet<T> &last = output_sets.back();
@@ -1878,7 +1911,7 @@ namespace Legion {
         }
         return;
       }
-      for (typename LegionMap<T,FieldMask>::aligned::const_iterator pit = 
+      for (typename LegionMap<T,FieldMask>::const_iterator pit = 
             inputs.begin(); pit != inputs.end(); pit++)
       {
         bool inserted = false;
@@ -1888,7 +1921,7 @@ namespace Legion {
           universe_mask -= pit->second;
         FieldMask remaining = pit->second;
         // Insert this event into the precondition sets 
-        for (typename LegionList<FieldSet<T> >::aligned::iterator it = 
+        for (typename LegionList<FieldSet<T> >::iterator it = 
               output_sets.begin(); it != output_sets.end(); it++)
         {
           // Easy case, check for equality
@@ -1957,20 +1990,26 @@ namespace Legion {
      * A template helper class for tracking collections of 
      * objects associated with different sets of fields
      */
-    template<typename T>
+    template<typename T, AllocationType A = UNTRACKED_ALLOC>
     class FieldMaskSet : 
       public LegionHeapify<FieldMaskSet<T> > {
     public:
       // forward declaration
       class const_iterator;
-      class iterator : public std::iterator<std::input_iterator_tag,
-                              std::pair<T*const,FieldMask> > {
+      class iterator {
       public:
+        // explicitly set iterator traits
+        typedef std::input_iterator_tag iterator_category;
+        typedef std::pair<T*const,FieldMask> value_type;
+        typedef std::ptrdiff_t difference_type;
+        typedef std::pair<T*const,FieldMask> *pointer;
+        typedef std::pair<T*const,FieldMask>& reference;
+
         iterator(FieldMaskSet *_set, 
             std::pair<T*const,FieldMask> *_result)
           : set(_set), result(_result), single(true) { }
         iterator(FieldMaskSet *_set,
-            typename LegionMap<T*,FieldMask>::aligned::iterator _it,
+            typename LegionMap<T*,FieldMask,A>::iterator _it,
             bool end = false)
           : set(_set), result(end ? NULL : &(*_it)), it(_it), single(false) { }
       public:
@@ -2055,7 +2094,7 @@ namespace Legion {
             result->second.clear();
           }
       public:
-        inline void erase(typename LegionMap<T*,FieldMask>::aligned &target)
+        inline void erase(LegionMap<T*,FieldMask,A> &target)
         {
 #ifdef DEBUG_LEGION
           assert(!single);
@@ -2070,18 +2109,24 @@ namespace Legion {
         friend class const_iterator;
         FieldMaskSet *set;
         std::pair<T*const,FieldMask> *result;
-        typename LegionMap<T*,FieldMask>::aligned::iterator it;
+        typename LegionMap<T*,FieldMask,A>::iterator it;
         bool single;
       };
     public:
-      class const_iterator : public std::iterator<std::input_iterator_tag,
-                              std::pair<T*const,FieldMask> > {
+      class const_iterator {
       public:
+        // explicitly set iterator traits
+        typedef std::input_iterator_tag iterator_category;
+        typedef std::pair<T*const,FieldMask> value_type;
+        typedef std::ptrdiff_t difference_type;
+        typedef std::pair<T*const,FieldMask> *pointer;
+        typedef std::pair<T*const,FieldMask>& reference;
+
         const_iterator(const FieldMaskSet *_set, 
             const std::pair<T*const,FieldMask> *_result)
           : set(_set), result(_result), single(true) { }
         const_iterator(const FieldMaskSet *_set,
-            typename LegionMap<T*,FieldMask>::aligned::const_iterator _it,
+            typename LegionMap<T*,FieldMask,A>::const_iterator _it,
             bool end = false)
           : set(_set), result(end ? NULL : &(*_it)), it(_it), single(false) { }
       public:
@@ -2157,7 +2202,7 @@ namespace Legion {
       private:
         const FieldMaskSet *set;
         const std::pair<T*const,FieldMask> *result;
-        typename LegionMap<T*,FieldMask>::aligned::const_iterator it;
+        typename LegionMap<T*,FieldMask,A>::const_iterator it;
         bool single;
       };
     public:
@@ -2198,7 +2243,7 @@ namespace Legion {
       inline const_iterator end(void) const;
     public:
       inline void compute_field_sets(FieldMask universe_mask,
-          typename LegionList<FieldSet<T*> >::aligned &output_sets) const;
+                    LegionList<FieldSet<T*> > &output_sets) const;
     protected:
       // Fun with C, keep these two fields first and in this order
       // so that a FieldMaskSet of size 1 looks the same as an entry
@@ -2206,7 +2251,7 @@ namespace Legion {
       // provides goodness for the iterator
       union {
         T *single_entry;
-        typename LegionMap<T*,FieldMask>::aligned *multi_entries;
+        LegionMap<T*,FieldMask,A> *multi_entries;
       } entries;
       // This can be an overapproximation if we have multiple entries
       FieldMask valid_fields;
@@ -2214,23 +2259,23 @@ namespace Legion {
     };
 
     //--------------------------------------------------------------------------
-    template<typename T>
-    inline FieldMaskSet<T>::FieldMaskSet(const FieldMaskSet<T> &rhs)
+    template<typename T, AllocationType A>
+    inline FieldMaskSet<T,A>::FieldMaskSet(const FieldMaskSet<T,A> &rhs)
       : valid_fields(rhs.valid_fields), single(rhs.single)
     //--------------------------------------------------------------------------
     {
       if (single)
         entries.single_entry = rhs.entries.single_entry;
       else
-        entries.multi_entries = new typename LegionMap<T*,FieldMask>::aligned(
+        entries.multi_entries = new LegionMap<T*,FieldMask,A>(
             rhs.entries.multi_entries->begin(),
             rhs.entries.multi_entries->end());
     }
 
     //--------------------------------------------------------------------------
-    template<typename T>
-    inline FieldMaskSet<T>& FieldMaskSet<T>::operator=(
-                                                     const FieldMaskSet<T> &rhs)
+    template<typename T, AllocationType A>
+    inline FieldMaskSet<T,A>& FieldMaskSet<T,A>::operator=(
+                                                   const FieldMaskSet<T,A> &rhs)
     //--------------------------------------------------------------------------
     {
       // Check our current state
@@ -2239,7 +2284,7 @@ namespace Legion {
         // Different data structures
         if (single)
         {
-          entries.multi_entries = new typename LegionMap<T*,FieldMask>::aligned(
+          entries.multi_entries = new LegionMap<T*,FieldMask,A>(
               rhs.entries.multi_entries->begin(),
               rhs.entries.multi_entries->end());
         }
@@ -2269,15 +2314,15 @@ namespace Legion {
     }
 
     //--------------------------------------------------------------------------
-    template<typename T>
-    inline const FieldMask& FieldMaskSet<T>::tighten_valid_mask(void)
+    template<typename T, AllocationType A>
+    inline const FieldMask& FieldMaskSet<T,A>::tighten_valid_mask(void)
     //--------------------------------------------------------------------------
     {
       // If we're single then there is nothing to do as we're already tight
       if (single)
         return valid_fields;
       valid_fields.clear();
-      for (typename LegionMap<T*,FieldMask>::aligned::const_iterator it = 
+      for (typename LegionMap<T*,FieldMask,A>::const_iterator it = 
             entries.multi_entries->begin(); it !=
             entries.multi_entries->end(); it++)
         valid_fields |= it->second;
@@ -2285,32 +2330,32 @@ namespace Legion {
     }
 
     //--------------------------------------------------------------------------
-    template<typename T>
-    inline void FieldMaskSet<T>::relax_valid_mask(const FieldMask &m)
+    template<typename T, AllocationType A>
+    inline void FieldMaskSet<T,A>::relax_valid_mask(const FieldMask &m)
     //--------------------------------------------------------------------------
     {
       valid_fields |= m;
     }
 
     //--------------------------------------------------------------------------
-    template<typename T>
-    inline void FieldMaskSet<T>::filter_valid_mask(const FieldMask &m)
+    template<typename T, AllocationType A>
+    inline void FieldMaskSet<T,A>::filter_valid_mask(const FieldMask &m)
     //--------------------------------------------------------------------------
     {
       valid_fields -= m;
     }
 
     //--------------------------------------------------------------------------
-    template<typename T>
-    inline void FieldMaskSet<T>::restrict_valid_mask(const FieldMask &m)
+    template<typename T, AllocationType A>
+    inline void FieldMaskSet<T,A>::restrict_valid_mask(const FieldMask &m)
     //--------------------------------------------------------------------------
     {
       valid_fields &= m;
     }
 
     //--------------------------------------------------------------------------
-    template<typename T>
-    inline const FieldMask& FieldMaskSet<T>::operator[](T *entry) const
+    template<typename T, AllocationType A>
+    inline const FieldMask& FieldMaskSet<T,A>::operator[](T *entry) const
     //--------------------------------------------------------------------------
     {
       if (single)
@@ -2322,7 +2367,7 @@ namespace Legion {
       }
       else
       {
-        typename LegionMap<T*,FieldMask>::aligned::const_iterator finder =
+        typename LegionMap<T*,FieldMask,A>::const_iterator finder =
           entries.multi_entries->find(entry);
 #ifdef DEBUG_LEGION
         assert(finder != entries.multi_entries->end());
@@ -2332,8 +2377,8 @@ namespace Legion {
     }
 
     //--------------------------------------------------------------------------
-    template<typename T>
-    inline bool FieldMaskSet<T>::insert(T *entry, const FieldMask &mask)
+    template<typename T, AllocationType A>
+    inline bool FieldMaskSet<T,A>::insert(T *entry, const FieldMask &mask)
     //--------------------------------------------------------------------------
     {
       bool result = true;
@@ -2352,8 +2397,7 @@ namespace Legion {
         else
         {
           // Go to multi
-          typename LegionMap<T*,FieldMask>::aligned *multi = 
-            new typename LegionMap<T*,FieldMask>::aligned();
+          LegionMap<T*,FieldMask,A> *multi = new LegionMap<T*,FieldMask,A>();
           (*multi)[entries.single_entry] = valid_fields;
           (*multi)[entry] = mask;
           entries.multi_entries = multi;
@@ -2366,7 +2410,7 @@ namespace Legion {
  #ifdef DEBUG_LEGION
         assert(entries.multi_entries != NULL);
 #endif   
-        typename LegionMap<T*,FieldMask>::aligned::iterator finder = 
+        typename LegionMap<T*,FieldMask,A>::iterator finder = 
           entries.multi_entries->find(entry);
         if (finder == entries.multi_entries->end())
           (*entries.multi_entries)[entry] = mask;
@@ -2381,8 +2425,8 @@ namespace Legion {
     }
 
     //--------------------------------------------------------------------------
-    template<typename T>
-    inline void FieldMaskSet<T>::filter(const FieldMask &filter)
+    template<typename T, AllocationType A>
+    inline void FieldMaskSet<T,A>::filter(const FieldMask &filter)
     //--------------------------------------------------------------------------
     {
       if (single)
@@ -2408,7 +2452,7 @@ namespace Legion {
         {
           // Manually remove entries
           typename std::vector<T*> to_delete;
-          for (typename LegionMap<T*,FieldMask>::aligned::iterator it = 
+          for (typename LegionMap<T*,FieldMask,A>::iterator it = 
                 entries.multi_entries->begin(); it !=
                 entries.multi_entries->end(); it++)
           {
@@ -2429,7 +2473,7 @@ namespace Legion {
             }
             else if (entries.multi_entries->size() == 1)
             {
-              typename LegionMap<T*,FieldMask>::aligned::iterator last = 
+              typename LegionMap<T*,FieldMask,A>::iterator last = 
                 entries.multi_entries->begin();     
               T *temp = last->first; 
               valid_fields = last->second;
@@ -2443,8 +2487,8 @@ namespace Legion {
     }
 
     //--------------------------------------------------------------------------
-    template<typename T>
-    inline void FieldMaskSet<T>::erase(T *to_erase)
+    template<typename T, AllocationType A>
+    inline void FieldMaskSet<T,A>::erase(T *to_erase)
     //--------------------------------------------------------------------------
     {
       if (single)
@@ -2457,7 +2501,7 @@ namespace Legion {
       }
       else
       {
-        typename LegionMap<T*,FieldMask>::aligned::iterator finder = 
+        typename LegionMap<T*,FieldMask,A>::iterator finder = 
           entries.multi_entries->find(to_erase);
 #ifdef DEBUG_LEGION
         assert(finder != entries.multi_entries->end());
@@ -2477,8 +2521,8 @@ namespace Legion {
     }
 
     //--------------------------------------------------------------------------
-    template<typename T>
-    inline void FieldMaskSet<T>::clear(void)
+    template<typename T, AllocationType A>
+    inline void FieldMaskSet<T,A>::clear(void)
     //--------------------------------------------------------------------------
     {
       if (single)
@@ -2496,8 +2540,8 @@ namespace Legion {
     }
 
     //--------------------------------------------------------------------------
-    template<typename T>
-    inline size_t FieldMaskSet<T>::size(void) const
+    template<typename T, AllocationType A>
+    inline size_t FieldMaskSet<T,A>::size(void) const
     //--------------------------------------------------------------------------
     {
       if (single)
@@ -2512,8 +2556,8 @@ namespace Legion {
     }
 
     //--------------------------------------------------------------------------
-    template<typename T>
-    inline void FieldMaskSet<T>::swap(FieldMaskSet &other)
+    template<typename T, AllocationType A>
+    inline void FieldMaskSet<T,A>::swap(FieldMaskSet &other)
     //--------------------------------------------------------------------------
     {
       // Just use single, doesn't matter for swap
@@ -2531,8 +2575,8 @@ namespace Legion {
     }
 
     //--------------------------------------------------------------------------
-    template<typename T>
-    inline typename FieldMaskSet<T>::iterator FieldMaskSet<T>::begin(void)
+    template<typename T, AllocationType A>
+    inline typename FieldMaskSet<T,A>::iterator FieldMaskSet<T,A>::begin(void)
     //--------------------------------------------------------------------------
     {
       // Scariness!
@@ -2552,8 +2596,8 @@ namespace Legion {
     }
 
     //--------------------------------------------------------------------------
-    template<typename T>
-    inline typename FieldMaskSet<T>::iterator FieldMaskSet<T>::find(T *e)
+    template<typename T, AllocationType A>
+    inline typename FieldMaskSet<T,A>::iterator FieldMaskSet<T,A>::find(T *e)
     //--------------------------------------------------------------------------
     {
       if (single)
@@ -2568,7 +2612,7 @@ namespace Legion {
       }
       else
       {
-        typename LegionMap<T*,FieldMask>::aligned::iterator finder = 
+        typename LegionMap<T*,FieldMask,A>::iterator finder = 
           entries.multi_entries->find(e);
         if (finder == entries.multi_entries->end())
           return end();
@@ -2577,8 +2621,8 @@ namespace Legion {
     }
 
     //--------------------------------------------------------------------------
-    template<typename T>
-    inline void FieldMaskSet<T>::erase(iterator &it)
+    template<typename T, AllocationType A>
+    inline void FieldMaskSet<T,A>::erase(iterator &it)
     //--------------------------------------------------------------------------
     {
 #ifdef DEBUG_LEGION
@@ -2598,7 +2642,7 @@ namespace Legion {
         if (entries.multi_entries->size() == 1)
         {
           // go back to single
-          typename LegionMap<T*,FieldMask>::aligned::iterator finder = 
+          typename LegionMap<T*,FieldMask,A>::iterator finder = 
             entries.multi_entries->begin();
           valid_fields = finder->second;
           T *first = finder->first;
@@ -2610,8 +2654,8 @@ namespace Legion {
     }
 
     //--------------------------------------------------------------------------
-    template<typename T>
-    inline typename FieldMaskSet<T>::iterator FieldMaskSet<T>::end(void)
+    template<typename T, AllocationType A>
+    inline typename FieldMaskSet<T,A>::iterator FieldMaskSet<T,A>::end(void)
     //--------------------------------------------------------------------------
     {
       if (single)
@@ -2621,9 +2665,9 @@ namespace Legion {
     }
 
     //--------------------------------------------------------------------------
-    template<typename T>
-    inline typename FieldMaskSet<T>::const_iterator 
-                                              FieldMaskSet<T>::begin(void) const
+    template<typename T, AllocationType A>
+    inline typename FieldMaskSet<T,A>::const_iterator 
+                                            FieldMaskSet<T,A>::begin(void) const
     //--------------------------------------------------------------------------
     {
       // Scariness!
@@ -2643,9 +2687,9 @@ namespace Legion {
     }
 
     //--------------------------------------------------------------------------
-    template<typename T>
-    inline typename FieldMaskSet<T>::const_iterator 
-                                               FieldMaskSet<T>::find(T *e) const
+    template<typename T, AllocationType A>
+    inline typename FieldMaskSet<T,A>::const_iterator 
+                                             FieldMaskSet<T,A>::find(T *e) const
     //--------------------------------------------------------------------------
     {
       if (single)
@@ -2660,7 +2704,7 @@ namespace Legion {
       }
       else
       {
-        typename LegionMap<T*,FieldMask>::aligned::const_iterator finder = 
+        typename LegionMap<T*,FieldMask,A>::const_iterator finder = 
           entries.multi_entries->find(e);
         if (finder == entries.multi_entries->end())
           return end();
@@ -2669,9 +2713,9 @@ namespace Legion {
     }
 
     //--------------------------------------------------------------------------
-    template<typename T>
-    inline typename FieldMaskSet<T>::const_iterator 
-                                                FieldMaskSet<T>::end(void) const
+    template<typename T, AllocationType A>
+    inline typename FieldMaskSet<T,A>::const_iterator 
+                                              FieldMaskSet<T,A>::end(void) const
     //--------------------------------------------------------------------------
     {
       if (single)
@@ -2681,9 +2725,9 @@ namespace Legion {
     }
 
     //--------------------------------------------------------------------------
-    template<typename T>
-    inline void FieldMaskSet<T>::compute_field_sets(FieldMask universe_mask,
-                 typename LegionList<FieldSet<T*> >::aligned &output_sets) const
+    template<typename T, AllocationType A>
+    inline void FieldMaskSet<T,A>::compute_field_sets(FieldMask universe_mask,
+                                   LegionList<FieldSet<T*> > &output_sets) const
     //--------------------------------------------------------------------------
     {
       // Handle special cases for single entry and single fields
@@ -2744,7 +2788,7 @@ namespace Legion {
           universe_mask -= pit->second;
         FieldMask remaining = pit->second;
         // Insert this event into the precondition sets 
-        for (typename LegionList<FieldSet<T*> >::aligned::iterator it = 
+        for (typename LegionList<FieldSet<T*> >::iterator it = 
               output_sets.begin(); it != output_sets.end(); it++)
         {
           // Easy case, check for equality
