@@ -1,4 +1,4 @@
-/* Copyright 2021 Stanford University, NVIDIA Corporation
+/* Copyright 2023 Stanford University, NVIDIA Corporation
  *
  * Licensed under the Apache License, Version 2.0 (the "License");
  * you may not use this file except in compliance with the License.
@@ -107,9 +107,8 @@ namespace Legion {
     class InstanceView : public LogicalView {
     public:
       typedef LegionMap<ApEvent,
-                FieldMaskSet<IndexSpaceExpression> >::aligned EventFieldExprs; 
-      typedef LegionMap<ApEvent,FieldMaskSet<PhysicalUser> >::aligned 
-                                                              EventFieldUsers;
+                FieldMaskSet<IndexSpaceExpression> > EventFieldExprs; 
+      typedef LegionMap<ApEvent,FieldMaskSet<PhysicalUser> > EventFieldUsers;
       typedef FieldMaskSet<PhysicalUser> EventUsers;
     public:
       struct DeferFindCopyPreconditionArgs : 
@@ -225,13 +224,12 @@ namespace Legion {
     public:
       void find_atomic_reservations(const FieldMask &mask, Operation *op, 
                                     const unsigned index, bool exclusive);
-      void find_field_reservations(const std::vector<FieldID> &needed_fields,
+      void find_field_reservations(const FieldMask &mask,
                                    std::vector<Reservation> &results);
       static void handle_send_atomic_reservation_request(Runtime *runtime,
                                   Deserializer &derez, AddressSpaceID source);
-      void update_field_reservations(
-                            const std::vector<FieldID> &fields,
-                            const std::vector<Reservation> &reservations);
+      void update_field_reservations(const FieldMask &mask,
+                                  const std::vector<Reservation> &reservations);
       static void handle_send_atomic_reservation_response(Runtime *runtime,
                                                           Deserializer &derez);
     public:
@@ -264,7 +262,7 @@ namespace Legion {
       // Keep track of the locks used for managing atomic coherence
       // on individual fields of this materialized view. Only the
       // top-level view for an instance needs to track this.
-      std::map<FieldID,Reservation> atomic_reservations;
+      std::map<unsigned,Reservation> atomic_reservations;
     };
 
     /**
@@ -299,9 +297,8 @@ namespace Legion {
                      public CollectableView, public Collectable {
     public:
       typedef LegionMap<ApEvent,
-                FieldMaskSet<IndexSpaceExpression> >::aligned EventFieldExprs; 
-      typedef LegionMap<ApEvent,FieldMaskSet<PhysicalUser> >::aligned 
-                                                              EventFieldUsers;
+                FieldMaskSet<IndexSpaceExpression> > EventFieldExprs; 
+      typedef LegionMap<ApEvent,FieldMaskSet<PhysicalUser> > EventFieldUsers;
       typedef FieldMaskSet<PhysicalUser> EventUsers;
     public:
       ExprView(RegionTreeForest *ctx, PhysicalManager *manager,
@@ -338,8 +335,7 @@ namespace Legion {
       void find_tightest_subviews(IndexSpaceExpression *expr,
                                   FieldMask &expr_mask,
                                   LegionMap<std::pair<size_t,
-                                    ExprView*>,FieldMask>::aligned 
-                                    &bounding_views);
+                                    ExprView*>,FieldMask> &bounding_views);
       void add_partial_user(const RegionUsage &usage,
                             UniqueID op_id, unsigned index,
                             FieldMask user_mask,
@@ -443,6 +439,9 @@ namespace Legion {
       InstanceView *const inst_view;
       IndexSpaceExpression *const view_expr;
       const size_t view_volume;
+#if defined(DEBUG_LEGION_GC) || defined(LEGION_GC)
+      const DistributedID view_did;
+#endif
       // This is publicly mutable and protected by expr_lock from
       // the owner inst_view
       FieldMask invalid_fields;
@@ -539,7 +538,7 @@ namespace Legion {
       static const unsigned user_cache_timeout = 1024;
     public:
       typedef LegionMap<VersionID,FieldMaskSet<IndexSpaceExpression>,
-                      PHYSICAL_VERSION_ALLOC>::track_aligned VersionFieldExprs;  
+                        PHYSICAL_VERSION_ALLOC> VersionFieldExprs;  
     public:
       struct DeferMaterializedViewArgs : 
         public LgTaskArgs<DeferMaterializedViewArgs> {
@@ -685,7 +684,7 @@ namespace Legion {
       unsigned expr_cache_uses;
       // Helping with making sure that there are no outstanding users being
       // added for when we go to invalidate the cache and clean the views
-      unsigned outstanding_additions;
+      std::atomic<unsigned> outstanding_additions;
       RtUserEvent clean_waiting; 
 #ifdef ENABLE_VIEW_REPLICATION
     protected:
@@ -698,8 +697,8 @@ namespace Legion {
       // On the owner node we also need to keep track of our set of 
       // which nodes have replicated copies for which field
       union {
-        LegionMap<AddressSpaceID,FieldMask>::aligned *replicated_copies;
-        LegionMap<RtUserEvent,FieldMask>::aligned *replicated_requests;
+        LegionMap<AddressSpaceID,FieldMask> *replicated_copies;
+        LegionMap<RtUserEvent,FieldMask> *replicated_requests;
       } repl_ptr;
       // For remote copies we track which fields have seen requests
       // in the past epoch of user adds so that we can reduce our 
@@ -1055,8 +1054,8 @@ namespace Legion {
       const PredEvent false_guard;
       InnerContext *const owner_context;
     protected:
-      LegionMap<LogicalView*,FieldMask>::aligned true_views;
-      LegionMap<LogicalView*,FieldMask>::aligned false_views;
+      LegionMap<LogicalView*,FieldMask> true_views;
+      LegionMap<LogicalView*,FieldMask> false_views;
     };
 
     //--------------------------------------------------------------------------
