@@ -1,4 +1,4 @@
-/* Copyright 2021 Stanford University, NVIDIA Corporation
+/* Copyright 2023 Stanford University, NVIDIA Corporation
  *
  * Licensed under the Apache License, Version 2.0 (the "License");
  * you may not use this file except in compliance with the License.
@@ -405,24 +405,23 @@ namespace Legion {
                                     InstanceManager *manager, bool created);
       void release_acquired_instance(MappingCallInfo *info,
                                      InstanceManager *manager);
-      void check_region_consistency(MappingCallInfo *info, const char *call,
+      bool check_region_consistency(MappingCallInfo *info, const char *call,
                                     const std::vector<LogicalRegion> &regions);
-      bool perform_local_acquires(MappingCallInfo *info,
-                                  const std::vector<MappingInstance> &instances,
-                       std::map<MemoryManager*,AcquireStatus> &acquire_requests,
-                                  std::vector<unsigned> *to_erase,
-                                  const bool filter_acquired_instances = false);
-      bool perform_remote_acquires(MappingCallInfo *info,
-                      std::map<MemoryManager*,AcquireStatus> &acquire_requests);
+      bool perform_acquires(MappingCallInfo *info,
+                            const std::vector<MappingInstance> &instances,
+                            std::vector<unsigned> *to_erase = NULL,
+                            const bool filter_acquired_instances = false);
     public:
       IndexSpace create_index_space(MappingCallInfo *info, const Domain &domain,
-                                    TypeTag type_tag);
+                                    TypeTag type_tag, const char *provenance);
       IndexSpace union_index_spaces(MappingCallInfo *info, 
-                                    const std::vector<IndexSpace> &sources);
+                                    const std::vector<IndexSpace> &sources,
+                                    const char *provenance);
       IndexSpace intersect_index_spaces(MappingCallInfo *info,
-                                        const std::vector<IndexSpace> &sources);
-      IndexSpace subtract_index_spaces(MappingCallInfo *info,
-                                       IndexSpace left, IndexSpace right);
+                                        const std::vector<IndexSpace> &sources,
+                                        const char *provenance);
+      IndexSpace subtract_index_spaces(MappingCallInfo *info, IndexSpace left,
+                                       IndexSpace right,const char *provenance);
       bool is_index_space_empty(MappingCallInfo *info, IndexSpace handle);
       bool index_spaces_overlap(MappingCallInfo *info, 
                                 IndexSpace one, IndexSpace two);
@@ -628,9 +627,9 @@ namespace Legion {
       // Whether or not we are currently supporting reentrant calls
       bool permit_reentrant;
       // A flag checking whether we have a pending paused mapper call
-      volatile bool pending_pause_call;
+      std::atomic<bool> pending_pause_call;
       // A flag checking whether we have a pending finished call
-      volatile bool pending_finish_call;
+      std::atomic<bool> pending_finish_call;
     };
 
     /**
