@@ -1,4 +1,4 @@
-/* Copyright 2021 Stanford University, NVIDIA Corporation
+/* Copyright 2023 Stanford University, NVIDIA Corporation
  *
  * Licensed under the Apache License, Version 2.0 (the "License");
  * you may not use this file except in compliance with the License.
@@ -1072,6 +1072,7 @@ namespace Legion {
       //------------------------------------------------------------------------
       {
         switch (kind) {
+          case Memory::NO_MEMKIND: return "NO_MEMKIND";
           case Memory::GLOBAL_MEM: return "GLOBAL_MEM";
           case Memory::SYSTEM_MEM: return "SYSTEM_MEM";
           case Memory::REGDMA_MEM: return "REGDMA_MEM";
@@ -1084,6 +1085,8 @@ namespace Legion {
           case Memory::LEVEL3_CACHE: return "LEVEL3_CACHE";
           case Memory::LEVEL2_CACHE: return "LEVEL2_CACHE";
           case Memory::LEVEL1_CACHE: return "LEVEL1_CACHE";
+          case Memory::GPU_MANAGED_MEM: return "GPU_MANAGED_MEM";
+          case Memory::GPU_DYNAMIC_MEM: return "GPU_DYNAMIC_MEM";
           default: assert(false); return "";
         }
       }
@@ -1150,6 +1153,20 @@ namespace Legion {
         return "";
       }
 
+      //------------------------------------------------------------------------
+      std::string to_string(MapperRuntime* runtime,
+                            const MapperContext ctx,
+                            LogicalRegion lr)
+      //------------------------------------------------------------------------
+      {
+        std::stringstream ss;
+        ss << "(" << lr.get_tree_id() << ",("
+           << lr.get_index_space().get_id() << ","
+           << lr.get_index_space().get_tree_id() << "),"
+           << lr.get_field_space().get_id() << ")";
+        return ss.str();
+      }
+
       //------------------------------------------------------------------------
       std::string to_string(MapperRuntime* runtime,
                             const MapperContext ctx,
@@ -1237,7 +1254,7 @@ namespace Legion {
       //------------------------------------------------------------------------
       {
         std::stringstream ss;
-        ss << "PhysicalInstance";
+        ss << "Instance";
         if (inst.is_virtual_instance()) {
           ss << "(VIRTUAL)";
           return ss.str();
@@ -1252,7 +1269,9 @@ namespace Legion {
         if (inst.is_collective_instance()) {
           ss << "COLLECTIVE,";
         }
-        ss << "memory=" << inst.get_location();
+        ss << "region=(" << inst.get_tree_id() << ",*,"
+           << inst.get_field_space().get_id() << ")";
+        ss << ",memory=" << inst.get_location();
         ss << ",domain=" << to_string(runtime, ctx, inst.get_instance_domain());
         std::set<FieldID> fields;
         inst.get_fields(fields);
@@ -1272,7 +1291,7 @@ namespace Legion {
       //------------------------------------------------------------------------
       {
         std::stringstream ss;
-        ss << "RegionRequirement";
+        ss << "Requirement";
         ss << "[" << req_idx << "]";
         ss << "(privilege=" << to_string(req.privilege);
         if (req.is_restricted()) {
@@ -1281,21 +1300,12 @@ namespace Legion {
         if (req.prop != LEGION_EXCLUSIVE) {
           ss << ",prop=" << to_string(req.prop);
         }
-        RegionTreeID tree;
-        IndexSpace is;
-        FieldSpace fs;
-        if (req.region.exists()) {
-          tree = req.region.get_tree_id();
-          is = req.region.get_index_space();
-          fs = req.region.get_field_space();
-        } else {
-          assert(req.partition.exists());
-          tree = req.partition.get_tree_id();
-          is = runtime->get_parent_index_space(
-              ctx, req.partition.get_index_partition());
-          fs = req.partition.get_field_space();
-        }
-        ss << ",tree=" << tree;
+        LogicalRegion lr =
+          req.region.exists() ? req.region
+          : runtime->get_parent_logical_region(ctx, req.partition);
+        IndexSpace is = lr.get_index_space();
+        FieldSpace fs = lr.get_field_space();
+        ss << ",region=" << to_string(runtime, ctx, lr);
         ss << ",domain=" << to_string(runtime, ctx, is);
         ss << ",fields=" << to_string(runtime, ctx, fs, req.privilege_fields);
         ss << ")";
@@ -1314,9 +1324,36 @@ namespace Legion {
         if (include_index_point && task.is_index_space) {
           ss << "(index_point=" << task.index_point << ")";
         }
+        ss << "<" << task.get_unique_id() << ">";
         return ss.str();
       }
 
+      //------------------------------------------------------------------------
+      std::string to_string(MapperRuntime* runtime,
+                            const MapperContext ctx,
+                            const InlineMapping& inline_op)
+      //------------------------------------------------------------------------
+      {
+        std::stringstream ss;
+        ss << "InlineMapping" << "<" << inline_op.get_unique_id() << ">";
+        return ss.str();
+      }
+
+      //------------------------------------------------------------------------
+      std::string to_string(MapperRuntime* runtime,
+                            const MapperContext ctx,
+                            const Copy& copy,
+                            bool include_index_point)
+      //------------------------------------------------------------------------
+      {
+        std::stringstream ss;
+        ss << "Copy";
+        if (include_index_point && copy.is_index_space) {
+          ss << "(index_point=" << copy.index_point << ")";
+        }
+        ss << "<" << copy.get_unique_id() << ">";
+        return ss.str();
+      }
     }; // namespace Utilities
   }; // namespace Mapping
 }; // namespace Legion
