@@ -1,4 +1,4 @@
-/* Copyright 2021 Stanford University, NVIDIA Corporation
+/* Copyright 2023 Stanford University, NVIDIA Corporation
  *
  * Licensed under the Apache License, Version 2.0 (the "License");
  * you may not use this file except in compliance with the License.
@@ -866,15 +866,15 @@ namespace Legion {
         unsigned num_constraints;
         ignore_result(fread(&num_constraints, 
                             sizeof(num_constraints), 1, f));
-        layout.splitting_constraints.resize(num_constraints);
+        layout.tiling_constraints.resize(num_constraints);
         for (unsigned idx = 0; idx < num_constraints; idx++)
         {
-          SplittingConstraint &split = layout.splitting_constraints[idx];
-          ignore_result(fread(&split.kind, sizeof(split.kind), 1, f));
-          ignore_result(fread(&split.value, sizeof(split.value), 1, f));
-          unsigned chunks;
-          ignore_result(fread(&chunks, sizeof(chunks), 1, f));
-          split.chunks = (chunks != 0);
+          TilingConstraint &tile = layout.tiling_constraints[idx];
+          ignore_result(fread(&tile.dim, sizeof(tile.dim), 1, f));
+          ignore_result(fread(&tile.value, sizeof(tile.value), 1, f));
+          unsigned tiles;
+          ignore_result(fread(&tiles, sizeof(tiles), 1, f));
+          tile.tiles= (tiles != 0);
         }
       }
       {
@@ -1094,21 +1094,41 @@ namespace Legion {
       ignore_result(fread(string, string_length, 1, f));
       // Now convert the hex string back into the value
       unsigned byte_index = 0;
-      unsigned *target = (unsigned*)tunable->tunable_value;
-      for (unsigned word_idx = 0; 
-            word_idx < (tunable->tunable_size/4); word_idx++)
       {
-        unsigned next_word = 0;
-        for (unsigned i = 0; i < (8*sizeof(next_word)/4); i++, byte_index++)
+        unsigned *target = (unsigned*)tunable->tunable_value;
+        for (unsigned word_idx = 0;
+             word_idx < (tunable->tunable_size/4); word_idx++)
         {
-          unsigned next = 0;
-          if (string[byte_index] >= 'A')
-            next = (string[byte_index] - 'A') + 10;
-          else
-            next = string[byte_index] - '0';
-          next_word |= (next << (i*4));
+          unsigned next_word = 0;
+          for (unsigned i = 0; i < (8*sizeof(next_word)/4); i++, byte_index++)
+          {
+            unsigned next = 0;
+            if (string[byte_index] >= 'A')
+              next = (string[byte_index] - 'A') + 10;
+            else
+              next = string[byte_index] - '0';
+            next_word |= (next << (i*4));
+          }
+          target[word_idx] = next_word;
+        }
+      }
+      {
+        char *target = (char*)tunable->tunable_value;
+        for (unsigned char_idx = (tunable->tunable_size/4)*4;
+             char_idx < tunable->tunable_size; ++char_idx)
+        {
+          char next_char = 0;
+          for (unsigned i = 0; i < 2; i++, byte_index++)
+          {
+            char next = 0;
+            if (string[byte_index] >= 'A')
+              next = (string[byte_index] - 'A') + 10;
+            else
+              next = string[byte_index] - '0';
+            next_char |= (next << (i*4));
+          }
+          target[char_idx] = next_char;
         }
-        target[word_idx] = next_word;
       }
 
       free(string);
