@@ -1,4 +1,4 @@
-/* Copyright 2021 Stanford University, NVIDIA Corporation
+/* Copyright 2023 Stanford University, NVIDIA Corporation
  *
  * Licensed under the Apache License, Version 2.0 (the "License");
  * you may not use this file except in compliance with the License.
@@ -168,6 +168,14 @@ namespace Legion {
       return parent_task;
     }
 
+    //--------------------------------------------------------------------------
+    const std::string& ShimMapper::Task::get_provenance_string(bool human) const
+    //--------------------------------------------------------------------------
+    {
+      assert(false);
+      return *(new std::string());
+    }
+
     //--------------------------------------------------------------------------
     const char* ShimMapper::Task::get_task_name(void) const
     //--------------------------------------------------------------------------
@@ -175,6 +183,15 @@ namespace Legion {
       return task_name;
     }
 
+    //--------------------------------------------------------------------------
+    Domain ShimMapper::Task::get_slice_domain(void) const
+    //--------------------------------------------------------------------------
+    {
+      // This method didn't exist back then anyway
+      assert(false);
+      return Domain::NO_DOMAIN;
+    }
+
     //--------------------------------------------------------------------------
     bool ShimMapper::Task::has_trace(void) const
     //--------------------------------------------------------------------------
@@ -255,6 +272,15 @@ namespace Legion {
     {
       return parent_task;
     }
+
+    //--------------------------------------------------------------------------
+    const std::string& ShimMapper::Inline::get_provenance_string(
+                                                               bool human) const
+    //--------------------------------------------------------------------------
+    {
+      assert(false);
+      return *(new std::string());
+    }
     
     //--------------------------------------------------------------------------
     ShimMapper::Copy::Copy(const Legion::Copy &c)
@@ -337,6 +363,14 @@ namespace Legion {
       return parent_task;
     }
 
+    //--------------------------------------------------------------------------
+    const std::string& ShimMapper::Copy::get_provenance_string(bool human) const
+    //--------------------------------------------------------------------------
+    {
+      assert(false);
+      return *(new std::string());
+    }
+
     //--------------------------------------------------------------------------
     bool ShimMapper::TaskVariantCollection::has_variant(Processor::Kind kind, 
                                                   bool single, bool index_space)
