@@ -1,4 +1,4 @@
-/* Copyright 2021 Stanford University, NVIDIA Corporation
+/* Copyright 2023 Stanford University, NVIDIA Corporation
  *
  * Licensed under the Apache License, Version 2.0 (the "License");
  * you may not use this file except in compliance with the License.
@@ -16,13 +16,22 @@
 #ifndef REALM_CUDA_INTERNAL_H
 #define REALM_CUDA_INTERNAL_H
 
-#include <cuda.h>
-
-// We don't actually use the cuda runtime, but
-// we need all its declarations so we have all the right types
-#include <cuda_runtime.h>
-
 #include "realm/realm_config.h"
+
+#include <memory>
+#include <cuda.h>
+#include <nvml.h>
+#if defined(REALM_USE_CUDART_HIJACK)
+#include <cuda_runtime_api.h>   // For cudaDeviceProp
+#endif
+
+// For CUDA runtime's dim3 definition
+#include <vector_types.h>
+
+#if defined(REALM_CUDA_DYNAMIC_LOAD) && (CUDA_VERSION >= 11030)
+#include <cudaTypedefs.h>
+#endif
+
 #include "realm/operation.h"
 #include "realm/threads.h"
 #include "realm/circ_queue.h"
@@ -31,27 +40,35 @@
 #include "realm/mem_impl.h"
 #include "realm/bgwork.h"
 #include "realm/transfer/channel.h"
+#include "realm/transfer/ib_memory.h"
 
-#define CHECK_CUDART(cmd) do { \
-  cudaError_t ret = (cmd); \
-  if(ret != cudaSuccess) { \
-    fprintf(stderr, "CUDART: %s = %d (%s)\n", #cmd, ret, cudaGetErrorString(ret)); \
-    assert(0); \
-    exit(1); \
-  } \
-} while(0)
+#if CUDART_VERSION < 11000
+#define CHECK_CUDART(cmd)                                                      \
+  do {                                                                         \
+    int ret = (int)(cmd);                                                      \
+    if (ret != 0) {                                                            \
+      fprintf(stderr, "CUDART: %s = %d\n", #cmd, ret);                         \
+      assert(0);                                                               \
+      exit(1);                                                                 \
+    }                                                                          \
+  } while (0)
+#else
+// Since CUDA TK11.0, runtime and driver error codes are 1:1 correlated
+#define CHECK_CUDART(cmd) CHECK_CU((CUresult)(cmd))
+#endif
 
 // Need CUDA 6.5 or later for good error reporting
 #if CUDA_VERSION >= 6050
 #define REPORT_CU_ERROR(cmd, ret) \
   do { \
     const char *name, *str; \
-    cuGetErrorName(ret, &name); \
-    cuGetErrorString(ret, &str); \
+    CUDA_DRIVER_FNPTR(cuGetErrorName)(ret, &name);      \
+    CUDA_DRIVER_FNPTR(cuGetErrorString)(ret, &str);                 \
     fprintf(stderr, "CU: %s = %d (%s): %s\n", cmd, ret, name, str); \
     abort(); \
   } while(0)
 #else
+#define REPORT_CU_ERROR(cmd, ret) \
   do { \
     fprintf(stderr, "CU: %s = %d\n", cmd, ret); \
     abort(); \
@@ -63,20 +80,42 @@
   if(ret != CUDA_SUCCESS) REPORT_CU_ERROR(#cmd, ret); \
 } while(0)
 
+#define REPORT_NVML_ERROR(cmd, ret)                                                      \
+  do {                                                                                   \
+    fprintf(stderr, "NVML: %s = %d\n", cmd, ret);                                        \
+    abort();                                                                             \
+  } while(0)
+
+#define CHECK_NVML(cmd)                                                                  \
+  do {                                                                                   \
+    nvmlReturn_t ret = (cmd);                                                            \
+    if(ret != NVML_SUCCESS)                                                              \
+      REPORT_NVML_ERROR(#cmd, ret);                                                      \
+  } while(0)
+
 namespace Realm {
 
   namespace Cuda {
 
-    struct GPUInfo {
+    struct GPUInfo 
+    {
       int index;  // index used by CUDA runtime
       CUdevice device;
-
-      static const size_t MAX_NAME_LEN = 64;
+      nvmlDevice_t nvml_dev;
+      CUuuid uuid;
+      int major;
+      int minor;
+      static const size_t MAX_NAME_LEN = 256;
       char name[MAX_NAME_LEN];
-
-      int compute_major, compute_minor;
-      size_t total_mem;
+      size_t totalGlobalMem;
+      static const size_t MAX_NUMA_NODE_LEN = 20;
+      bool has_numa_preference;
+      unsigned long numa_node_affinity[MAX_NUMA_NODE_LEN];
       std::set<CUdevice> peers;  // other GPUs we can do p2p copies with
+
+      #ifdef REALM_USE_CUDART_HIJACK
+      cudaDeviceProp prop;
+      #endif
     };
 
     enum GPUMemcpyKind {
@@ -92,9 +131,12 @@ namespace Realm {
     class GPUStream;
     class GPUFBMemory;
     class GPUZCMemory;
+    class GPUFBIBMemory;
     class GPU;
     class CudaModule;
 
+    extern CudaModule *cuda_module_singleton;
+
     // an interface for receiving completion notification for a GPU operation
     //  (right now, just copies)
     class GPUCompletionNotification {
@@ -319,7 +361,7 @@ namespace Realm {
     //  with when async work needs doing
     class GPUStream {
     public:
-      GPUStream(GPU *_gpu, GPUWorker *_worker);
+      GPUStream(GPU *_gpu, GPUWorker *_worker, int rel_priority = 0);
       ~GPUStream(void);
 
       GPU *get_gpu(void) const;
@@ -332,6 +374,12 @@ namespace Realm {
       void add_notification(GPUCompletionNotification *notification);
       void wait_on_streams(const std::set<GPUStream*> &other_streams);
 
+      // atomically checks rate limit counters and returns true if 'bytes'
+      //  worth of copies can be submitted or false if not (in which case
+      //  the progress counter on the xd will be updated when it should try
+      //  again)
+      bool ok_to_submit_copy(size_t bytes, XferDes *xd);
+
       // to be called by a worker (that should already have the GPU context
       //   current) - returns true if any work remains
       bool issue_copies(TimeLimit work_until);
@@ -401,7 +449,7 @@ namespace Realm {
       bool process_streams(bool sleep_on_empty);
 
       Mutex lock;
-      CondVar condvar;
+      Mutex::CondVar condvar;
 
       typedef CircularQueue<GPUStream *, 16> ActiveStreamQueue;
       ActiveStreamQueue active_streams;
@@ -456,7 +504,7 @@ namespace Realm {
       CUcontext context;
       int max_threads;
       Mutex mutex;
-      CondVar condvar;
+      Mutex::CondVar condvar;
       bool shutdown_flag;
       GPUWorkFence::FenceList fences;
       int total_threads, sleeping_threads, syncing_threads;
@@ -474,8 +522,7 @@ namespace Realm {
     class GPU {
     public:
       GPU(CudaModule *_module, GPUInfo *_info, GPUWorker *worker,
-	  CUcontext _context,
-	  int num_streams);
+	  CUcontext _context);
       ~GPU(void);
 
       void push_context(void);
@@ -491,7 +538,8 @@ namespace Realm {
 #endif
 
       void create_processor(RuntimeImpl *runtime, size_t stack_size);
-      void create_fb_memory(RuntimeImpl *runtime, size_t size);
+      void create_fb_memory(RuntimeImpl *runtime, size_t size, size_t ib_size);
+      void create_dynamic_fb_memory(RuntimeImpl *runtime, size_t max_size);
 
       void create_dma_channels(Realm::RuntimeImpl *r);
 
@@ -582,6 +630,7 @@ namespace Realm {
       GPUStream *find_stream(CUstream stream) const;
       GPUStream *get_null_task_stream(void) const;
       GPUStream *get_next_task_stream(bool create = false);
+      GPUStream *get_next_d2d_stream();
     protected:
       CUmodule load_cuda_module(const void *data);
 
@@ -591,9 +640,10 @@ namespace Realm {
       GPUWorker *worker;
       GPUProcessor *proc;
       GPUFBMemory *fbmem;
+      GPUFBIBMemory *fb_ibmem;
 
       CUcontext context;
-      CUdeviceptr fbmem_base;
+      CUdeviceptr fbmem_base, fb_ibmem_base;
 
       // which system memories have been registered and can be used for cuMemcpyAsync
       std::set<Memory> pinned_sysmems;
@@ -608,12 +658,28 @@ namespace Realm {
       GPUStream *host_to_device_stream;
       GPUStream *device_to_host_stream;
       GPUStream *device_to_device_stream;
+      std::vector<GPUStream *> device_to_device_streams;
       std::vector<GPUStream *> peer_to_peer_streams; // indexed by target
       std::vector<GPUStream *> task_streams;
-      atomic<unsigned> next_stream;
+      atomic<unsigned> next_task_stream, next_d2d_stream;
 
       GPUEventPool event_pool;
 
+      // this can technically be different in each context (but probably isn't
+      //  in practice)
+      int least_stream_priority, greatest_stream_priority;
+
+      struct CudaIpcMapping {
+        NodeID owner;
+        Memory mem;
+        uintptr_t local_base;
+        uintptr_t address_offset; // add to convert from original to local base
+      };
+      std::vector<CudaIpcMapping> cudaipc_mappings;
+      std::map<NodeID, GPUStream *> cudaipc_streams;
+
+      const CudaIpcMapping *find_ipc_mapping(Memory mem) const;
+
 #ifdef REALM_USE_CUDART_HIJACK
       std::map<const FatBin *, CUmodule> device_modules;
       std::map<const void *, CUfunction> device_functions;
@@ -650,49 +716,49 @@ namespace Realm {
                                   size_t *shared_size, void *stream);
 #endif
 
-      void stream_wait_on_event(cudaStream_t stream, cudaEvent_t event);
-      void stream_synchronize(cudaStream_t stream);
+      void stream_wait_on_event(CUstream stream, CUevent event);
+      void stream_synchronize(CUstream stream);
       void device_synchronize(void);
 
 #ifdef REALM_USE_CUDART_HIJACK
-      void event_create(cudaEvent_t *event, int flags);
-      void event_destroy(cudaEvent_t event);
-      void event_record(cudaEvent_t event, cudaStream_t stream);
-      void event_synchronize(cudaEvent_t event);
-      void event_elapsed_time(float *ms, cudaEvent_t start, cudaEvent_t end);
+      void event_create(CUevent *event, int flags);
+      void event_destroy(CUevent event);
+      void event_record(CUevent event, CUstream stream);
+      void event_synchronize(CUevent event);
+      void event_elapsed_time(float *ms, CUevent start, CUevent end);
       
       void configure_call(dim3 grid_dim, dim3 block_dim,
-			  size_t shared_memory, cudaStream_t stream);
+			  size_t shared_memory, CUstream stream);
       void setup_argument(const void *arg, size_t size, size_t offset);
       void launch(const void *func);
       void launch_kernel(const void *func, dim3 grid_dim, dim3 block_dim, 
                          void **args, size_t shared_memory, 
-                         cudaStream_t stream, bool cooperative = false);
+                         CUstream stream, bool cooperative = false);
 #endif
 
-      void gpu_memcpy(void *dst, const void *src, size_t size, cudaMemcpyKind kind);
+      void gpu_memcpy(void *dst, const void *src, size_t size);
       void gpu_memcpy_async(void *dst, const void *src, size_t size,
-			    cudaMemcpyKind kind, cudaStream_t stream);
+                            CUstream stream);
 #ifdef REALM_USE_CUDART_HIJACK
       void gpu_memcpy2d(void *dst, size_t dpitch, const void *src, size_t spitch,
-                        size_t width, size_t height, cudaMemcpyKind kind);
+                        size_t width, size_t height);
       void gpu_memcpy2d_async(void *dst, size_t dpitch, const void *src, 
                               size_t spitch, size_t width, size_t height, 
-                              cudaMemcpyKind kind, cudaStream_t stream);
+                              CUstream stream);
       void gpu_memcpy_to_symbol(const void *dst, const void *src, size_t size,
-				size_t offset, cudaMemcpyKind kind);
+				size_t offset);
       void gpu_memcpy_to_symbol_async(const void *dst, const void *src, size_t size,
-				      size_t offset, cudaMemcpyKind kind,
-				      cudaStream_t stream);
+				      size_t offset,
+				      CUstream stream);
       void gpu_memcpy_from_symbol(void *dst, const void *src, size_t size,
-				  size_t offset, cudaMemcpyKind kind);
+				  size_t offset);
       void gpu_memcpy_from_symbol_async(void *dst, const void *src, size_t size,
-					size_t offset, cudaMemcpyKind kind,
-					cudaStream_t stream);
+					size_t offset,
+					CUstream stream);
 #endif
 
       void gpu_memset(void *dst, int value, size_t count);
-      void gpu_memset_async(void *dst, int value, size_t count, cudaStream_t stream);
+      void gpu_memset_async(void *dst, int value, size_t count, CUstream stream);
     public:
       GPU *gpu;
 
@@ -704,8 +770,8 @@ namespace Realm {
 	LaunchConfig(dim3 _grid, dim3 _block, size_t _shared);
       };
       struct CallConfig : public LaunchConfig {
-        cudaStream_t stream; 
-        CallConfig(dim3 _grid, dim3 _block, size_t _shared, cudaStream_t _stream);
+        CUstream stream; 
+        CallConfig(dim3 _grid, dim3 _block, size_t _shared, CUstream _stream);
       };
       std::vector<CallConfig> launch_configs;
       std::vector<char> kernel_args;
@@ -716,6 +782,20 @@ namespace Realm {
       Realm::CoreReservation *core_rsrv;
     };
 
+    // this can be attached to any MemoryImpl if the underlying memory is
+    //  guaranteed to belong to a given CUcontext - this will allow that
+    //  context's processor and dma channels to work with it
+    // the creator is expected to know what CUcontext they want but need
+    //  not know which GPU object that corresponds to
+    class CudaDeviceMemoryInfo : public ModuleSpecificInfo
+    {
+    public:
+      CudaDeviceMemoryInfo(CUcontext _context);
+
+      CUcontext context;
+      GPU *gpu;
+    };
+
     class GPUFBMemory : public LocalManagedMemory {
     public:
       GPUFBMemory(Memory _me, GPU *_gpu, CUdeviceptr _base, size_t _size);
@@ -728,12 +808,67 @@ namespace Realm {
 
       virtual void *get_direct_ptr(off_t offset, size_t size);
 
+      // GPUFBMemory supports ExternalCudaMemoryResource and
+      //  ExternalCudaArrayResource
+      virtual bool attempt_register_external_resource(RegionInstanceImpl *inst,
+                                                      size_t& inst_offset);
+      virtual void unregister_external_resource(RegionInstanceImpl *inst);
+
+      // for re-registration purposes, generate an ExternalInstanceResource *
+      //  (if possible) for a given instance, or a subset of one
+      virtual ExternalInstanceResource *generate_resource_info(RegionInstanceImpl *inst,
+							       const IndexSpaceGeneric *subspace,
+							       span<const FieldID> fields,
+							       bool read_only);
+
     public:
       GPU *gpu;
       CUdeviceptr base;
       NetworkSegment local_segment;
     };
 
+    class GPUDynamicFBMemory : public MemoryImpl {
+    public:
+      GPUDynamicFBMemory(Memory _me, GPU *_gpu, size_t _max_size);
+
+      virtual ~GPUDynamicFBMemory(void);
+
+      // deferred allocation not supported
+      virtual AllocationResult allocate_storage_immediate(RegionInstanceImpl *inst,
+							  bool need_alloc_result,
+							  bool poisoned,
+							  TimeLimit work_until);
+
+      virtual void release_storage_immediate(RegionInstanceImpl *inst,
+					     bool poisoned,
+					     TimeLimit work_until);
+
+      // these work, but they are SLOW
+      virtual void get_bytes(off_t offset, void *dst, size_t size);
+      virtual void put_bytes(off_t offset, const void *src, size_t size);
+
+      virtual void *get_direct_ptr(off_t offset, size_t size);
+
+      // GPUDynamicFBMemory supports ExternalCudaMemoryResource and
+      //  ExternalCudaArrayResource
+      virtual bool attempt_register_external_resource(RegionInstanceImpl *inst,
+                                                      size_t& inst_offset);
+      virtual void unregister_external_resource(RegionInstanceImpl *inst);
+
+      // for re-registration purposes, generate an ExternalInstanceResource *
+      //  (if possible) for a given instance, or a subset of one
+      virtual ExternalInstanceResource *generate_resource_info(RegionInstanceImpl *inst,
+							       const IndexSpaceGeneric *subspace,
+							       span<const FieldID> fields,
+							       bool read_only);
+
+    public:
+      GPU *gpu;
+      Mutex mutex;
+      size_t cur_size;
+      std::map<RegionInstance, std::pair<CUdeviceptr, size_t> > alloc_bases;
+    };
+
     class GPUZCMemory : public LocalManagedMemory {
     public:
       GPUZCMemory(Memory _me, CUdeviceptr _gpu_base,
@@ -748,12 +883,34 @@ namespace Realm {
 
       virtual void *get_direct_ptr(off_t offset, size_t size);
 
+      // GPUZCMemory supports ExternalCudaPinnedHostResource
+      virtual bool attempt_register_external_resource(RegionInstanceImpl *inst,
+                                                      size_t& inst_offset);
+      virtual void unregister_external_resource(RegionInstanceImpl *inst);
+
+      // for re-registration purposes, generate an ExternalInstanceResource *
+      //  (if possible) for a given instance, or a subset of one
+      virtual ExternalInstanceResource *generate_resource_info(RegionInstanceImpl *inst,
+							       const IndexSpaceGeneric *subspace,
+							       span<const FieldID> fields,
+							       bool read_only);
+
     public:
       CUdeviceptr gpu_base;
       char *cpu_base;
       NetworkSegment local_segment;
     };
 
+    class GPUFBIBMemory : public IBMemory {
+    public:
+      GPUFBIBMemory(Memory _me, GPU *_gpu, CUdeviceptr _base, size_t _size);
+
+    public:
+      GPU *gpu;
+      CUdeviceptr base;
+      NetworkSegment local_segment;
+    };
+
     class GPURequest;
 
     class GPUCompletionEvent : public GPUCompletionNotification {
@@ -789,6 +946,30 @@ namespace Realm {
       size_t write_offset, write_size;
     };
 
+    class MemSpecificCudaArray : public MemSpecificInfo {
+    public:
+      MemSpecificCudaArray(CUarray _array);
+      virtual ~MemSpecificCudaArray();
+
+      CUarray array;
+    };
+
+    class AddressInfoCudaArray : public TransferIterator::AddressInfoCustom {
+    public:
+      virtual int set_rect(const RegionInstanceImpl *inst,
+                           const InstanceLayoutPieceBase *piece,
+                           size_t field_size, size_t field_offset,
+                           int ndims,
+                           const int64_t lo[/*ndims*/],
+                           const int64_t hi[/*ndims*/],
+                           const int order[/*ndims*/]);
+
+      CUarray array;
+      int dim;
+      size_t pos[3];
+      size_t width_in_bytes, height, depth;
+    };
+
     class GPUChannel;
 
     class GPUXferDes : public XferDes {
@@ -799,27 +980,13 @@ namespace Realm {
 		 const std::vector<XferDesPortInfo>& outputs_info,
 		 int _priority);
 
-      ~GPUXferDes()
-      {
-        while (!available_reqs.empty()) {
-          GPURequest* gpu_req = (GPURequest*) available_reqs.front();
-          available_reqs.pop();
-          delete gpu_req;
-        }
-      }
-
       long get_requests(Request** requests, long nr);
-      void notify_request_read_done(Request* req);
-      void notify_request_write_done(Request* req);
-      void flush();
 
       bool progress_xd(GPUChannel *channel, TimeLimit work_until);
 
     private:
-      //GPURequest* gpu_reqs;
-      //char *src_buf_base;
-      //char *dst_buf_base;
-      GPU *dst_gpu, *src_gpu;
+      std::vector<GPU *> src_gpus, dst_gpus;
+      std::vector<bool> dst_is_ipc;
     };
 
     class GPUChannel : public SingleXDQChannel<GPUChannel, GPUXferDes> {
@@ -839,7 +1006,8 @@ namespace Realm {
 				       const std::vector<XferDesPortInfo>& outputs_info,
 				       int priority,
 				       XferDesRedopInfo redop_info,
-				       const void *fill_data, size_t fill_size);
+				       const void *fill_data, size_t fill_size,
+                                       size_t fill_total);
 
       long submit(Request** requests, long nr);
 
@@ -857,7 +1025,8 @@ namespace Realm {
 		     const std::vector<XferDesPortInfo>& inputs_info,
 		     const std::vector<XferDesPortInfo>& outputs_info,
 		     int _priority,
-		     const void *_fill_data, size_t _fill_size);
+		     const void *_fill_data, size_t _fill_size,
+                     size_t _fill_total);
 
       long get_requests(Request** requests, long nr);
 
@@ -881,7 +1050,8 @@ namespace Realm {
 				       const std::vector<XferDesPortInfo>& outputs_info,
 				       int priority,
 				       XferDesRedopInfo redop_info,
-				       const void *fill_data, size_t fill_size);
+				       const void *fill_data, size_t fill_size,
+                                       size_t fill_total);
 
       long submit(Request** requests, long nr);
 
@@ -909,11 +1079,8 @@ namespace Realm {
     protected:
       XferDesRedopInfo redop_info;
       const ReductionOpUntyped *redop;
-#if defined(REALM_USE_CUDART_HIJACK) || (CUDA_VERSION >= 11000)
       CUfunction kernel;
-#else
       const void *kernel_host_proxy;
-#endif
       GPUStream *stream;
     };
 
@@ -929,13 +1096,16 @@ namespace Realm {
 
       // override this because we have to be picky about which reduction ops
       //  we support
-      virtual bool supports_path(Memory src_mem, Memory dst_mem,
-				 CustomSerdezID src_serdez_id,
-				 CustomSerdezID dst_serdez_id,
-				 ReductionOpID redop_id,
-				 XferDesKind *kind_ret = 0,
-				 unsigned *bw_ret = 0,
-				 unsigned *lat_ret = 0);
+      virtual uint64_t supports_path(Memory src_mem, Memory dst_mem,
+                                     CustomSerdezID src_serdez_id,
+                                     CustomSerdezID dst_serdez_id,
+                                     ReductionOpID redop_id,
+                                     size_t total_bytes,
+                                     const std::vector<size_t> *src_frags,
+                                     const std::vector<size_t> *dst_frags,
+                                     XferDesKind *kind_ret = 0,
+                                     unsigned *bw_ret = 0,
+                                     unsigned *lat_ret = 0);
 
       virtual RemoteChannelInfo *construct_remote_info() const;
 
@@ -946,7 +1116,8 @@ namespace Realm {
 				       const std::vector<XferDesPortInfo>& outputs_info,
 				       int priority,
 				       XferDesRedopInfo redop_info,
-				       const void *fill_data, size_t fill_size);
+				       const void *fill_data, size_t fill_size,
+                                       size_t fill_total);
 
       long submit(Request** requests, long nr);
 
@@ -980,15 +1151,152 @@ namespace Realm {
 
       GPUreduceRemoteChannel(uintptr_t _remote_ptr);
 
-      virtual bool supports_path(Memory src_mem, Memory dst_mem,
-				 CustomSerdezID src_serdez_id,
-				 CustomSerdezID dst_serdez_id,
-				 ReductionOpID redop_id,
-				 XferDesKind *kind_ret = 0,
-				 unsigned *bw_ret = 0,
-				 unsigned *lat_ret = 0);
+      virtual uint64_t supports_path(Memory src_mem, Memory dst_mem,
+                                     CustomSerdezID src_serdez_id,
+                                     CustomSerdezID dst_serdez_id,
+                                     ReductionOpID redop_id,
+                                     size_t total_bytes,
+                                     const std::vector<size_t> *src_frags,
+                                     const std::vector<size_t> *dst_frags,
+                                     XferDesKind *kind_ret = 0,
+                                     unsigned *bw_ret = 0,
+                                     unsigned *lat_ret = 0);
     };
 
+
+    // active messages for establishing cuda ipc mappings
+
+    struct CudaIpcRequest {
+#ifdef REALM_ON_LINUX
+      long hostid;  // POSIX hostid
+#endif
+
+      static void handle_message(NodeID sender, const CudaIpcRequest& args,
+                                 const void *data, size_t datalen);
+    };
+
+    struct CudaIpcResponse {
+      unsigned count;
+
+      static void handle_message(NodeID sender, const CudaIpcResponse& args,
+                                 const void *data, size_t datalen);
+    };
+
+    struct CudaIpcRelease {
+
+      static void handle_message(NodeID sender, const CudaIpcRelease& args,
+                                 const void *data, size_t datalen);
+    };
+
+    class GPUReplHeapListener : public ReplicatedHeap::Listener {
+    public:
+      GPUReplHeapListener(CudaModule *_module);
+
+      virtual void chunk_created(void *base, size_t bytes);
+      virtual void chunk_destroyed(void *base, size_t bytes);
+
+    protected:
+      CudaModule *module;
+    };
+
+
+#ifdef REALM_CUDA_DYNAMIC_LOAD
+  // cuda driver and/or runtime entry points
+#define CUDA_DRIVER_FNPTR(name) (name##_fnptr)
+
+#define CUDA_DRIVER_APIS(__op__)                                               \
+  __op__(cuModuleGetFunction);                                                 \
+  __op__(cuCtxEnablePeerAccess);                                               \
+  __op__(cuCtxGetFlags);                                                       \
+  __op__(cuCtxGetStreamPriorityRange);                                         \
+  __op__(cuCtxPopCurrent);                                                     \
+  __op__(cuCtxPushCurrent);                                                    \
+  __op__(cuCtxSynchronize);                                                    \
+  __op__(cuDeviceCanAccessPeer);                                               \
+  __op__(cuDeviceGet);                                                         \
+  __op__(cuDeviceGetUuid);                                                     \
+  __op__(cuDeviceGetAttribute);                                                \
+  __op__(cuDeviceGetCount);                                                    \
+  __op__(cuDeviceGetName);                                                     \
+  __op__(cuDevicePrimaryCtxRelease);                                           \
+  __op__(cuDevicePrimaryCtxRetain);                                            \
+  __op__(cuDevicePrimaryCtxSetFlags);                                          \
+  __op__(cuDeviceTotalMem);                                                    \
+  __op__(cuEventCreate);                                                       \
+  __op__(cuEventDestroy);                                                      \
+  __op__(cuEventQuery);                                                        \
+  __op__(cuEventRecord);                                                       \
+  __op__(cuGetErrorName);                                                      \
+  __op__(cuGetErrorString);                                                    \
+  __op__(cuInit);                                                              \
+  __op__(cuIpcCloseMemHandle);                                                 \
+  __op__(cuIpcGetMemHandle);                                                   \
+  __op__(cuIpcOpenMemHandle);                                                  \
+  __op__(cuLaunchKernel);                                                      \
+  __op__(cuMemAllocManaged);                                                   \
+  __op__(cuMemAlloc);                                                          \
+  __op__(cuMemcpy2DAsync);                                                     \
+  __op__(cuMemcpy3DAsync);                                                     \
+  __op__(cuMemcpyAsync);                                                       \
+  __op__(cuMemcpyDtoDAsync);                                                   \
+  __op__(cuMemcpyDtoH);                                                        \
+  __op__(cuMemcpyDtoHAsync);                                                   \
+  __op__(cuMemcpyHtoD);                                                        \
+  __op__(cuMemcpyHtoDAsync);                                                   \
+  __op__(cuMemFreeHost);                                                       \
+  __op__(cuMemFree);                                                           \
+  __op__(cuMemGetInfo);                                                        \
+  __op__(cuMemHostAlloc);                                                      \
+  __op__(cuMemHostGetDevicePointer);                                           \
+  __op__(cuMemHostRegister);                                                   \
+  __op__(cuMemHostUnregister);                                                 \
+  __op__(cuMemsetD16Async);                                                    \
+  __op__(cuMemsetD2D16Async);                                                  \
+  __op__(cuMemsetD2D32Async);                                                  \
+  __op__(cuMemsetD2D8Async);                                                   \
+  __op__(cuMemsetD32Async);                                                    \
+  __op__(cuMemsetD8Async);                                                     \
+  __op__(cuModuleLoadDataEx);                                                  \
+  __op__(cuStreamAddCallback);                                                 \
+  __op__(cuStreamCreate);                                                      \
+  __op__(cuStreamCreateWithPriority);                                          \
+  __op__(cuStreamDestroy);                                                     \
+  __op__(cuStreamSynchronize);                                                 \
+  __op__(cuStreamWaitEvent)
+
+  #if CUDA_VERSION >= 11030
+    // cuda 11.3+ gives us handy PFN_... types
+    #define DECL_FNPTR_EXTERN(name) \
+      extern PFN_ ## name name ## _fnptr;
+  #else
+    // before cuda 11.3, we have to rely on typeof/decltype
+    #define DECL_FNPTR_EXTERN(name) \
+      extern decltype(&name) name ## _fnptr;
+  #endif
+    CUDA_DRIVER_APIS(DECL_FNPTR_EXTERN);
+  #undef DECL_FNPTR_EXTERN
+
+#else
+  #define CUDA_DRIVER_FNPTR(name) (name)
+#endif
+
+#define NVML_FNPTR(name) (name##_fnptr)
+
+#if NVML_API_VERSION >= 11
+#define NVML_11_APIS(__op__) __op__(nvmlDeviceGetMemoryAffinity);
+#else
+#define NVML_11_APIS(__op__)
+#endif
+
+#define NVML_APIS(__op__)                                                                \
+  __op__(nvmlInit);                                                                      \
+  __op__(nvmlDeviceGetHandleByUUID);                                                     \
+  NVML_11_APIS(__op__);
+
+#define DECL_FNPTR_EXTERN(name) extern decltype(&name) name##_fnptr;
+    NVML_APIS(DECL_FNPTR_EXTERN)
+#undef DECL_FNPTR_EXTERN
+
   }; // namespace Cuda
 
 }; // namespace Realm 
