@@ -18,7 +18,7 @@
 #include <cuda_runtime.h>
 
 #include "realm/cuda/cudart_hijack.h"
-
+#include "realm/cuda/cuda_module.h"
 #include "realm/cuda/cuda_internal.h"
 #include "realm/logging.h"
 
@@ -33,9 +33,9 @@ namespace Realm {
       cudart_hijack_active = true;
 
       GPUProcessor *p = GPUProcessor::get_current_gpu_proc();
-      if(!p) {
-	log_cudart.fatal() << funcname << "() called outside CUDA task";
-	assert(false);
+      if (!p) {
+        log_cudart.fatal() << funcname << "() called outside CUDA task";
+        assert(false);
       }
       return p;
     }
@@ -649,7 +649,7 @@ extern "C" {
 			 size_t size, cudaMemcpyKind kind)
   {
     GPUProcessor *p = get_gpu_or_die("cudaMemcpy");
-    p->gpu_memcpy(dst, src, size, kind);
+    p->gpu_memcpy(dst, src, size);
     return cudaSuccess;
   }
 
@@ -659,7 +659,7 @@ extern "C" {
 			      cudaStream_t stream)
   {
     GPUProcessor *p = get_gpu_or_die("cudaMemcpyAsync");
-    p->gpu_memcpy_async(dst, src, size, kind, stream);
+    p->gpu_memcpy_async(dst, src, size, stream);
     return cudaSuccess;
   }
 
@@ -668,7 +668,7 @@ extern "C" {
 			   size_t width, size_t height, cudaMemcpyKind kind)
   {
     GPUProcessor *p = get_gpu_or_die("cudaMemcpy2D");
-    p->gpu_memcpy2d(dst, dpitch, src, spitch, width, height, kind);
+    p->gpu_memcpy2d(dst, dpitch, src, spitch, width, height);
     return cudaSuccess;
   }
 
@@ -678,7 +678,7 @@ extern "C" {
 				cudaMemcpyKind kind, cudaStream_t stream)
   {
     GPUProcessor *p = get_gpu_or_die("cudaMemcpy2DAsync");
-    p->gpu_memcpy2d_async(dst, dpitch, src, spitch, width, height, kind, stream);
+    p->gpu_memcpy2d_async(dst, dpitch, src, spitch, width, height, stream);
     return cudaSuccess;
   }
 
@@ -745,7 +745,7 @@ extern "C" {
 				 cudaMemcpyKind kind)
   {
     GPUProcessor *p = get_gpu_or_die("cudaMemcpyToSymbol");
-    p->gpu_memcpy_to_symbol(dst, src, size, offset, kind);
+    p->gpu_memcpy_to_symbol(dst, src, size, offset);
     return cudaSuccess;
   }
 
@@ -755,7 +755,7 @@ extern "C" {
 				      cudaMemcpyKind kind, cudaStream_t stream)
   {
     GPUProcessor *p = get_gpu_or_die("cudaMemcpyToSymbolAsync");
-    p->gpu_memcpy_to_symbol_async(dst, src, size, offset, kind, stream);
+    p->gpu_memcpy_to_symbol_async(dst, src, size, offset, stream);
     return cudaSuccess;
   }
 
@@ -765,7 +765,7 @@ extern "C" {
 				   cudaMemcpyKind kind)
   {
     GPUProcessor *p = get_gpu_or_die("cudaMemcpyFromSymbol");
-    p->gpu_memcpy_from_symbol(dst, src, size, offset, kind);
+    p->gpu_memcpy_from_symbol(dst, src, size, offset);
     return cudaSuccess;
   }
       
@@ -775,7 +775,7 @@ extern "C" {
 					cudaMemcpyKind kind, cudaStream_t stream)
   {
     GPUProcessor *p = get_gpu_or_die("cudaMemcpyFromSymbolAsync");
-    p->gpu_memcpy_from_symbol_async(dst, src, size, offset, kind, stream);
+    p->gpu_memcpy_from_symbol_async(dst, src, size, offset, stream);
     return cudaSuccess;
   }
       
@@ -803,83 +803,135 @@ extern "C" {
     return cudaSuccess;
   }
 
+#define CUDA_DEVICE_ATTRIBUTES_PRE_8_0(__op__) \
+  __op__(major, COMPUTE_CAPABILITY_MAJOR)                  \
+  __op__(minor, COMPUTE_CAPABILITY_MINOR)                  \
+  __op__(sharedMemPerBlock, MAX_SHARED_MEMORY_PER_BLOCK)   \
+  __op__(regsPerBlock,      MAX_REGISTERS_PER_BLOCK)       \
+  __op__(warpSize,          WARP_SIZE)                     \
+  __op__(memPitch,          MAX_PITCH)                     \
+  __op__(maxThreadsPerBlock, MAX_THREADS_PER_BLOCK)        \
+  __op__(maxThreadsDim[0], MAX_BLOCK_DIM_X)                \
+  __op__(maxThreadsDim[1], MAX_BLOCK_DIM_Y)                \
+  __op__(maxThreadsDim[2], MAX_BLOCK_DIM_Z)                \
+  __op__(maxGridSize[0], MAX_GRID_DIM_X)                   \
+  __op__(maxGridSize[1], MAX_GRID_DIM_Y)                   \
+  __op__(maxGridSize[2], MAX_GRID_DIM_Z)                   \
+  __op__(maxSurface1D,  MAXIMUM_SURFACE1D_WIDTH)           \
+  __op__(maxSurface2D[0], MAXIMUM_SURFACE2D_WIDTH)         \
+  __op__(maxSurface2D[1], MAXIMUM_SURFACE2D_HEIGHT)        \
+  __op__(maxSurface3D[0], MAXIMUM_SURFACE3D_WIDTH)         \
+  __op__(maxSurface3D[1], MAXIMUM_SURFACE3D_HEIGHT)        \
+  __op__(maxSurface3D[2], MAXIMUM_SURFACE3D_DEPTH)         \
+  __op__(maxSurface1DLayered[0],  MAXIMUM_SURFACE1D_LAYERED_WIDTH)           \
+  __op__(maxSurface1DLayered[1],  MAXIMUM_SURFACE1D_LAYERED_LAYERS)          \
+  __op__(maxSurface2DLayered[0], MAXIMUM_SURFACE2D_LAYERED_WIDTH)            \
+  __op__(maxSurface2DLayered[1], MAXIMUM_SURFACE2D_LAYERED_HEIGHT)           \
+  __op__(maxSurface2DLayered[2], MAXIMUM_SURFACE2D_LAYERED_LAYERS)           \
+  __op__(maxSurfaceCubemap, MAXIMUM_SURFACECUBEMAP_WIDTH)                    \
+  __op__(maxSurfaceCubemapLayered[0], MAXIMUM_SURFACECUBEMAP_LAYERED_WIDTH)  \
+  __op__(maxSurfaceCubemapLayered[1], MAXIMUM_SURFACECUBEMAP_LAYERED_LAYERS) \
+  __op__(clockRate, CLOCK_RATE)                          \
+  __op__(totalConstMem, TOTAL_CONSTANT_MEMORY)           \
+  __op__(deviceOverlap, GPU_OVERLAP)                     \
+  __op__(multiProcessorCount, MULTIPROCESSOR_COUNT)      \
+  __op__(kernelExecTimeoutEnabled, KERNEL_EXEC_TIMEOUT)  \
+  __op__(integrated, INTEGRATED)                         \
+  __op__(canMapHostMemory, CAN_MAP_HOST_MEMORY)          \
+  __op__(computeMode, COMPUTE_MODE)                      \
+  __op__(concurrentKernels, CONCURRENT_KERNELS)          \
+  __op__(ECCEnabled, ECC_ENABLED)                        \
+  __op__(pciBusID, PCI_BUS_ID)                           \
+  __op__(pciDeviceID, PCI_DEVICE_ID)                     \
+  __op__(pciDomainID, PCI_DOMAIN_ID)                     \
+  __op__(tccDriver, TCC_DRIVER)                          \
+  __op__(asyncEngineCount, ASYNC_ENGINE_COUNT)           \
+  __op__(unifiedAddressing, UNIFIED_ADDRESSING)          \
+  __op__(memoryClockRate, MEMORY_CLOCK_RATE)             \
+  __op__(memoryBusWidth, GLOBAL_MEMORY_BUS_WIDTH)        \
+  __op__(l2CacheSize, L2_CACHE_SIZE)                     \
+  __op__(maxThreadsPerMultiProcessor, MAX_THREADS_PER_MULTIPROCESSOR) \
+  __op__(streamPrioritiesSupported, STREAM_PRIORITIES_SUPPORTED)      \
+  __op__(globalL1CacheSupported, GLOBAL_L1_CACHE_SUPPORTED)           \
+  __op__(localL1CacheSupported, LOCAL_L1_CACHE_SUPPORTED)             \
+  __op__(sharedMemPerMultiprocessor, MAX_SHARED_MEMORY_PER_MULTIPROCESSOR)  \
+  __op__(regsPerMultiprocessor, MAX_REGISTERS_PER_MULTIPROCESSOR)           \
+  __op__(managedMemory, MANAGED_MEMORY)                                     \
+  __op__(isMultiGpuBoard, MULTI_GPU_BOARD)                                  \
+  __op__(multiGpuBoardGroupID, MULTI_GPU_BOARD_GROUP_ID)
+
+#if CUDA_VERSION >= 8000
+#define CUDA_DEVICE_ATTRIBUTES_8_0(__op__)                                        \
+  __op__(singleToDoublePrecisionPerfRatio, SINGLE_TO_DOUBLE_PRECISION_PERF_RATIO) \
+  __op__(pageableMemoryAccess, PAGEABLE_MEMORY_ACCESS)                            \
+  __op__(concurrentManagedAccess, CONCURRENT_MANAGED_ACCESS)
+#else
+#define CUDA_DEVICE_ATTRIBUTES_8_0(__op__)
+#endif
+
+#if CUDA_VERSION >= 9000
+#define CUDA_DEVICE_ATTRIBUTES_9_0(__op__)                                            \
+  __op__(computePreemptionSupported, COMPUTE_PREEMPTION_SUPPORTED)                    \
+  __op__(canUseHostPointerForRegisteredMem, CAN_USE_HOST_POINTER_FOR_REGISTERED_MEM)  \
+  __op__(cooperativeLaunch, COOPERATIVE_LAUNCH)                                       \
+  __op__(cooperativeMultiDeviceLaunch, COOPERATIVE_MULTI_DEVICE_LAUNCH)               \
+  __op__(sharedMemPerBlockOptin, MAX_SHARED_MEMORY_PER_BLOCK_OPTIN)
+#else
+#define CUDA_DEVICE_ATTRIBUTES_9_0(__op__)
+#endif
+
+#if CUDA_VERSION >= 9200
+#define CUDA_DEVICE_ATTRIBUTES_9_2(__op__)                                                      \
+  __op__(pageableMemoryAccessUsesHostPageTables, PAGEABLE_MEMORY_ACCESS_USES_HOST_PAGE_TABLES)  \
+  __op__(directManagedMemAccessFromHost, DIRECT_MANAGED_MEM_ACCESS_FROM_HOST)
+#else
+#define CUDA_DEVICE_ATTRIBUTES_9_2(__op__)
+#endif
+#if CUDA_VERSION >= 11000
+#define CUDA_DEVICE_ATTRIBUTES_11_0(__op__)                         \
+  __op__(maxBlocksPerMultiProcessor, MAX_BLOCKS_PER_MULTIPROCESSOR) \
+  __op__(accessPolicyMaxWindowSize, MAX_ACCESS_POLICY_WINDOW_SIZE)
+#else
+#define CUDA_DEVICE_ATTRIBUTES_11_0(__op__)
+#endif
+
+#define CUDA_DEVICE_ATTRIBUTES(__op__)   \
+  CUDA_DEVICE_ATTRIBUTES_PRE_8_0(__op__) \
+  CUDA_DEVICE_ATTRIBUTES_8_0 (__op__)    \
+  CUDA_DEVICE_ATTRIBUTES_9_0 (__op__)    \
+  CUDA_DEVICE_ATTRIBUTES_9_2 (__op__)    \
+  CUDA_DEVICE_ATTRIBUTES_11_0(__op__)
+
   REALM_PUBLIC_API
   cudaError_t cudaGetDeviceProperties(cudaDeviceProp *prop, int index)
   {
-    // doesn't need a current device - the index is supplied
-    CUdevice device;
-    CHECK_CU( cuDeviceGet(&device, index) );
-    CHECK_CU( cuDeviceGetName(prop->name, 255, device) );
-    CHECK_CU( cuDeviceTotalMem(&(prop->totalGlobalMem), device) );
-#define GET_DEVICE_PROP(member, name)					\
-    do {								\
-      int tmp;								\
-      CHECK_CU( cuDeviceGetAttribute(&tmp, CU_DEVICE_ATTRIBUTE_##name, device) ); \
-      prop->member = tmp;						\
-    } while(0)
-    // SCREW TEXTURES AND SURFACES FOR NOW!
-    GET_DEVICE_PROP(sharedMemPerBlock, MAX_SHARED_MEMORY_PER_BLOCK);
-    GET_DEVICE_PROP(regsPerBlock, MAX_REGISTERS_PER_BLOCK);
-    GET_DEVICE_PROP(warpSize, WARP_SIZE);
-    GET_DEVICE_PROP(memPitch, MAX_PITCH);
-    GET_DEVICE_PROP(maxThreadsPerBlock, MAX_THREADS_PER_BLOCK);
-    GET_DEVICE_PROP(maxThreadsDim[0], MAX_BLOCK_DIM_X);
-    GET_DEVICE_PROP(maxThreadsDim[1], MAX_BLOCK_DIM_Y);
-    GET_DEVICE_PROP(maxThreadsDim[2], MAX_BLOCK_DIM_Z);
-    GET_DEVICE_PROP(maxGridSize[0], MAX_GRID_DIM_X);
-    GET_DEVICE_PROP(maxGridSize[1], MAX_GRID_DIM_Y);
-    GET_DEVICE_PROP(maxGridSize[2], MAX_GRID_DIM_Z);
-    GET_DEVICE_PROP(clockRate, CLOCK_RATE);
-    GET_DEVICE_PROP(totalConstMem, TOTAL_CONSTANT_MEMORY);
-    GET_DEVICE_PROP(major, COMPUTE_CAPABILITY_MAJOR);
-    GET_DEVICE_PROP(minor, COMPUTE_CAPABILITY_MINOR);
-    GET_DEVICE_PROP(deviceOverlap, GPU_OVERLAP);
-    GET_DEVICE_PROP(multiProcessorCount, MULTIPROCESSOR_COUNT);
-    GET_DEVICE_PROP(kernelExecTimeoutEnabled, KERNEL_EXEC_TIMEOUT);
-    GET_DEVICE_PROP(integrated, INTEGRATED);
-    GET_DEVICE_PROP(canMapHostMemory, CAN_MAP_HOST_MEMORY);
-    GET_DEVICE_PROP(computeMode, COMPUTE_MODE);
-    GET_DEVICE_PROP(concurrentKernels, CONCURRENT_KERNELS);
-    GET_DEVICE_PROP(ECCEnabled, ECC_ENABLED);
-    GET_DEVICE_PROP(pciBusID, PCI_BUS_ID);
-    GET_DEVICE_PROP(pciDeviceID, PCI_DEVICE_ID);
-    GET_DEVICE_PROP(pciDomainID, PCI_DOMAIN_ID);
-    GET_DEVICE_PROP(tccDriver, TCC_DRIVER);
-    GET_DEVICE_PROP(asyncEngineCount, ASYNC_ENGINE_COUNT);
-    GET_DEVICE_PROP(unifiedAddressing, UNIFIED_ADDRESSING);
-    GET_DEVICE_PROP(memoryClockRate, MEMORY_CLOCK_RATE);
-    GET_DEVICE_PROP(memoryBusWidth, GLOBAL_MEMORY_BUS_WIDTH);
-    GET_DEVICE_PROP(l2CacheSize, L2_CACHE_SIZE);
-    GET_DEVICE_PROP(maxThreadsPerMultiProcessor, MAX_THREADS_PER_MULTIPROCESSOR);
-    GET_DEVICE_PROP(streamPrioritiesSupported, STREAM_PRIORITIES_SUPPORTED);
-    GET_DEVICE_PROP(globalL1CacheSupported, GLOBAL_L1_CACHE_SUPPORTED);
-    GET_DEVICE_PROP(localL1CacheSupported, LOCAL_L1_CACHE_SUPPORTED);
-    GET_DEVICE_PROP(sharedMemPerMultiprocessor, MAX_SHARED_MEMORY_PER_MULTIPROCESSOR);
-    GET_DEVICE_PROP(regsPerMultiprocessor, MAX_REGISTERS_PER_MULTIPROCESSOR);
-    GET_DEVICE_PROP(managedMemory, MANAGED_MEMORY);
-    GET_DEVICE_PROP(isMultiGpuBoard, MULTI_GPU_BOARD);
-    GET_DEVICE_PROP(multiGpuBoardGroupID, MULTI_GPU_BOARD_GROUP_ID);
-#if CUDA_VERSION >= 8000
-    GET_DEVICE_PROP(singleToDoublePrecisionPerfRatio, SINGLE_TO_DOUBLE_PRECISION_PERF_RATIO);
-    GET_DEVICE_PROP(pageableMemoryAccess, PAGEABLE_MEMORY_ACCESS);
-    GET_DEVICE_PROP(concurrentManagedAccess, CONCURRENT_MANAGED_ACCESS);
-#endif
-#if CUDA_VERSION >= 9000
-    GET_DEVICE_PROP(computePreemptionSupported, COMPUTE_PREEMPTION_SUPPORTED);
-    GET_DEVICE_PROP(canUseHostPointerForRegisteredMem, CAN_USE_HOST_POINTER_FOR_REGISTERED_MEM);
-    GET_DEVICE_PROP(cooperativeLaunch, COOPERATIVE_LAUNCH);
-    GET_DEVICE_PROP(cooperativeMultiDeviceLaunch, COOPERATIVE_MULTI_DEVICE_LAUNCH);
-#endif
-#if CUDA_VERSION >= 9200
-    GET_DEVICE_PROP(pageableMemoryAccessUsesHostPageTables, PAGEABLE_MEMORY_ACCESS_USES_HOST_PAGE_TABLES);
-    GET_DEVICE_PROP(directManagedMemAccessFromHost, DIRECT_MANAGED_MEM_ACCESS_FROM_HOST);
-#endif
-#if CUDA_VERSION >= 11000
-    GET_DEVICE_PROP(maxBlocksPerMultiProcessor, MAX_BLOCKS_PER_MULTIPROCESSOR);
-    GET_DEVICE_PROP(accessPolicyMaxWindowSize, MAX_ACCESS_POLICY_WINDOW_SIZE);
-#endif
-#undef GET_DEVICE_PROP
-    return cudaSuccess;
+    GPUProcessor *p = get_gpu_or_die("cudaGetDeviceProperties");
+    const std::vector<GPUInfo*> &infos = p->gpu->module->gpu_info;
+    for (GPUInfo* info : infos) {
+      if (info->index != index)
+        continue;
+
+      if (info->prop.totalGlobalMem != info->totalGlobalMem) {
+        // TODO: Replace with cuDeviceGetAttributes to batch
+        #define CUDA_GET_DEVICE_PROP(name, attr)                     \
+          do {								                                       \
+            int val = 0;                                             \
+            CHECK_CU( CUDA_DRIVER_FNPTR(cuDeviceGetAttribute)        \
+                      (&val, CU_DEVICE_ATTRIBUTE_##attr, info->device) );     \
+            info->prop.name = val;                                           \
+          } while(0);
+        CUDA_DEVICE_ATTRIBUTES(CUDA_GET_DEVICE_PROP);
+
+        strncpy(info->prop.name, info->name, sizeof(info->prop.name));
+        info->prop.totalGlobalMem = info->totalGlobalMem;
+        #undef CUDA_GET_DEVICE_PROP
+      }
+
+      memcpy(prop, &info->prop, sizeof(*prop));
+
+      return cudaSuccess;
+    }
+    return cudaErrorInvalidDevice;
   }
       
   REALM_PUBLIC_API
@@ -893,7 +945,7 @@ extern "C" {
     CUdevice_attribute cu_attr = (CUdevice_attribute)attr;
     CHECK_CU( cuDeviceGetAttribute(value, cu_attr, device) );
     return cudaSuccess;
-  }
+  } 
 
   REALM_PUBLIC_API
   cudaError_t cudaFuncGetAttributes(cudaFuncAttributes *attr, const void *func)
@@ -927,6 +979,127 @@ extern "C" {
     return cudaSuccess;
   }
 
+#if CUDA_VERSION >= 9000
+  REALM_PUBLIC_API
+  cudaError_t cudaFuncSetAttribute(const void *func, cudaFuncAttribute attr, int value)
+  {
+    GPUProcessor *p = get_gpu_or_die("cudaFuncSetAttribute");
+
+    CUfunction handle = p->gpu->lookup_function(func);
+    switch (attr)
+    {
+      case cudaFuncAttributeMaxDynamicSharedMemorySize:
+        {
+          CHECK_CU( cuFuncSetAttribute(handle, CU_FUNC_ATTRIBUTE_MAX_DYNAMIC_SHARED_SIZE_BYTES, value) );
+          return cudaSuccess;
+        }
+      case cudaFuncAttributePreferredSharedMemoryCarveout:
+        {
+          CHECK_CU( cuFuncSetAttribute(handle, CU_FUNC_ATTRIBUTE_PREFERRED_SHARED_MEMORY_CARVEOUT, value) );
+          return cudaSuccess;
+        }
+      default:
+        break;
+    }
+    return cudaErrorInvalidValue;
+  }
+#endif
+
+  REALM_PUBLIC_API
+  cudaError_t cudaFuncSetCacheConfig(const void *func, cudaFuncCache cacheConfig)
+  {
+    GPUProcessor *p = get_gpu_or_die("cudaFuncSetCacheConfig");
+
+    CUfunction handle = p->gpu->lookup_function(func);
+    switch (cacheConfig)
+    {
+      case cudaFuncCachePreferNone:
+        {
+          CHECK_CU( cuFuncSetCacheConfig(handle, CU_FUNC_CACHE_PREFER_NONE) ); 
+          return cudaSuccess;
+        }
+      case cudaFuncCachePreferShared:
+        {
+          CHECK_CU( cuFuncSetCacheConfig(handle, CU_FUNC_CACHE_PREFER_SHARED) ); 
+          return cudaSuccess;
+        }
+      case cudaFuncCachePreferL1:
+        {
+          CHECK_CU( cuFuncSetCacheConfig(handle, CU_FUNC_CACHE_PREFER_L1) ); 
+          return cudaSuccess;
+        }
+      case cudaFuncCachePreferEqual:
+        {
+          CHECK_CU( cuFuncSetCacheConfig(handle, CU_FUNC_CACHE_PREFER_EQUAL) ); 
+          return cudaSuccess;
+        }
+      default:
+        break;
+    }
+    return cudaErrorInvalidValue;
+  }
+
+  REALM_PUBLIC_API
+  cudaError_t cudaDeviceSetCacheConfig(cudaFuncCache config)
+  {
+    get_gpu_or_die("cudaDeviceSetCacheConfig");
+    switch (config)
+    {
+      case cudaFuncCachePreferNone:
+        {
+          CHECK_CU( cuCtxSetCacheConfig(CU_FUNC_CACHE_PREFER_NONE) ); 
+          return cudaSuccess;
+        }
+      case cudaFuncCachePreferShared:
+        {
+          CHECK_CU( cuCtxSetCacheConfig(CU_FUNC_CACHE_PREFER_SHARED) ); 
+          return cudaSuccess;
+        }
+      case cudaFuncCachePreferL1:
+        {
+          CHECK_CU( cuCtxSetCacheConfig(CU_FUNC_CACHE_PREFER_L1) ); 
+          return cudaSuccess;
+        }
+      case cudaFuncCachePreferEqual:
+        {
+          CHECK_CU( cuCtxSetCacheConfig(CU_FUNC_CACHE_PREFER_EQUAL) ); 
+          return cudaSuccess;
+        }
+      default:
+        break;
+    }
+    return cudaErrorInvalidValue;
+  }
+
+  REALM_PUBLIC_API
+  cudaError_t cudaFuncSetSharedMemConfig(const void *func, cudaSharedMemConfig config)
+  {
+    GPUProcessor *p = get_gpu_or_die("cudaFuncSetSharedMemConfig");
+
+    CUfunction handle = p->gpu->lookup_function(func);
+    switch (config)
+    {
+      case cudaSharedMemBankSizeDefault:
+        {
+          CHECK_CU( cuFuncSetSharedMemConfig(handle, CU_SHARED_MEM_CONFIG_DEFAULT_BANK_SIZE) );
+          return cudaSuccess;
+        }
+      case cudaSharedMemBankSizeFourByte:
+        {
+          CHECK_CU( cuFuncSetSharedMemConfig(handle, CU_SHARED_MEM_CONFIG_FOUR_BYTE_BANK_SIZE) );
+          return cudaSuccess;
+        }
+      case cudaSharedMemBankSizeEightByte:
+        {
+          CHECK_CU( cuFuncSetSharedMemConfig(handle, CU_SHARED_MEM_CONFIG_EIGHT_BYTE_BANK_SIZE) );
+          return cudaSuccess;
+        }
+      default:
+        break;
+    }
+    return cudaErrorInvalidValue;
+  }
+
 #if CUDA_VERSION >= 11000
   REALM_PUBLIC_API
   cudaError_t cudaGetFuncBySymbol(cudaFunction_t *funcPtr, const void *symbolPtr)
@@ -1023,23 +1196,7 @@ extern "C" {
     // Ignore calls to set the device here since we already
     // know which device we are running on
     return cudaSuccess;
-  }
-
-  REALM_PUBLIC_API
-  cudaError_t cudaFuncSetCacheConfig(const void *func, cudaFuncCache config)
-  {
-    get_gpu_or_die("cudaFuncSetCacheConfig");
-    // TODO: actually do something with this
-    return cudaSuccess;
-  }
-
-  REALM_PUBLIC_API
-  cudaError_t cudaDeviceSetCacheConfig(cudaFuncCache config)
-  {
-    get_gpu_or_die("cudaDeviceSetCacheConfig");
-    // TODO: actually do something with this
-    return cudaSuccess;
-  }
+  } 
 
   REALM_PUBLIC_API
   cudaError_t cudaThreadSetCacheConfig(cudaFuncCache config)
@@ -1141,6 +1298,7 @@ extern "C" {
     }
   }
 
+  REALM_PUBLIC_API
   cudaError_t cudaSetDeviceFlags(unsigned flags)
   {
     get_gpu_or_die("cudaSetDeviceFlags");
@@ -1150,6 +1308,7 @@ extern "C" {
     return cudaErrorSetOnActiveProcess;
   }
 
+  REALM_PUBLIC_API
   cudaError_t cudaGetDeviceFlags(unsigned *flags)
   {
     get_gpu_or_die("cudaGetDeviceFlags");
@@ -1165,4 +1324,137 @@ extern "C" {
     return (cudaError_t)cuCtxGetFlags(flags);
   }
 
+  REALM_PUBLIC_API
+  cudaError_t cudaMalloc3DArray(cudaArray_t *array,
+                                const cudaChannelFormatDesc *desc,
+                                cudaExtent extent, unsigned flags)
+  {
+    get_gpu_or_die("cudaMalloc3DArray");
+
+    CUarray handle;
+    CUDA_ARRAY3D_DESCRIPTOR d;
+    d.Width = extent.width;
+    d.Height = extent.height;
+    d.Depth = extent.depth;
+
+    // runtime and driver describe channel count/format very differently
+    int channel_count = 0;
+    int channel_width = 0;
+    if(desc->x != 0) {
+      channel_count++;
+      channel_width = desc->x;
+    }
+    if(desc->y != 0) {
+      channel_count++;
+      if(channel_width && (desc->y != channel_width))
+        return cudaErrorInvalidValue;
+      channel_width = desc->y;
+    }
+    if(desc->z != 0) {
+      channel_count++;
+      if(channel_width && (desc->z != channel_width))
+        return cudaErrorInvalidValue;
+      channel_width = desc->z;
+    }
+    if(desc->w != 0) {
+      channel_count++;
+      if(channel_width && (desc->w != channel_width))
+        return cudaErrorInvalidValue;
+      channel_width = desc->w;
+    }
+
+    switch(desc->f) {
+    case cudaChannelFormatKindSigned:
+      {
+        switch(channel_width) {
+        case 8 : { d.Format = CU_AD_FORMAT_SIGNED_INT8; break; }
+        case 16 : { d.Format = CU_AD_FORMAT_SIGNED_INT16; break; }
+        case 32 : { d.Format = CU_AD_FORMAT_SIGNED_INT32; break; }
+        default: return cudaErrorInvalidValue;
+        }
+        break;
+      }
+    case cudaChannelFormatKindUnsigned:
+      {
+        switch(channel_width) {
+        case 8 : { d.Format = CU_AD_FORMAT_UNSIGNED_INT8; break; }
+        case 16 : { d.Format = CU_AD_FORMAT_UNSIGNED_INT16; break; }
+        case 32 : { d.Format = CU_AD_FORMAT_UNSIGNED_INT32; break; }
+        default: return cudaErrorInvalidValue;
+        }
+        break;
+      }
+    case cudaChannelFormatKindFloat:
+      {
+        switch(channel_width) {
+        case 16 : { d.Format = CU_AD_FORMAT_HALF; break; }
+        case 32 : { d.Format = CU_AD_FORMAT_FLOAT; break; }
+        default: return cudaErrorInvalidValue;
+        }
+        break;
+      }
+    default: return cudaErrorInvalidValue;
+    }
+
+    d.NumChannels = channel_count;
+
+    d.Flags = 0;
+    if((flags & cudaArrayLayered) != 0)
+      d.Flags |= CUDA_ARRAY3D_LAYERED;
+    if((flags & cudaArrayCubemap) != 0)
+      d.Flags |= CUDA_ARRAY3D_CUBEMAP;
+    if((flags & cudaArraySurfaceLoadStore) != 0)
+      d.Flags |= CUDA_ARRAY3D_SURFACE_LDST;
+    if((flags & cudaArrayTextureGather) != 0)
+      d.Flags |= CUDA_ARRAY3D_TEXTURE_GATHER;
+    // ignore flags we don't understand and hope for the best
+
+    CUresult ret = cuArray3DCreate(&handle, &d);
+    if(ret == CUDA_SUCCESS) {
+      *array = (struct cudaArray *)handle;
+      return cudaSuccess;
+    } else
+      return (cudaError_t)ret;
+  }
+
+  REALM_PUBLIC_API
+  cudaError_t cudaFreeArray(cudaArray_t array)
+  {
+    get_gpu_or_die("cudaFreeArray");
+
+    return (cudaError_t)cuArrayDestroy((CUarray)array);
+  }
+
+  REALM_PUBLIC_API
+  cudaError_t cudaCreateSurfaceObject(cudaSurfaceObject_t *object,
+                                      const cudaResourceDesc *desc)
+  {
+    get_gpu_or_die("cudaCreateSurfaceObject");
+
+    if(desc->resType != cudaResourceTypeArray)
+      return cudaErrorInvalidValue;
+
+    CUDA_RESOURCE_DESC d;
+    memset(&d, 0, sizeof(d));
+    d.resType = CU_RESOURCE_TYPE_ARRAY;
+    d.res.array.hArray = (CUarray)(desc->res.array.array);
+    d.flags = 0;
+
+    CUsurfObject surf;
+    CUresult ret = cuSurfObjectCreate(&surf, &d);
+    if(ret == CUDA_SUCCESS) {
+      *object = (cudaSurfaceObject_t)surf;
+      return cudaSuccess;
+    } else
+      return (cudaError_t)ret;
+  }
+
+  REALM_PUBLIC_API
+  cudaError_t cudaDestroySurfaceObject(cudaSurfaceObject_t object)
+  {
+    get_gpu_or_die("cudaDestroySurfaceObject");
+
+    return (cudaError_t)cuSurfObjectDestroy((CUsurfObject)object);
+  }
+
 }; // extern "C"
