@@ -1,4 +1,4 @@
-/* Copyright 2021 Stanford University, NVIDIA Corporation
+/* Copyright 2023 Stanford University, NVIDIA Corporation
  *
  * Licensed under the Apache License, Version 2.0 (the "License");
  * you may not use this file except in compliance with the License.
@@ -28,62 +28,38 @@ namespace Realm {
   extern Logger log_part;
   extern Logger log_uop_timing;
 
-
   template <int N, typename T>
   template <int N2, typename T2>
-  Event IndexSpace<N,T>::create_subspaces_by_image(const std::vector<FieldDataDescriptor<IndexSpace<N2,T2>,Point<N,T> > >& field_data,
-							   const std::vector<IndexSpace<N2,T2> >& sources,
-							   std::vector<IndexSpace<N,T> >& images,
-							   const ProfilingRequestSet &reqs,
-							   Event wait_on /*= Event::NO_EVENT*/) const
-  {
-    // output vector should start out empty
-    assert(images.empty());
+  Event IndexSpace<N, T>::create_subspaces_by_image(
+      const DomainTransform<N, T, N2, T2> &domain_transform,
+      const std::vector<IndexSpace<N2, T2>> &sources,
+      std::vector<IndexSpace<N, T>> &images, const ProfilingRequestSet &reqs,
+      Event wait_on) const {
+   // output vector should start out empty
+   assert(images.empty());
 
-    GenEventImpl *finish_event = GenEventImpl::create_genevent();
-    Event e = finish_event->current_event();
-    ImageOperation<N,T,N2,T2> *op = new ImageOperation<N,T,N2,T2>(*this, field_data, reqs, finish_event, ID(e).event_generation());
+   GenEventImpl *finish_event = GenEventImpl::create_genevent();
+   Event e = finish_event->current_event();
 
-    size_t n = sources.size();
-    images.resize(n);
-    for(size_t i = 0; i < n; i++) {
-      images[i] = op->add_source(sources[i]);
-      log_dpops.info() << "image: " << *this << " src=" << sources[i] << " -> " << images[i] << " (" << e << ")";
-    }
+   ImageOperation<N, T, N2, T2> *op = new ImageOperation<N, T, N2, T2>(
+       *this, domain_transform, reqs, finish_event, ID(e).event_generation());
 
-    op->launch(wait_on);
-    return e;
+   size_t n = sources.size();
+   images.resize(n);
+   for (size_t i = 0; i < n; i++) {
+    images[i] = op->add_source(sources[i]);
+    log_dpops.info() << "image: " << *this << " src=" << sources[i] << " -> "
+                     << images[i] << " (" << e << ")";
+   }
+
+   op->launch(wait_on);
+   return e;
   }
 
   template <int N, typename T>
   template <int N2, typename T2>
-  Event IndexSpace<N,T>::create_subspaces_by_image(const std::vector<FieldDataDescriptor<IndexSpace<N2,T2>,Rect<N,T> > >& field_data,
-							   const std::vector<IndexSpace<N2,T2> >& sources,
-							   std::vector<IndexSpace<N,T> >& images,
-							   const ProfilingRequestSet &reqs,
-							   Event wait_on /*= Event::NO_EVENT*/) const
-  {
-    // output vector should start out empty
-    assert(images.empty());
-
-    GenEventImpl *finish_event = GenEventImpl::create_genevent();
-    Event e = finish_event->current_event();
-    ImageOperation<N,T,N2,T2> *op = new ImageOperation<N,T,N2,T2>(*this, field_data, reqs, finish_event, ID(e).event_generation());
-
-    size_t n = sources.size();
-    images.resize(n);
-    for(size_t i = 0; i < n; i++) {
-      images[i] = op->add_source(sources[i]);
-      log_dpops.info() << "image: " << *this << " src=" << sources[i] << " -> " << images[i] << " (" << e << ")";
-    }
-
-    op->launch(wait_on);
-    return e;
-  }
-
-  template <int N, typename T>
-  template <int N2, typename T2>
-  Event IndexSpace<N,T>::create_subspaces_by_image_with_difference(const std::vector<FieldDataDescriptor<IndexSpace<N2,T2>,Point<N,T> > >& field_data,
+  Event IndexSpace<N,T>::create_subspaces_by_image_with_difference(
+      const DomainTransform<N, T, N2, T2> &domain_transform,
 							   const std::vector<IndexSpace<N2,T2> >& sources,
 							   const std::vector<IndexSpace<N,T> >& diff_rhss,
 							   std::vector<IndexSpace<N,T> >& images,
@@ -95,7 +71,7 @@ namespace Realm {
 
     GenEventImpl *finish_event = GenEventImpl::create_genevent();
     Event e = finish_event->current_event();
-    ImageOperation<N,T,N2,T2> *op = new ImageOperation<N,T,N2,T2>(*this, field_data, reqs, finish_event, ID(e).event_generation());
+    ImageOperation<N,T,N2,T2> *op = new ImageOperation<N,T,N2,T2>(*this, domain_transform, reqs, finish_event, ID(e).event_generation());
 
     size_t n = sources.size();
     images.resize(n);
@@ -356,8 +332,14 @@ namespace Realm {
       return;
     }
 
-    // instance index spaces should always be valid
-    assert(inst_space.is_valid(true /*precise*/));
+    // Need valid data for the instance space
+    if (!inst_space.dense()) {
+      // it's safe to add the count after the registration only because we initialized
+      //  the count to 2 instead of 1
+      bool registered = SparsityMapImpl<N2,T2>::lookup(inst_space.sparsity)->add_waiter(this, true /*precise*/);
+      if(registered)
+        wait_count.fetch_add(1);
+    }
 
     // need valid data for each source
     for(size_t i = 0; i < sources.size(); i++) {
@@ -438,26 +420,14 @@ namespace Realm {
   // class ImageOperation<N,T,N2,T2>
 
   template <int N, typename T, int N2, typename T2>
-  ImageOperation<N,T,N2,T2>::ImageOperation(const IndexSpace<N,T>& _parent,
-					    const std::vector<FieldDataDescriptor<IndexSpace<N2,T2>,Point<N,T> > >& _field_data,
-					    const ProfilingRequestSet &reqs,
-					    GenEventImpl *_finish_event,
-					    EventImpl::gen_t _finish_gen)
-    : PartitioningOperation(reqs, _finish_event, _finish_gen)
-    , parent(_parent)
-    , ptr_data(_field_data)
-  {}
-
-  template <int N, typename T, int N2, typename T2>
-  ImageOperation<N,T,N2,T2>::ImageOperation(const IndexSpace<N,T>& _parent,
-					    const std::vector<FieldDataDescriptor<IndexSpace<N2,T2>,Rect<N,T> > >& _field_data,
-					    const ProfilingRequestSet &reqs,
-					    GenEventImpl *_finish_event,
-					    EventImpl::gen_t _finish_gen)
-    : PartitioningOperation(reqs, _finish_event, _finish_gen)
-    , parent(_parent)
-    , range_data(_field_data)
-  {}
+  ImageOperation<N, T, N2, T2>::ImageOperation(
+      const IndexSpace<N, T> &_parent,
+      const DomainTransform<N, T, N2, T2> &_domain_transform,
+      const ProfilingRequestSet &reqs, GenEventImpl *_finish_event,
+      EventImpl::gen_t _finish_gen)
+      : PartitioningOperation(reqs, _finish_event, _finish_gen),
+        parent(_parent),
+        domain_transform(_domain_transform) {}
 
   template <int N, typename T, int N2, typename T2>
   ImageOperation<N,T,N2,T2>::~ImageOperation(void)
@@ -476,14 +446,15 @@ namespace Realm {
 
     // if the source has a sparsity map, use the same node - otherwise
     // get a sparsity ID by round-robin'ing across the nodes that have field data
-    int target_node;
+    int target_node = 0;
     if(!source.dense())
       target_node = ID(source.sparsity).sparsity_creator_node();
     else
-      if(!ptr_data.empty())
-	target_node = ID(ptr_data[sources.size() % ptr_data.size()].inst).instance_owner_node();
+      if(!domain_transform.ptr_data.empty())
+	target_node = ID(domain_transform.ptr_data[sources.size() % domain_transform.ptr_data.size()].inst).instance_owner_node();
       else
-	target_node = ID(range_data[sources.size() % range_data.size()].inst).instance_owner_node();
+	target_node = ID(domain_transform.range_data[sources.size() % domain_transform.range_data.size()].inst).instance_owner_node();
+
     SparsityMap<N,T> sparsity = get_runtime()->get_available_sparsity_impl(target_node)->me.convert<SparsityMap<N,T> >();
     image.sparsity = sparsity;
 
@@ -511,10 +482,10 @@ namespace Realm {
     if(!source.dense())
       target_node = ID(source.sparsity).sparsity_creator_node();
     else
-      if(!ptr_data.empty())
-	target_node = ID(ptr_data[sources.size() % ptr_data.size()].inst).instance_owner_node();
+      if(!domain_transform.ptr_data.empty())
+	target_node = ID(domain_transform.ptr_data[sources.size() % domain_transform.ptr_data.size()].inst).instance_owner_node();
       else
-	target_node = ID(range_data[sources.size() % range_data.size()].inst).instance_owner_node();
+	target_node = ID(domain_transform.range_data[sources.size() % domain_transform.range_data.size()].inst).instance_owner_node();
     SparsityMap<N,T> sparsity = get_runtime()->get_available_sparsity_impl(target_node)->me.convert<SparsityMap<N,T> >();
     image.sparsity = sparsity;
 
@@ -526,60 +497,80 @@ namespace Realm {
   }
 
   template <int N, typename T, int N2, typename T2>
-  void ImageOperation<N,T,N2,T2>::execute(void)
-  {
-    if(!DeppartConfig::cfg_disable_intersection_optimization) {
-      // build the overlap tester based on the field index spaces - they're more likely to be known and
-      //  denser
-      ComputeOverlapMicroOp<N2,T2> *uop = new ComputeOverlapMicroOp<N2,T2>(this);
+  void ImageOperation<N, T, N2, T2>::execute(void) {
+   if (domain_transform.type ==
+       DomainTransform<N, T, N2, T2>::DomainTransformType::STRUCTURED) {
+    for (size_t i = 0; i < sources.size(); i++) {
+     SparsityMapImpl<N, T>::lookup(images[i])->set_contributor_count(1);
+    }
 
-      for(size_t i = 0; i < ptr_data.size(); i++)
-	uop->add_input_space(ptr_data[i].index_space);
+    StructuredImageMicroOp<N, T, N2, T2> *micro_op =
+        new StructuredImageMicroOp<N, T, N2, T2>(
+            parent, domain_transform.structured_transform);
 
-      for(size_t i = 0; i < range_data.size(); i++)
-	uop->add_input_space(range_data[i].index_space);
+    for (size_t j = 0; j < sources.size(); j++) {
+     micro_op->add_sparsity_output(sources[j], images[j]);
+    }
 
-      // we will ask this uop to also prefetch the sources we will intersect test against it
-      for(size_t i = 0; i < sources.size(); i++)
-	uop->add_extra_dependency(sources[i]);
+    micro_op->dispatch(this, /*inline_ok=*/true);
+   } else {
+    if (!DeppartConfig::cfg_disable_intersection_optimization) {
+     // build the overlap tester based on the field index spaces - they're more
+     // likely to be known and
+     //  denser
+     ComputeOverlapMicroOp<N2, T2> *uop =
+         new ComputeOverlapMicroOp<N2, T2>(this);
+
+     for (size_t i = 0; i < domain_transform.ptr_data.size(); i++)
+      uop->add_input_space(domain_transform.ptr_data[i].index_space);
+
+     for (size_t i = 0; i < domain_transform.range_data.size(); i++)
+      uop->add_input_space(domain_transform.range_data[i].index_space);
+
+     // we will ask this uop to also prefetch the sources we will intersect test
+     // against it
+     for (size_t i = 0; i < sources.size(); i++)
+      uop->add_extra_dependency(sources[i]);
+
+     uop->dispatch(this, true /* ok to run in this thread */);
+    } else {
+     // launch full cross-product of image micro ops right away
+     for (size_t i = 0; i < sources.size(); i++)
+      SparsityMapImpl<N, T>::lookup(images[i])->set_contributor_count(
+          domain_transform.ptr_data.size() +
+          domain_transform.range_data.size());
+
+     for (size_t i = 0; i < domain_transform.ptr_data.size(); i++) {
+      ImageMicroOp<N, T, N2, T2> *uop = new ImageMicroOp<N, T, N2, T2>(
+          parent, domain_transform.ptr_data[i].index_space,
+          domain_transform.ptr_data[i].inst,
+          domain_transform.ptr_data[i].field_offset, false /*ptrs*/);
+      for (size_t j = 0; j < sources.size(); j++)
+       if (diff_rhss.empty())
+        uop->add_sparsity_output(sources[j], images[j]);
+       else
+        uop->add_sparsity_output_with_difference(sources[j], diff_rhss[j],
+                                                 images[j]);
 
       uop->dispatch(this, true /* ok to run in this thread */);
-    } else {
-      // launch full cross-product of image micro ops right away
-      for(size_t i = 0; i < sources.size(); i++)
-	SparsityMapImpl<N,T>::lookup(images[i])->set_contributor_count(ptr_data.size() +
-								       range_data.size());
+     }
 
-      for(size_t i = 0; i < ptr_data.size(); i++) {
-	ImageMicroOp<N,T,N2,T2> *uop = new ImageMicroOp<N,T,N2,T2>(parent,
-								   ptr_data[i].index_space,
-								   ptr_data[i].inst,
-								   ptr_data[i].field_offset,
-								   false /*ptrs*/);
-	for(size_t j = 0; j < sources.size(); j++)
-          if(diff_rhss.empty())
-	    uop->add_sparsity_output(sources[j], images[j]);
-          else
-	    uop->add_sparsity_output_with_difference(sources[j], diff_rhss[j], images[j]);
+     for (size_t i = 0; i < domain_transform.range_data.size(); i++) {
+      ImageMicroOp<N, T, N2, T2> *uop = new ImageMicroOp<N, T, N2, T2>(
+          parent, domain_transform.range_data[i].index_space,
+          domain_transform.range_data[i].inst,
+          domain_transform.range_data[i].field_offset, true /*ranges*/);
+      for (size_t j = 0; j < sources.size(); j++)
+       if (diff_rhss.empty())
+        uop->add_sparsity_output(sources[j], images[j]);
+       else
+        uop->add_sparsity_output_with_difference(sources[j], diff_rhss[j],
+                                                 images[j]);
 
-	uop->dispatch(this, true /* ok to run in this thread */);
-      }
-
-      for(size_t i = 0; i < range_data.size(); i++) {
-	ImageMicroOp<N,T,N2,T2> *uop = new ImageMicroOp<N,T,N2,T2>(parent,
-								   range_data[i].index_space,
-								   range_data[i].inst,
-								   range_data[i].field_offset,
-								   true /*ranges*/);
-	for(size_t j = 0; j < sources.size(); j++)
-          if(diff_rhss.empty())
-	    uop->add_sparsity_output(sources[j], images[j]);
-          else
-	    uop->add_sparsity_output_with_difference(sources[j], diff_rhss[j], images[j]);
-
-	uop->dispatch(this, true /* ok to run in this thread */);
-      }
+      uop->dispatch(this, true /* ok to run in this thread */);
+     }
     }
+   }
   }
 
   template <int N, typename T, int N2, typename T2>
@@ -589,8 +580,8 @@ namespace Realm {
 
     // we asked the overlap tester to prefetch all the source data we need, so we can use it
     //  right away (and then delete it)
-    std::vector<std::set<int> > overlaps_by_field_data(ptr_data.size() +
-						       range_data.size());
+    std::vector<std::set<int> > overlaps_by_field_data(domain_transform.ptr_data.size() +
+						       domain_transform.range_data.size());
     for(size_t i = 0; i < sources.size(); i++) {
       std::set<int> overlaps_by_source;
 
@@ -608,15 +599,15 @@ namespace Realm {
     }
     delete overlap_tester;
 
-    for(size_t i = 0; i < ptr_data.size(); i++) {
+    for(size_t i = 0; i < domain_transform.ptr_data.size(); i++) {
       const std::set<int>& overlaps = overlaps_by_field_data[i];
       size_t n = overlaps.size();
       if(n == 0) continue;
 
       ImageMicroOp<N,T,N2,T2> *uop = new ImageMicroOp<N,T,N2,T2>(parent,
-								 ptr_data[i].index_space,
-								 ptr_data[i].inst,
-								 ptr_data[i].field_offset,
+								 domain_transform.ptr_data[i].index_space,
+								 domain_transform.ptr_data[i].inst,
+								 domain_transform.ptr_data[i].field_offset,
 								 false /*ptrs*/);
       for(std::set<int>::const_iterator it = overlaps.begin();
 	  it != overlaps.end();
@@ -630,15 +621,15 @@ namespace Realm {
       uop->dispatch(this, true /* ok to run in this thread */);
     }
 
-    for(size_t i = 0; i < range_data.size(); i++) {
-      const std::set<int>& overlaps = overlaps_by_field_data[i + ptr_data.size()];
+    for(size_t i = 0; i < domain_transform.range_data.size(); i++) {
+      const std::set<int>& overlaps = overlaps_by_field_data[i + domain_transform.ptr_data.size()];
       size_t n = overlaps.size();
       if(n == 0) continue;
 
       ImageMicroOp<N,T,N2,T2> *uop = new ImageMicroOp<N,T,N2,T2>(parent,
-								 range_data[i].index_space,
-								 range_data[i].inst,
-								 range_data[i].field_offset,
+								 domain_transform.range_data[i].index_space,
+								 domain_transform.range_data[i].inst,
+								 domain_transform.range_data[i].field_offset,
 								 true /*ranges*/);
       for(std::set<int>::const_iterator it = overlaps.begin();
 	  it != overlaps.end();
@@ -659,6 +650,128 @@ namespace Realm {
     os << "ImageOperation(" << parent << ")";
   }
 
+  ////////////////////////////////////////////////////////////////////////
+  //
+  // class StructuredImageMicroOp<N, T, N2, T2>
+
+  template <int N, typename T, int N2, typename T2>
+  StructuredImageMicroOp<N, T, N2, T2>::StructuredImageMicroOp(
+      const IndexSpace<N, T> &_parent,
+      const StructuredTransform<N, T, N2, T2> &_transform)
+      : parent_space(_parent), transform(_transform) {}
+
+  template <int N, typename T, int N2, typename T2>
+  StructuredImageMicroOp<N, T, N2, T2>::~StructuredImageMicroOp() {}
+
+  template <int N, typename T, int N2, typename T2>
+  void StructuredImageMicroOp<N, T, N2, T2>::dispatch(
+      PartitioningOperation *op, bool inline_ok) {
+    for (size_t i = 0; i < sources.size(); i++) {
+      if (!sources[i].dense()) {
+        bool registered = SparsityMapImpl<N2, T2>::lookup(sources[i].sparsity)
+                              ->add_waiter(this, true /*precise*/);
+        if (registered) wait_count.fetch_add(1);
+      }
+    }
+
+    if (!parent_space.dense()) {
+      bool registered = SparsityMapImpl<N, T>::lookup(parent_space.sparsity)
+                            ->add_waiter(this, true /*precise*/);
+      if (registered) wait_count.fetch_add(1);
+    }
+    this->finish_dispatch(op, inline_ok);
+  }
+
+  template <int N, typename T, int N2, typename T2>
+  void StructuredImageMicroOp<N, T, N2, T2>::add_sparsity_output(
+      IndexSpace<N2, T2> _source, SparsityMap<N, T> _sparsity) {
+   sources.push_back(_source);
+   sparsity_outputs.push_back(_sparsity);
+  }
+
+  template <int N, typename T, int N2, typename T2>
+  void StructuredImageMicroOp<N, T, N2, T2>::execute(void) {
+    TimeStamp ts("StructuredImageMicroOp::execute", true, &log_uop_timing);
+
+    if (!sparsity_outputs.empty()) {
+      std::map<int, HybridRectangleList<N, T> *> rect_map;
+
+      populate(rect_map);
+
+#ifdef DEBUG_PARTITIONING
+      std::cout << rect_map.size() << " non-empty images present in instance "
+                << inst << std::endl;
+      for (typename std::map<int, DenseRectangleList<N, T> *>::const_iterator
+               it = rect_map.begin();
+           it != rect_map.end(); it++)
+        std::cout << "  " << sources[it->first] << " = "
+                  << it->second->rects.size() << " rectangles" << std::endl;
+#endif
+
+      // iterate over sparsity outputs and contribute to all (even if we didn't
+      // have any points found for it)
+      for (size_t i = 0; i < sparsity_outputs.size(); i++) {
+        SparsityMapImpl<N, T> *impl =
+            SparsityMapImpl<N, T>::lookup(sparsity_outputs[i]);
+        typename std::map<int, HybridRectangleList<N, T> *>::const_iterator
+            it2 = rect_map.find(i);
+        if (it2 != rect_map.end()) {
+          impl->contribute_dense_rect_list(it2->second->convert_to_vector(),
+                                           false /*!disjoint*/);
+          delete it2->second;
+        } else
+          impl->contribute_nothing();
+      }
+    }
+  }
+
+  template <int N, typename T, int N2, typename T2>
+  void StructuredImageMicroOp<N, T, N2, T2>::populate(
+      std::map<int, HybridRectangleList<N, T> *> &bitmasks) {
+   // TODO(apyakhin@): Just for now we effectively defaulting to affine
+   // an transform. Consider adding both more generalized version that
+   // works for any type of structured transforms as well as specialized
+   // version.
+   std::vector<Rect<N, T>> parent_rects;
+   if (this->parent_space.dense()) {
+    parent_rects.push_back(this->parent_space.bounds);
+   } else {
+    for (IndexSpaceIterator<N, T> parent_it(this->parent_space);
+         parent_it.valid; parent_it.step()) {
+     parent_rects.push_back(parent_it.rect);
+    }
+   }
+
+   assert(!parent_rects.empty());
+   Rect<N, T> parent_bbox = parent_rects[0];
+   for (size_t i = 1; i < parent_rects.size(); i++) {
+    parent_bbox = parent_bbox.union_bbox(parent_rects[i]);
+   }
+
+   for (size_t i = 0; i < this->sources.size(); i++) {
+    for (IndexSpaceIterator<N2, T2> it2(this->sources[i]); it2.valid;
+         it2.step()) {
+     for (PointInRectIterator<N2, T2> point(it2.rect); point.valid;
+          point.step()) {
+      Point<N, T> source_point = transform[point.p];
+
+      if (!parent_bbox.contains(source_point)) continue;
+
+      for (const auto &parent_rect : parent_rects) {
+       if (parent_rect.contains(source_point)) {
+        HybridRectangleList<N, T> **bmpp = 0;
+        if (!bmpp) bmpp = &bitmasks[i];
+        if (!*bmpp) *bmpp = new HybridRectangleList<N, T>;
+        (*bmpp)->add_point(source_point);
+       }
+      }
+     }
+    }
+   }
+  }
+
+  ////////////////////////////////////////////////////////////////////////
+
   // instantiations of templates handled in image_tmpl.cc
 
-};
+  };  // namespace Realm
