@@ -1,4 +1,4 @@
-/* Copyright 2021 Stanford University, NVIDIA Corporation
+/* Copyright 2023 Stanford University, NVIDIA Corporation
  *
  * Licensed under the Apache License, Version 2.0 (the "License");
  * you may not use this file except in compliance with the License.
@@ -563,7 +563,7 @@ namespace Realm {
 	  //  from the next count down and starting a new clump
 
 	  size_t best_waste = ~size_t(0);
-	  size_t merge_idx;
+	  size_t merge_idx = 0;
 	  Rect<N,T> merge_rect;
 
 	  // if state was illegal, no modifications to it are legal
@@ -1251,11 +1251,12 @@ namespace Realm {
       // send partial messages first
       while(remaining > max_to_send) {
 	size_t bytes = max_to_send * sizeof(Rect<N,T>);
-	ActiveMessage<RemoteSparsityContrib> amsg(requestor, rdata, bytes);
+	ActiveMessage<RemoteSparsityContrib> amsg(requestor, bytes);
 	amsg->sparsity = me;
 	amsg->piece_count = 0;
         amsg->disjoint = true; // we've already de-overlapped everything
         amsg->total_count = total_count;
+	amsg.add_payload(rdata, bytes, PAYLOAD_COPY);
 	amsg.commit();
 
 	num_pieces++;
@@ -1265,11 +1266,12 @@ namespace Realm {
 
       // final message includes the count of all messages (including this one!)
       size_t bytes = remaining * sizeof(Rect<N,T>);
-      ActiveMessage<RemoteSparsityContrib> amsg(requestor, rdata, bytes);
+      ActiveMessage<RemoteSparsityContrib> amsg(requestor, bytes);
       amsg->sparsity = me;
       amsg->piece_count = num_pieces + 1;
       amsg->disjoint = true; // we've already de-overlapped everything
       amsg->total_count = total_count;
+      amsg.add_payload(rdata, bytes, PAYLOAD_COPY);
       amsg.commit();
     }
   }
