@@ -1,4 +1,4 @@
-/* Copyright 2021 Stanford University, NVIDIA Corporation
+/* Copyright 2023 Stanford University, NVIDIA Corporation
  *
  * Licensed under the Apache License, Version 2.0 (the "License");
  * you may not use this file except in compliance with the License.
@@ -252,7 +252,9 @@ namespace Realm {
 
       // external waiters on this node are notifies via a condition variable
       bool has_external_waiters;
-      CondVar external_waiter_condvar;
+      // use kernel mutex for timedwait functionality
+      KernelMutex external_waiter_mutex;
+      KernelMutex::CondVar external_waiter_condvar;
 
       // remote waiters are kept in a bitmask for the current generation - this is
       //  only maintained on the owner, who never has to worry about more than one
@@ -331,7 +333,8 @@ namespace Realm {
       bool get_result(gen_t result_gen, void *value, size_t value_size);
 
     public: //protected:
-      gen_t generation, gen_subscribed;
+      atomic<gen_t> generation; // can be read without holding mutex
+      atomic<gen_t> gen_subscribed;
       gen_t first_generation;
       BarrierImpl *next_free;
 
@@ -360,7 +363,9 @@ namespace Realm {
 
       // external waiters on this node are notifies via a condition variable
       bool has_external_waiters;
-      CondVar external_waiter_condvar;
+      // use kernel mutex for timedwait functionality
+      KernelMutex external_waiter_mutex;
+      KernelMutex::CondVar external_waiter_condvar;
 
       // a list of remote waiters and the latest generation they're interested in
       // also the latest generation that each node (that has ever subscribed) has been told about
@@ -416,7 +421,7 @@ namespace Realm {
 	RemotePopRequest(Event *_events, size_t _capacity);
 
 	Mutex mutex;
-	CondVar condvar;
+        Mutex::CondVar condvar;
 	bool completed;
 	size_t count, capacity;
 	Event *events;
