@@ -1,4 +1,4 @@
-/* Copyright 2021 Stanford University, NVIDIA Corporation
+/* Copyright 2023 Stanford University, NVIDIA Corporation
  *
  * Licensed under the Apache License, Version 2.0 (the "License");
  * you may not use this file except in compliance with the License.
@@ -497,7 +497,7 @@ public:
 public:
   size_t last_message_count;
   Realm::Mutex mutex;
-  Realm::CondVar condvar;
+  Realm::Mutex::CondVar condvar;
   int messages_received;
   bool is_quiescent;
 };
