@@ -1,4 +1,4 @@
-/* Copyright 2021 Stanford University, NVIDIA Corporation
+/* Copyright 2023 Stanford University, NVIDIA Corporation
  *
  * Licensed under the Apache License, Version 2.0 (the "License");
  * you may not use this file except in compliance with the License.
@@ -73,6 +73,17 @@
   #endif
 #endif
 
+// the GASNet-EX API defines the GEX_FLAG_IMMEDIATE flag to be a best-effort
+//  thing, with calls that accept the flag still being allowed to block -
+//  as of 2022.3.0, for any conduit other than aries "best effort" is actually
+//  "no effort" for RMA operations and we want to avoid using them in
+//  immediate-mode situations
+// NOTE: as with the NPAM stuff above, we'll pretend that MPI honors it as
+//  well so that we get code coverage in CI tests
+#if defined(GASNET_CONDUIT_ARIES) || defined(GASNET_CONDUIT_MPI)
+  #define REALM_GEX_RMA_HONORS_IMMEDIATE_FLAG
+#endif
+
 // eliminate GASNet warnings for unused static functions
 #include <gasnet_tools.h>
 REALM_ATTR_UNUSED(static const void *ignore_gasnet_warning1) = (void *)_gasneti_threadkey_init;
@@ -342,7 +353,7 @@ namespace Realm {
     };
     Strategy strategy;
     gex_Rank_t target;
-    gex_EP_Index_t target_ep_index;
+    gex_EP_Index_t source_ep_index, target_ep_index;
     unsigned short msgid;
     uintptr_t dest_payload_addr;
     void *temp_buffer;
@@ -411,6 +422,13 @@ namespace Realm {
 
     void enqueue_put_header(PendingPutHeader *put);
 
+    // adds the xpair to the injector ready list or the poller critical
+    //  pair list as appropriate, eventually resulting in a call to
+    //  push_packets - MUST NOT be called until the push_packets
+    //  that resulted from the previous enqueue is done-ish (i.e. not going
+    //  to push anything else)
+    void request_push(bool force_critical);
+
     void push_packets(bool immediate_mode, TimeLimit work_until);
 
     long long time_since_failure() const;
@@ -456,6 +474,9 @@ namespace Realm {
     gex_EP_Index_t tgt_ep_index;
     atomic<size_t> packets_reserved, packets_sent;
     Mutex mutex;
+    // we don't hold the mutex while pushing packets, but we need definitely
+    //  don't want multiple threads trying to push for the same src/dst pair
+    MutexChecker push_mutex_check;
     atomic<OutbufMetadata *> first_pbuf;  // read without mutex
     OutbufMetadata *cur_pbuf;
     atomic<unsigned> imm_fail_count;
@@ -511,6 +532,9 @@ namespace Realm {
     GASNetEXEvent& set_databuf(OutbufMetadata *_databuf);
     GASNetEXEvent& set_rget(PendingReverseGet *_rget);
     GASNetEXEvent& set_put(PendingPutHeader *_put);
+    GASNetEXEvent& set_leaf(GASNetEXEvent *_leaf);
+
+    void propagate_to_leaves();
 
     void trigger(GASNetEXInternal *internal);
 
@@ -525,6 +549,7 @@ namespace Realm {
     OutbufMetadata *databuf;
     PendingReverseGet *rget;
     PendingPutHeader *put;
+    GASNetEXEvent *leaf;
   };
 
   // an injector tries to send packets, but is not allowed to stall -
@@ -569,13 +594,30 @@ namespace Realm {
     GASNetEXInternal *internal;
     Mutex mutex;
     atomic<bool> shutdown_flag;  // set/cleared inside mutex, but tested outside
-    CondVar shutdown_cond;
+    Mutex::CondVar shutdown_cond;
     atomic<bool> pollwait_flag;  // set/cleared inside mutex, but tested outside
-    CondVar pollwait_cond;
+    Mutex::CondVar pollwait_cond;
     XmitSrcDestPair::XmitPairList critical_xpairs;
     GASNetEXEvent::EventList pending_events;
   };
 
+  class GASNetEXCompleter : public BackgroundWorkItem {
+  public:
+    GASNetEXCompleter(GASNetEXInternal *_internal);
+
+    void add_ready_events(GASNetEXEvent::EventList& newly_ready);
+
+    bool has_work_remaining();
+
+    virtual bool do_work(TimeLimit work_until);
+
+  protected:
+    GASNetEXInternal *internal;
+    Mutex mutex;
+    atomic<bool> has_work;  // can be read without mutex
+    GASNetEXEvent::EventList ready_events;
+  };
+
   struct PendingPutHeader {
   protected:
     // should not be directly allocated
@@ -731,6 +773,7 @@ namespace Realm {
     friend class XmitSrcDestPair;
     friend class GASNetEXEvent;
     friend class GASNetEXPoller;
+    friend class GASNetEXCompleter;
 
     // callbacks from IncomingMessageManager
     static void short_message_complete(NodeID sender, uintptr_t objptr,
@@ -769,6 +812,7 @@ namespace Realm {
 
     GASNetEXPoller poller;
     GASNetEXInjector injector;
+    GASNetEXCompleter completer;
     ReverseGetter rgetter;
     PendingCompletionManager compmgr;
     OutbufManager obmgr;
