@@ -1,4 +1,4 @@
-/* Copyright 2021 Stanford University, NVIDIA Corporation
+/* Copyright 2023 Stanford University, NVIDIA Corporation
  *
  * Licensed under the Apache License, Version 2.0 (the "License");
  * you may not use this file except in compliance with the License.
@@ -25,6 +25,7 @@
 #include "realm/runtime_impl.h"
 #include "realm/mem_impl.h"
 #include "realm/logging.h"
+#include "realm/transfer/ib_memory.h"
 
 #ifdef REALM_GASNETEX_MODULE_DYNAMIC
 REGISTER_REALM_NETWORK_MODULE_DYNAMIC(Realm::GASNetEXModule);
@@ -660,7 +661,8 @@ namespace Realm {
     // if checksums are enabled, we'll tack it on to the end of the header
     //  to avoid any alignment issues
     if(cfg_do_checksums)
-      header_size += sizeof(gex_AM_Arg_t);
+      header_size = roundup_pow2(header_size + sizeof(gex_AM_Arg_t),
+                                 sizeof(gex_AM_Arg_t));
 
     assert(storage_size >= sizeof(GASNetEXMessageImpl));
     GASNetEXMessageImpl *impl = new(storage_base) GASNetEXMessageImpl(internal,
@@ -689,7 +691,8 @@ namespace Realm {
     // if checksums are enabled, we'll tack it on to the end of the header
     //  to avoid any alignment issues
     if(cfg_do_checksums)
-      header_size += sizeof(gex_AM_Arg_t);
+      header_size = roundup_pow2(header_size + sizeof(gex_AM_Arg_t),
+                                 sizeof(gex_AM_Arg_t));
 
     assert(storage_size >= sizeof(GASNetEXMessageImpl));
     GASNetEXMessageImpl *impl = new(storage_base) GASNetEXMessageImpl(internal,
@@ -718,7 +721,8 @@ namespace Realm {
     // if checksums are enabled, we'll tack it on to the end of the header
     //  to avoid any alignment issues
     if(cfg_do_checksums)
-      header_size += sizeof(gex_AM_Arg_t);
+      header_size = roundup_pow2(header_size + sizeof(gex_AM_Arg_t),
+                                 sizeof(gex_AM_Arg_t));
 
     assert(storage_size >= sizeof(GASNetEXMessageImpl));
     if(targets.size() == 1) {
@@ -753,7 +757,8 @@ namespace Realm {
 						 size_t header_size)
   {
     if(cfg_do_checksums)
-      header_size += sizeof(gex_AM_Arg_t);
+      header_size = roundup_pow2(header_size + sizeof(gex_AM_Arg_t),
+                                 sizeof(gex_AM_Arg_t));
 
     return internal->recommended_max_payload(target, 0 /*ep_index*/,
 					     with_congestion,
@@ -766,7 +771,8 @@ namespace Realm {
 						 size_t header_size)
   {
     if(cfg_do_checksums)
-      header_size += sizeof(gex_AM_Arg_t);
+      header_size = roundup_pow2(header_size + sizeof(gex_AM_Arg_t),
+                                 sizeof(gex_AM_Arg_t));
 
     if(targets.size() == 1) {
       // optimization - if there's exactly 1 target, redirect to the unicast mode
@@ -788,7 +794,8 @@ namespace Realm {
 						 size_t header_size)
   {
     if(cfg_do_checksums)
-      header_size += sizeof(gex_AM_Arg_t);
+      header_size = roundup_pow2(header_size + sizeof(gex_AM_Arg_t),
+                                 sizeof(gex_AM_Arg_t));
 
     return internal->recommended_max_payload(target,
 					     dest_payload_addr.extra,
