@@ -1,4 +1,4 @@
-/* Copyright 2021 Stanford University, NVIDIA Corporation
+/* Copyright 2023 Stanford University, NVIDIA Corporation
  *
  * Licensed under the Apache License, Version 2.0 (the "License");
  * you may not use this file except in compliance with the License.
@@ -97,32 +97,40 @@ namespace Realm {
       return 0; // cannot provide a pointer for it.
     }
 
+    // HDF5Memory supports ExternalHDF5Resource
+    bool HDF5Memory::attempt_register_external_resource(RegionInstanceImpl *inst,
+                                                        size_t& inst_offset)
+    {
+      {
+        ExternalHDF5Resource *res = dynamic_cast<ExternalHDF5Resource *>(inst->metadata.ext_resource);
+        if(res) {
+          // instant success - we won't try to open the file until it's accessed
+          inst_offset = 0;
+          return true;
+        }
+      }
+
+      // not a kind we recognize
+      return false;
+    }
+
+    void HDF5Memory::unregister_external_resource(RegionInstanceImpl *inst)
+    {
+      // nothing to do here
+    }
+
     MemoryImpl::AllocationResult HDF5Memory::allocate_storage_immediate(RegionInstanceImpl *inst,
 							    bool need_alloc_result,
 							    bool poisoned,
 							    TimeLimit work_until)
     {
-      // if the allocation request doesn't include an external HDF5 resource,
-      //  we fail it immediately
-      ExternalHDF5Resource *res = dynamic_cast<ExternalHDF5Resource *>(inst->metadata.ext_resource);
-      if(res == 0) {
-	if(inst->metadata.ext_resource)
-	  log_inst.warning() << "attempt to register non-hdf5 resource: mem=" << me << " resource=" << *(inst->metadata.ext_resource);
-	else
-	  log_inst.warning() << "attempt to allocate memory in hdf5 memory: layout=" << *(inst->metadata.layout);
-	inst->notify_allocation(ALLOC_INSTANT_FAILURE, 0, work_until);
-	return ALLOC_INSTANT_FAILURE;
-      }
+      // we can't actually allocate anything in an HDF5Memory
+      if(inst->metadata.ext_resource)
+        log_inst.warning() << "attempt to register non-hdf5 resource: mem=" << me << " resource=" << *(inst->metadata.ext_resource);
+      else
+        log_inst.warning() << "attempt to allocate memory in hdf5 memory: layout=" << *(inst->metadata.layout);
 
-      // poisoned preconditions cancel the allocation request
-      if(poisoned) {
-	inst->notify_allocation(ALLOC_CANCELLED, 0, work_until);
-	return ALLOC_CANCELLED;
-      }
-
-      // TODO: try to open the file once here to make sure it exists?
-
-      AllocationResult result = ALLOC_INSTANT_SUCCESS;
+      AllocationResult result = ALLOC_INSTANT_FAILURE;
       size_t inst_offset = 0;
       inst->notify_allocation(result, inst_offset, work_until);
 
@@ -137,8 +145,15 @@ namespace Realm {
       if(poisoned)
 	return;
 
-      // we didn't save anything on the instance, so just ack and return
-      inst->notify_deallocation();
+      // for external instances, all we have to do is ack the destruction
+      if(inst->metadata.ext_resource != 0) {
+        unregister_external_resource(inst);
+        inst->notify_deallocation();
+	return;
+      }
+
+      // shouldn't get here - no allocation
+      assert(0);
     }
 
 
@@ -148,6 +163,7 @@ namespace Realm {
 
     int AddressInfoHDF5::set_rect(const RegionInstanceImpl *inst,
                                   const InstanceLayoutPieceBase *piece,
+                                  size_t field_size, size_t field_offset,
                                   int ndims,
                                   const int64_t lo[/*ndims*/],
                                   const int64_t hi[/*ndims*/],
@@ -263,6 +279,10 @@ namespace Realm {
         long idx = 0;
 	
 	while((idx < nr) && request_available()) {	  
+	  // TODO: we really shouldn't even be trying if the iteration
+	  //   is already done
+	  if(iteration_completed.load()) break;
+
 	  // TODO: use control stream to determine which input/output ports
 	  //  to use
 	  int in_port_idx = 0;
@@ -276,7 +296,7 @@ namespace Realm {
 	    // non-ib iterators should end at the same time
 	    assert((in_port->peer_guid != XFERDES_NO_GUID) || in_port->iter->done());
 	    assert((out_port->peer_guid != XFERDES_NO_GUID) || out_port->iter->done());
-	    iteration_completed.store_release(true);
+	    begin_completion();
 	    break;
 	  }
 
@@ -300,7 +320,7 @@ namespace Realm {
 	      // otherwise, this shouldn't happen - we should detect this case
 	      //  on the the transfer of those last bytes
 	      assert(0);
-	      iteration_completed.store_release(true);
+	      begin_completion();
 	      break;
 	    }
 	    if(pre_max < max_bytes) {
@@ -413,6 +433,7 @@ namespace Realm {
 	  new_req->write_seq_pos = out_port->local_bytes_total;
 	  new_req->write_seq_count = hdf5_bytes;
 	  out_port->local_bytes_total += hdf5_bytes;
+          out_port->local_bytes_cons.fetch_add(hdf5_bytes);
 
 	  requests[idx++] = new_req;
 
@@ -420,7 +441,7 @@ namespace Realm {
 	  //  process the request (so that multi-hop successors are notified
 	  //  properly)
 	  if(hdf5_iter->done())
-	    iteration_completed.store_release(true);
+	    begin_completion();
 	}
 
 	return idx;
@@ -524,7 +545,7 @@ namespace Realm {
                     output_control.eos_received);
 
           if(done)
-            iteration_completed.store_release(true);
+            begin_completion();
 
           if(done || work_until.is_expired())
             break;
@@ -577,34 +598,31 @@ namespace Realm {
     //
     // class HDF5Channel
 
-      static const Memory::Kind cpu_mem_kinds[] = { Memory::SYSTEM_MEM,
-						    Memory::REGDMA_MEM,
-						    Memory::Z_COPY_MEM,
-                                                    Memory::SOCKET_MEM };
-      static const size_t num_cpu_mem_kinds = sizeof(cpu_mem_kinds) / sizeof(cpu_mem_kinds[0]);
-
       HDF5Channel::HDF5Channel(BackgroundWorkManager *bgwork)
 	: SingleXDQChannel<HDF5Channel, HDF5XferDes>(bgwork,
 						     XFER_NONE /*FIXME*/,
 						     "hdf5 channel")
       {
-	unsigned bw = 0; // TODO
-	unsigned latency = 0;
-	// any combination of SYSTEM/REGDMA/Z_COPY_MEM
-	for(size_t i = 0; i < num_cpu_mem_kinds; i++) {
-	  add_path(Memory::HDF_MEM, false,
-		   cpu_mem_kinds[i], false,
-		   bw, latency, false, false, XFER_HDF5_READ);
+        unsigned bw = 10; // HACK - estimate 10 MB/s
+        unsigned latency = 10000; // HACK - estimate 10 us
+        unsigned frag_overhead = 10000; // HACK - estimate 10 us
 
-	  add_path(cpu_mem_kinds[i], false,
-		   Memory::HDF_MEM, false,
-		   bw, latency, false, false, XFER_HDF5_WRITE);
-	}
+        // all local cpu memories are valid sources/dests
+        std::vector<Memory> local_cpu_mems;
+        MemcpyChannel::enumerate_local_cpu_memories(local_cpu_mems);
+
+        add_path(Memory::HDF_MEM, false,
+                 local_cpu_mems,
+                 bw, latency, frag_overhead, XFER_HDF5_READ);
+
+        add_path(local_cpu_mems,
+                 Memory::HDF_MEM, false,
+                 bw, latency, frag_overhead, XFER_HDF5_WRITE);
 
         // also indicate willingness to handle fills to HDF5 (src == NO_MEMORY)
         add_path(Memory::NO_MEMORY,
                  Memory::HDF_MEM, false,
-                 bw, latency, false, false, XFER_HDF5_WRITE);
+                 bw, latency, frag_overhead, XFER_HDF5_WRITE);
       }
 
       HDF5Channel::~HDF5Channel() {}
@@ -616,7 +634,9 @@ namespace Realm {
 					    const std::vector<XferDesPortInfo>& outputs_info,
 					    int priority,
 					    XferDesRedopInfo redop_info,
-					    const void *fill_data, size_t fill_size)
+					    const void *fill_data,
+                                            size_t fill_size,
+                                            size_t fill_total)
       {
 	assert(redop_info.id == 0);
 	return new HDF5XferDes(dma_op, this, launch_node, guid,
