@@ -1,4 +1,4 @@
-/* Copyright 2021 Stanford University, NVIDIA Corporation
+/* Copyright 2023 Stanford University, NVIDIA Corporation
  *                Los Alamos National Laboratory
  *
  * Licensed under the Apache License, Version 2.0 (the "License");
@@ -20,9 +20,6 @@
 #include <hip/hip_runtime.h>
 #ifdef __HIP_PLATFORM_NVCC__
 #define hipDeviceScheduleBlockingSync CU_CTX_SCHED_BLOCKING_SYNC 
-typedef CUdeviceptr hipDeviceCharptr_t;
-#else
-typedef char* hipDeviceCharptr_t;
 #endif
 
 #include "realm/realm_config.h"
@@ -34,28 +31,29 @@ typedef char* hipDeviceCharptr_t;
 #include "realm/mem_impl.h"
 #include "realm/bgwork.h"
 #include "realm/transfer/channel.h"
+#include "realm/transfer/ib_memory.h"
 
 #define CHECK_CUDART(cmd) do { \
   hipError_t ret = (cmd); \
   if(ret != hipSuccess) { \
-    fprintf(stderr, "CUDART: %s = %d (%s)\n", #cmd, ret, hipGetErrorString(ret)); \
+    fprintf(stderr, "HIP: %s = %d (%s)\n", #cmd, ret, hipGetErrorString(ret)); \
     assert(0); \
     exit(1); \
   } \
 } while(0)
   
-#define REPORT_CU_ERROR(cmd, ret) \
+#define REPORT_HIP_ERROR(cmd, ret) \
   do { \
     const char *name, *str; \
     name = hipGetErrorName(ret); \
     str = hipGetErrorString(ret); \
-    fprintf(stderr, "CU: %s = %d (%s): %s\n", cmd, ret, name, str); \
+    fprintf(stderr, "HIP: %s = %d (%s): %s\n", cmd, ret, name, str); \
     abort(); \
   } while(0)
 
-#define CHECK_CU(cmd) do {                      \
+#define CHECK_HIP(cmd) do {                      \
   hipError_t ret = (cmd); \
-  if(ret != hipSuccess) REPORT_CU_ERROR(#cmd, ret); \
+  if(ret != hipSuccess) REPORT_HIP_ERROR(#cmd, ret); \
 } while(0)
 
 
@@ -63,15 +61,21 @@ namespace Realm {
   
   namespace Hip {
 
-    struct GPUInfo {
+    struct GPUInfo 
+#ifdef REALM_USE_HIP_HIJACK
+      : public hipDeviceProp_t
+#endif
+    {
       int index;  // index used by HIP runtime
       hipDevice_t device;
 
       static const size_t MAX_NAME_LEN = 64;
+#ifndef REALM_USE_HIP_HIJACK
       char name[MAX_NAME_LEN];
 
-      int compute_major, compute_minor;
-      size_t total_mem;
+      int major, minor;
+      size_t totalGlobalMem;
+#endif
       std::set<hipDevice_t> peers;  // other GPUs we can do p2p copies with
     };
 
@@ -88,9 +92,12 @@ namespace Realm {
     class GPUStream;
     class GPUFBMemory;
     class GPUZCMemory;
+    class GPUFBIBMemory;
     class GPU;
     class HipModule;
 
+    extern HipModule *hip_module_singleton;
+
     // an interface for receiving completion notification for a GPU operation
     //  (right now, just copies)
     class GPUCompletionNotification {
@@ -315,7 +322,7 @@ namespace Realm {
     //  with when async work needs doing
     class GPUStream {
     public:
-      GPUStream(GPU *_gpu, GPUWorker *_worker);
+      GPUStream(GPU *_gpu, GPUWorker *_worker, int rel_priority = 0);
       ~GPUStream(void);
 
       GPU *get_gpu(void) const;
@@ -327,6 +334,12 @@ namespace Realm {
       void add_start_event(GPUWorkStart *start);
       void add_notification(GPUCompletionNotification *notification);
       void wait_on_streams(const std::set<GPUStream*> &other_streams);
+      
+      // atomically checks rate limit counters and returns true if 'bytes'
+      //  worth of copies can be submitted or false if not (in which case
+      //  the progress counter on the xd will be updated when it should try
+      //  again)
+      bool ok_to_submit_copy(size_t bytes, XferDes *xd);
 
       // to be called by a worker (that should already have the GPU context
       //   current) - returns true if any work remains
@@ -397,7 +410,7 @@ namespace Realm {
       bool process_streams(bool sleep_on_empty);
       
       Mutex lock;
-      CondVar condvar;
+      Mutex::CondVar condvar;
       
       typedef CircularQueue<GPUStream *, 16> ActiveStreamQueue;
       ActiveStreamQueue active_streams;
@@ -453,7 +466,7 @@ namespace Realm {
       int device_id;
       int max_threads;
       Mutex mutex;
-      CondVar condvar;
+      Mutex::CondVar condvar;
       bool shutdown_flag;
       GPUWorkFence::FenceList fences;
       int total_threads, sleeping_threads, syncing_threads;
@@ -471,8 +484,7 @@ namespace Realm {
     class GPU {
     public:
       GPU(HipModule *_module, GPUInfo *_info, GPUWorker *worker,
-	        int _device_id,
-          int num_streams);
+	  int _device_id);
       ~GPU(void);
 
       void push_context(void);
@@ -484,11 +496,12 @@ namespace Realm {
       void register_function(const RegisteredFunction *func);
 
       hipFunction_t lookup_function(const void *func);
-      hipDeviceCharptr_t lookup_variable(const void *var);
+      char* lookup_variable(const void *var);
 #endif
 
       void create_processor(RuntimeImpl *runtime, size_t stack_size);
-      void create_fb_memory(RuntimeImpl *runtime, size_t size);
+      void create_fb_memory(RuntimeImpl *runtime, size_t size, size_t ib_size);
+      void create_dynamic_fb_memory(RuntimeImpl *runtime, size_t max_size);
 
       void create_dma_channels(Realm::RuntimeImpl *r);
 
@@ -579,6 +592,7 @@ namespace Realm {
       GPUStream *find_stream(hipStream_t stream) const;
       GPUStream *get_null_task_stream(void) const;
       GPUStream *get_next_task_stream(bool create = false);
+      GPUStream *get_next_d2d_stream();
     protected:
       hipModule_t load_hip_module(const void *data);
 
@@ -588,10 +602,11 @@ namespace Realm {
       GPUWorker *worker;
       GPUProcessor *proc;
       GPUFBMemory *fbmem;
+      GPUFBIBMemory *fb_ibmem;
 
       //hipCtx_t context;
       int device_id;
-      hipDeviceCharptr_t fbmem_base;
+      char *fbmem_base, *fb_ibmem_base;
 
       // which system memories have been registered and can be used for cuMemcpyAsync
       std::set<Memory> pinned_sysmems;
@@ -603,16 +618,32 @@ namespace Realm {
       GPUStream *host_to_device_stream;
       GPUStream *device_to_host_stream;
       GPUStream *device_to_device_stream;
+      std::vector<GPUStream *> device_to_device_streams;
       std::vector<GPUStream *> peer_to_peer_streams; // indexed by target
       std::vector<GPUStream *> task_streams;
-      atomic<unsigned> next_stream;
+      atomic<unsigned> next_task_stream, next_d2d_stream;
 
       GPUEventPool event_pool;
 
+      // this can technically be different in each context (but probably isn't
+      //  in practice)
+      int least_stream_priority, greatest_stream_priority;
+
+      struct HipIpcMapping {
+        NodeID owner;
+        Memory mem;
+        uintptr_t local_base;
+        uintptr_t address_offset; // add to convert from original to local base
+      };
+      std::vector<HipIpcMapping> hipipc_mappings;
+      std::map<NodeID, GPUStream *> hipipc_streams;
+
+      const HipIpcMapping *find_ipc_mapping(Memory mem) const;
+
 #ifdef REALM_USE_HIP_HIJACK
       std::map<const FatBin *, hipModule_t> device_modules;
       std::map<const void *, hipFunction_t> device_functions;
-      std::map<const void *, hipDeviceCharptr_t> device_variables;
+      std::map<const void *, char *> device_variables;
 #endif
     };
 
@@ -706,9 +737,23 @@ namespace Realm {
       Realm::CoreReservation *core_rsrv;
     };
 
+    // this can be attached to any MemoryImpl if the underlying memory is
+    //  guaranteed to belong to a given device - this will allow that
+    //  context's processor and dma channels to work with it
+    // the creator is expected to know what device they want but need
+    //  not know which GPU object that corresponds to
+    class HipDeviceMemoryInfo : public ModuleSpecificInfo
+    {
+    public:
+      HipDeviceMemoryInfo(int _device_id);
+
+      int device_id;
+      GPU *gpu;
+    };
+
     class GPUFBMemory : public LocalManagedMemory {
     public:
-      GPUFBMemory(Memory _me, GPU *_gpu, hipDeviceCharptr_t _base, size_t _size);
+      GPUFBMemory(Memory _me, GPU *_gpu, char *_base, size_t _size);
 
       virtual ~GPUFBMemory(void);
 
@@ -718,15 +763,72 @@ namespace Realm {
 
       virtual void *get_direct_ptr(off_t offset, size_t size);
 
+      // GPUFBMemory supports ExternalHipMemoryResource and
+      //  ExternalHipArrayResource (not implemented)
+      virtual bool attempt_register_external_resource(RegionInstanceImpl *inst,
+                                                      size_t& inst_offset);
+      virtual void unregister_external_resource(RegionInstanceImpl *inst);
+
+      // for re-registration purposes, generate an ExternalInstanceResource *
+      //  (if possible) for a given instance, or a subset of one
+      virtual ExternalInstanceResource *generate_resource_info(RegionInstanceImpl *inst,
+                                                               const IndexSpaceGeneric *subspace,
+                                                               span<const FieldID> fields,
+                                                               bool read_only);
+
     public:
       GPU *gpu;
-      hipDeviceCharptr_t base;
+      char *base;
       NetworkSegment local_segment;
     };
 
+    class GPUDynamicFBMemory : public MemoryImpl {
+    public:
+      GPUDynamicFBMemory(Memory _me, GPU *_gpu, size_t _max_size);
+
+      virtual ~GPUDynamicFBMemory(void);
+
+      // deferred allocation not supported
+      virtual AllocationResult allocate_storage_immediate(RegionInstanceImpl *inst,
+							  bool need_alloc_result,
+							  bool poisoned,
+							  TimeLimit work_until);
+
+      virtual void release_storage_immediate(RegionInstanceImpl *inst,
+					     bool poisoned,
+					     TimeLimit work_until);
+
+      // these work, but they are SLOW
+      virtual void get_bytes(off_t offset, void *dst, size_t size);
+      virtual void put_bytes(off_t offset, const void *src, size_t size);
+
+      virtual void *get_direct_ptr(off_t offset, size_t size);
+
+      // GPUDynamicFBMemory supports ExternalHipMemoryResource and
+      //  ExternalHipArrayResource (not implemented)
+      virtual bool attempt_register_external_resource(RegionInstanceImpl *inst,
+                                                      size_t& inst_offset);
+      virtual void unregister_external_resource(RegionInstanceImpl *inst);
+
+      // for re-registration purposes, generate an ExternalInstanceResource *
+      //  (if possible) for a given instance, or a subset of one
+      virtual ExternalInstanceResource *generate_resource_info(RegionInstanceImpl *inst,
+                                                               const IndexSpaceGeneric *subspace,
+                                                               span<const FieldID> fields,
+                                                               bool read_only);
+
+    public:
+      GPU *gpu;
+      Mutex mutex;
+      size_t cur_size;
+      std::map<RegionInstance, void*> alloc_bases;
+    };
+
     class GPUZCMemory : public LocalManagedMemory {
     public:
-      GPUZCMemory(Memory _me, hipDeviceCharptr_t _gpu_base, void *_cpu_base, size_t _size);
+      GPUZCMemory(Memory _me, char *_gpu_base, 
+                  void *_cpu_base, size_t _size,
+                  MemoryKind _kind, Memory::Kind _lowlevel_kind);
 
       virtual ~GPUZCMemory(void);
 
@@ -736,12 +838,34 @@ namespace Realm {
 
       virtual void *get_direct_ptr(off_t offset, size_t size);
 
+      // GPUZCMemory supports ExternalHipPinnedHostResource
+      virtual bool attempt_register_external_resource(RegionInstanceImpl *inst,
+                                                      size_t& inst_offset);
+      virtual void unregister_external_resource(RegionInstanceImpl *inst);
+
+      // for re-registration purposes, generate an ExternalInstanceResource *
+      //  (if possible) for a given instance, or a subset of one
+      virtual ExternalInstanceResource *generate_resource_info(RegionInstanceImpl *inst,
+                                                               const IndexSpaceGeneric *subspace,
+                                                               span<const FieldID> fields,
+                                                               bool read_only);
+
     public:
-      hipDeviceCharptr_t gpu_base;
+      char *gpu_base;
       char *cpu_base;
       NetworkSegment local_segment;
     };
     
+    class GPUFBIBMemory : public IBMemory {
+    public:
+      GPUFBIBMemory(Memory _me, GPU *_gpu, char *_base, size_t _size);
+
+    public:
+      GPU *gpu;
+      char *base;
+      NetworkSegment local_segment;
+    };
+    
     class GPURequest;
 
     class GPUCompletionEvent : public GPUCompletionNotification {
@@ -782,32 +906,18 @@ namespace Realm {
     class GPUXferDes : public XferDes {
     public:
       GPUXferDes(uintptr_t _dma_op, Channel *_channel,
-		 NodeID _launch_node, XferDesID _guid,
-		 const std::vector<XferDesPortInfo>& inputs_info,
-		 const std::vector<XferDesPortInfo>& outputs_info,
-		 int _priority);
-
-      ~GPUXferDes()
-      {
-        while (!available_reqs.empty()) {
-          GPURequest* gpu_req = (GPURequest*) available_reqs.front();
-          available_reqs.pop();
-          delete gpu_req;
-        }
-      }
+                 NodeID _launch_node, XferDesID _guid,
+                 const std::vector<XferDesPortInfo>& inputs_info,
+                 const std::vector<XferDesPortInfo>& outputs_info,
+                 int _priority);
 
       long get_requests(Request** requests, long nr);
-      void notify_request_read_done(Request* req);
-      void notify_request_write_done(Request* req);
-      void flush();
 
       bool progress_xd(GPUChannel *channel, TimeLimit work_until);
 
     private:
-      //GPURequest* gpu_reqs;
-      //char *src_buf_base;
-      //char *dst_buf_base;
-      GPU *dst_gpu, *src_gpu;
+      std::vector<GPU *> src_gpus, dst_gpus;
+      std::vector<bool> dst_is_ipc;
     };
 
     class GPUChannel : public SingleXDQChannel<GPUChannel, GPUXferDes> {
@@ -827,7 +937,8 @@ namespace Realm {
       				       const std::vector<XferDesPortInfo>& outputs_info,
       				       int priority,
       				       XferDesRedopInfo redop_info,
-      				       const void *fill_data, size_t fill_size);                        
+      				       const void *fill_data, size_t fill_size,
+                                       size_t fill_total);
 
       long submit(Request** requests, long nr);
 
@@ -845,7 +956,8 @@ namespace Realm {
 		     const std::vector<XferDesPortInfo>& inputs_info,
 		     const std::vector<XferDesPortInfo>& outputs_info,
 		     int _priority,
-		     const void *_fill_data, size_t _fill_size);
+		     const void *_fill_data, size_t _fill_size,
+                     size_t _fill_total);
 
       long get_requests(Request** requests, long nr);
 
@@ -869,7 +981,8 @@ namespace Realm {
 				       const std::vector<XferDesPortInfo>& outputs_info,
 				       int priority,
 				       XferDesRedopInfo redop_info,
-				       const void *fill_data, size_t fill_size);
+				       const void *fill_data, size_t fill_size,
+                                       size_t fill_total);
 
       long submit(Request** requests, long nr);
 
@@ -878,6 +991,137 @@ namespace Realm {
 
       GPU* gpu;
     };
+    
+    class GPUreduceChannel;
+
+    class GPUreduceXferDes : public XferDes {
+    public:
+      GPUreduceXferDes(uintptr_t _dma_op, Channel *_channel,
+                       NodeID _launch_node, XferDesID _guid,
+                       const std::vector<XferDesPortInfo>& inputs_info,
+                       const std::vector<XferDesPortInfo>& outputs_info,
+                       int _priority,
+                       XferDesRedopInfo _redop_info);
+
+      long get_requests(Request** requests, long nr);
+
+      bool progress_xd(GPUreduceChannel *channel, TimeLimit work_until);
+
+    protected:
+      XferDesRedopInfo redop_info;
+      const ReductionOpUntyped *redop;
+#if defined(REALM_USE_HIP_HIJACK)
+      void *kernel;
+#else
+      const void *kernel_host_proxy;
+#endif
+      GPUStream *stream;
+    };
+
+    class GPUreduceChannel : public SingleXDQChannel<GPUreduceChannel, GPUreduceXferDes> {
+    public:
+      GPUreduceChannel(GPU* _gpu, BackgroundWorkManager *bgwork);
+
+      // multiple concurrent cuda reduces ok
+      static const bool is_ordered = false;
+      
+      // helper method here so that GPUreduceRemoteChannel can use it too
+      static bool is_gpu_redop(ReductionOpID redop_id);
+
+      // override this because we have to be picky about which reduction ops
+      //  we support
+      virtual uint64_t supports_path(Memory src_mem, Memory dst_mem,
+                                     CustomSerdezID src_serdez_id,
+                                     CustomSerdezID dst_serdez_id,
+                                     ReductionOpID redop_id,
+                                     size_t total_bytes,
+                                     const std::vector<size_t> *src_frags,
+                                     const std::vector<size_t> *dst_frags,
+                                     XferDesKind *kind_ret = 0,
+                                     unsigned *bw_ret = 0,
+                                     unsigned *lat_ret = 0);
+         
+      virtual RemoteChannelInfo *construct_remote_info() const;
+
+      virtual XferDes *create_xfer_des(uintptr_t dma_op,
+				       NodeID launch_node,
+				       XferDesID guid,
+				       const std::vector<XferDesPortInfo>& inputs_info,
+				       const std::vector<XferDesPortInfo>& outputs_info,
+				       int priority,
+				       XferDesRedopInfo redop_info,
+				       const void *fill_data, size_t fill_size,
+               size_t fill_total);
+
+      long submit(Request** requests, long nr);
+
+    protected:
+      friend class GPUreduceXferDes;
+
+      GPU* gpu;
+    };
+    
+    class GPUreduceRemoteChannelInfo : public SimpleRemoteChannelInfo {
+    public:
+      GPUreduceRemoteChannelInfo(NodeID _owner, XferDesKind _kind,
+                                 uintptr_t _remote_ptr,
+                                 const std::vector<Channel::SupportedPath>& _paths);
+
+      virtual RemoteChannel *create_remote_channel();
+
+      template <typename S>
+      bool serialize(S& serializer) const;
+
+      template <typename S>
+      static RemoteChannelInfo *deserialize_new(S& deserializer);
+
+    protected:
+      static Serialization::PolymorphicSerdezSubclass<RemoteChannelInfo,
+                                                      GPUreduceRemoteChannelInfo> serdez_subclass;
+    };
+
+    class GPUreduceRemoteChannel : public RemoteChannel {
+      friend class GPUreduceRemoteChannelInfo;
+
+      GPUreduceRemoteChannel(uintptr_t _remote_ptr);
+
+      virtual uint64_t supports_path(Memory src_mem, Memory dst_mem,
+                                     CustomSerdezID src_serdez_id,
+                                     CustomSerdezID dst_serdez_id,
+                                     ReductionOpID redop_id,
+                                     size_t total_bytes,
+                                     const std::vector<size_t> *src_frags,
+                                     const std::vector<size_t> *dst_frags,
+                                     XferDesKind *kind_ret = 0,
+                                     unsigned *bw_ret = 0,
+                                     unsigned *lat_ret = 0);
+
+    };
+
+
+    // active messages for establishing cuda ipc mappings
+
+    struct HipIpcRequest {
+#ifdef REALM_ON_LINUX
+      long hostid;  // POSIX hostid
+#endif
+
+      static void handle_message(NodeID sender, const HipIpcRequest& args,
+                                 const void *data, size_t datalen);
+    };
+
+    struct HipIpcResponse {
+      unsigned count;
+
+      static void handle_message(NodeID sender, const HipIpcResponse& args,
+                                 const void *data, size_t datalen);
+    };
+
+    struct HipIpcRelease {
+
+      static void handle_message(NodeID sender, const HipIpcRelease& args,
+                                 const void *data, size_t datalen);
+    };
 
   }; // namespace Hip
   
