@@ -1,4 +1,4 @@
-/* Copyright 2021 Stanford University, NVIDIA Corporation
+/* Copyright 2023 Stanford University, NVIDIA Corporation
  *
  * Licensed under the Apache License, Version 2.0 (the "License");
  * you may not use this file except in compliance with the License.
@@ -18,11 +18,13 @@
 #include "realm/proc_impl.h"
 #include "realm/logging.h"
 #include "realm/serialize.h"
+#include "realm/idx_impl.h"
 #include "realm/inst_impl.h"
 #include "realm/runtime_impl.h"
 #include "realm/profiling.h"
 #include "realm/utils.h"
 #include "realm/activemsg.h"
+#include "realm/transfer/transfer.h"
 
 namespace Realm {
 
@@ -85,6 +87,7 @@ namespace Realm {
 			   NetworkSegment *_segment)
       : me(_me), size(_size), kind(_kind), lowlevel_kind(_lowlevel_kind)
       , segment(_segment)
+      , module_specific(0)
     {}
 
     MemoryImpl::~MemoryImpl(void)
@@ -105,6 +108,19 @@ namespace Realm {
 	    delete *it2;
 	delete it->second;
       }
+
+      // free any module-specific info we have
+      while(module_specific) {
+        ModuleSpecificInfo *next = module_specific->next;
+        delete module_specific;
+        module_specific = next;
+      }
+    }
+
+    void MemoryImpl::add_module_specific(ModuleSpecificInfo *info)
+    {
+      info->next = module_specific;
+      module_specific = info;
     }
 
     // default implementation handles deferral, but falls through to
@@ -120,19 +136,48 @@ namespace Realm {
 
       // check precondition on allocation
       bool alloc_poisoned = false;
+      AllocationResult result;
+      size_t inst_offset = 0;
       if(precondition.has_triggered_faultaware(alloc_poisoned)) {
-	// attempt immediate allocation (it'll handle poison)
-	return allocate_storage_immediate(inst, need_alloc_result,
-					  alloc_poisoned,
-					  TimeLimit::responsive());
+	if(alloc_poisoned) {
+	  // a poisoned creation works a lot like a failed creation
+	  inst->notify_allocation(ALLOC_CANCELLED,
+				  RegionInstanceImpl::INSTOFFSET_FAILED,
+				  TimeLimit::responsive());
+	  return ALLOC_INSTANT_FAILURE;
+        }
+
+        if(inst->metadata.ext_resource != 0) {
+          // hopefully this memory can handle this kind of external resource
+          if(attempt_register_external_resource(inst, inst_offset)) {
+            result = ALLOC_INSTANT_SUCCESS;
+          } else {
+            log_inst.warning() << "attempt to register unsupported external resource: mem=" << me << " resource=" << *(inst->metadata.ext_resource);
+            result = ALLOC_INSTANT_FAILURE;
+          }
+        } else {
+          // attempt immediate allocation (this will notify as needed on
+          //  its own)
+          return allocate_storage_immediate(inst, need_alloc_result,
+                                            false /*!alloc_poisoned*/,
+                                            TimeLimit::responsive());
+        }
       } else {
 	// defer allocation attempt
 	inst->metadata.inst_offset = RegionInstanceImpl::INSTOFFSET_DELAYEDALLOC;
 	inst->deferred_create.defer(inst, this,
 				    need_alloc_result,
 				    precondition);
-	return ALLOC_DEFERRED /*asynchronous notification*/;
+	result = ALLOC_DEFERRED /*asynchronous notification*/;
       }
+
+      // if we needed an alloc result, send deferred responses too
+      if((result != ALLOC_DEFERRED) || need_alloc_result) {
+        inst->notify_allocation(result, inst_offset,
+                                TimeLimit::responsive());
+      }
+
+      return result;
     }
 
     void MemoryImpl::release_storage_deferrable(RegionInstanceImpl *inst,
@@ -154,6 +199,30 @@ namespace Realm {
       }
     }
 
+    bool MemoryImpl::attempt_register_external_resource(RegionInstanceImpl *inst,
+                                                        size_t& inst_offset)
+    {
+      // nothing supported in base memory implementation
+      return false;
+    }
+
+    void MemoryImpl::unregister_external_resource(RegionInstanceImpl *inst)
+    {
+      // nothing to do
+    }
+
+    // for re-registration purposes, generate an ExternalInstanceResource *
+    //  (if possible) for a given instance, or a subset of one
+    ExternalInstanceResource *MemoryImpl::generate_resource_info(RegionInstanceImpl *inst,
+								 const IndexSpaceGeneric *subspace,
+								 span<const FieldID> fields,
+								 bool read_only)
+    {
+      // we don't know about any specific types of external resources in
+      //  the base class
+      return 0;
+    }
+
 #if 0
     off_t MemoryImpl::alloc_bytes_local(size_t size)
     {
@@ -449,196 +518,12 @@ namespace Realm {
 
   ////////////////////////////////////////////////////////////////////////
   //
-  // class IBMemory
+  // class MemSpecificInfo
   //
 
-    IBMemory::IBMemory(Memory _me, size_t _size,
-		       MemoryKind _kind, Memory::Kind _lowlevel_kind,
-		       void *prealloc_base, NetworkSegment *_segment)
-      : MemoryImpl(_me, _size, _kind, _lowlevel_kind, _segment)
-      , base(static_cast<char *>(prealloc_base))
-    {
-      free_blocks[0] = _size;
-    }
-
-    IBMemory::~IBMemory()
-    {
-    }
-
-    // old-style allocation used by IB memories
-    // make bad offsets really obvious (+1 PB)
-    static const off_t ZERO_SIZE_INSTANCE_OFFSET = 1ULL << ((sizeof(off_t) == 8) ? 50 : 30);
-
-    off_t IBMemory::alloc_bytes_local(size_t size)
-    {
-      AutoLock<> al(mutex);
-
-      // for zero-length allocations, return a special "offset"
-      if(size == 0) {
-	return this->size + ZERO_SIZE_INSTANCE_OFFSET;
-      }
-
-      const size_t alignment = 256;
-
-      if(alignment > 0) {
-	off_t leftover = size % alignment;
-	if(leftover > 0) {
-	  log_malloc.info("padding allocation from %zd to %zd",
-			  size, (size_t)(size + (alignment - leftover)));
-	  size += (alignment - leftover);
-	}
-      }
-      // HACK: pad the size by a bit to see if we have people falling off
-      //  the end of their allocations
-      size += 0;
-
-      // try to minimize footprint by allocating at the highest address possible
-      if(!free_blocks.empty()) {
-	std::map<off_t, off_t>::iterator it = free_blocks.end();
-	do {
-	  --it;  // predecrement since we started at the end
-
-	  if(it->second == (off_t)size) {
-	    // perfect match
-	    off_t retval = it->first;
-	    free_blocks.erase(it);
-	    log_malloc.info("alloc full block: mem=" IDFMT " size=%zd ofs=%zd", me.id, size, (ssize_t)retval);
-#if 0
-	    usage += size;
-	    if(usage > peak_usage) peak_usage = usage;
-	    size_t footprint = this->size - retval;
-	    if(footprint > peak_footprint) peak_footprint = footprint;
-#endif
-	    return retval;
-	  }
-	
-	  if(it->second > (off_t)size) {
-	    // some left over
-	    off_t leftover = it->second - size;
-	    off_t retval = it->first + leftover;
-	    it->second = leftover;
-	    log_malloc.info("alloc partial block: mem=" IDFMT " size=%zd ofs=%zd", me.id, size, (ssize_t)retval);
-#if 0
-	    usage += size;
-	    if(usage > peak_usage) peak_usage = usage;
-	    size_t footprint = this->size - retval;
-	    if(footprint > peak_footprint) peak_footprint = footprint;
-#endif
-	    return retval;
-	  }
-	} while(it != free_blocks.begin());
-      }
-
-      // no blocks large enough - boo hoo
-      log_malloc.info("alloc FAILED: mem=" IDFMT " size=%zd", me.id, size);
-      return -1;
-    }
-
-    void IBMemory::free_bytes_local(off_t offset, size_t size)
-    {
-      log_malloc.info() << "free block: mem=" << me << " size=" << size << " ofs=" << offset;
-      AutoLock<> al(mutex);
-
-      // frees of zero bytes should have the special offset
-      if(size == 0) {
-	assert((size_t)offset == this->size + ZERO_SIZE_INSTANCE_OFFSET);
-	return;
-      }
-
-      const size_t alignment = 256;
-
-      if(alignment > 0) {
-	off_t leftover = size % alignment;
-	if(leftover > 0) {
-	  log_malloc.info("padding free from %zd to %zd",
-			  size, (size_t)(size + (alignment - leftover)));
-	  size += (alignment - leftover);
-	}
-      }
-
-#if 0
-      usage -= size;
-      // only made things smaller, so can't impact the peak usage
-#endif
-
-      if(free_blocks.size() > 0) {
-	// find the first existing block that comes _after_ us
-	std::map<off_t, off_t>::iterator after = free_blocks.lower_bound(offset);
-	if(after != free_blocks.end()) {
-	  // found one - is it the first one?
-	  if(after == free_blocks.begin()) {
-	    // yes, so no "before"
-	    assert((offset + (off_t)size) <= after->first); // no overlap!
-	    if((offset + (off_t)size) == after->first) {
-	      // merge the ranges by eating the "after"
-	      size += after->second;
-	      free_blocks.erase(after);
-	    }
-	    free_blocks[offset] = size;
-	  } else {
-	    // no, get range that comes before us too
-	    std::map<off_t, off_t>::iterator before = after; before--;
-
-	    // if we're adjacent to the after, merge with it
-	    assert((offset + (off_t)size) <= after->first); // no overlap!
-	    if((offset + (off_t)size) == after->first) {
-	      // merge the ranges by eating the "after"
-	      size += after->second;
-	      free_blocks.erase(after);
-	    }
-
-	    // if we're adjacent with the before, grow it instead of adding
-	    //  a new range
-	    assert((before->first + before->second) <= offset);
-	    if((before->first + before->second) == offset) {
-	      before->second += size;
-	    } else {
-	      free_blocks[offset] = size;
-	    }
-	  }
-	} else {
-	  // nothing's after us, so just see if we can merge with the range
-	  //  that's before us
-
-	  std::map<off_t, off_t>::iterator before = after; before--;
-
-	  // if we're adjacent with the before, grow it instead of adding
-	  //  a new range
-	  assert((before->first + before->second) <= offset);
-	  if((before->first + before->second) == offset) {
-	    before->second += size;
-	  } else {
-	    free_blocks[offset] = size;
-	  }
-	}
-      } else {
-	// easy case - nothing was free, so now just our block is
-	free_blocks[offset] = size;
-      }
-    }
-
-    void *IBMemory::get_direct_ptr(off_t offset, size_t size)
-    {
-      assert(NodeID(ID(me).memory_owner_node()) == Network::my_node_id);
-      assert((offset >= 0) && ((size_t)(offset + size) <= this->size));
-      return (base + offset);
-    }
-
-    // not used by IB memories
-    MemoryImpl::AllocationResult IBMemory::allocate_storage_immediate(RegionInstanceImpl *inst,
-							bool need_alloc_result,
-							bool poisoned,
-							TimeLimit work_until)
-    {
-      abort();
-    }
-
-    void IBMemory::release_storage_immediate(RegionInstanceImpl *inst,
-					     bool poisoned,
-					     TimeLimit work_until)
-    {
-      abort();
-    }
+  MemSpecificInfo::MemSpecificInfo()
+    : next(0)
+  {}
 
 
   ////////////////////////////////////////////////////////////////////////
@@ -704,19 +589,11 @@ namespace Realm {
       AllocationResult result;
       size_t inst_offset = 0;
       if(inst->metadata.ext_resource != 0) {
-	// this is an external allocation - it had better be a memory resource
-	ExternalMemoryResource *res = dynamic_cast<ExternalMemoryResource *>(inst->metadata.ext_resource);
-	if(res != 0) {
-	  // automatic success - make the "offset" be the difference between the
-	  //  base address we were given and our own allocation's base
-	  void *mem_base = get_direct_ptr(0, 0); // only our subclasses know this
-	  assert(mem_base != 0);
-	  // underflow is ok here - it'll work itself out when we add the mem_base
-	  //  back in on accesses
-	  inst_offset = res->base - reinterpret_cast<uintptr_t>(mem_base);
+        // hopefully this memory can handle this kind of external resource
+        if(attempt_register_external_resource(inst, inst_offset)) {
 	  result = ALLOC_INSTANT_SUCCESS;
 	} else {
-	  log_inst.warning() << "attempt to register non-memory resource: mem=" << me << " resource=" << *(inst->metadata.ext_resource);
+	  log_inst.warning() << "attempt to register unsupported external resource: mem=" << me << " resource=" << *(inst->metadata.ext_resource);
 	  result = ALLOC_INSTANT_FAILURE;
 	}
       } else {
@@ -827,7 +704,9 @@ namespace Realm {
 
       // ignore external instances here - we can't reuse their memory for
       //  future allocations
-      if(inst->metadata.ext_resource == 0) {
+      if(inst->metadata.ext_resource != 0) {
+        unregister_external_resource(inst);
+      } else {
 	// this release may satisfy pending allocation requests
 	std::vector<std::pair<RegionInstanceImpl *, size_t> > successful_allocs;
 
@@ -1124,8 +1003,10 @@ namespace Realm {
       // for external instances, all we have to do is ack the destruction (assuming
       //  it wasn't poisoned)
       if(inst->metadata.ext_resource != 0) {
-	if(!poisoned)
+	if(!poisoned) {
+          unregister_external_resource(inst);
 	  inst->notify_deallocation();
+        }
 	return;
       }
 
@@ -1390,28 +1271,33 @@ namespace Realm {
       base = (char *)prealloc_base;
       prealloced = true;
     } else {
-      // allocate our own space
-      // enforce alignment on the whole memory range
-      base_orig = static_cast<char *>(malloc(_size + ALIGNMENT - 1));
-      if(!base_orig) {
-	log_malloc.fatal() << "insufficient system memory: "
-			   << size << " bytes needed (from -ll:csize)";
-	abort();
-      }
-      size_t ofs = reinterpret_cast<size_t>(base_orig) % ALIGNMENT;
-      if(ofs > 0) {
-	base = base_orig + (ALIGNMENT - ofs);
-      } else {
-	base = base_orig;
-      }
-      prealloced = false;
+      if(_size > 0) {
+        // allocate our own space
+        // enforce alignment on the whole memory range
+        base_orig = static_cast<char *>(malloc(_size + ALIGNMENT - 1));
+        if(!base_orig) {
+          log_malloc.fatal() << "insufficient system memory: "
+                             << size << " bytes needed (from -ll:csize)";
+          abort();
+        }
+        size_t ofs = reinterpret_cast<size_t>(base_orig) % ALIGNMENT;
+        if(ofs > 0) {
+          base = base_orig + (ALIGNMENT - ofs);
+        } else {
+          base = base_orig;
+        }
+        prealloced = false;
 
-      // we should not have been given a NetworkSegment by our caller
-      assert(!segment);
-      // advertise our allocation in case the network can register it
-      local_segment.assign(NetworkSegmentInfo::HostMem,
-			   base, _size);
-      segment = &local_segment;
+        // we should not have been given a NetworkSegment by our caller
+        assert(!segment);
+        // advertise our allocation in case the network can register it
+        local_segment.assign(NetworkSegmentInfo::HostMem,
+                             base, _size);
+        segment = &local_segment;
+      } else {
+        base = 0;
+        prealloced = true;
+      }
     }
     log_malloc.debug("CPU memory at %p, size = %zd%s%s", base, _size, 
 		     prealloced ? " (prealloced)" : "",
@@ -1424,6 +1310,77 @@ namespace Realm {
       free(base_orig);
   }
 
+  // LocalCPUMemory supports ExternalMemoryResource
+  bool LocalCPUMemory::attempt_register_external_resource(RegionInstanceImpl *inst,
+                                                          size_t& inst_offset)
+  {
+    ExternalMemoryResource *res = dynamic_cast<ExternalMemoryResource *>(inst->metadata.ext_resource);
+    if(res != 0) {
+      // automatic success - make the "offset" be the difference between the
+      //  base address we were given and our own allocation's base
+      void *mem_base = get_direct_ptr(0, 0); // only our subclasses know this
+      // underflow is ok here - it'll work itself out when we add the mem_base
+      //  back in on accesses
+      inst_offset = res->base - reinterpret_cast<uintptr_t>(mem_base);
+      return true;
+    }
+
+    // not a kind we recognize
+    return false;
+  }
+
+  void LocalCPUMemory::unregister_external_resource(RegionInstanceImpl *inst)
+  {
+    // nothing actually to clean up
+  }
+
+  // for re-registration purposes, generate an ExternalInstanceResource *
+  //  (if possible) for a given instance, or a subset of one
+  ExternalInstanceResource *LocalCPUMemory::generate_resource_info(RegionInstanceImpl *inst,
+								   const IndexSpaceGeneric *subspace,
+								   span<const FieldID> fields,
+								   bool read_only)
+  {
+    // TODO: handle subspaces
+    //assert(subspace == 0);
+
+    // compute the bounds of the instance relative to our base
+    assert(inst->metadata.is_valid() &&
+	   "instance metadata must be valid before accesses are performed");
+    assert(inst->metadata.layout);
+    InstanceLayoutGeneric *ilg = inst->metadata.layout;
+    uintptr_t rel_base, extent;
+    if(subspace == 0) {
+      // want full instance
+      rel_base = 0;
+      extent = ilg->bytes_used;
+    } else {
+      assert(!fields.empty());
+      uintptr_t limit;
+      for(size_t i = 0; i < fields.size(); i++) {
+        uintptr_t f_base, f_limit;
+        if(!subspace->impl->compute_affine_bounds(ilg, fields[i], f_base, f_limit))
+          return 0;
+        if(i == 0) {
+          rel_base = f_base;
+          limit = f_limit;
+        } else {
+          rel_base = std::min(rel_base, f_base);
+          limit = std::max(limit, f_limit);
+        }
+      }
+      extent = limit - rel_base;
+    }
+
+    void *mem_base = get_direct_ptr(inst->metadata.inst_offset + rel_base,
+                                    extent); // only our subclasses know this
+    if(!mem_base)
+      return 0;
+
+    return new ExternalMemoryResource(reinterpret_cast<uintptr_t>(mem_base),
+                                      extent, read_only);
+  }
+
   void LocalCPUMemory::get_bytes(off_t offset, void *dst, size_t size)
   {
     memcpy(dst, base+offset, size);
