@@ -1,4 +1,4 @@
-/* Copyright 2021 Stanford University, NVIDIA Corporation
+/* Copyright 2023 Stanford University, NVIDIA Corporation
  *
  * Licensed under the Apache License, Version 2.0 (the "License");
  * you may not use this file except in compliance with the License.
@@ -44,6 +44,16 @@ extern "C" {
       return 1;
   }
 
+  REALM_PUBLIC_API
+  int omp_get_num_places(void)
+  {
+    // Some newer versions of the OpenMP runtime enable control over OpenMP
+    // threads ploacement onto different sockets or cores. This sort of thing
+    // is controlled by Realm in our implementation, so just return the number
+    // of threads allocated to this worker pool.
+    return omp_get_num_threads();
+  }
+
   REALM_PUBLIC_API
   int omp_get_max_threads(void)
   {
@@ -54,6 +64,12 @@ extern "C" {
       return 1;
   }
 
+  REALM_PUBLIC_API
+  int omp_get_num_procs(void)
+  {
+    return omp_get_max_threads();
+  }
+
   REALM_PUBLIC_API
   int omp_get_thread_num(void)
   {
@@ -276,7 +292,7 @@ extern "C" {
       // give back the whole loop and hope for the best
       *istart = start;
       *iend = end;
-      return (start < end);
+      return ((incr > 0) ? (start < end) : (start > end));
     }
 
     // loops must be inside work items
@@ -288,7 +304,7 @@ extern "C" {
 
     wi->work_item->schedule.start_dynamic(start, end, incr, chunk);
     int64_t span_start, span_end;
-    long stride = 0; // not used
+    int64_t stride = 0; // not used
     bool more = wi->work_item->schedule.next_dynamic(span_start, span_end, stride);
     if(more) {
       *istart = span_start;
@@ -297,6 +313,43 @@ extern "C" {
     return more;
   }
 
+  REALM_PUBLIC_API
+  bool GOMP_loop_ull_dynamic_start(bool up,
+                                   uint64_t start, uint64_t end,
+                                   uint64_t incr, uint64_t chunk,
+                                   uint64_t *istart, uint64_t *iend)
+  {
+    Realm::ThreadPool::WorkerInfo *wi = Realm::ThreadPool::get_worker_info(true);
+    if(!wi) {
+      // give back the whole loop and hope for the best
+      *istart = start;
+      *iend = end;
+      return (up ? (start < end) : (start > end));
+    }
+
+    // loops must be inside work items
+    assert(wi->work_item != 0);
+
+    log_omp.debug() << "loop dynamic start: start=" << start
+		    << " end=" << end << " incr=" << (int64_t)incr
+		    << " chunk=" << chunk;
+
+    // shift values down to int64_t rank for LoopSchedule
+    int64_t start_shifted = static_cast<int64_t>(start - (uint64_t(1) << 63));
+    int64_t end_shifted = static_cast<int64_t>(end - (uint64_t(1) << 63));
+
+    wi->work_item->schedule.start_dynamic(start_shifted, end_shifted, incr, chunk);
+    int64_t span_start, span_end;
+    int64_t stride = 0; // not used
+    bool more = wi->work_item->schedule.next_dynamic(span_start, span_end, stride);
+    if(more) {
+      // shift from int64_t back to uint64_t range
+      *istart = static_cast<uint64_t>(span_start) + (uint64_t(1) << 63);
+      *iend = static_cast<uint64_t>(span_end) + (uint64_t(1) << 63);
+    }
+    return more;
+  }
+
   REALM_PUBLIC_API
   void GOMP_loop_end_nowait(void)
   {
@@ -379,6 +432,35 @@ extern "C" {
     return more;
   }
 
+  REALM_PUBLIC_API
+  bool GOMP_loop_ull_dynamic_next(uint64_t *istart, uint64_t *iend)
+  {
+    Realm::ThreadPool::WorkerInfo *wi = Realm::ThreadPool::get_worker_info(false);
+    if(!wi)
+      return false;  // complained already above
+
+    // loops must be inside work items
+    assert(wi->work_item != 0);
+
+    log_omp.debug() << "loop dynamic next: pstart=" << *istart
+		    << " pend=" << *iend;
+
+    int64_t span_start, span_end, stride;
+    bool more = wi->work_item->schedule.next_dynamic(span_start, span_end,
+						     stride);
+
+    if(more) {
+      // shift from int64_t back to uint64_t range
+      *istart = static_cast<uint64_t>(span_start) + (uint64_t(1) << 63);
+      *iend = static_cast<uint64_t>(span_end) + (uint64_t(1) << 63);
+      log_omp.debug() << "loop dynamic next: start=" << *istart
+		      << " end=" << *iend;
+    } else
+      log_omp.debug() << "loop dynamic next: done";
+
+    return more;
+  }
+
   static unsigned hash_gomp_critical_name(void **pptr)
   {
     uintptr_t v = reinterpret_cast<uintptr_t>(pptr);
@@ -1188,18 +1270,22 @@ namespace Realm {
        (wi->num_threads == 1) ||
        ((incr > 0) && (*plower > *pupper)) ||
        ((incr < 0) && (*plower < *pupper))) {
+      *pstride = *pupper - *plower + ((incr > 0) ? 1 : -1);
       return;
     }
 
     //printf("static_init(%p, %d, %d)\n", loc, global_tid, schedtype);
     switch(schedtype) {
     case 34 /* kmp_sch_static */:
+    case 33 /* kmp_sch_static_chunked - (chunk ignored - TODO) */:
       {
 	T iters;
 	if(incr > 0) {
 	  iters = 1 + (*pupper - *plower) / incr;
+          *pstride = *pupper - *plower + 1;
 	} else {
 	  iters = 1 + (*plower - *pupper) / -incr;
+          *pstride = *pupper - *plower - 1;
 	}
 	T whole = iters / wi->num_threads;
 	T leftover = iters - (whole * wi->num_threads);
@@ -1210,7 +1296,8 @@ namespace Realm {
 	// special case for when some threads get no iterations at all
 	*plastiter = ((whole > 0) ? (wi->thread_id == (wi->num_threads - 1)) :
 		                    (((T)(wi->thread_id)) == (leftover - 1)));
-	//printf("static(%d, %d, %d, %d, %d)\n", *plower, *pupper, *pstride, incr, *plastiter);
+        //log_omp.print() << "static: " << *plower << " " << *pupper << " " << *pstride
+        //                << " " << incr << " " << *plastiter;
 	return;
       }
 
