@@ -1,4 +1,4 @@
-/* Copyright 2021 Stanford University, NVIDIA Corporation
+/* Copyright 2023 Stanford University, NVIDIA Corporation
  *
  * Licensed under the Apache License, Version 2.0 (the "License");
  * you may not use this file except in compliance with the License.
@@ -80,21 +80,10 @@ namespace Realm {
 			   //std::set<RegionInstance> instances_needed,
 			   Event wait_on, int priority) const
     {
-      DetailedTimer::ScopedPush sp(TIME_LOW_LEVEL);
       ProcessorImpl *p = get_runtime()->get_processor_impl(*this);
 
       GenEventImpl *finish_event = GenEventImpl::create_genevent();
       Event e = finish_event->current_event();
-#ifdef EVENT_GRAPH_TRACE
-      Event enclosing = find_enclosing_termination_event();
-      log_event_graph.info("Task Request: %d " IDFMT 
-                            " (" IDFMT ",%d) (" IDFMT ",%d)"
-                            " (" IDFMT ",%d) %d %p %ld",
-                            func_id, id, e.id, e.gen,
-                            wait_on.id, wait_on.gen,
-                            enclosing.id, enclosing.gen,
-                            priority, args, arglen);
-#endif
 
       p->spawn_task(func_id, args, arglen, ProfilingRequestSet(),
 		    wait_on, finish_event, ID(e).event_generation(), priority);
@@ -105,21 +94,10 @@ namespace Realm {
                            const ProfilingRequestSet &reqs,
 			   Event wait_on, int priority) const
     {
-      DetailedTimer::ScopedPush sp(TIME_LOW_LEVEL);
       ProcessorImpl *p = get_runtime()->get_processor_impl(*this);
 
       GenEventImpl *finish_event = GenEventImpl::create_genevent();
       Event e = finish_event->current_event();
-#ifdef EVENT_GRAPH_TRACE
-      Event enclosing = find_enclosing_termination_event();
-      log_event_graph.info("Task Request: %d " IDFMT 
-                            " (" IDFMT ",%d) (" IDFMT ",%d)"
-                            " (" IDFMT ",%d) %d %p %ld",
-                            func_id, id, e.id, e.gen,
-                            wait_on.id, wait_on.gen,
-                            enclosing.id, enclosing.gen,
-                            priority, args, arglen);
-#endif
 
       p->spawn_task(func_id, args, arglen, reqs,
 		    wait_on, finish_event, ID(e).event_generation(), priority);
@@ -225,7 +203,8 @@ namespace Realm {
 	    it != local_procs.end();
 	    it++) {
 	  ProcessorImpl *p = get_runtime()->get_processor_impl(*it);
-	  p->register_task(func_id, tro->codedesc, tro->userdata);
+	  bool ok = p->register_task(func_id, tro->codedesc, tro->userdata);
+	  assert(ok); // TODO: poison completion instead
 	}
       }
 
@@ -301,7 +280,8 @@ namespace Realm {
 	    it != local_procs.end();
 	    it++) {
 	  ProcessorImpl *p = get_runtime()->get_processor_impl(*it);
-	  p->register_task(func_id, tro->codedesc, tro->userdata);
+	  bool ok = p->register_task(func_id, tro->codedesc, tro->userdata);
+	  assert(ok); // TODO: poison completion instead
 	}
       }
 
@@ -515,12 +495,13 @@ namespace Realm {
       assert(0);
     }
 
-    void ProcessorImpl::register_task(Processor::TaskFuncID func_id,
+    bool ProcessorImpl::register_task(Processor::TaskFuncID func_id,
 				      CodeDescriptor& codedesc,
 				      const ByteArrayRef& user_data)
     {
       // should never be called
       assert(0);
+      return false;
     }
 
     // helper function for spawn implementations
@@ -867,14 +848,16 @@ namespace Realm {
 	  it != local_procs.end();
 	  it++) {
 	ProcessorImpl *p = get_runtime()->get_processor_impl(*it);
-	p->register_task(args.func_id, codedesc, userdata);
+	bool ok = p->register_task(args.func_id, codedesc, userdata);
+	assert(ok); // TODO: poison completion instead
       }
     } else {
       for(std::vector<Processor>::const_iterator it = procs.begin();
 	  it != procs.end();
 	  it++) {
 	ProcessorImpl *p = get_runtime()->get_processor_impl(*it);
-	p->register_task(args.func_id, codedesc, userdata);
+	bool ok = p->register_task(args.func_id, codedesc, userdata);
+	assert(ok); // TODO: poison completion instead
       }
     }
 
@@ -1075,7 +1058,7 @@ namespace Realm {
     enqueue_or_defer_task(task, start_event, &deferred_spawn_cache);
   }
 
-  void LocalTaskProcessor::register_task(Processor::TaskFuncID func_id,
+  bool LocalTaskProcessor::register_task(Processor::TaskFuncID func_id,
 					 CodeDescriptor& codedesc,
 					 const ByteArrayRef& user_data)
   {
@@ -1110,7 +1093,7 @@ namespace Realm {
       // first, make sure we haven't seen this task id before
       if(task_table.count(func_id) > 0) {
         log_taskreg.fatal() << "duplicate task registration: proc=" << me << " func=" << func_id;
-        assert(0);
+        return false;
       }
 
       TaskTableEntry &tte = task_table[func_id];
@@ -1119,6 +1102,8 @@ namespace Realm {
     }
 
     log_taskreg.info() << "task " << func_id << " registered on " << me << ": " << codedesc;
+
+    return true;
   }
 
   void LocalTaskProcessor::execute_task(Processor::TaskFuncID func_id,
@@ -1389,7 +1374,6 @@ namespace Realm {
 						      const void *data,
 						      size_t datalen)
   {
-    DetailedTimer::ScopedPush sp(TIME_LOW_LEVEL);
     ProcessorImpl *p = get_runtime()->get_processor_impl(args.proc);
 
     log_task.debug() << "received remote spawn request:"
