@@ -1,4 +1,4 @@
-/* Copyright 2021 Stanford University, NVIDIA Corporation
+/* Copyright 2023 Stanford University, NVIDIA Corporation
  *
  * Licensed under the Apache License, Version 2.0 (the "License");
  * you may not use this file except in compliance with the License.
@@ -60,6 +60,7 @@ namespace Realm {
 #ifdef USE_PYGILSTATE_CALLS
     get_symbol(this->PyGILState_Ensure, "PyGILState_Ensure");
     get_symbol(this->PyGILState_Release, "PyGILState_Release");
+    get_symbol(this->PyGILState_Check, "PyGILState_Check");
 #else
     get_symbol(this->PyThreadState_New, "PyThreadState_New");
     get_symbol(this->PyThreadState_Clear, "PyThreadState_Clear");
@@ -391,12 +392,12 @@ namespace Realm {
 
     // take lock and go into normal task scheduler loop
     {
-      AutoLock<> al(lock);
+      AutoLock<FIFOMutex> al(lock);
       KernelThreadTaskScheduler::scheduler_loop();
     }
 #if 0
     // now go into main scheduler loop, holding scheduler lock for whole thing
-    AutoLock<> al(lock);
+    AutoLock<FIFOMutex> al(lock);
     while(true) {
       // remember the work counter value before we start so that we don't iterate
       //   unnecessarily
@@ -557,7 +558,7 @@ namespace Realm {
     // if this gets called before we're done initializing the interpreter,
     //  we need a simple blocking wait
     if(!interpreter_ready) {
-      AutoLock<> al(lock);
+      AutoLock<FIFOMutex> al(lock);
 
       log_py.debug() << "waiting during initialization";
       bool really_blocked = try_update_thread_state(thread,
@@ -580,23 +581,20 @@ namespace Realm {
     }
 
     // if we got here through a cffi call, the GIL has already been released,
-    //  so try to handle that case here - a call PyEval_SaveThread
-    //  if the GIL is not held will assert-fail, and while a call to
-    //  PyThreadState_Swap is technically illegal (and unsafe if python-created
-    //  threads exist), it does what we want for now
+    //  so try to handle that case here - first, we need to check if the GIL
+    //  is still held, if yes, we call PyEval_SaveThread to save the thread
+    //  and release the GIL; if not, the GIL has been released by cffi. 
     // NOTE: we use PyEval_{Save,Restore}Thread here even if USE_PYGILSTATE_CALLS
     //  is defined, as a call to PyGILState_Release will destroy a thread
     //  context - the Save/Restore take care of the actual lock, and since we
     //  restore each python thread on the OS thread that owned it intially, the
     //  PyGILState TLS stuff should remain consistent
-    PyThreadState *saved = (pyproc->interpreter->api->PyThreadState_Swap)(0);
-    if(saved != 0) {
+    PyThreadState *saved = 0;
+    if((pyproc->interpreter->api->PyGILState_Check)() == 1) {
       log_py.info() << "python worker sleeping - releasing GIL";
-      // put it back so we can save it properly
-      (pyproc->interpreter->api->PyThreadState_Swap)(saved);
       // would like to sanity-check that this returns the expected thread state,
       //  but that would require taking the PythonThreadTaskScheduler's lock
-      (pyproc->interpreter->api->PyEval_SaveThread)();
+      saved = (pyproc->interpreter->api->PyEval_SaveThread)();
       log_py.debug() << "SaveThread -> " << saved;
     } else
       log_py.info() << "python worker sleeping - GIL already released";
@@ -606,6 +604,7 @@ namespace Realm {
     if(saved) {
       log_py.info() << "python worker awake - acquiring GIL";
       log_py.debug() << "RestoreThread <- " << saved;
+      assert( (pyproc->interpreter->api->PyGILState_Check)() == 0 );
       (pyproc->interpreter->api->PyEval_RestoreThread)(saved);
     } else
       log_py.info() << "python worker awake - not acquiring GIL";
@@ -615,7 +614,7 @@ namespace Realm {
   {
     // handle the wakening of the initialization thread specially
     if(!interpreter_ready) {
-      AutoLock<> al(lock);
+      AutoLock<FIFOMutex> al(lock);
       resumable_workers.put(thread, 0);
     } else {
       KernelThreadTaskScheduler::thread_ready(thread);
@@ -702,7 +701,7 @@ namespace Realm {
 #ifdef REALM_USE_OPENMP
     if(_omp_workers > 0) {
       // create a pool (except for one thread, which is the main task thread)
-      omp_threadpool = new ThreadPool(_omp_workers - 1,
+      omp_threadpool = new ThreadPool(me, _omp_workers - 1,
 				      name, -1 /*numa_node*/, _stack_size, crs);
     } else
       omp_threadpool = 0;
@@ -749,6 +748,7 @@ namespace Realm {
     interpreter = new PythonInterpreter;
     // the call to PyEval_InitThreads in the PythonInterpreter constructor
     //  acquired the GIL on our behalf already
+    assert( (interpreter->api->PyGILState_Check)() == 1 );
     master_thread = (interpreter->api->PyThreadState_Get)();
 
     // always need the python threading module
@@ -798,6 +798,22 @@ namespace Realm {
     //  to deal with the case where 'import threading' never got called
     (interpreter->api->PyRun_SimpleString)("__import__('threading').current_thread()");
 
+    // Python > 3.9.7 requires the main thread to collapse the threading module,
+    // but we're in a non-master thread when tearing down the interpreter here,
+    // which leads to a hang caused by bpo-1596321. As a workaround, we forcibly
+    // unlock the master thread's shutdown lock, so that when Python's threading
+    // module tries to lock and unlock it to emulate the thread join, there won't
+    // be a deadlock. See this GitHub issue for the detail:
+    //   https://github.com/nv-legate/cunumeric/issues/187
+    (interpreter->api->PyRun_SimpleString)(
+      "[main_thread._tstate_lock.release() "
+      "if v.major >= 3 and (v.minor > 9 or (v.minor == 9 and v.micro > 7)) "
+      "and main_thread != curr_thread "
+      "else None "
+      "for main_thread in (__import__('threading').main_thread(),) "
+      "for curr_thread in (__import__('threading').current_thread(),) "
+      "for v in (__import__('sys').version_info,)]");
+
     delete interpreter;
     interpreter = 0;
     master_thread = 0;
@@ -912,7 +928,7 @@ namespace Realm {
     sched->remove_task_queue(&group->task_queue);
   }
 
-  void LocalPythonProcessor::register_task(Processor::TaskFuncID func_id,
+  bool LocalPythonProcessor::register_task(Processor::TaskFuncID func_id,
                                            CodeDescriptor& codedesc,
                                            const ByteArrayRef& user_data)
   {
@@ -922,6 +938,8 @@ namespace Realm {
     treg->codedesc = new CodeDescriptor(codedesc);
     treg->user_data = user_data;
     sched->add_internal_task(treg);
+    // TODO: failure wouldn't be detected until later...
+    return true;
   }
 
   void LocalPythonProcessor::execute_task(Processor::TaskFuncID func_id,
@@ -1127,7 +1145,7 @@ namespace Realm {
 						       cfg_init_scripts);
           runtime->add_processor(pi);
 
-          // create affinities between this processor and system/reg memories
+          // create affinities between this processor and system, numa, reg, and zc memories
           // if the memory is one we created, use the kernel-reported distance
           // to adjust the answer
           std::vector<MemoryImpl *>& local_mems = runtime->nodes[Network::my_node_id].memories;
@@ -1135,7 +1153,7 @@ namespace Realm {
               it2 != local_mems.end();
               ++it2) {
             Memory::Kind kind = (*it2)->get_kind();
-            if((kind != Memory::SYSTEM_MEM) && 
+            if((kind != Memory::SYSTEM_MEM) && (kind != Memory::SOCKET_MEM) &&
                (kind != Memory::REGDMA_MEM) && (kind != Memory::Z_COPY_MEM))
               continue;
 
