@@ -1,4 +1,4 @@
-/* Copyright 2021 Stanford University, NVIDIA Corporation
+/* Copyright 2023 Stanford University, NVIDIA Corporation
  *
  * Licensed under the Apache License, Version 2.0 (the "License");
  * you may not use this file except in compliance with the License.
@@ -168,7 +168,6 @@ namespace Realm {
     Event Reservation::acquire(unsigned mode /* = 0 */, bool exclusive /* = true */,
 		     Event wait_on /* = Event::NO_EVENT */) const
     {
-      DetailedTimer::ScopedPush sp(TIME_LOW_LEVEL);
       //printf("LOCK(" IDFMT ", %d, %d, " IDFMT ") -> ", id, mode, exclusive, wait_on.id);
       // early out - if the event has obviously triggered (or is NO_EVENT)
       //  don't build up continuation
@@ -190,8 +189,6 @@ namespace Realm {
     Event Reservation::try_acquire(bool retry, unsigned mode /* = 0 */, bool exclusive /* = true */,
 				   Event wait_on /* = Event::NO_EVENT */) const
     {
-      DetailedTimer::ScopedPush sp(TIME_LOW_LEVEL);
-
       ReservationImpl *impl = get_runtime()->get_lock_impl(*this);
 
       // if we have an unsatisfied precondition, we need to wait for that before actually trying
@@ -216,7 +213,6 @@ namespace Realm {
     // releases a held lock - release can be deferred until an event triggers
     void Reservation::release(Event wait_on /* = Event::NO_EVENT */) const
     {
-      DetailedTimer::ScopedPush sp(TIME_LOW_LEVEL);
       // early out - if the event has obviously triggered (or is NO_EVENT)
       //  don't build up continuation
       if(wait_on.has_triggered()) {
@@ -229,11 +225,8 @@ namespace Realm {
     }
 
     // Create a new lock, destroy an existing lock
-    /*static*/ Reservation Reservation::create_reservation(size_t _data_size /*= 0*/)
+    /*static*/ Reservation Reservation::create_reservation()
     {
-      DetailedTimer::ScopedPush sp(TIME_LOW_LEVEL);
-      //DetailedTimer::ScopedPush sp(18);
-
       // see if the freelist has an event we can reuse
       ReservationImpl *impl = get_runtime()->local_reservation_free_list->alloc_entry();
       assert(impl);
@@ -259,7 +252,7 @@ namespace Realm {
       return Reservation::NO_RESERVATION;
     }
 
-    void Reservation::destroy_reservation()
+    void Reservation::destroy_reservation(Event wait_on)
     {
       log_reservation.info() << "reservation destroyed: rsrv=" << *this;
 
@@ -268,16 +261,17 @@ namespace Realm {
 	ActiveMessage<DestroyLockMessage> amsg(ID(*this).rsrv_creator_node());
 	amsg->actual = *this;
 	amsg->dummy = *this;
+        amsg->wait_on = wait_on;
 	amsg.commit();
 	return;
       }
 
       // to destroy a local lock, we first must lock it (exclusively)
-      ReservationImpl *lock_impl = get_runtime()->get_lock_impl(*this);
-      Event e = lock_impl->acquire(0, true, ReservationImpl::ACQUIRE_BLOCKING);
+      Event e = acquire(0/*mode*/, true/*exclusive*/, wait_on);
       if(!e.has_triggered()) {
 	EventImpl::add_waiter(e, new DeferredLockDestruction(*this));
       } else {
+        ReservationImpl *lock_impl = get_runtime()->get_lock_impl(*this);
 	// got grant immediately - can release reservation now
 	lock_impl->release_reservation();
       }
@@ -293,8 +287,7 @@ namespace Realm {
       init(Reservation::NO_RESERVATION, (unsigned)-1);
     }
 
-    void ReservationImpl::init(Reservation _me, unsigned _init_owner,
-			  size_t _data_size /*= 0*/)
+    void ReservationImpl::init(Reservation _me, unsigned _init_owner)
     {
       me = _me;
       owner = _init_owner;
@@ -305,21 +298,11 @@ namespace Realm {
       remote_waiter_mask = NodeSet(); 
       remote_sharer_mask = NodeSet();
       requested = false;
-      if(_data_size) {
-	local_data = malloc(_data_size);
-	local_data_size = _data_size;
-        own_local = true;
-      } else {
-        local_data = 0;
-	local_data_size = 0;
-        own_local = false;
-      }
     }
 
     /*static*/ void LockReleaseMessage::handle_message(NodeID sender, const LockReleaseMessage &msg,
 						       const void *data, size_t datalen)
     {
-      DetailedTimer::ScopedPush sp(TIME_LOW_LEVEL);
       assert(0);
     }
 
@@ -328,7 +311,6 @@ namespace Realm {
 						     const void *data, size_t datalen,
 						     TimeLimit work_until)
     {
-      DetailedTimer::ScopedPush sp(TIME_LOW_LEVEL);
       log_reservation.debug(          "reservation request granted: reservation=" IDFMT " mode=%d", // mask=%lx",
 	       args.lock.id, args.mode); //, args.remote_waiter_mask);
 
@@ -347,15 +329,11 @@ namespace Realm {
 	const int *pos = (const int *)data;
 
 	size_t waiter_count = *pos++;
-	assert(datalen == (((waiter_count+1) * sizeof(int)) + impl->local_data_size));
+	assert(datalen == ((waiter_count+1) * sizeof(int)));
 	impl->remote_waiter_mask.clear();
 	for(size_t i = 0; i < waiter_count; i++)
 	  impl->remote_waiter_mask.add(*pos++);
 
-	// is there local data to grab?
-	if(impl->local_data_size > 0)
-          memcpy(impl->local_data, pos, impl->local_data_size);
-
 	if(args.mode == 0) // take ownership if given exclusive access
 	  impl->owner = Network::my_node_id;
 	impl->mode = args.mode;
@@ -681,7 +659,7 @@ namespace Realm {
       {
         // Make a buffer for storing our waiter mask and the the local data
 	size_t waiter_count = copy_waiters.size();
-        size_t payload_size = ((waiter_count+1) * sizeof(int)) + local_data_size;
+        size_t payload_size = ((waiter_count+1) * sizeof(int));
         int *payload = (int*)malloc(payload_size);
 	int *pos = payload;
 	*pos++ = waiter_count;
@@ -689,10 +667,6 @@ namespace Realm {
 	    it != copy_waiters.end();
 	    ++it)
 	  *pos++ = *it;
-	//for(int i = 0; i < MAX_NUM_NODES; i++)
-	//  if(copy_waiters.contains(i))
-	//    *pos++ = i;
-        memcpy(pos, local_data, local_data_size);
 	ActiveMessage<LockGrantMessage> amsg(grant_target, payload_size);
 	amsg->lock = me;
 	amsg->mode = 0; // TODO: figure out shared cases
@@ -743,12 +717,6 @@ namespace Realm {
 	assert(local_shared.empty());
 	assert(retries.empty());
 	assert(in_use);
-        // Mark that we no longer own our data
-        if (own_local)
-          free(local_data);
-        local_data = NULL;
-        local_data_size = 0;
-        own_local = false;
       	in_use = false;
 	count = ZERO_COUNT;
       }
@@ -761,7 +729,7 @@ namespace Realm {
 						       const void *data, size_t datalen)
     {
       Reservation a = args.actual;
-      a.destroy_reservation();
+      a.destroy_reservation(args.wait_on);
     }
 
 
@@ -781,7 +749,7 @@ namespace Realm {
     Event rsrv_ready;             // ready event for a pending rsrv request
     unsigned sleeper_count;
     Event sleeper_event;
-    CondVar condvar;        // for external waiters
+    Mutex::CondVar condvar;        // for external waiters
 
     // pointer math to obtain FastRsrvState reference
     static FastRsrvState& get_frs(FastReservation& frsv);
@@ -861,7 +829,7 @@ namespace Realm {
     }
     // mutex, condvar must be manually constructed
     new(&frs.mutex) Mutex;
-    new(&frs.condvar) CondVar(frs.mutex);
+    new(&frs.condvar) Mutex::CondVar(frs.mutex);
     frs.rsrv_ready = Event::NO_EVENT;
     frs.sleeper_count = 0;
     frs.sleeper_event = Event::NO_EVENT;
@@ -889,8 +857,8 @@ namespace Realm {
       }
     }
     // mutex, condvar must be manually destroyed
-    frs.condvar.~CondVar();
-    frs.mutex.~Mutex();
+    call_destructor(&frs.condvar);
+    call_destructor(&frs.mutex);
   }
 
   // NOT copyable
@@ -1483,7 +1451,6 @@ namespace Realm {
   void LockRequestMessage::handle_message(NodeID sender, const LockRequestMessage &args,
 					     const void *data, size_t datalen)
   {
-    DetailedTimer::ScopedPush sp(TIME_LOW_LEVEL);
     ReservationImpl *impl = get_runtime()->get_lock_impl(args.lock);
 
     log_reservation.debug("reservation request: reservation=" IDFMT ", node=%d, mode=%d",
@@ -1549,7 +1516,7 @@ namespace Realm {
       {
         // Make a buffer for storing our waiter mask and the the local data
 	size_t waiter_count = copy_waiters.size();
-        size_t payload_size = ((waiter_count+1) * sizeof(int)) + impl->local_data_size;
+        size_t payload_size = ((waiter_count+1) * sizeof(int));
         int *payload = (int*)malloc(payload_size);
 	int *pos = payload;
 	*pos++ = waiter_count;
@@ -1557,10 +1524,6 @@ namespace Realm {
 	    it != copy_waiters.end();
 	    ++it)
 	  *pos++ = *it;
-	//for(int i = 0; i < MAX_NUM_NODES; i++)
-	//  if(copy_waiters.contains(i))
-	//    *pos++ = i;
-        memcpy(pos, impl->local_data, impl->local_data_size);
 	ActiveMessage<LockGrantMessage> amsg(grant_target, payload_size);
 	amsg->lock = args.lock;
 	amsg->mode = 0; // always grant exclusive for now
