@@ -1,4 +1,4 @@
-/* Copyright 2021 Stanford University, NVIDIA Corporation
+/* Copyright 2023 Stanford University, NVIDIA Corporation
  *
  * Licensed under the Apache License, Version 2.0 (the "License");
  * you may not use this file except in compliance with the License.
@@ -20,6 +20,7 @@
 #include "realm/proc_impl.h"
 #include "realm/mem_impl.h"
 #include "realm/inst_impl.h"
+#include "realm/transfer/ib_memory.h"
 
 #include "realm/activemsg.h"
 #include "realm/deppart/preimage.h"
@@ -41,6 +42,10 @@
 #include "realm/kokkos_interop.h"
 #endif
 
+#ifdef REALM_USE_NVTX
+#include "realm/nvtx.h"
+#endif
+
 #include <string.h>
 
 #if defined(REALM_ON_LINUX) || defined(REALM_ON_MACOS) || defined(REALM_ON_FREEBSD)
@@ -49,6 +54,7 @@
 #endif
 
 #ifdef REALM_ON_WINDOWS
+#include <windows.h>
 #include <processthreadsapi.h>
 #include <synchapi.h>
 
@@ -255,8 +261,8 @@ namespace Realm {
 	if (b->generations.empty())
 	  continue;
 
-	os << "Barrier " << b->me << ": gen=" << b->generation
-	   << " subscr=" << b->gen_subscribed << "\n";
+	os << "Barrier " << b->me << ": gen=" << b->generation.load()
+	   << " subscr=" << b->gen_subscribed.load() << "\n";
 	for (std::map<EventImpl::gen_t, BarrierImpl::Generation*>::const_iterator git = 
 	       b->generations.begin(); git != b->generations.end(); git++) {
 	  const EventWaiter::EventWaiterList &waiters = git->second->local_waiters;
@@ -384,7 +390,7 @@ namespace Realm {
       return realm_library_version;
     }
 
-#if defined(REALM_USE_MPI) || defined(REALM_USE_GASNET1) || defined(REALM_USE_GASNETEX) || defined(REALM_USE_KOKKOS)
+#if defined(REALM_USE_UCX) || defined(REALM_USE_MPI) || defined(REALM_USE_GASNET1) || defined(REALM_USE_GASNETEX) || defined(REALM_USE_KOKKOS)
     // global flag that tells us if a realm runtime has already been
     //  initialized in this process - some underlying libraries (e.g. mpi,
     //  gasnet, kokkos) do not permit reinitialization
@@ -396,9 +402,12 @@ namespace Realm {
     //  (instead of e.g. mpi spawner information)
     bool Runtime::network_init(int *argc, char ***argv)
     {
-#if defined(REALM_USE_MPI) || defined(REALM_USE_GASNET1) || defined(REALM_USE_GASNETEX) || defined(REALM_USE_KOKKOS)
+#if defined(REALM_USE_UCX) || defined(REALM_USE_MPI) || defined(REALM_USE_GASNET1) || defined(REALM_USE_GASNETEX) || defined(REALM_USE_KOKKOS)
       if(runtime_initialized) {
         fprintf(stderr, "ERROR: reinitialization not supported by these Realm components:"
+#ifdef REALM_USE_UCX
+                " ucx"
+#endif
 #ifdef REALM_USE_MPI
                 " mpi"
 #endif
@@ -661,6 +670,7 @@ namespace Realm {
     , pin_util_procs(false)
     , cpu_bgwork_timeslice(0)
     , util_bgwork_timeslice(0)
+    , use_ext_sysmem(true)
   {}
 
   CoreModule::~CoreModule(void)
@@ -682,6 +692,7 @@ namespace Realm {
       .add_option_bool("-ll:pin_util", m->pin_util_procs)
       .add_option_int("-ll:cpu_bgwork", m->cpu_bgwork_timeslice)
       .add_option_int("-ll:util_bgwork", m->util_bgwork_timeslice)
+      .add_option_int("-ll:ext_sysmem", m->use_ext_sysmem)
       .parse_command_line(cmdline);
 
     return m;
@@ -693,12 +704,27 @@ namespace Realm {
   {
     Module::create_memories(runtime);
 
+    MemoryImpl *sysmem;
     if(sysmem_size > 0) {
       Memory m = runtime->next_local_memory_id();
-      MemoryImpl *mi = new LocalCPUMemory(m, sysmem_size,
-          -1/*don't care numa domain*/, Memory::SYSTEM_MEM);
-      runtime->add_memory(mi);
-    }
+      sysmem = new LocalCPUMemory(m, sysmem_size,
+                                  -1/*don't care numa domain*/,
+                                  Memory::SYSTEM_MEM);
+      runtime->add_memory(sysmem);
+    } else
+      sysmem = 0;
+
+    // create a memory that will hold external instances (the sysmem above
+    //  might get registered with network and/or gpus, but external instances
+    //  usually won't have those affinities)
+    if(use_ext_sysmem || !sysmem) {
+      Memory m = runtime->next_local_memory_id();
+      ext_sysmem = new LocalCPUMemory(m, 0 /*size*/,
+                                      -1 /*don't care numa domain*/,
+                                      Memory::SYSTEM_MEM);
+      runtime->add_memory(ext_sysmem);
+    } else
+      ext_sysmem = sysmem;
   }
 
   // create any processors provided by the module (default == do nothing)
@@ -809,7 +835,8 @@ namespace Realm {
 	sampling_profiler(true /*system default*/),
 	num_local_memories(0), num_local_ib_memories(0),
 	num_local_processors(0),
-	module_registrar(this)
+	module_registrar(this),
+        modules_created(false)
     {
       machine = new MachineImpl;
     }
@@ -906,6 +933,30 @@ namespace Realm {
       return code_translators;
     }
 
+    Module *RuntimeImpl::get_module_untyped(const char *name) const
+    {
+      if(!modules_created) {
+        log_runtime.fatal() << "request for '" << name
+                            << "' module before all modules have been created";
+        abort();
+      }
+
+      // TODO: worth building a map here instead?
+      for(std::vector<Module *>::const_iterator it = modules.begin();
+          it != modules.end();
+          ++it)
+        if(!strcmp(name, (*it)->get_name().c_str()))
+          return *it;
+
+      for(std::vector<NetworkModule *>::const_iterator it = network_modules.begin();
+          it != network_modules.end();
+          ++it)
+        if(!strcmp(name, (*it)->get_name().c_str()))
+          return *it;
+
+      return 0;
+    }
+
     static void add_proc_mem_affinities(MachineImpl *machine,
 					const std::set<Processor>& procs,
 					const std::set<Memory>& mems,
@@ -958,8 +1009,6 @@ namespace Realm {
 
     bool RuntimeImpl::network_init(int *argc, char ***argv)
     {
-      DetailedTimer::init_timers();
-
       // if we're given empty or non-existent argc/argv, start from a
       //  dummy command line with a single string (which is supposed to be
       //  the name of the binary) so that the network module and/or the
@@ -1174,6 +1223,32 @@ namespace Realm {
       // very first thing - let the logger initialization happen
       Logger::configure_from_cmdline(cmdline);
 
+      // calibrate timers
+      int use_cpu_tsc = -1; // dont care
+      uint64_t force_cpu_tsq_freq = 0; // no force
+      {
+        CommandLineParser cp;
+        cp.add_option_int("-ll:cputsc", use_cpu_tsc);
+        cp.add_option_int_units("-ll:tscfreq", force_cpu_tsq_freq, 'm', false/*!binary*/);
+        bool ok = cp.parse_command_line(cmdline);
+        assert(ok);
+      }
+      Clock::calibrate(use_cpu_tsc, force_cpu_tsq_freq);
+
+#ifdef REALM_USE_NVTX
+      // need to init nvtx at the very beginning
+      std::vector<std::string> nvtx_module_list;
+      {
+        CommandLineParser cp;
+        // modules are defined as the key of the nvtx_categories_predefined in nvtx.cc
+        //   if all is passed, all modules will be enabled. 
+        cp.add_option_stringlist("-ll:nvtx_modules", nvtx_module_list);
+        bool ok = cp.parse_command_line(cmdline);
+        assert(ok);
+      }
+      init_nvtx(nvtx_module_list);
+#endif
+
       // start up the threading subsystem - modules will likely want threads
       if(!Threading::initialize()) exit(1);
 
@@ -1186,6 +1261,7 @@ namespace Realm {
       // now load modules
       module_registrar.create_static_modules(cmdline, modules);
       module_registrar.create_dynamic_modules(cmdline, modules);
+      modules_created = true;
 
       PartitioningOpQueue::configure_from_cmdline(cmdline);
 
@@ -1224,6 +1300,10 @@ namespace Realm {
       int bitset_twolevel = -1024; // i.e. yes if > 1024 nodes
       int active_msg_handler_threads = 0; // default is none (use bgwork)
       bool active_msg_handler_bgwork = true;
+      // This dummy network list is actually handled in network_init()
+      // this is just here to help verify low-level arguement
+      std::vector<std::string> dummy_network_list;
+      size_t replheap_size = 16 << 20;
 
       CommandLineParser cp;
       cp.add_option_int_units("-ll:rsize", reg_mem_size, 'm')
@@ -1259,6 +1339,11 @@ namespace Realm {
       cp.add_option_int("-ll:aminline", Config::max_inline_message_time);
       cp.add_option_int("-ll:ahandlers", active_msg_handler_threads);
       cp.add_option_int("-ll:handler_bgwork", active_msg_handler_bgwork);
+      cp.add_option_stringlist("-ll:networks", dummy_network_list);
+      cp.add_option_int_units("-ll:replheap", replheap_size);
+
+      // The default of path_cache_size is 0, when it is set to non-zero, the caching is enabled.
+      cp.add_option_int("-ll:path_cache_size", Config::path_cache_lru_size);
 
       bool cmdline_ok = cp.parse_command_line(cmdline);
 
@@ -1564,11 +1649,6 @@ namespace Realm {
       filemem = new FileMemory(get_runtime()->next_local_memory_id());
       get_runtime()->add_memory(filemem);
 
-      for(std::vector<Module *>::const_iterator it = modules.begin();
-	  it != modules.end();
-	  it++)
-	(*it)->create_dma_channels(this);
-
       for(std::vector<Module *>::const_iterator it = modules.begin();
 	  it != modules.end();
 	  it++)
@@ -1596,6 +1676,15 @@ namespace Realm {
 	exit(1);
       }
 
+      // create the "replicated heap" that puts instance layouts and sparsity
+      //  maps where non-CPU devices can see them
+      repl_heap.init(replheap_size, 1 /*chunks*/);
+
+      for(std::vector<Module *>::const_iterator it = modules.begin();
+	  it != modules.end();
+	  it++)
+	(*it)->create_dma_channels(this);
+
       {
         // iterate over all local processors and add affinities for them
 	// all of this should eventually be moved into appropriate modules
@@ -1828,6 +1917,11 @@ namespace Realm {
 #endif
       }
 
+      if (Config::path_cache_lru_size) {
+        assert(Config::path_cache_lru_size > 0);
+        init_path_cache();
+      }
+
       return true;
     }
 
@@ -2265,6 +2359,15 @@ namespace Realm {
 	(*it)->shutdown();
       stop_dma_system();
 
+      repl_heap.cleanup();
+
+      // let network-dependent cleanup happen before we detach
+      for(std::vector<Module *>::iterator it = modules.begin();
+          it != modules.end();
+          it++) {
+        (*it)->pre_detach_cleanup();
+      }
+
       // detach from the network
       for(std::vector<NetworkModule *>::const_iterator it = network_modules.begin();
 	  it != network_modules.end();
@@ -2313,12 +2416,6 @@ namespace Realm {
                rt->local_index_space_free_list->next_alloc,
                rt->local_proc_group_free_list->next_alloc);
       }
-#endif
-#ifdef EVENT_GRAPH_TRACE
-      {
-        //FILE *log_file = Logger::get_log_file();
-        show_event_waiters(/*log_file*/);
-      }
 #endif
       cleanup_query_caches();
       // delete processors, memories, nodes, etc.
@@ -2384,6 +2481,15 @@ namespace Realm {
       // very last step - unregister our signal handlers
       unregister_error_signal_handler();
 
+      if (Config::path_cache_lru_size) {
+        finalize_path_cache();
+      }
+
+#ifdef REALM_USE_NVTX
+      // finalize nvtx
+      finalize_nvtx();
+#endif
+
       return shutdown_result_code;
     }
 
