@@ -1,4 +1,4 @@
-/* Copyright 2021 Stanford University, NVIDIA Corporation
+/* Copyright 2023 Stanford University, NVIDIA Corporation
  *
  * Licensed under the Apache License, Version 2.0 (the "License");
  * you may not use this file except in compliance with the License.
@@ -251,10 +251,6 @@ namespace Realm {
     log_util(((func_id == 3) ? LEVEL_SPEW : LEVEL_INFO), 
 	     "task start: %d (%p) (%s)", func_id, fptr, argstr);
 #endif
-#ifdef EVENT_GRAPH_TRACE
-    start_enclosing(finish_event);
-    unsigned long long start = TimeStamp::get_current_time_in_micros();
-#endif
 
     // does the profiler want to know where it was run?
     if(measurements.wants_measurement<ProfilingMeasurements::OperationProcessorUsage>()) {
@@ -314,14 +310,6 @@ namespace Realm {
       // and clear the TLS when we're done
       // TODO: get this right when using user threads
       //ThreadLocal::current_processor = Processor::NO_PROC;
-
-#ifdef EVENT_GRAPH_TRACE
-      unsigned long long stop = TimeStamp::get_current_time_in_micros();
-      finish_enclosing();
-      log_event_graph.debug("Task Time: (" IDFMT ",%d) %lld",
-			    finish_event.id, finish_event.gen,
-			    (stop - start));
-#endif
     } else {
       // !ok_to_run
       thread->stop_operation(this);
@@ -435,14 +423,14 @@ namespace Realm {
   void TaskQueue::add_subscription(NotificationCallback *callback,
 				   priority_t higher_than /*= PRI_NEG_INF*/)
   {
-    AutoLock<> al(mutex);
+    AutoLock<FIFOMutex> al(mutex);
     callbacks.push_back(callback);
     callback_priorities.push_back(higher_than);
   }
 
   void TaskQueue::remove_subscription(NotificationCallback *callback)
   {
-    AutoLock<> al(mutex);
+    AutoLock<FIFOMutex> al(mutex);
     std::vector<NotificationCallback *>::iterator cit = callbacks.begin();
     std::vector<priority_t>::iterator cpit = callback_priorities.begin();
     while (cit != callbacks.end()) {
@@ -479,7 +467,7 @@ namespace Realm {
 	it++) {
       Task *new_task;
       {
-	AutoLock<> al((*it)->mutex);
+	AutoLock<FIFOMutex> al((*it)->mutex);
 	new_task = (*it)->ready_task_list.pop_front(task_priority+1);
       }
       if(new_task) {
@@ -489,7 +477,7 @@ namespace Realm {
 	// if we got something better, put back the old thing (if any)
 	if(task) {
 	  {
-	    AutoLock<> al(task_source->mutex);
+	    AutoLock<FIFOMutex> al(task_source->mutex);
 	    task_source->ready_task_list.push_front(task);
 	  }
 	  if(task_source->task_count_gauge)
@@ -517,7 +505,7 @@ namespace Realm {
 //#endif
 
       {
-	AutoLock<> al(mutex);
+	AutoLock<FIFOMutex> al(mutex);
 	if(ready_task_list.empty(task->priority))
 	  notify_priority = task->priority;
 	ready_task_list.push_back(task);
@@ -564,7 +552,7 @@ namespace Realm {
     priority_t notify_priority = tasks.front()->priority;
 
     {
-      AutoLock<> al(mutex);
+      AutoLock<FIFOMutex> al(mutex);
       // cancel notification if we already have equal/higher priority tasks
       if(!ready_task_list.empty(notify_priority))
 	notify_priority = PRI_NEG_INF;
@@ -588,7 +576,8 @@ namespace Realm {
   //
 
   ThreadedTaskScheduler::WorkCounter::WorkCounter(void)
-    : counter(0), wait_value(-1), interrupt_flag(0), condvar(mutex)
+    : counter(0)
+    , interrupt_flag(0)
   {}
 
   ThreadedTaskScheduler::WorkCounter::~WorkCounter(void)
@@ -605,125 +594,84 @@ namespace Realm {
     if(interrupt_flag)
       interrupt_flag->store(true);
 
-    // common case is that we'll bump the counter and nobody cares, so do
-    //  this without a lock - if the LSB is set, we'll need to look at the
-    //  wait value (and the RMW on counter makes that read synchronize
-    //  properly with the write)
-    long long old_value = counter.fetch_add_acqrel(2);
-    if(REALM_LIKELY((old_value & 1) == 0)) return;
+    // bump the counter and atomically see if any sleepers exist
+    uint64_t old_value = counter.fetch_add_acqrel(1 << SLEEPER_BITS);
+    uint64_t sleepers = (old_value & ((1 << SLEEPER_BITS) - 1));
 
-    // second non-locked check - read the wait_value and skip out if it's
-    //  not our job to signal it
-    long long wv_snapshot = wait_value.load();
-    if(REALM_LIKELY(wv_snapshot != old_value)) return;
+    // no sleepers?  all done
+    if(REALM_LIKELY(sleepers == 0))
+      return;
 
-    {
-      AutoLock<> al(mutex);
+    // try to claim sleepers to wake up
+    uint64_t expected = old_value + (1 << SLEEPER_BITS);
+    while(true) {
+      uint64_t newval = expected - sleepers;
 
-      // now that we hold the lock, wait_value cannot change - it might not
-      //  be the one expect though if a later waiter signaled ours for us
-      long long wv_reread = wait_value.load();
-#ifdef DEBUG_REALM
-      assert(wv_reread >= old_value);
-#endif
-      if(wv_reread == old_value) {
-	long long check = counter.fetch_add(1);  // clear LSB, moving it forward
-	assert((check & 1) != 0);
-	wait_value.store(-1);
-	condvar.broadcast();
+      if(counter.compare_exchange(expected, newval)) {
+        // success - we must now notify exactly that many sleepers on the
+        //   doorbell list, but we have to get exclusive popping access first
+
+        uint64_t tstate;
+        uint64_t act_pops = db_mutex.attempt_enter(sleepers, tstate);
+        while(act_pops != 0) {
+          db_list.notify_newest(act_pops, true /*prefer_spinning*/);
+
+          // try to release mutex, but loop in case work was delegated to us
+          act_pops = db_mutex.attempt_exit(tstate);
+        }
+
+        break;
+      } else {
+        // two failure cases:
+        //  1) somebody else has bumped the work counter in the mean time -
+        //      this is great, because waking the sleepers is now their problem
+        //  2) additional sleepers (on the _post-incremented_) count have shown
+        //      up - it's probably futile to wake the older sleepers, but let's
+        //      try it anyway
+        if((expected >> SLEEPER_BITS) != ((old_value >> SLEEPER_BITS) + 1)) {
+          // case 1
+          break;
+        } else {
+          // case 2
+          continue;
+        }
       }
     }
   }
 
-  // waits until new work arrives - this will possibly take the counter lock and 
-  // sleep, so should not be called while holding another lock
-  void ThreadedTaskScheduler::WorkCounter::wait_for_work(long long old_counter)
+  // waits until new work arrives - this will possibly go to sleep,
+  //  so should not be called while holding another lock
+  void ThreadedTaskScheduler::WorkCounter::wait_for_work(uint64_t old_counter)
   {
-    // the old counter with the LSB set will be our wait value
-    long long oc_w_lsb = old_counter | 1;
-    
-    // we assume the caller tried check_for_work() before dropping
-    //  their locks and calling us, so take and hold the lock the entire time
-    AutoLock<> al(mutex);
+    // use a CAS to add to the sleeper count, but only if the counter hasn't
+    //  moved on
+    uint64_t oldval = counter.load_acquire();
+    while(true) {
+      if((oldval >> SLEEPER_BITS) != old_counter) {
+        // counter moved - no need to wait
+        return;
+      }
 
-    // see if there's already a waiter
-    long long wv_read = wait_value.load();
-
-    // unlikely case - there's a waiter on a FUTURE counter
-    if(wv_read > oc_w_lsb) {
-#ifdef DEBUG_REALM
-      assert(counter.load() > oc_w_lsb);
-#endif
-      return;
+      if(counter.compare_exchange(oldval, oldval + 1)) {
+        // success - we can add our doorbell to the list and wait on it below
+        break;
+      }
     }
 
-    // waiters on an older counter should be woken before we go to sleep
-    if((wv_read >= 0) && (wv_read < oc_w_lsb)) {
-      condvar.broadcast();
-#ifdef DEBUG_REALM
-      // LSB should already be set, but we still have to do the fetch_or below
-      assert((counter.load() & 1) == 1);
-#endif
-    }
-
-    if(wv_read == oc_w_lsb) {
-      // somebody's already waiting on this counter, and the eventual waker
-      //   is already going to know it, so we can just sleep along with them
-      long long wc_reread = counter.load();
-#ifdef DEBUG_REALM
-      assert((wc_reread & 1) == 1);
-#endif
-      // if we observe the count has moved on, we can return early - somebody
-      //  else will still signal the other waiters and clear wait_value
-      if(wc_reread > oc_w_lsb)
-	return;
+    Doorbell *db = Doorbell::get_thread_doorbell();
+    db->prepare();
+    if(db_list.add_doorbell(db)) {
+      db->wait();
     } else {
-      wait_value.store(oc_w_lsb);
-      long long wc_prev = counter.fetch_or_acqrel(1);
+      // signal came first, so cancel our wait
+      db->cancel();
+    }
 #ifdef DEBUG_REALM
-      // LSB should not have already been set
-      assert((wc_prev & 1) == 0);
+    // sanity-check that counter did move
+    uint64_t checkval = counter.load();
+    assert((checkval >> SLEEPER_BITS) != old_counter);
+    (void)checkval;
 #endif
-      // if the counter has already moved on, cancel the wait ourselves
-      if(wc_prev > oc_w_lsb) {
-	wait_value.store(-1);
-	counter.fetch_add(1);
-	return;
-      }
-    }
-
-    // we've already done our early out on a counter bump, so just spin here
-    //  until the wait_value is changed by somebody else
-    while(wait_value.load() == oc_w_lsb) {
-#define WORK_COUNTER_TIMEOUT_CHECK
-#ifdef WORK_COUNTER_TIMEOUT_CHECK
-      bool awakened = condvar.timedwait(1000000000LL);
-      if(!awakened && (counter.load() != oc_w_lsb)) {
-	static atomic<int> warncount(0);
-	static const int MAX_WARNINGS = 10;
-	int c = warncount.fetch_add(1) + 1;
-	if(c <= MAX_WARNINGS)
-	  log_task.warning() << "missed work counter wakeup?"
-			     << ((c == MAX_WARNINGS) ? " - suppressing further messages" : "");
-	// don't clear wait_value, since there may be other threads that did
-	//  want to get waken up, but we don't want to fall through to the
-	//  checks below either, so just return
-	return;
-      }
-#else
-      condvar.wait();
-#endif
-    }
-
-//#ifdef DEBUG_REALM
-    // wait value should have moved on, or been set back to -1
-    long long wv_check = wait_value.load();
-    if(!((wv_check == -1) || (wv_check > oc_w_lsb))) {
-      log_task.fatal() << "HELP: wv_check=" << wv_check << " oc_w_lsb=" << oc_w_lsb << " wv_read=" << wv_read << " counter=" << counter.load();
-      abort();
-    }
-    assert((wv_check == -1) || (wv_check > oc_w_lsb));
-//#endif
   }
 
 
@@ -759,7 +707,7 @@ namespace Realm {
 
   void ThreadedTaskScheduler::add_task_queue(TaskQueue *queue)
   {
-    AutoLock<> al(lock);
+    AutoLock<FIFOMutex> al(lock);
 
     task_queues.push_back(queue);
 
@@ -769,7 +717,7 @@ namespace Realm {
 
   void ThreadedTaskScheduler::remove_task_queue(TaskQueue *queue)
   {
-    AutoLock<> al(lock);
+    AutoLock<FIFOMutex> al(lock);
     for (std::vector<TaskQueue *>::iterator it = task_queues.begin(); it != task_queues.end();++it) {
       if (*it == queue) {
         //found; we erase and exit
@@ -782,6 +730,13 @@ namespace Realm {
     queue->remove_subscription(&wcu_task_queues);
   }
 
+  void ThreadedTaskScheduler::add_task_context(const TaskContextManager *_manager)
+  {
+    AutoLock<FIFOMutex> al(lock);
+
+    context_managers.push_back(_manager);
+  }
+
   void ThreadedTaskScheduler::configure_bgworker(BackgroundWorkManager *manager,
 						 long long max_timeslice,
 						 int numa_domain)
@@ -828,7 +783,7 @@ namespace Realm {
   {
     // there's a potential race between a thread blocking and being reawakened,
     //  so take the scheduler lock and THEN try to mark the thread as blocked
-    AutoLock<> al(lock);
+    AutoLock<FIFOMutex> al(lock);
 
     bool really_blocked = try_update_thread_state(thread,
 						  Thread::STATE_BLOCKING,
@@ -846,7 +801,7 @@ namespace Realm {
       log_sched.debug() << "thread w/ scheduler lock spinning: " << thread;
       spinning_workers.insert(thread);
       while(true) {
-	long long old_work_counter = work_counter.read_counter();
+	uint64_t old_work_counter = work_counter.read_counter();
 	switch(thread->get_state()) {
 	case Thread::STATE_READY:
 	  {
@@ -901,7 +856,7 @@ namespace Realm {
 
       // remember the work counter value before we start so that we don't iterate
       //   unnecessarily
-      //long long old_work_counter = work_counter.read_counter();
+      //uint64_t old_work_counter = work_counter.read_counter();
 
       // first choice - is there a resumable worker we can yield to?
       if(!resumable_workers.empty()) {
@@ -973,7 +928,7 @@ namespace Realm {
 
     // TODO: might be nice to do this in a lock-free way, since this is called by
     //  some other thread
-    AutoLock<> al(lock);
+    AutoLock<FIFOMutex> al(lock);
 
     // if this was a spinning thread, remove it from the list and poke the
     //  work counter in cases its execution resource is napping
@@ -1018,7 +973,7 @@ namespace Realm {
     int old_priority;
 
     {
-      AutoLock<> al(lock);
+      AutoLock<FIFOMutex> al(lock);
       std::map<Thread *, int>::iterator it = worker_priorities.find(thread);
       assert(it != worker_priorities.end());
       old_priority = it->second;
@@ -1049,7 +1004,7 @@ namespace Realm {
       while(true) {
 	// remember the work counter value before we start so that we don't iterate
 	//   unnecessarily
-	long long old_work_counter = work_counter.read_counter();
+	uint64_t old_work_counter = work_counter.read_counter();
 
 	// internal tasks always take precedence
 	while(!internal_tasks.empty()) {
@@ -1071,9 +1026,20 @@ namespace Realm {
 	  //  scheduler lock
 	  ThreadLocal::scheduler_lock++;
 
+	  // if we have any context managers, create the necessary contexts
+	  std::vector<void *> contexts(context_managers.size(), 0);
+	  if(!context_managers.empty())
+	    for(size_t i = 0; i < context_managers.size(); i++)
+	      contexts[i] = context_managers[i]->create_context(0 /*task*/);
+
 	  execute_internal_task(itask);
 
-	  ThreadLocal::scheduler_lock--;
+          // destroy contexts in reverse order
+	  if(!context_managers.empty())
+	    for(size_t i = context_managers.size(); i > 0; i--)
+	      context_managers[i-1]->destroy_context(0 /*task*/, contexts[i-1]);
+
+          ThreadLocal::scheduler_lock--;
 
 	  // we don't delete the internal task object - it can do that itself
 	  //  if it wants, or the requestor of the operation can do it once
@@ -1121,12 +1087,32 @@ namespace Realm {
 	  // release the lock while we run the task
 	  lock.unlock();
 
+	  // if we have any context managers, create the necessary contexts
+	  std::vector<void *> contexts(context_managers.size(), 0);
+	  if(!context_managers.empty()) {
+	    for(size_t i = 0; i < context_managers.size(); i++)
+	      contexts[i] = context_managers[i]->create_context(task);
+	    // add a reference to the task to prevent it from being deleted
+	    //  before we destroy our contexts
+	    // NOTE: this does NOT prevent the task from finishing - a context
+	    //  should add an AsyncWorkItem to the task if that is needed
+	    task->add_reference();
+	  }
+
 #ifndef NDEBUG
 	  bool ok =
 #endif
 	    execute_task(task);
 	  assert(ok);  // no fault recovery yet
 
+	  if(!context_managers.empty()) {
+	    // destroy contexts in reverse order
+	    for(size_t i = context_managers.size(); i > 0; i--)
+	      context_managers[i-1]->destroy_context(task, contexts[i-1]);
+	    // and only then remove the extra reference we were holding
+	    task->remove_reference();
+	  }
+
 	  lock.lock();
 
 	  worker_priorities.erase(Thread::self());
@@ -1227,11 +1213,11 @@ namespace Realm {
   // an entry point that takes the scheduler lock explicitly
   void ThreadedTaskScheduler::scheduler_loop_wlock(void)
   {
-    AutoLock<> al(lock);
+    AutoLock<FIFOMutex> al(lock);
     scheduler_loop();
   }
 
-  void ThreadedTaskScheduler::wait_for_work(long long old_work_counter)
+  void ThreadedTaskScheduler::wait_for_work(uint64_t old_work_counter)
   {
     // clear the interrupt flag (if we use it) before we check the work counter
     //  to avoid a missed interrupt
@@ -1292,7 +1278,7 @@ namespace Realm {
   {
     // fire up the minimum number of workers
     {
-      AutoLock<> al(lock);
+      AutoLock<FIFOMutex> al(lock);
 
       update_worker_count(cfg_min_active_workers, cfg_min_active_workers);
 
@@ -1310,7 +1296,7 @@ namespace Realm {
 
     // wait for all workers to finish
     {
-      AutoLock<> al(lock);
+      AutoLock<FIFOMutex> al(lock);
 
       while(!all_workers.empty() || !terminating_workers.empty())
 	shutdown_condvar.wait();
@@ -1325,11 +1311,11 @@ namespace Realm {
 
     // see if we're supposed to be active yet
     {
-      AutoLock<> al(lock);
+      AutoLock<FIFOMutex> al(lock);
 
       if(active_workers.count(thread) == 0) {
 	// nope, sleep on a CV until we are
-	CondVar my_cv(lock);
+        FIFOMutex::CondVar my_cv(lock);
 	sleeping_threads[thread] = &my_cv;
 
 	while(active_workers.count(thread) == 0)
@@ -1344,7 +1330,7 @@ namespace Realm {
   {
     log_sched.info() << "scheduler worker terminating: sched=" << this << " worker=" << thread;
 
-    AutoLock<> al(lock);
+    AutoLock<FIFOMutex> al(lock);
 
     // if the thread is still in our all_workers list, this was unexpected
     if(all_workers.count(thread) > 0) {
@@ -1416,7 +1402,7 @@ namespace Realm {
       active_workers.erase(Thread::self());
     assert(count == 1);
 
-    CondVar my_cv(lock);
+    FIFOMutex::CondVar my_cv(lock);
     sleeping_threads[Thread::self()] = &my_cv;
 
     // with kernel threads, sleeping and waking are separable actions
@@ -1438,7 +1424,7 @@ namespace Realm {
     active_workers.insert(to_wake);
 
     // if they have a CV (they might not yet), poke that
-    std::map<Thread *, CondVar *>::const_iterator it = sleeping_threads.find(to_wake);
+    std::map<Thread *, FIFOMutex::CondVar *>::const_iterator it = sleeping_threads.find(to_wake);
     if(it != sleeping_threads.end())
       it->second->signal();
   }
@@ -1469,7 +1455,7 @@ namespace Realm {
       worker_wake(switch_to);
   }
   
-  void KernelThreadTaskScheduler::wait_for_work(long long old_work_counter)
+  void KernelThreadTaskScheduler::wait_for_work(uint64_t old_work_counter)
   {
     // if we have a dedicated core and we don't care about power, we can spin-wait here
     bool spin_wait = false;
@@ -1531,7 +1517,7 @@ namespace Realm {
 
     // fire up the host threads (which will fire up initial workers)
     {
-      AutoLock<> al(lock);
+      AutoLock<FIFOMutex> al(lock);
 
       update_worker_count(cfg_num_host_threads, cfg_num_host_threads);
 
@@ -1555,7 +1541,7 @@ namespace Realm {
   {
     log_sched.info() << "scheduler shutdown requested: sched=" << this;
     // set the shutdown flag and wait for all the host threads to exit
-    AutoLock<> al(lock);
+    AutoLock<FIFOMutex> al(lock);
 
     // make sure everybody actually started before we tell them to shut down
     while(host_startups_remaining > 0) {
@@ -1603,7 +1589,7 @@ namespace Realm {
   void UserThreadTaskScheduler::host_thread_loop(void)
   {
     log_sched.debug() << "host thread started: sched=" << this << " thread=" << Thread::self();
-    AutoLock<> al(lock);
+    AutoLock<FIFOMutex> al(lock);
 
     // create a user worker thread - it won't start right away
     Thread *worker = worker_create(false);
@@ -1736,7 +1722,7 @@ namespace Realm {
     assert(0);
   }
 
-  void UserThreadTaskScheduler::wait_for_work(long long old_work_counter)
+  void UserThreadTaskScheduler::wait_for_work(uint64_t old_work_counter)
   {
     // if we have a dedicated core and we don't care about power, we can spin-wait here
     bool spin_wait = false;
