@@ -1,4 +1,4 @@
-/* Copyright 2021 Stanford University, NVIDIA Corporation
+/* Copyright 2023 Stanford University, NVIDIA Corporation
  *
  * Licensed under the Apache License, Version 2.0 (the "License");
  * you may not use this file except in compliance with the License.
@@ -21,6 +21,10 @@
 #include "realm/faults.h"
 #include "realm/operation.h"
 
+#ifdef REALM_USE_NVTX
+#include "realm/nvtx.h"
+#endif
+
 #ifdef DEBUG_USWITCH
 #include <stdio.h>
 #endif
@@ -747,6 +751,9 @@ namespace Realm {
 #ifdef REALM_USE_PTHREADS
   /*static*/ void *KernelThread::pthread_entry(void *data)
   {
+#ifdef REALM_USE_NVTX
+    init_nvtx_thread("RealmKernalThread");
+#endif
     KernelThread *thread = (KernelThread *)data;
 
 #ifdef REALM_USE_ALTSTACK
@@ -804,7 +811,11 @@ namespace Realm {
     // this is last so that the scheduler can delete us if it wants to
     if(thread->scheduler)
       thread->scheduler->thread_terminating(thread);
-    
+
+#ifdef REALM_USE_NVTX
+    finalize_nvtx_thread();
+#endif
+
     return 0;
   }
 #endif
@@ -812,6 +823,9 @@ namespace Realm {
 #ifdef REALM_ON_WINDOWS
   /*static*/ DWORD WINAPI KernelThread::winthread_entry(LPVOID data)
   {
+#ifdef REALM_USE_NVTX
+    init_nvtx_thread("RealmKernalThread");
+#endif    
     KernelThread *thread = (KernelThread *)data;
 
     // set up TLS so people can find us
@@ -834,6 +848,10 @@ namespace Realm {
     if (thread->scheduler)
       thread->scheduler->thread_terminating(thread);
 
+#ifdef REALM_USE_NVTX
+    finalize_nvtx_thread();
+#endif
+
     return 0;
   }
 #endif
@@ -1041,7 +1059,7 @@ namespace Realm {
       CHECK_PTHREAD( pthread_attr_destroy(&attr) );
 
       // sanity-check the resulting value
-      if(minstack < PTHREAD_STACK_MIN) break;
+      if(minstack < (size_t)PTHREAD_STACK_MIN) break;
 
       static_tls_size = minstack - PTHREAD_STACK_MIN;
       log_thread.debug() << "static tls size = " << static_tls_size << " (from glibc __pthread_get_minstack)";
@@ -1565,11 +1583,7 @@ namespace Realm {
   /*static*/ void Thread::yield(void)
   {
 #ifdef REALM_USE_PTHREADS
-#ifdef REALM_ON_MACOS
     sched_yield();
-#else
-    pthread_yield();
-#endif
 #endif
 #ifdef REALM_ON_WINDOWS
     SwitchToThread();
