@@ -1,5 +1,5 @@
-/* Copyright 2021 Stanford University
- * Copyright 2021 Los Alamos National Laboratory
+/* Copyright 2023 Stanford University
+ * Copyright 2023 Los Alamos National Laboratory
  *
  * Licensed under the Apache License, Version 2.0 (the "License");
  * you may not use this file except in compliance with the License.
@@ -38,6 +38,7 @@
 #include "realm/mem_impl.h"
 #include "realm/inst_impl.h"
 #include "realm/bgwork.h"
+#include "realm/utils.h"
 
 namespace Realm {
 
@@ -113,6 +114,12 @@ namespace Realm {
       //  modified during this call
       void swap(SequenceAssembler& other);
 
+      // imports data from this assembler into another (this is thread-safe
+      //  on the `other` but assumes no changes being made on `this`)
+      void import(SequenceAssembler& other) const;
+
+      bool empty() const;
+
       // asks if a span exists - return value is number of bytes from the
       //  start that do
       size_t span_exists(size_t start, size_t count);
@@ -122,9 +129,11 @@ namespace Realm {
       size_t add_span(size_t pos, size_t count);
 
     protected:
+      Mutex *ensure_mutex();
+
       atomic<size_t> contig_amount_x2;  // everything from [0, contig_amount) is covered - LSB indicates potential presence of noncontig spans
-      atomic<size_t> first_noncontig; // nothing in [contig_amount, first_noncontig) 
-      Mutex *mutex;
+      atomic<size_t> first_noncontig; // nothing in [contig_amount, first_noncontig)
+      atomic<Mutex *> mutex; // created on first use
       std::map<size_t, size_t> spans;  // noncontiguous spans
     };
 
@@ -172,12 +181,13 @@ namespace Realm {
       ReductionOpID id;
       bool is_fold;
       bool in_place;
+      bool is_exclusive;
 
       // default constructor == no reduction requested
-      XferDesRedopInfo() : id(0), is_fold(false), in_place(false) {}
+      XferDesRedopInfo() : id(0), is_fold(false), in_place(false), is_exclusive(false) {}
 
-      XferDesRedopInfo(ReductionOpID _id, bool _is_fold, bool _in_place)
-      : id(_id), is_fold(_is_fold), in_place(_in_place) {}
+      XferDesRedopInfo(ReductionOpID _id, bool _is_fold, bool _in_place, bool _is_exclusive)
+      : id(_id), is_fold(_is_fold), in_place(_in_place), is_exclusive(_is_exclusive) {}
     };
 
     class AddressList {
@@ -207,10 +217,10 @@ namespace Realm {
 
       void set_addrlist(AddressList *_addrlist);
 
-      int get_dim();
-      uintptr_t get_offset();
-      uintptr_t get_stride(int dim);
-      size_t remaining(int dim);
+      int get_dim() const;
+      uintptr_t get_offset() const;
+      uintptr_t get_stride(int dim) const;
+      size_t remaining(int dim) const;
       void advance(int dim, size_t amount);
 
       void skip_bytes(size_t bytes);
@@ -218,11 +228,16 @@ namespace Realm {
     protected:
       AddressList *addrlist;
       bool partial;
-      static const int MAX_DIM = 8;
+      // we need to be one larger than any index space realm supports, since
+      //  we use the contiguous bytes within a field as a "dimension" in some
+      //  cases
+      static const int MAX_DIM = REALM_MAX_DIM + 1;
       int partial_dim;
       size_t pos[MAX_DIM];
     };
 
+    std::ostream& operator<<(std::ostream& os, const AddressListCursor& alc);
+
     // a control port is used to steer inputs/outputs of transfer descriptors -
     //   the information is encoded into 32b packets which may be read/written
     //   at different times due to flow control, so the encoder and decoder
@@ -286,6 +301,7 @@ namespace Realm {
       NodeID launch_node;
       //uint64_t /*bytes_submit, */bytes_read, bytes_write/*, bytes_total*/;
       atomic<bool> iteration_completed;
+      atomic<int64_t> bytes_write_pending;
       atomic<bool> transfer_completed;
       // current input and output port mask
       uint64_t current_in_port_mask, current_out_port_mask;
@@ -404,7 +420,9 @@ namespace Realm {
       void update_pre_bytes_total(int port_idx, size_t pre_bytes_total);
       void update_next_bytes_read(int port_idx, size_t offset, size_t size);
 
-      bool is_completed(void);
+      // called once iteration is complete, but we need to track in flight
+      //  writes, flush byte counts, etc.
+      void begin_completion();
 
       void mark_completed();
 
@@ -499,6 +517,9 @@ namespace Realm {
       // as a side effect, the input/output control information is updated - the
       //  actual input/output ports involved in the next transfer are stored there
       size_t get_addresses(size_t min_xfer_size, ReadSequenceCache *rseqcache);
+      size_t get_addresses(size_t min_xfer_size, ReadSequenceCache *rseqcache,
+                           const InstanceLayoutPieceBase *&in_nonaffine,
+                           const InstanceLayoutPieceBase *&out_nonaffine);
 
       // after a call to 'get_addresses', this call updates the various data
       //  structures to record that transfers for 'total_{read,write}_bytes' bytes
@@ -548,7 +569,8 @@ namespace Realm {
 		     const std::vector<XferDesPortInfo>& inputs_info,
 		     const std::vector<XferDesPortInfo>& outputs_info,
 		     int _priority,
-		     const void *_fill_data, size_t _fill_size);
+		     const void *_fill_data, size_t _fill_size,
+                     size_t _fill_total);
 
       long get_requests(Request** requests, long nr);
 
@@ -669,7 +691,8 @@ namespace Realm {
 				   const std::vector<XferDesPortInfo>& outputs_info,
 				   int priority,
 				   XferDesRedopInfo redop_info,
-				   const void *fill_data, size_t fill_size) = 0;
+				   const void *fill_data, size_t fill_size,
+                                   size_t fill_total) = 0;
     };
 
     struct XferDesCreateMessageBase {
@@ -703,7 +726,8 @@ namespace Realm {
 				   const std::vector<XferDesPortInfo>& outputs_info,
 				   int priority,
 				   XferDesRedopInfo redop_info,
-				   const void *fill_data, size_t fill_size);
+				   const void *fill_data, size_t fill_size,
+                                   size_t fill_total);
 
     protected:
       uintptr_t channel;
@@ -760,32 +784,62 @@ namespace Realm {
 	  GLOBAL_KIND,
 	  LOCAL_RDMA,
 	  REMOTE_RDMA,
+          MEMORY_BITMASK,
 	};
 	SrcDstType src_type, dst_type;
+        struct MemBitmask {
+          NodeID node;
+          static const int BITMASK_SIZE = (1 << ID::MEMORY_INDEX_WIDTH) >> 6;
+          uint64_t mems[BITMASK_SIZE], ib_mems[BITMASK_SIZE];
+        };
 	union {
 	  Memory src_mem;
 	  Memory::Kind src_kind;
+          MemBitmask src_bitmask;
 	};
 	union {
 	  Memory dst_mem;
 	  Memory::Kind dst_kind;
+          MemBitmask dst_bitmask;
 	};
 	XferDesKind xd_kind;
 	unsigned bandwidth; // units = MB/s = B/us
 	unsigned latency;   // units = ns
+        unsigned frag_overhead; // units = ns
+        unsigned char max_src_dim, max_dst_dim;
 	bool redops_allowed; // TODO: list of redops?
 	bool serdez_allowed; // TODO: list of serdez ops?
+
+        // mutators to modify less-common fields
+        SupportedPath& set_max_dim(int src_and_dst_dim);
+        SupportedPath& set_max_dim(int src_dim, int dst_dim);
+        SupportedPath& allow_redops();
+        SupportedPath& allow_serdez();
+
+        // only valid when a SupportedPath is modifiable by the above methods
+        //  (i.e. only on the creator node and only until another path is added)
+        SupportedPath *chain;
+
+        void populate_memory_bitmask(span <const Memory> mems,
+                                     NodeID node,
+                                     MemBitmask& bitmask);
       };
 
       const std::vector<SupportedPath>& get_paths(void) const;
 
-      virtual bool supports_path(Memory src_mem, Memory dst_mem,
-				 CustomSerdezID src_serdez_id,
-				 CustomSerdezID dst_serdez_id,
-				 ReductionOpID redop_id,
-				 XferDesKind *kind_ret = 0,
-				 unsigned *bw_ret = 0,
-				 unsigned *lat_ret = 0);
+      // returns 0 if the path is not supported, or a strictly-positive
+      //  estimate of the time required (in nanoseconds) to transfer data
+      //  along a supported path
+      virtual uint64_t supports_path(Memory src_mem, Memory dst_mem,
+                                     CustomSerdezID src_serdez_id,
+                                     CustomSerdezID dst_serdez_id,
+                                     ReductionOpID redop_id,
+                                     size_t total_bytes,
+                                     const std::vector<size_t> *src_frags,
+                                     const std::vector<size_t> *dst_frags,
+                                     XferDesKind *kind_ret = 0,
+                                     unsigned *bw_ret = 0,
+                                     unsigned *lat_ret = 0);
 
       virtual RemoteChannelInfo *construct_remote_info() const;
 
@@ -795,24 +849,33 @@ namespace Realm {
       virtual void wakeup_xd(XferDes *xd) = 0;
 
     protected:
-      void add_path(Memory src_mem, Memory dst_mem,
-		    unsigned bandwidth, unsigned latency,
-		    bool redops_allowed, bool serdez_allowed,
-		    XferDesKind xd_kind);
-      void add_path(Memory src_mem, Memory::Kind dst_kind, bool dst_global,
-		    unsigned bandwidth, unsigned latency,
-		    bool redops_allowed, bool serdez_allowed,
-		    XferDesKind xd_kind);
-      void add_path(Memory::Kind src_kind, bool src_global,
-		    Memory::Kind dst_kind, bool dst_global,
-		    unsigned bandwidth, unsigned latency,
-		    bool redops_allowed, bool serdez_allowed,
-		    XferDesKind xd_kind);
+      // returns the added path for further modification, but reference is
+      //  only valid until the next call to 'add_path'
+      SupportedPath& add_path(span<const Memory> src_mems,
+                              span<const Memory> dst_mems,
+                              unsigned bandwidth, unsigned latency,
+                              unsigned frag_overhead,
+                              XferDesKind xd_kind);
+      SupportedPath& add_path(span<const Memory> src_mems,
+                              Memory::Kind dst_kind, bool dst_global,
+                              unsigned bandwidth, unsigned latency,
+                              unsigned frag_overhead,
+                              XferDesKind xd_kind);
+      SupportedPath& add_path(Memory::Kind src_kind, bool src_global,
+                              span<const Memory> dst_mems,
+                              unsigned bandwidth, unsigned latency,
+                              unsigned frag_overhead,
+                              XferDesKind xd_kind);
+      SupportedPath& add_path(Memory::Kind src_kind, bool src_global,
+                              Memory::Kind dst_kind, bool dst_global,
+                              unsigned bandwidth, unsigned latency,
+                              unsigned frag_overhead,
+                              XferDesKind xd_kind);
       // TODO: allow rdma path to limit by kind?
-      void add_path(bool local_loopback,
-		    unsigned bandwidth, unsigned latency,
-		    bool redops_allowed, bool serdez_allowed,
-		    XferDesKind xd_kind);
+      SupportedPath& add_path(bool local_loopback,
+                              unsigned bandwidth, unsigned latency,
+                              unsigned frag_overhead,
+                              XferDesKind xd_kind);
 
       std::vector<SupportedPath> paths;
     };
@@ -831,7 +894,8 @@ namespace Realm {
 				       const std::vector<XferDesPortInfo>& outputs_info,
 				       int priority,
 				       XferDesRedopInfo redop_info,
-				       const void *fill_data, size_t fill_size) = 0;
+				       const void *fill_data, size_t fill_size,
+                                       size_t fill_total) = 0;
 
       virtual XferDesFactory *get_factory();
 
@@ -908,13 +972,16 @@ namespace Realm {
        */
       virtual long available();
 
-      virtual bool supports_path(Memory src_mem, Memory dst_mem,
-				 CustomSerdezID src_serdez_id,
-				 CustomSerdezID dst_serdez_id,
-				 ReductionOpID redop_id,
-				 XferDesKind *kind_ret = 0,
-				 unsigned *bw_ret = 0,
-				 unsigned *lat_ret = 0);
+      virtual uint64_t supports_path(Memory src_mem, Memory dst_mem,
+                                     CustomSerdezID src_serdez_id,
+                                     CustomSerdezID dst_serdez_id,
+                                     ReductionOpID redop_id,
+                                     size_t total_bytes,
+                                     const std::vector<size_t> *src_frags,
+                                     const std::vector<size_t> *dst_frags,
+                                     XferDesKind *kind_ret = 0,
+                                     unsigned *bw_ret = 0,
+                                     unsigned *lat_ret = 0);
 
       virtual void enqueue_ready_xd(XferDes *xd) { assert(0); }
       virtual void wakeup_xd(XferDes *xd) { assert(0); }
@@ -972,13 +1039,20 @@ namespace Realm {
 
       ~MemcpyChannel();
 
-      virtual bool supports_path(Memory src_mem, Memory dst_mem,
-				 CustomSerdezID src_serdez_id,
-				 CustomSerdezID dst_serdez_id,
-				 ReductionOpID redop_id,
-				 XferDesKind *kind_ret = 0,
-				 unsigned *bw_ret = 0,
-				 unsigned *lat_ret = 0);
+      // helper to list all memories that can be reached by load/store instructions
+      //  on the cpu in the current process
+      static void enumerate_local_cpu_memories(std::vector<Memory>& mems);
+
+      virtual uint64_t supports_path(Memory src_mem, Memory dst_mem,
+                                     CustomSerdezID src_serdez_id,
+                                     CustomSerdezID dst_serdez_id,
+                                     ReductionOpID redop_id,
+                                     size_t total_bytes,
+                                     const std::vector<size_t> *src_frags,
+                                     const std::vector<size_t> *dst_frags,
+                                     XferDesKind *kind_ret = 0,
+                                     unsigned *bw_ret = 0,
+                                     unsigned *lat_ret = 0);
 
       virtual XferDes *create_xfer_des(uintptr_t dma_op,
 				       NodeID launch_node,
@@ -987,7 +1061,8 @@ namespace Realm {
 				       const std::vector<XferDesPortInfo>& outputs_info,
 				       int priority,
 				       XferDesRedopInfo redop_info,
-				       const void *fill_data, size_t fill_size);
+				       const void *fill_data, size_t fill_size,
+                                       size_t fill_total);
 
       virtual long submit(Request** requests, long nr);
 
@@ -1010,7 +1085,8 @@ namespace Realm {
 				       const std::vector<XferDesPortInfo>& outputs_info,
 				       int priority,
 				       XferDesRedopInfo redop_info,
-				       const void *fill_data, size_t fill_size);
+				       const void *fill_data, size_t fill_size,
+                                       size_t fill_total);
 
       virtual long submit(Request** requests, long nr);
 
@@ -1025,13 +1101,16 @@ namespace Realm {
       static const bool is_ordered = false;
 
       // override because we don't want to claim non-reduction copies
-      virtual bool supports_path(Memory src_mem, Memory dst_mem,
-				 CustomSerdezID src_serdez_id,
-				 CustomSerdezID dst_serdez_id,
-				 ReductionOpID redop_id,
-				 XferDesKind *kind_ret = 0,
-				 unsigned *bw_ret = 0,
-				 unsigned *lat_ret = 0);
+      virtual uint64_t supports_path(Memory src_mem, Memory dst_mem,
+                                     CustomSerdezID src_serdez_id,
+                                     CustomSerdezID dst_serdez_id,
+                                     ReductionOpID redop_id,
+                                     size_t total_bytes,
+                                     const std::vector<size_t> *src_frags,
+                                     const std::vector<size_t> *dst_frags,
+                                     XferDesKind *kind_ret = 0,
+                                     unsigned *bw_ret = 0,
+                                     unsigned *lat_ret = 0);
 
       virtual XferDes *create_xfer_des(uintptr_t dma_op,
 				       NodeID launch_node,
@@ -1040,7 +1119,8 @@ namespace Realm {
 				       const std::vector<XferDesPortInfo>& outputs_info,
 				       int priority,
 				       XferDesRedopInfo redop_info,
-				       const void *fill_data, size_t fill_size);
+				       const void *fill_data, size_t fill_size,
+                                       size_t fill_total);
 
       virtual long submit(Request** requests, long nr);
 
@@ -1062,7 +1142,8 @@ namespace Realm {
 				       const std::vector<XferDesPortInfo>& outputs_info,
 				       int priority,
 				       XferDesRedopInfo redop_info,
-				       const void *fill_data, size_t fill_size);
+				       const void *fill_data, size_t fill_size,
+                                       size_t fill_total);
 
       long submit(Request** requests, long nr);
     };
@@ -1082,7 +1163,8 @@ namespace Realm {
 				       const std::vector<XferDesPortInfo>& outputs_info,
 				       int priority,
 				       XferDesRedopInfo redop_info,
-				       const void *fill_data, size_t fill_size);
+				       const void *fill_data, size_t fill_size,
+                                       size_t fill_total);
 
       long submit(Request** requests, long nr);
     };
@@ -1122,7 +1204,8 @@ namespace Realm {
 				       const std::vector<XferDesPortInfo>& outputs_info,
 				       int priority,
 				       XferDesRedopInfo redop_info,
-				       const void *fill_data, size_t fill_size) { assert(0); return 0; }
+				       const void *fill_data, size_t fill_size,
+                                       size_t fill_total) { assert(0); return 0; }
 
       virtual long submit(Request** requests, long nr) { assert(0); return 0; }
 
@@ -1246,7 +1329,7 @@ namespace Realm {
     struct UpdateBytesWriteMessage {
       XferDesID guid;
       int port_idx;
-      size_t span_start, span_size, pre_bytes_total;
+      size_t span_start, span_size;
 
       static void handle_message(NodeID sender,
 				 const UpdateBytesWriteMessage &args,
@@ -1255,15 +1338,13 @@ namespace Realm {
 
       static void send_request(NodeID target, XferDesID guid,
 			       int port_idx,
-			       size_t span_start, size_t span_size,
-			       size_t pre_bytes_total)
+			       size_t span_start, size_t span_size)
       {
 	ActiveMessage<UpdateBytesWriteMessage> amsg(target);
         amsg->guid = guid;
 	amsg->port_idx = port_idx;
 	amsg->span_start = span_start;
 	amsg->span_size = span_size;
-	amsg->pre_bytes_total = pre_bytes_total;
 	amsg.commit();
       }
     };
@@ -1291,14 +1372,38 @@ namespace Realm {
       }
     };
 
+    // object used to hold input progress (pre_write and bytes_total) before
+    //  we've actually created the correct xd
+    class XferDesPlaceholder {
+    public:
+      XferDesPlaceholder();
+
+    protected:
+      ~XferDesPlaceholder();
+
+    public:
+      void add_reference();
+      void remove_reference();
+
+      void update_pre_bytes_write(int port_idx,
+				  size_t span_start, size_t span_size);
+      void update_pre_bytes_total(int port_idx, size_t pre_bytes_total);
+
+      void set_real_xd(XferDes *_xd);
+
+    protected:
+      static const int INLINE_PORTS = 4;
+      atomic<unsigned> refcount;
+      XferDes *xd;
+      size_t inline_bytes_total[INLINE_PORTS];
+      SequenceAssembler inline_pre_write[INLINE_PORTS];
+      Mutex extra_mutex;
+      std::map<int, size_t> extra_bytes_total;
+      std::map<int, SequenceAssembler> extra_pre_write;
+    };
+
     class XferDesQueue {
     public:
-      struct XferDesWithUpdates{
-        XferDesWithUpdates(void): xd(NULL) {}
-        XferDes* xd;
-	std::map<int, size_t> pre_bytes_total;
-	std::map<int, SequenceAssembler> seq_pre_write;
-      };
       enum {
         NODE_BITS = 16,
         INDEX_BITS = 32
@@ -1332,26 +1437,18 @@ namespace Realm {
       void update_next_bytes_read(XferDesID xd_guid, int port_idx,
 				  size_t span_start, size_t span_size);
 
-      void destroy_xferDes(XferDesID guid) {
-	XferDes *xd;
-	{
-	  RWLock::AutoWriterLock al(guid_lock);
-	  std::map<XferDesID, XferDesWithUpdates>::iterator it = guid_to_xd.find(guid);
-	  assert(it != guid_to_xd.end());
-	  assert(it->second.xd != NULL);
-	  xd = it->second.xd;
-	  guid_to_xd.erase(it);
-	}
-	xd->remove_reference();
-      }
+      void destroy_xferDes(XferDesID guid);
 
       // returns true if xd is ready, false if enqueue has been deferred
       bool enqueue_xferDes_local(XferDes* xd, bool add_to_queue = true);
 
     protected:
-      std::map<XferDesID, XferDesWithUpdates> guid_to_xd;
+      // guid_to_xd maps a guid to either an XferDes * (as a uintptr_t) or
+      //  a XferDesPlaceholder * (as a uintptr_t with the LSB set)
+      Mutex guid_lock;
+      std::map<XferDesID, uintptr_t> guid_to_xd;
+
       Mutex queues_lock;
-      RWLock guid_lock;
       atomic<XferDesID> next_to_assign_idx;
     };
 
