@@ -1,6 +1,6 @@
 #!/usr/bin/env python3
 
-# Copyright 2021 Stanford University
+# Copyright 2023 Stanford University
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -67,8 +67,11 @@ legion_cxx_tests = [
     ['examples/attach_index_space/index_space_attach', []],
     ['examples/predication/predication', []],
     ['examples/layout_constraints/transpose', []],
+    ['examples/padded_instances/padded_instances', []],
     ['examples/inline_tasks/inline_tasks', []],
     ['examples/local_function_tasks/local_function_tasks', []],
+    ['examples/provenance/provenance', []],
+    ['examples/tiling/tiling', []],
     # Comment this test out until it works everywhere
     #['examples/implicit_top_task/implicit_top_task', []],
 
@@ -77,6 +80,10 @@ legion_cxx_tests = [
     ['test/legion_stl/test_stl', []],
 ]
 
+legion_cxx_provenance_tests = [
+    ['examples/provenance/provenance', []],
+]
+
 legion_fortran_tests = [
     ['tutorial/fortran/00_hello_world/hello_world_fortran', []],
     ['tutorial/fortran/01_tasks_and_futures/tasks_and_futures_fortran', []],
@@ -152,6 +159,10 @@ legion_python_cxx_tests = [
     ['bindings/python/legion_python', ['tests/pass/privileges.py', '-ll:py', '1', '-ll:cpu', '0']],
     ['bindings/python/legion_python', ['tests/pass/no_access.py', '-ll:py', '1', '-ll:cpu', '0']],
 
+    # Tests for Package Import
+    ['bindings/python/legion_python', ['-m', 'tests.pass.test_package1.a.b.c', '-ll:py', '1', '-ll:cpu', '0']],
+    ['bindings/python/legion_python', ['-m', 'tests.pass.test_package2.a.b.c', '-ll:py', '1', '-ll:cpu', '0']],
+
     # Examples
     ['examples/python_interop/python_interop', ['-ll:py', '1']],
 
@@ -273,7 +284,7 @@ def precompile_regent(tests, flags, launcher, root_dir, env, thread_count):
 
 def run_test_legion_cxx(launcher, root_dir, tmp_dir, bin_dir, env, thread_count, timelimit):
     flags = ['-logfile', 'out_%.log']
-    if env['USE_CUDA'] == '1':
+    if env['USE_CUDA'] == '1' or env['USE_HIP'] == '1':
         flags.extend(['-ll:gpu', '1'])
     if (env['USE_KOKKOS'] == '1') and (env['USE_OPENMP'] == '1'):
         flags.extend(['-ll:ocpu', '1', '-ll:onuma', '0' ])
@@ -281,7 +292,7 @@ def run_test_legion_cxx(launcher, root_dir, tmp_dir, bin_dir, env, thread_count,
 
 def run_test_legion_network_cxx(launcher, root_dir, tmp_dir, bin_dir, env, thread_count, timelimit):
     flags = ['-logfile', 'out_%.log']
-    if env['USE_CUDA'] == '1':
+    if env['USE_CUDA'] == '1' or env['USE_HIP'] == '1':
         flags.extend(['-ll:gpu', '1'])
     if (env['USE_KOKKOS'] == '1') and (env['USE_OPENMP'] == '1'):
         flags.extend(['-ll:ocpu', '1', '-ll:onuma', '0' ])
@@ -289,7 +300,7 @@ def run_test_legion_network_cxx(launcher, root_dir, tmp_dir, bin_dir, env, threa
 
 def run_test_legion_openmp_cxx(launcher, root_dir, tmp_dir, bin_dir, env, thread_count, timelimit):
     flags = ['-logfile', 'out_%.log']
-    if env['USE_CUDA'] == '1':
+    if env['USE_CUDA'] == '1' or env['USE_HIP'] == '1':
         flags.extend(['-ll:gpu', '1'])
     if (env['USE_KOKKOS'] == '1') and (env['USE_OPENMP'] == '1'):
         flags.extend(['-ll:ocpu', '1', '-ll:onuma', '0' ])
@@ -297,7 +308,7 @@ def run_test_legion_openmp_cxx(launcher, root_dir, tmp_dir, bin_dir, env, thread
 
 def run_test_legion_kokkos_cxx(launcher, root_dir, tmp_dir, bin_dir, env, thread_count, timelimit):
     flags = ['-logfile', 'out_%.log']
-    if env['USE_CUDA'] == '1':
+    if env['USE_CUDA'] == '1' or env['USE_HIP'] == '1':
         flags.extend(['-ll:gpu', '1'])
     if (env['USE_KOKKOS'] == '1') and (env['USE_OPENMP'] == '1'):
         flags.extend(['-ll:ocpu', '1', '-ll:onuma', '0' ])
@@ -318,9 +329,46 @@ def run_test_legion_python_cxx(launcher, root_dir, tmp_dir, bin_dir, env, thread
     run_cxx(legion_python_cxx_tests, flags, launcher, root_dir, bin_dir, env, thread_count, timelimit)
     cmd([make_exe, '-C', python_dir, 'clean'], env=env)
 
+def run_test_legion_jupyter_cxx(launcher, root_dir, tmp_dir, bin_dir, env, thread_count, timelimit):
+    # Hack: legion_python currently requires the module name to come first
+    flags = [] # ['-logfile', 'out_%.log']
+    python_dir = os.path.join(root_dir, 'bindings', 'python')
+    # Hack: Fix up the environment so that Python can find all the examples.
+    env = dict(list(env.items()) + [
+        ('PYTHONPATH', ':'.join([python_dir])),
+        ('LD_LIBRARY_PATH', ':'.join([python_dir])),
+    ])
+    # Clean up around python because we are going to make shared objects
+    # which is not something that anyone else does
+    cmd([make_exe, '-C', python_dir, 'clean'], env=env)
+    cmd([make_exe, '-C', python_dir, '-j', str(thread_count)], env=env)
+    jupyter_dir = os.path.join(root_dir, 'jupyter_notebook')
+    jupyter_install_cmd = [sys.executable, './install_jupyter.py', '--legion-prefix', python_dir, '--verbose']
+    cmd(jupyter_install_cmd, env=env, cwd=jupyter_dir)
+    jupyter_test_file = os.path.join(root_dir, 'jupyter_notebook', 'ci_test.py')
+    jupyter_test_cmd = ['jupyter', 'run', '--kernel', 'legion_kernel_nocr', jupyter_test_file]
+    cmd(jupyter_test_cmd, env=env)
+    canonical_jupyter_test_file = os.path.join(root_dir, 'jupyter_notebook', 'test_canonical.py')
+    canonical_jupyter_test_cmd = ['jupyter', 'run', '--kernel', 'python3', canonical_jupyter_test_file]
+    cmd(canonical_jupyter_test_cmd, env=env)
+    canonical_python_test_cmd = [sys.executable, canonical_jupyter_test_file]
+    cmd(canonical_python_test_cmd, env=env)
+    cmd([make_exe, '-C', python_dir, 'clean'], env=env)
+
+def run_test_legion_provenance_cxx(launcher, root_dir, tmp_dir, bin_dir, env, thread_count, timelimit):
+    flags = ['-lg:prof','1', '-lg:prof_logfile', 'prof_%.gz']
+    run_cxx(legion_cxx_provenance_tests, flags, launcher, root_dir, bin_dir, env, thread_count, timelimit)
+    provenance_test_dir = os.path.join(root_dir, 'examples', 'provenance')
+    test_cmd = [sys.executable, 'test.py']
+    env = dict(list(env.items()) + [
+        ('LEGION_DIR', root_dir),
+        ('TMP_DIR', tmp_dir),
+    ])
+    cmd(test_cmd, env=env, cwd=provenance_test_dir)
+
 def run_test_legion_hdf_cxx(launcher, root_dir, tmp_dir, bin_dir, env, thread_count, timelimit):
     flags = ['-logfile', 'out_%.log']
-    if env['USE_CUDA'] == '1':
+    if env['USE_CUDA'] == '1' or env['USE_HIP'] == '1':
         flags.extend(['-ll:gpu', '1'])
     if (env['USE_KOKKOS'] == '1') and (env['USE_OPENMP'] == '1'):
         flags.extend(['-ll:ocpu', '1', '-ll:onuma', '0' ])
@@ -328,7 +376,7 @@ def run_test_legion_hdf_cxx(launcher, root_dir, tmp_dir, bin_dir, env, thread_co
 
 def run_test_legion_fortran(launcher, root_dir, tmp_dir, bin_dir, env, thread_count, timelimit):
     flags = ['-logfile', 'out_%.log']
-    if env['USE_CUDA'] == '1':
+    if env['USE_CUDA'] == '1' or env['USE_HIP'] == '1':
         flags.extend(['-ll:gpu', '1'])
     run_cxx(legion_fortran_tests, flags, launcher, root_dir, bin_dir, env, thread_count, timelimit)
 
@@ -381,7 +429,8 @@ def run_test_external1(launcher, root_dir, tmp_dir, bin_dir, env, thread_count,
     makefile = os.path.join(root_dir, 'apps/Makefile.template')
     cmd([make_exe, '-f', makefile, '-C', stencil_dir, '-j', str(thread_count)], env=stencil_env)
     stencil = os.path.join(stencil_dir, 'stencil')
-    cmd([stencil, '4', '10', '1000'], timelimit=timelimit)
+    # HACK: work around stencil mapper issue with -ll:ext_sysmem 0
+    cmd([stencil, '4', '10', '1000', '-ll:ext_sysmem', '0'], timelimit=timelimit)
 
     # SNAP
     # Contact: Mike Bauer <mbauer@nvidia.com>
@@ -502,7 +551,7 @@ def run_test_ctest(launcher, root_dir, tmp_dir, bin_dir, env, thread_count, time
     build_dir = os.path.join(tmp_dir, 'build')
     args = ['ctest', '--output-on-failure']
     # do not run tests in parallel if they use GPUs - might not all fit
-    if env['USE_CUDA'] != '1':
+    if env['USE_CUDA'] != '1' and env['USE_HIP'] != '1':
         args.extend(['-j', str(thread_count)])
     if timelimit:
         args.extend(['--timeout', str(timelimit)])
@@ -510,6 +559,20 @@ def run_test_ctest(launcher, root_dir, tmp_dir, bin_dir, env, thread_count, time
         env=env,
         cwd=build_dir)
 
+def run_test_legion_prof_mypy(root_dir):
+    mypy_cmd = [
+        "mypy",
+        "--disallow-any-unimported",
+        "--disallow-any-explicit",
+        "--disallow-untyped-defs",
+        "--disallow-incomplete-defs",
+        "--warn-redundant-casts",
+        "--warn-unused-ignores",
+        os.path.join(root_dir, 'tools', 'legion_prof.py'),
+    ]
+    print('Running mypy test:', cmd)
+    cmd(mypy_cmd)
+
 def hostname():
     return subprocess.check_output(['hostname']).strip()
 
@@ -705,6 +768,12 @@ def build_cmake(root_dir, tmp_dir, env, thread_count,
     cmdline.append('-DLegion_USE_CUDA=%s' % ('ON' if env['USE_CUDA'] == '1' else 'OFF'))
     if 'GPU_ARCH' in env:
         cmdline.append('-DLegion_CUDA_ARCH=%s' % env['GPU_ARCH'])
+    cmdline.append('-DLegion_USE_HIP=%s' % ('ON' if env['USE_HIP'] == '1' else 'OFF'))
+    if 'HIP_ARCH' in env:
+        cmdline.append('-DLegion_HIP_ARCH=%s' % env['HIP_ARCH'])
+    if 'THRUST_PATH' in env and env['USE_COMPLEX'] == '1':
+        cmdline.append('-DHIP_THRUST_ROOT_DIR=%s' % env['THRUST_PATH'])
+    cmdline.append('-DLegion_USE_NVTX=%s' % ('ON' if env['USE_NVTX'] == '1' else 'OFF'))
     cmdline.append('-DLegion_USE_OpenMP=%s' % ('ON' if env['USE_OPENMP'] == '1' else 'OFF'))
     cmdline.append('-DLegion_USE_Kokkos=%s' % ('ON' if env['USE_KOKKOS'] == '1' else 'OFF'))
     cmdline.append('-DLegion_USE_Python=%s' % ('ON' if env['USE_PYTHON'] == '1' else 'OFF'))
@@ -721,7 +790,7 @@ def build_cmake(root_dir, tmp_dir, env, thread_count,
         cmdline.append('-DLegion_ENABLE_TESTING=ON')
         if 'LAUNCHER' in env:
             cmdline.append('-DLegion_TEST_LAUNCHER=%s' % env['LAUNCHER'])
-        if env['USE_CUDA'] == '1':
+        if env['USE_CUDA'] == '1' or env['USE_HIP'] == '1':
             cmdline.append('-DLegion_TEST_ARGS=-ll:gpu 1')
     else:
         cmdline.append('-DLegion_ENABLE_TESTING=OFF')
@@ -741,6 +810,9 @@ def build_cmake(root_dir, tmp_dir, env, thread_count,
     # if MARCH is set in the environment, give that to cmake as BUILD_MARCH
     if 'MARCH' in env:
         cmdline.append('-DBUILD_MARCH=' + env['MARCH'])
+    # add cxx standard
+    if 'CXX_STANDARD' in env:
+        cmdline.append('-DCMAKE_CXX_STANDARD=' + env['CXX_STANDARD'])
     # cmake before 3.16 doesn't know how to look for CUDAHOSTCXX
     if 'CUDAHOSTCXX' in env:
         cmdline.append('-DCMAKE_CUDA_HOST_COMPILER=' + env['CUDAHOSTCXX'])
@@ -756,10 +828,15 @@ def build_cmake(root_dir, tmp_dir, env, thread_count,
     return os.path.join(build_dir, 'bin')
 
 def build_legion_prof_rs(root_dir, tmp_dir, env):
+    legion_prof_dir = os.path.join(root_dir, 'tools', 'legion_prof_rs')
     cmd(['cargo', 'install',
-         '--path', os.path.join(root_dir, 'tools', 'legion_prof_rs'),
+         '--all-features',
+         '--locked',
+         '--path', legion_prof_dir,
          '--root', tmp_dir],
         env=env)
+    cmd(['cargo', 'test', '--all-features'], env=env, cwd=legion_prof_dir)
+    cmd(['cargo', 'fmt', '--all', '--', '--check'], env=env, cwd=legion_prof_dir)
 
 def build_regent(root_dir, env):
     cmd([os.path.join(root_dir, 'language/travis.py'), '--install-only'], env=env)
@@ -816,12 +893,12 @@ class Stage(object):
 def report_mode(debug, max_dim, launcher,
                 test_regent, test_legion_cxx, test_fuzzer, test_realm,
                 test_external1, test_external2, test_private,
-                test_perf, test_ctest, networks,
-                use_cuda, use_openmp, use_kokkos, use_python, use_llvm,
+                test_perf, test_ctest, test_jupyter, networks,
+                use_cuda, use_hip, use_openmp, use_kokkos, use_python, use_llvm,
                 use_hdf, use_fortran, use_spy, use_prof,
                 use_bounds_checks, use_privilege_checks, use_complex,
                 use_shared_objects,
-                use_gcov, use_cmake, use_rdir):
+                use_gcov, use_cmake, use_rdir, use_nvtx, cxx_standard):
     print()
     print('#'*60)
     print('### Test Suite Configuration')
@@ -842,10 +919,12 @@ def report_mode(debug, max_dim, launcher,
     print('###   * Private:    %s' % test_private)
     print('###   * Perf:       %s' % test_perf)
     print('###   * CTest:      %s' % test_ctest)
+    print('###   * Jupyter:    %s' % test_jupyter)
     print('###')
     print('### Build Flags:')
     print('###   * Networks:   %s' % networks)
     print('###   * CUDA:       %s' % use_cuda)
+    print('###   * HIP:        %s' % use_hip)
     print('###   * OpenMP:     %s' % use_openmp)
     print('###   * Kokkos:     %s' % use_kokkos)
     print('###   * Python:     %s' % use_python)
@@ -861,7 +940,9 @@ def report_mode(debug, max_dim, launcher,
     print('###   * Gcov:       %s' % use_gcov)
     print('###   * CMake:      %s' % use_cmake)
     print('###   * RDIR:       %s' % use_rdir)
+    print('###   * NVTX:       %s' % use_nvtx)
     print('###   * Max DIM:    %s' % max_dim)
+    print('###   * CXX STD:    %s' % cxx_standard)
     print('#'*60)
     print()
     sys.stdout.flush()
@@ -874,6 +955,7 @@ def run_tests(test_modules=None,
               launcher=None,
               thread_count=None,
               root_dir=None,
+              tmp_dir=None,
               check_ownership=False,
               keep_tmp_dir=False,
               timelimit=None,
@@ -906,12 +988,14 @@ def run_tests(test_modules=None,
     test_private = module_enabled('private', False)
     test_perf = module_enabled('perf', False)
     test_ctest = module_enabled('ctest', False)
+    test_jupyter = module_enabled('jupyter', False)
 
     # Determine which features to build with.
     def feature_enabled(feature, default=True, prefix='USE_', **kwargs):
         return option_enabled(feature, use_features, default,
                               envprefix=prefix, **kwargs)
     use_cuda = feature_enabled('cuda', False)
+    use_hip = feature_enabled('hip', False)
     use_openmp = feature_enabled('openmp', False)
     use_kokkos = feature_enabled('kokkos', False)
     use_python = feature_enabled('python', False)
@@ -928,6 +1012,7 @@ def run_tests(test_modules=None,
     use_gcov = feature_enabled('gcov', False)
     use_cmake = feature_enabled('cmake', False)
     use_rdir = feature_enabled('rdir', True)
+    use_nvtx = feature_enabled('nvtx', False)
     use_shared_objects = feature_enabled('shared', False,
                                          envname='SHARED_OBJECTS')
 
@@ -949,10 +1034,19 @@ def run_tests(test_modules=None,
     if test_ctest and not use_cmake:
         raise Exception('CTest cannot be used without CMake')
 
+    if test_jupyter and not use_python:
+        raise Exception('Jupyter requires Python')
+
     if networks and launcher is None:
         raise Exception('Network(s) is enabled but launcher is not set (use --launcher or LAUNCHER)')
     launcher = launcher.split() if launcher is not None else []
 
+    # CXX Standard
+    cxx_standard = os.environ['CXX_STANDARD'] if 'CXX_STANDARD' in os.environ else ''
+    # if not use cmake, let's add -std=c++NN to CXXFLAGS
+    if use_cmake == False:
+        os.environ['CXXFLAGS'] += " -std=c++" + os.environ['CXX_STANDARD']
+
     gcov_flags = ' -ftest-coverage -fprofile-arcs'
 
     if check_ownership:
@@ -962,15 +1056,21 @@ def run_tests(test_modules=None,
     report_mode(debug, max_dim, launcher,
                 test_regent, test_legion_cxx, test_fuzzer, test_realm,
                 test_external1, test_external2, test_private,
-                test_perf, test_ctest,
+                test_perf, test_ctest, test_jupyter,
                 networks,
-                use_cuda, use_openmp, use_kokkos, use_python, use_llvm,
+                use_cuda, use_hip, use_openmp, use_kokkos, use_python, use_llvm,
                 use_hdf, use_fortran, use_spy, use_prof,
                 use_bounds_checks, use_privilege_checks, use_complex,
                 use_shared_objects,
-                use_gcov, use_cmake, use_rdir)
+                use_gcov, use_cmake, use_rdir, use_nvtx, cxx_standard)
+
+    if not tmp_dir:
+        tmp_dir = tempfile.mkdtemp(dir=root_dir)
+    else:
+        if os.path.exists(tmp_dir):
+            shutil.rmtree(tmp_dir)
+        os.mkdir(tmp_dir)
 
-    tmp_dir = tempfile.mkdtemp(dir=root_dir)
     if verbose:
         print('Using build directory: %s' % tmp_dir)
         print()
@@ -982,6 +1082,8 @@ def run_tests(test_modules=None,
         ('REALM_NETWORKS', networks),
         ('USE_CUDA', '1' if use_cuda else '0'),
         ('TEST_CUDA', '1' if use_cuda else '0'),
+        ('USE_HIP', '1' if use_hip else '0'),
+        ('TEST_HIP', '1' if use_hip else '0'),
         ('USE_OPENMP', '1' if use_openmp else '0'),
         ('TEST_OPENMP', '1' if use_openmp else '0'),
         ('USE_KOKKOS', '1' if use_kokkos else '0'),
@@ -1003,6 +1105,7 @@ def run_tests(test_modules=None,
         ('SHARED_OBJECTS', '1' if use_shared_objects else '0'),
         ('TEST_GCOV', '1' if use_gcov else '0'),
         ('USE_RDIR', '1' if use_rdir else '0'),
+        ('USE_NVTX', '1' if use_nvtx else '0'),
         ('MAX_DIM', str(max_dim)),
         ('LG_RT_DIR', os.path.join(root_dir, 'runtime')),
         ('DEFINE_HEADERS_DIR', os.path.join(root_dir, 'runtime')),
@@ -1020,9 +1123,15 @@ def run_tests(test_modules=None,
     try:
         # Build tests.
         with Stage('build'):
-            if use_prof:
+            if use_prof or use_spy:
                 build_legion_prof_rs(root_dir, tmp_dir, env)
+            if use_prof:
+                run_test_legion_prof_mypy(root_dir)
             if use_cmake:
+                # We should always be using ctest if we're building with
+                # cmake, except for some unusual cases with Regent
+                # (ask @eslaught for details about Regent cases)
+                assert test_ctest or test_regent
                 bin_dir = build_cmake(
                     root_dir, tmp_dir, env, thread_count,
                     test_regent, test_legion_cxx, test_external1,
@@ -1043,6 +1152,8 @@ def run_tests(test_modules=None,
         if test_legion_cxx:
             with Stage('legion_cxx'):
                 run_test_legion_cxx(launcher, root_dir, tmp_dir, bin_dir, env, thread_count, timelimit)
+                if use_prof:
+                    run_test_legion_provenance_cxx(launcher, root_dir, tmp_dir, bin_dir, env, thread_count, timelimit)
                 if networks:
                     run_test_legion_network_cxx(launcher, root_dir, tmp_dir, bin_dir, env, thread_count, timelimit)
                 if use_openmp:
@@ -1080,6 +1191,9 @@ def run_tests(test_modules=None,
         if test_ctest:
             with Stage('ctest'):
                 run_test_ctest(launcher, root_dir, tmp_dir, bin_dir, env, thread_count, timelimit)
+        if test_jupyter:
+            with Stage('jupyter'):
+                run_test_legion_jupyter_cxx(launcher, root_dir, tmp_dir, bin_dir, env, thread_count, timelimit)
     finally:
         if keep_tmp_dir:
             print('Leaving build directory:')
@@ -1131,7 +1245,7 @@ def driver():
         '--test', dest='test_modules', action=ExtendAction,
         choices=MultipleChoiceList('regent', 'legion_cxx', 'fuzzer',
                                    'realm', 'external1', 'external2',
-                                   'private', 'perf', 'ctest'),
+                                   'private', 'perf', 'ctest', 'jupyter'),
         type=lambda s: s.split(','),
         default=None,
         help='Test modules to run (also via TEST_*).')
@@ -1150,10 +1264,10 @@ def driver():
         help='Maximum number of dimensions (also via MAX_DIM).')
     parser.add_argument(
         '--use', dest='use_features', action=ExtendAction,
-        choices=MultipleChoiceList('gasnet', 'cuda', 'openmp', 'kokkos',
+        choices=MultipleChoiceList('gasnet', 'cuda', 'hip', 'openmp', 'kokkos',
                                    'python', 'llvm', 'hdf', 'fortran', 'spy', 'prof',
                                    'bounds', 'privilege', 'complex',
-                                   'gcov', 'cmake', 'rdir'),
+                                   'gcov', 'cmake', 'rdir', 'nvtx'),
         type=lambda s: s.split(','),
         default=None,
         help='Build Legion with features (also via USE_*).')
@@ -1170,6 +1284,10 @@ def driver():
         '-C', '--directory', dest='root_dir', metavar='DIR', action='store', required=False,
         help='Legion root directory.')
 
+    parser.add_argument(
+        '--tmp-dir', dest='tmp_dir', metavar='DIR', action='store', required=False,
+        help='Temporary directory path for out-of-source builds')
+
     parser.add_argument(
         '-j', dest='thread_count', nargs='?', type=int,
         help='Number threads used to compile.')
