@@ -1,4 +1,4 @@
-/* Copyright 2021 NVIDIA Corporation
+/* Copyright 2023 NVIDIA Corporation
  *
  * Licensed under the Apache License, Version 2.0 (the "License");
  * you may not use this file except in compliance with the License.
@@ -67,8 +67,10 @@ class ProjectionOneLevel : public ProjectionFunctor {
 public:
   ProjectionOneLevel(int _index1) : index1(_index1) {}
   virtual bool is_functional(void) const { return true; }
+  virtual bool is_invertible(void) const { return true; }
   virtual unsigned get_depth(void) const { return 0; }
   using ProjectionFunctor::project;
+  using ProjectionFunctor::invert;
   virtual LogicalRegion project(LogicalPartition upper_bound,
 				const DomainPoint &point,
 				const Domain &launch_domain)
@@ -76,6 +78,20 @@ public:
     return runtime->get_logical_subregion_by_color(upper_bound,
 						   point[index1]);
   }
+  virtual void invert(LogicalRegion region, LogicalPartition upper_bound,
+                      const Domain &launch_domain,
+                      std::vector<DomainPoint> &ordered_points)
+  {
+    // Invert this the dumb way for now since these are likely to be
+    // small launch domains
+    for (Domain::DomainPointIterator itr(launch_domain); itr; itr++)
+    {
+      LogicalRegion lr = project(upper_bound, itr.p, launch_domain);
+      if (lr != region)
+        continue;
+      ordered_points.push_back(itr.p);
+    }
+  }
 
 protected:
   int index1;
@@ -1087,7 +1103,7 @@ public:
     , replicate(_replicate)
   {
     Machine::MemoryQuery mq(machine);
-    mq.only_kind(Memory::SYSTEM_MEM);
+    mq.only_kind(Memory::SYSTEM_MEM).has_capacity(1);
     for(Machine::MemoryQuery::iterator it = mq.begin();
 	it != mq.end();
 	++it) {
