@@ -1,5 +1,5 @@
-/* Copyright 2021 Stanford University
- * Copyright 2021 Los Alamos National Laboratory 
+/* Copyright 2023 Stanford University
+ * Copyright 2023 Los Alamos National Laboratory 
  *
  * Licensed under the Apache License, Version 2.0 (the "License");
  * you may not use this file except in compliance with the License.
@@ -34,7 +34,7 @@ namespace TestConfig {
 void dummy_task_body(const void *args, size_t arglen, 
 		     const void *userdata, size_t userlen, Processor p);
 
-#ifdef REALM_USE_CUDA
+#if defined(REALM_USE_CUDA) || defined(REALM_USE_HIP)
 void dummy_gpu_task(const void *args, size_t arglen, 
 		    const void *userdata, size_t userlen, Processor p);
 #endif
