@@ -1,4 +1,4 @@
-/* Copyright 2021 Stanford University, NVIDIA Corporation
+/* Copyright 2023 Stanford University, NVIDIA Corporation
  *
  * Licensed under the Apache License, Version 2.0 (the "License");
  * you may not use this file except in compliance with the License.
@@ -510,6 +510,9 @@ void top_level_task(const void *args, size_t arglen,
     Memory m = *it;
     if(m.capacity() == 0) continue;
 
+    // GPU_DYNAMIC_MEM does not support deferred allocation
+    if(m.kind() == Memory::GPU_DYNAMIC_MEM) continue;
+
     // directed tests
     if(config.directed_tests) {
       directed_test_memory(config, m, p, "simple capacity limit",
