@@ -11,14 +11,14 @@ use nom::{
     bytes::complete::{tag, take_till, take_while1},
     character::{is_alphanumeric, is_digit},
     combinator::{map, map_opt, map_res, opt},
-    multi::{many1, many_m_n, separated_nonempty_list},
+    multi::{many1, many_m_n, separated_list1},
     number::complete::{le_i32, le_u32, le_u64, le_u8},
     IResult,
 };
 
 use crate::state::{
-    EventID, FSpaceID, FieldID, IPartID, ISpaceID, InstID, MapperCallKindID, MemID, OpID, ProcID,
-    RuntimeCallKindID, TaskID, Timestamp, TreeID, VariantID,
+    EventID, FSpaceID, FieldID, IPartID, ISpaceID, InstID, InstUID, MapperCallKindID, MemID,
+    NodeID, OpID, ProcID, RuntimeCallKindID, State, TaskID, Timestamp, TreeID, VariantID,
 };
 
 #[derive(Debug, Copy, Clone, PartialEq, Eq)]
@@ -81,12 +81,13 @@ pub struct Point(pub Vec<u64>);
 pub enum Record {
     MapperCallDesc { kind: MapperCallKindID, name: String },
     RuntimeCallDesc { kind: RuntimeCallKindID, name: String },
-    MetaDesc { kind: VariantID, name: String },
+    MetaDesc { kind: VariantID, message: bool, ordered_vc: bool, name: String },
     OpDesc { kind: u32, name: String },
-    ProcDesc { proc_id: ProcID, kind: ProcKind },
     MaxDimDesc { max_dim: MaxDim },
+    MachineDesc { node_id: NodeID, num_nodes: u32 },
+    ProcDesc { proc_id: ProcID, kind: ProcKind },
     MemDesc { mem_id: MemID, kind: MemKind, capacity: u64 },
-    ProcMDesc { proc_id: ProcID, mem_id: MemID },
+    ProcMDesc { proc_id: ProcID, mem_id: MemID, bandwidth: u32, latency: u32 },
     IndexSpacePointDesc { ispace_id: ISpaceID, dim: u32, rem: Point },
     IndexSpaceRectDesc { ispace_id: ISpaceID, dim: u32, rem: Array },
     IndexSpaceEmptyDesc { ispace_id: ISpaceID },
@@ -98,12 +99,13 @@ pub enum Record {
     IndexPartitionDesc { parent_id: ISpaceID, unique_id: IPartID, disjoint: bool, point0: u64 },
     IndexSpaceSizeDesc { ispace_id: ISpaceID, dense_size: u64, sparse_size: u64, is_sparse: bool },
     LogicalRegionDesc { ispace_id: ISpaceID, fspace_id: u32, tree_id: TreeID, name: String },
-    PhysicalInstRegionDesc { op_id: OpID, inst_id: InstID, ispace_id: ISpaceID, fspace_id: u32, tree_id: TreeID },
-    PhysicalInstLayoutDesc { op_id: OpID, inst_id: InstID, field_id: FieldID, fspace_id: u32, has_align: bool, eqk: u32, align_desc: u32 },
-    PhysicalInstDimOrderDesc { op_id: OpID, inst_id: InstID, dim: u32, dim_kind: u32 },
+    PhysicalInstRegionDesc { inst_uid: InstUID, ispace_id: ISpaceID, fspace_id: u32, tree_id: TreeID },
+    PhysicalInstLayoutDesc { inst_uid: InstUID, field_id: FieldID, fspace_id: u32, has_align: bool, eqk: u32, align_desc: u32 },
+    PhysicalInstDimOrderDesc { inst_uid: InstUID, dim: u32, dim_kind: u32 },
+    PhysicalInstanceUsage { inst_uid: InstUID, op_id: OpID, index_id: u32, field_id: FieldID },
     TaskKind { task_id: TaskID, name: String, overwrite: bool },
     TaskVariant { task_id: TaskID, variant_id: VariantID, name: String },
-    OperationInstance { op_id: OpID, kind: u32 },
+    OperationInstance { op_id: OpID, parent_id: OpID, kind: u32, provenance: String },
     MultiTask { op_id: OpID, task_id: TaskID },
     SliceOwner { parent_id: UniqueID, op_id: OpID },
     TaskWaitInfo { op_id: OpID, task_id: TaskID, variant_id: VariantID, wait_start: Timestamp, wait_ready: Timestamp, wait_end: Timestamp },
@@ -111,12 +113,11 @@ pub enum Record {
     TaskInfo { op_id: OpID, task_id: TaskID, variant_id: VariantID, proc_id: ProcID, create: Timestamp, ready: Timestamp, start: Timestamp, stop: Timestamp },
     GPUTaskInfo { op_id: OpID, task_id: TaskID, variant_id: VariantID, proc_id: ProcID, create: Timestamp, ready: Timestamp, start: Timestamp, stop: Timestamp, gpu_start: Timestamp, gpu_stop: Timestamp },
     MetaInfo { op_id: OpID, lg_id: VariantID, proc_id: ProcID, create: Timestamp, ready: Timestamp, start: Timestamp, stop: Timestamp },
-    CopyInfo { op_id: OpID, src: MemID, dst: MemID, size: u64, create: Timestamp, ready: Timestamp, start: Timestamp, stop: Timestamp, fevent: EventID, num_requests: u32 },
-    CopyInstInfo { op_id: OpID, src_inst: InstID, dst_inst: InstID, fevent: EventID, num_fields: u32, request_type: u32, num_hops: u32 },
-    FillInfo { op_id: OpID, dst: MemID, create: Timestamp, ready: Timestamp, start: Timestamp, stop: Timestamp },
-    InstCreateInfo { op_id: OpID, inst_id: InstID, create: Timestamp },
-    InstUsageInfo { op_id: OpID, inst_id: InstID, mem_id: MemID, size: u64 },
-    InstTimelineInfo { op_id: OpID, inst_id: InstID, create: Timestamp, destroy: Timestamp },
+    CopyInfo { op_id: OpID, size: u64, create: Timestamp, ready: Timestamp, start: Timestamp, stop: Timestamp, num_hops: u32, request_type: u32, fevent: EventID },
+    CopyInstInfo { src: MemID, dst: MemID, src_fid: FieldID, dst_fid: FieldID, src_inst: InstUID, dst_inst: InstUID, fevent: EventID, indirect: bool },
+    FillInfo { op_id: OpID, size: u64, create: Timestamp, ready: Timestamp, start: Timestamp, stop: Timestamp, fevent: EventID },
+    FillInstInfo { dst: MemID, fid: FieldID, dst_inst: InstUID, fevent: EventID },
+    InstTimelineInfo { inst_uid: InstUID, inst_id: InstID, mem_id: MemID, size: u64, op_id: OpID, create: Timestamp, ready: Timestamp, destroy: Timestamp },
     PartitionInfo { op_id: OpID, part_op: DepPartOpKind, create: Timestamp, ready: Timestamp, start: Timestamp, stop: Timestamp },
     MapperCallInfo { kind: MapperCallKindID, op_id: OpID, start: Timestamp, stop: Timestamp, proc_id: ProcID },
     RuntimeCallInfo { kind: RuntimeCallKindID, start: Timestamp, stop: Timestamp, proc_id: ProcID },
@@ -226,14 +227,7 @@ fn parse_field_format(input: &[u8]) -> IResult<&[u8], FieldFormat> {
     let (input, value) = parse_value_format(input)?;
     let (input, _) = tag(":")(input)?;
     let (input, size) = parse_text_i32(input)?;
-    Ok((
-        input,
-        FieldFormat {
-            name: name,
-            value: value,
-            size: size,
-        },
-    ))
+    Ok((input, FieldFormat { name, value, size }))
 }
 
 fn parse_record_format(input: &[u8]) -> IResult<&[u8], RecordFormat> {
@@ -241,17 +235,10 @@ fn parse_record_format(input: &[u8]) -> IResult<&[u8], RecordFormat> {
     let (input, _) = tag(" {id:")(input)?;
     let (input, id) = parse_text_u32(input)?;
     let (input, _) = tag(", ")(input)?;
-    let (input, fields) = separated_nonempty_list(tag(", "), parse_field_format)(input)?;
+    let (input, fields) = separated_list1(tag(", "), parse_field_format)(input)?;
     let (input, _) = tag("}")(input)?;
     let (input, _) = newline(input)?;
-    Ok((
-        input,
-        RecordFormat {
-            id: id,
-            name: name,
-            fields: fields,
-        },
-    ))
+    Ok((input, RecordFormat { id, name, fields }))
 }
 
 ///
@@ -287,50 +274,52 @@ fn parse_string(input: &[u8]) -> IResult<&[u8], String> {
 ///
 
 fn parse_event_id(input: &[u8]) -> IResult<&[u8], EventID> {
-    map(le_u64, |x| EventID(x))(input)
+    map(le_u64, EventID)(input)
+}
+fn parse_inst_uid(input: &[u8]) -> IResult<&[u8], InstUID> {
+    map(le_u64, InstUID)(input)
 }
-
 fn parse_inst_id(input: &[u8]) -> IResult<&[u8], InstID> {
-    map(le_u64, |x| InstID(x))(input)
+    map(le_u64, InstID)(input)
 }
 fn parse_ipart_id(input: &[u8]) -> IResult<&[u8], IPartID> {
-    map(le_u64, |x| IPartID(x))(input)
+    map(le_u64, IPartID)(input)
 }
 fn parse_ispace_id(input: &[u8]) -> IResult<&[u8], ISpaceID> {
-    map(le_u64, |x| ISpaceID(x))(input)
+    map(le_u64, ISpaceID)(input)
 }
 fn parse_fspace_id(input: &[u8]) -> IResult<&[u8], FSpaceID> {
-    map(le_u64, |x| FSpaceID(x))(input)
+    map(le_u64, FSpaceID)(input)
 }
 fn parse_field_id(input: &[u8]) -> IResult<&[u8], FieldID> {
-    map(le_u32, |x| FieldID(x))(input)
+    map(le_u32, FieldID)(input)
 }
 fn parse_tree_id(input: &[u8]) -> IResult<&[u8], TreeID> {
-    map(le_u32, |x| TreeID(x))(input)
+    map(le_u32, TreeID)(input)
 }
 fn parse_mapper_call_kind_id(input: &[u8]) -> IResult<&[u8], MapperCallKindID> {
-    map(le_u32, |x| MapperCallKindID(x))(input)
+    map(le_u32, MapperCallKindID)(input)
 }
 fn parse_mem_id(input: &[u8]) -> IResult<&[u8], MemID> {
-    map(le_u64, |x| MemID(x))(input)
+    map(le_u64, MemID)(input)
 }
 fn parse_op_id(input: &[u8]) -> IResult<&[u8], OpID> {
-    map(le_u64, |x| OpID(x))(input)
+    map(le_u64, OpID)(input)
 }
 fn parse_proc_id(input: &[u8]) -> IResult<&[u8], ProcID> {
-    map(le_u64, |x| ProcID(x))(input)
+    map(le_u64, ProcID)(input)
 }
 fn parse_runtime_call_kind_id(input: &[u8]) -> IResult<&[u8], RuntimeCallKindID> {
-    map(le_u32, |x| RuntimeCallKindID(x))(input)
+    map(le_u32, RuntimeCallKindID)(input)
 }
 fn parse_task_id(input: &[u8]) -> IResult<&[u8], TaskID> {
-    map(le_u32, |x| TaskID(x))(input)
+    map(le_u32, TaskID)(input)
 }
 fn parse_timestamp(input: &[u8]) -> IResult<&[u8], Timestamp> {
-    map(le_u64, |x| Timestamp(x))(input)
+    map(le_u64, Timestamp)(input)
 }
 fn parse_variant_id(input: &[u8]) -> IResult<&[u8], VariantID> {
-    map(le_u32, |x| VariantID(x))(input)
+    map(le_u32, VariantID)(input)
 }
 
 ///
@@ -349,23 +338,39 @@ fn parse_runtime_call_desc(input: &[u8], _max_dim: i32) -> IResult<&[u8], Record
 }
 fn parse_meta_desc(input: &[u8], _max_dim: i32) -> IResult<&[u8], Record> {
     let (input, kind) = parse_variant_id(input)?;
+    let (input, message) = parse_bool(input)?;
+    let (input, ordered_vc) = parse_bool(input)?;
     let (input, name) = parse_string(input)?;
-    Ok((input, Record::MetaDesc { kind, name }))
+    Ok((
+        input,
+        Record::MetaDesc {
+            kind,
+            message,
+            ordered_vc,
+            name,
+        },
+    ))
 }
 fn parse_op_desc(input: &[u8], _max_dim: i32) -> IResult<&[u8], Record> {
     let (input, kind) = le_u32(input)?;
     let (input, name) = parse_string(input)?;
     Ok((input, Record::OpDesc { kind, name }))
 }
+fn parse_max_dim_desc(input: &[u8], _max_dim: i32) -> IResult<&[u8], Record> {
+    let (input, max_dim) = le_i32(input)?;
+    Ok((input, Record::MaxDimDesc { max_dim }))
+}
+fn parse_machine_desc(input: &[u8], _max_dim: i32) -> IResult<&[u8], Record> {
+    let (input, nodeid) = le_u32(input)?;
+    let (input, num_nodes) = le_u32(input)?;
+    let node_id = NodeID(u64::from(nodeid));
+    Ok((input, Record::MachineDesc { node_id, num_nodes }))
+}
 fn parse_proc_desc(input: &[u8], _max_dim: i32) -> IResult<&[u8], Record> {
     let (input, proc_id) = parse_proc_id(input)?;
     let (input, kind) = le_i32(input)?;
     Ok((input, Record::ProcDesc { proc_id, kind }))
 }
-fn parse_max_dim_desc(input: &[u8], _max_dim: i32) -> IResult<&[u8], Record> {
-    let (input, max_dim) = le_i32(input)?;
-    Ok((input, Record::MaxDimDesc { max_dim }))
-}
 fn parse_mem_desc(input: &[u8], _max_dim: i32) -> IResult<&[u8], Record> {
     let (input, mem_id) = parse_mem_id(input)?;
     let (input, kind) = le_i32(input)?;
@@ -382,7 +387,17 @@ fn parse_mem_desc(input: &[u8], _max_dim: i32) -> IResult<&[u8], Record> {
 fn parse_mem_proc_affinity_desc(input: &[u8], _max_dim: i32) -> IResult<&[u8], Record> {
     let (input, proc_id) = parse_proc_id(input)?;
     let (input, mem_id) = parse_mem_id(input)?;
-    Ok((input, Record::ProcMDesc { proc_id, mem_id }))
+    let (input, bandwidth) = le_u32(input)?;
+    let (input, latency) = le_u32(input)?;
+    Ok((
+        input,
+        Record::ProcMDesc {
+            proc_id,
+            mem_id,
+            bandwidth,
+            latency,
+        },
+    ))
 }
 fn parse_index_space_point_desc(input: &[u8], max_dim: i32) -> IResult<&[u8], Record> {
     let (input, ispace_id) = parse_ispace_id(input)?;
@@ -501,16 +516,14 @@ fn parse_logical_region_desc(input: &[u8], _max_dim: i32) -> IResult<&[u8], Reco
     ))
 }
 fn parse_physical_inst_region_desc(input: &[u8], _max_dim: i32) -> IResult<&[u8], Record> {
-    let (input, op_id) = parse_op_id(input)?;
-    let (input, inst_id) = parse_inst_id(input)?;
+    let (input, inst_uid) = parse_inst_uid(input)?;
     let (input, ispace_id) = parse_ispace_id(input)?;
     let (input, fspace_id) = le_u32(input)?;
     let (input, tree_id) = parse_tree_id(input)?;
     Ok((
         input,
         Record::PhysicalInstRegionDesc {
-            op_id,
-            inst_id,
+            inst_uid,
             ispace_id,
             fspace_id,
             tree_id,
@@ -518,8 +531,7 @@ fn parse_physical_inst_region_desc(input: &[u8], _max_dim: i32) -> IResult<&[u8]
     ))
 }
 fn parse_physical_inst_layout_desc(input: &[u8], _max_dim: i32) -> IResult<&[u8], Record> {
-    let (input, op_id) = parse_op_id(input)?;
-    let (input, inst_id) = parse_inst_id(input)?;
+    let (input, inst_uid) = parse_inst_uid(input)?;
     let (input, field_id) = parse_field_id(input)?;
     let (input, fspace_id) = le_u32(input)?;
     let (input, has_align) = parse_bool(input)?;
@@ -528,8 +540,7 @@ fn parse_physical_inst_layout_desc(input: &[u8], _max_dim: i32) -> IResult<&[u8]
     Ok((
         input,
         Record::PhysicalInstLayoutDesc {
-            op_id,
-            inst_id,
+            inst_uid,
             field_id,
             fspace_id,
             has_align,
@@ -539,20 +550,33 @@ fn parse_physical_inst_layout_desc(input: &[u8], _max_dim: i32) -> IResult<&[u8]
     ))
 }
 fn parse_physical_inst_layout_dim_desc(input: &[u8], _max_dim: i32) -> IResult<&[u8], Record> {
-    let (input, op_id) = parse_op_id(input)?;
-    let (input, inst_id) = parse_inst_id(input)?;
+    let (input, inst_uid) = parse_inst_uid(input)?;
     let (input, dim) = le_u32(input)?;
     let (input, dim_kind) = le_u32(input)?;
     Ok((
         input,
         Record::PhysicalInstDimOrderDesc {
-            op_id,
-            inst_id,
+            inst_uid,
             dim,
             dim_kind,
         },
     ))
 }
+fn parse_physical_inst_usage(input: &[u8], _max_dim: i32) -> IResult<&[u8], Record> {
+    let (input, inst_uid) = parse_inst_uid(input)?;
+    let (input, op_id) = parse_op_id(input)?;
+    let (input, index_id) = le_u32(input)?;
+    let (input, field_id) = parse_field_id(input)?;
+    Ok((
+        input,
+        Record::PhysicalInstanceUsage {
+            inst_uid,
+            op_id,
+            index_id,
+            field_id,
+        },
+    ))
+}
 fn parse_task_kind(input: &[u8], _max_dim: i32) -> IResult<&[u8], Record> {
     let (input, task_id) = parse_task_id(input)?;
     let (input, name) = parse_string(input)?;
@@ -581,8 +605,18 @@ fn parse_task_variant(input: &[u8], _max_dim: i32) -> IResult<&[u8], Record> {
 }
 fn parse_operation(input: &[u8], _max_dim: i32) -> IResult<&[u8], Record> {
     let (input, op_id) = parse_op_id(input)?;
+    let (input, parent_id) = parse_op_id(input)?;
     let (input, kind) = le_u32(input)?;
-    Ok((input, Record::OperationInstance { op_id, kind }))
+    let (input, provenance) = parse_string(input)?;
+    Ok((
+        input,
+        Record::OperationInstance {
+            op_id,
+            parent_id,
+            kind,
+            provenance,
+        },
+    ))
 }
 fn parse_multi_task(input: &[u8], _max_dim: i32) -> IResult<&[u8], Record> {
     let (input, op_id) = parse_op_id(input)?;
@@ -703,110 +737,107 @@ fn parse_meta_info(input: &[u8], _max_dim: i32) -> IResult<&[u8], Record> {
 }
 fn parse_copy_info(input: &[u8], _max_dim: i32) -> IResult<&[u8], Record> {
     let (input, op_id) = parse_op_id(input)?;
-    let (input, src) = parse_mem_id(input)?;
-    let (input, dst) = parse_mem_id(input)?;
     let (input, size) = le_u64(input)?;
     let (input, create) = parse_timestamp(input)?;
     let (input, ready) = parse_timestamp(input)?;
     let (input, start) = parse_timestamp(input)?;
     let (input, stop) = parse_timestamp(input)?;
+    let (input, num_hops) = le_u32(input)?;
+    let (input, request_type) = le_u32(input)?;
     let (input, fevent) = parse_event_id(input)?;
-    let (input, num_requests) = le_u32(input)?;
     Ok((
         input,
         Record::CopyInfo {
             op_id,
-            src,
-            dst,
             size,
             create,
             ready,
             start,
             stop,
+            num_hops,
+            request_type,
             fevent,
-            num_requests,
         },
     ))
 }
 fn parse_copy_inst_info(input: &[u8], _max_dim: i32) -> IResult<&[u8], Record> {
-    let (input, op_id) = parse_op_id(input)?;
-    let (input, src_inst) = parse_inst_id(input)?;
-    let (input, dst_inst) = parse_inst_id(input)?;
+    let (input, src) = parse_mem_id(input)?;
+    let (input, dst) = parse_mem_id(input)?;
+    let (input, src_fid) = parse_field_id(input)?;
+    let (input, dst_fid) = parse_field_id(input)?;
+    let (input, src_inst) = parse_inst_uid(input)?;
+    let (input, dst_inst) = parse_inst_uid(input)?;
     let (input, fevent) = parse_event_id(input)?;
-    let (input, num_fields) = le_u32(input)?;
-    let (input, request_type) = le_u32(input)?;
-    let (input, num_hops) = le_u32(input)?;
+    let (input, indirect) = parse_bool(input)?;
     Ok((
         input,
         Record::CopyInstInfo {
-            op_id,
+            src,
+            dst,
+            src_fid,
+            dst_fid,
             src_inst,
             dst_inst,
             fevent,
-            num_fields,
-            request_type,
-            num_hops,
+            indirect,
         },
     ))
 }
 fn parse_fill_info(input: &[u8], _max_dim: i32) -> IResult<&[u8], Record> {
     let (input, op_id) = parse_op_id(input)?;
-    let (input, dst) = parse_mem_id(input)?;
+    let (input, size) = le_u64(input)?;
     let (input, create) = parse_timestamp(input)?;
     let (input, ready) = parse_timestamp(input)?;
     let (input, start) = parse_timestamp(input)?;
     let (input, stop) = parse_timestamp(input)?;
+    let (input, fevent) = parse_event_id(input)?;
     Ok((
         input,
         Record::FillInfo {
             op_id,
-            dst,
+            size,
             create,
             ready,
             start,
             stop,
+            fevent,
         },
     ))
 }
-fn parse_inst_create(input: &[u8], _max_dim: i32) -> IResult<&[u8], Record> {
-    let (input, op_id) = parse_op_id(input)?;
-    let (input, inst_id) = parse_inst_id(input)?;
-    let (input, create) = parse_timestamp(input)?;
+fn parse_fill_inst_info(input: &[u8], _max_dim: i32) -> IResult<&[u8], Record> {
+    let (input, dst) = parse_mem_id(input)?;
+    let (input, fid) = parse_field_id(input)?;
+    let (input, dst_inst) = parse_inst_uid(input)?;
+    let (input, fevent) = parse_event_id(input)?;
     Ok((
         input,
-        Record::InstCreateInfo {
-            op_id,
-            inst_id,
-            create,
-        },
-    ))
-}
-fn parse_inst_usage(input: &[u8], _max_dim: i32) -> IResult<&[u8], Record> {
-    let (input, op_id) = parse_op_id(input)?;
-    let (input, inst_id) = parse_inst_id(input)?;
-    let (input, mem_id) = parse_mem_id(input)?;
-    let (input, size) = le_u64(input)?;
-    Ok((
-        input,
-        Record::InstUsageInfo {
-            op_id,
-            inst_id,
-            mem_id,
-            size,
+        Record::FillInstInfo {
+            dst,
+            fid,
+            dst_inst,
+            fevent,
         },
     ))
 }
 fn parse_inst_timeline(input: &[u8], _max_dim: i32) -> IResult<&[u8], Record> {
-    let (input, op_id) = parse_op_id(input)?;
+    let (input, inst_uid) = parse_inst_uid(input)?;
     let (input, inst_id) = parse_inst_id(input)?;
+    let (input, mem_id) = parse_mem_id(input)?;
+    let (input, size) = le_u64(input)?;
+    let (input, op_id) = parse_op_id(input)?;
     let (input, create) = parse_timestamp(input)?;
+    let (input, ready) = parse_timestamp(input)?;
     let (input, destroy) = parse_timestamp(input)?;
     Ok((
         input,
         Record::InstTimelineInfo {
-            op_id,
+            inst_uid,
             inst_id,
+            mem_id,
+            size,
+            op_id,
             create,
+            ready,
             destroy,
         },
     ))
@@ -878,6 +909,50 @@ fn parse_proftask_info(input: &[u8], _max_dim: i32) -> IResult<&[u8], Record> {
     ))
 }
 
+fn filter_record<'a>(
+    record: &'a Record,
+    visible_nodes: &'a Vec<NodeID>,
+    node_id: Option<NodeID>,
+) -> bool {
+    assert!(!visible_nodes.is_empty());
+    if let Some(nodeid) = node_id {
+        if !visible_nodes.contains(&nodeid) {
+            match record {
+                Record::ProcDesc { .. } => true,
+                Record::MemDesc { .. } => true,
+                Record::ProcMDesc { .. } => true,
+                Record::TaskInfo { proc_id, .. } => {
+                    State::is_on_visible_nodes(visible_nodes, proc_id.node_id())
+                }
+                Record::GPUTaskInfo { proc_id, .. } => {
+                    State::is_on_visible_nodes(visible_nodes, proc_id.node_id())
+                }
+                Record::MetaInfo { proc_id, .. } => {
+                    State::is_on_visible_nodes(visible_nodes, proc_id.node_id())
+                }
+                Record::CopyInfo { .. } => true,
+                Record::CopyInstInfo { src, dst, .. } => {
+                    State::is_on_visible_nodes(visible_nodes, src.node_id())
+                        || State::is_on_visible_nodes(visible_nodes, dst.node_id())
+                }
+                Record::FillInfo { .. } => true,
+                Record::FillInstInfo { dst, .. } => {
+                    State::is_on_visible_nodes(visible_nodes, dst.node_id())
+                }
+                Record::InstTimelineInfo { mem_id, .. } => {
+                    State::is_on_visible_nodes(visible_nodes, mem_id.node_id())
+                }
+                Record::PartitionInfo { .. } => true,
+                _ => false,
+            }
+        } else {
+            true
+        }
+    } else {
+        true
+    }
+}
+
 fn parse_record<'a>(
     input: &'a [u8],
     parsers: &BTreeMap<u32, fn(&[u8], i32) -> IResult<&[u8], Record>>,
@@ -888,7 +963,11 @@ fn parse_record<'a>(
     parser(input, max_dim)
 }
 
-fn parse(input: &[u8]) -> IResult<&[u8], Vec<Record>> {
+fn parse<'a>(
+    input: &'a [u8],
+    visible_nodes: &'a Vec<NodeID>,
+    filter_input: bool,
+) -> IResult<&'a [u8], Vec<Record>> {
     let (input, version) = parse_filetype(input)?;
     assert_eq!(version, (1, 0));
     let (input, record_formats) = many1(parse_record_format)(input)?;
@@ -903,8 +982,9 @@ fn parse(input: &[u8]) -> IResult<&[u8], Vec<Record>> {
     parsers.insert(ids["RuntimeCallDesc"], parse_runtime_call_desc);
     parsers.insert(ids["MetaDesc"], parse_meta_desc);
     parsers.insert(ids["OpDesc"], parse_op_desc);
-    parsers.insert(ids["ProcDesc"], parse_proc_desc);
     parsers.insert(ids["MaxDimDesc"], parse_max_dim_desc);
+    parsers.insert(ids["MachineDesc"], parse_machine_desc);
+    parsers.insert(ids["ProcDesc"], parse_proc_desc);
     parsers.insert(ids["MemDesc"], parse_mem_desc);
     parsers.insert(ids["ProcMDesc"], parse_mem_proc_affinity_desc);
     parsers.insert(ids["IndexSpacePointDesc"], parse_index_space_point_desc);
@@ -930,6 +1010,7 @@ fn parse(input: &[u8]) -> IResult<&[u8], Vec<Record>> {
         ids["PhysicalInstDimOrderDesc"],
         parse_physical_inst_layout_dim_desc,
     );
+    parsers.insert(ids["PhysicalInstanceUsage"], parse_physical_inst_usage);
     parsers.insert(ids["TaskKind"], parse_task_kind);
     parsers.insert(ids["TaskVariant"], parse_task_variant);
     parsers.insert(ids["OperationInstance"], parse_operation);
@@ -943,8 +1024,7 @@ fn parse(input: &[u8]) -> IResult<&[u8], Vec<Record>> {
     parsers.insert(ids["CopyInfo"], parse_copy_info);
     parsers.insert(ids["CopyInstInfo"], parse_copy_inst_info);
     parsers.insert(ids["FillInfo"], parse_fill_info);
-    parsers.insert(ids["InstCreateInfo"], parse_inst_create);
-    parsers.insert(ids["InstUsageInfo"], parse_inst_usage);
+    parsers.insert(ids["FillInstInfo"], parse_fill_inst_info);
     parsers.insert(ids["InstTimelineInfo"], parse_inst_timeline);
     parsers.insert(ids["PartitionInfo"], parse_partition_info);
     parsers.insert(ids["MapperCallInfo"], parse_mapper_call_info);
@@ -953,31 +1033,33 @@ fn parse(input: &[u8]) -> IResult<&[u8], Vec<Record>> {
 
     let mut input = input;
     let mut max_dim = -1;
+    let mut node_id: Option<NodeID> = None;
     let mut records = Vec::new();
-    loop {
-        match parse_record(input, &parsers, max_dim) {
-            Ok((input_, record)) => {
-                match &record {
-                    Record::MaxDimDesc { max_dim: d } => {
-                        max_dim = *d;
-                    }
-                    _ => {}
-                }
-                records.push(record);
-                input = input_;
-            }
-            _ => break,
+    while let Ok((input_, record)) = parse_record(input, &parsers, max_dim) {
+        if let Record::MaxDimDesc { max_dim: d } = &record {
+            max_dim = *d;
+        }
+        if let Record::MachineDesc { node_id: d, .. } = &record {
+            node_id = Some(*d);
+        }
+        input = input_;
+        if !filter_input || filter_record(&record, visible_nodes, node_id) {
+            records.push(record);
         }
     }
     Ok((input, records))
 }
 
-pub fn deserialize<P: AsRef<Path>>(path: P) -> io::Result<Vec<Record>> {
+pub fn deserialize<P: AsRef<Path>>(
+    path: P,
+    visible_nodes: &Vec<NodeID>,
+    filter_input: bool,
+) -> io::Result<Vec<Record>> {
     let mut gz = GzDecoder::new(File::open(path)?);
     let mut s = Vec::<u8>::new();
     gz.read_to_end(&mut s)?;
     // throw error here if parse failed
-    let (rest, records) = parse(&s).unwrap();
+    let (rest, records) = parse(&s, visible_nodes, filter_input).unwrap();
     assert_eq!(rest.len(), 0);
     Ok(records)
 }
